require 'spec_helper'
require 'puppet_x/puppetlabs/meep/config'

module PuppetX::Puppetlabs::Meep
  describe "Meep config modification" do
    include_context 'meep_config'

    let(:modify) { Modify.new(meep_config_path) }
    let(:pe_conf_path) { "#{meep_config_path}/conf.d/pe.conf" }

    context "#set_in_pe_conf" do
      context "when pe.conf exists" do
        let(:layout) { 'set-test.tmp' }
        let(:hieradata) do
          <<-HIERADATA
# This example deliberately uses HOCON features to ensure we can work outside
# of valid JSON

#--------------------------------------------------------------------------
# Required. The password to log into the Puppet Enterprise console.
#--------------------------------------------------------------------------
"console_admin_password": "puppet"

# These are some initial comments about node_roles
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}

# These are some comments about agent_platforms
"agent_platforms": [
     "%{::platform_tag}",
  #  "aix_53_power",
  #  "aix_61_power",
  #  "aix_71_power",
]

# Advanced parameters
#"pe_install::puppet_master_dnsaltnames": ["puppet"]
          HIERADATA
        end

        around(:each) do |example|
          run_with_custom_hieradata(layout, hieradata, &example)
        end

        def test_setting_key(key, value)
          expect(modify.set_in_pe_conf(key, value)).to eq(true)
          expect(modify.get_in_pe_conf(key)).to eq(value)
        end

        def test_raw_pe_conf_after_set(key, value, gold_data)
          expect(modify.set_in_pe_conf(key, value)).to eq(true)
          raw_pe_conf = File.read(pe_conf_path)
          expect(raw_pe_conf).to eq(gold_data)
        end

        it "can set strings" do
          test_setting_key("new::parameter", "new_value")
        end

        it "can set numbers" do
          test_setting_key("new::parameter", 123)
        end

        it "can set booleans" do
          test_setting_key("new::parameter", true)
        end

        it "can set arrays" do
          test_setting_key("new::parameter", ["value",123, false])
        end

        it "can set hashes" do
          test_setting_key("new::parameter", {"foo" => { "bar" => "baz" }})
        end

        it "can update existing keys" do
          test_setting_key("console_admin_password", "foobar")
          test_setting_key("node_roles",
            {
              "pe_role::monolithic::primary_master" => ["master.rspec"],
              "pe_role::monolithic::compile_master" => ["compile.rspec"],
            }
          )
          test_setting_key("agent_platforms",
            [
              'el_7_x86_64',
              'el_6_x86_64',
            ]
          )
        end

        it "can update an element within a hash" do
          compound_key = '"node_roles"."pe_role::monolithic::primary_master"'

          expect(modify.set_in_pe_conf(
            compound_key,
            ['master.rspec', 'other.rspec'])
          ).to eq(true)

          expect(modify.get_in_pe_conf("node_roles")).to match(
            {
              "pe_role::monolithic::primary_master" => match_array(['master.rspec', 'other.rspec'])
            }
          )
        end

        context "with existing comments" do
          it "preserves comments when adding a new key" do
            test_raw_pe_conf_after_set(
              "foo",
              "bar",
              hieradata << %Q{"foo" : "bar"\n}
            )
          end

          it "preserves comments when updating an existing key" do
            gold = hieradata.gsub(
              /^"console_admin_password": "puppet"\n/,
              %Q{"console_admin_password": "foobar"\n}
            )
            test_raw_pe_conf_after_set(
              "console_admin_password",
              "foobar",
              gold
            )
          end

          it "preserves comments when updating an existing hash" do
            gold = hieradata.gsub(
              %r|^"node_roles": {\n.*?}\n|m,
              <<-EOS
"node_roles": {
    "pe_role::monolithic::primary_master" : [
        "master.rspec"
    ],
    "pe_role::monolithic::primary_master_replica" : [
        "replica.rspec"
    ]
}
              EOS
            )

            test_raw_pe_conf_after_set(
              "node_roles",
              {
                "pe_role::monolithic::primary_master" => ["master.rspec"],
                "pe_role::monolithic::primary_master_replica" => ["replica.rspec"],
              },
              gold
            )
          end

          it "looses comments embedded in a complex value" do
            gold = hieradata.gsub(
              %r|^"agent_platforms": \[\n.*?\]\n|m,
              <<-EOS
"agent_platforms": [
    "el_7_x86_64",
    "el_6_x86_64"
]
              EOS
            )

            test_raw_pe_conf_after_set(
              "agent_platforms",
              [
                'el_7_x86_64',
                'el_6_x86_64',
              ],
              gold
            )
          end
        end

        context "with a non-standard pe.conf" do
          let(:hieradata) do
            <<-HIERADATA
# Perhaps the user has removed some quoting around keys...
{
  #--------------------------------------------------------------------------
  # Required. The password to log into the Puppet Enterprise console.
  #--------------------------------------------------------------------------
  console_admin_password: "puppet"

  # These are some initial comments about node_roles
  node_roles: {
    "pe_role::monolithic::primary_master": ["master.rspec"]
  }
}
            HIERADATA
          end

          it "still updates the correct key" do
            gold = hieradata.gsub(
              /^  console_admin_password: "puppet"\n/,
              %Q{  console_admin_password: "foobar"\n}
            )
            test_raw_pe_conf_after_set(
              "console_admin_password",
              "foobar",
              gold
            )
          end

          it "still updates the correct object" do
            gold = hieradata.gsub(
              %r|^  node_roles: {\n.*?}\n|m,
              <<-EOS
  node_roles: {
      "pe_role::monolithic::primary_master" : [
          "master.rspec"
      ],
      "pe_role::monolithic::primary_master_replica" : [
          "replica.rspec"
      ]
  }
              EOS
            )

            test_raw_pe_conf_after_set(
              "node_roles",
              {
                "pe_role::monolithic::primary_master" => ["master.rspec"],
                "pe_role::monolithic::primary_master_replica" => ["replica.rspec"],
              },
              gold
            )
          end

          it "can still updates an element within a hash" do
            compound_key = '"node_roles"."pe_role::monolithic::primary_master"'

            gold = hieradata.gsub(
              %r|^  node_roles: {\n.*?}\n|m,
              <<-EOS
  node_roles: {
    "pe_role::monolithic::primary_master": [
        "other.rspec"
    ]
  }
              EOS
            )

            test_raw_pe_conf_after_set(
              compound_key,
              ["other.rspec"],
              gold
            )
          end
        end

        context "with a mangled pe.conf" do
          let(:hieradata) { "invalid" }

          it "raises an error" do
            expect { modify.set_in_pe_conf("foo", "bar") }.to(
              raise_error(
                Hocon::ConfigError::ConfigParseError,
                /pe\.conf.*Key 'invalid' may not be followed by token: end of file/
              )
            )
          end
        end

        context "with an empty pe.conf" do
          let(:hieradata) { "" }

          it "can still set values" do
            test_setting_key("new::parameter", "new_value")
          end
        end
      end

      context "with a missing pe.conf" do
        let(:meep_config_path) { "/no/meep/dir" }

        it 'raises an error' do
          expect { modify.set_in_pe_conf("foo", "bar") }.to(
            raise_error(Puppet::Error, /No pe.conf file.*no.meep.dir/)
          )
        end
      end
    end

    context "#set_role" do
      let(:layout) { 'set-test.tmp' }
      let(:hieradata) do
        <<-HIERADATA
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": ["compile.rspec"]
  }
}
        HIERADATA
      end

      around(:each) do |example|
        run_with_custom_hieradata(layout, hieradata, &example)
      end

      def test_raw_pe_conf_after_set_role(role, value, gold_data)
        expect(modify.set_role(role, value)).to eq(true)
        raw_pe_conf = File.read(pe_conf_path)
        expect(raw_pe_conf).to eq(gold_data)
      end

      it "resets an existing role" do
        gold = <<-GOLD
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": [
        "compile1.rspec",
        "compile2.rspec"
    ]
  }
}
        GOLD

        test_raw_pe_conf_after_set_role(
          "pe_role::monolithic::compile_master",
          ["compile1.rspec", "compile2.rspec"],
          gold
        )
      end

      it "adds a new role" do
        gold = <<-GOLD
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": ["compile.rspec"]
    "pe_role::monolithic::primary_master_replica" : [
        "replica.rspec"
    ]
  }
}
        GOLD

        test_raw_pe_conf_after_set_role(
          "pe_role::monolithic::primary_master_replica",
          ["replica.rspec"],
          gold
        )
      end

      it "raises an error if it is not passed an array" do
        expect { modify.set_role("pe_role::monolithic::primary_master_replica", "replica.rspec") }.to(
          raise_error(Puppet::Error, /Expected an array/)
        )
      end

      context "with custom_roles" do
        let(:hieradata) do
          <<-HIERADATA
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": ["compile.rspec"]
  }

  "custom_roles": {
    "compile_with_puppetdb": [
      "puppet_enterprise::profile::master",
      "puppet_enterprise::profile::puppetdb"
    ]
  }
}
          HIERADATA
        end

        it "adds a custom role" do
          gold = hieradata.gsub(
            /(compile.rspec.*?\n)/,
            <<-GOLD
\\1    "compile_with_puppetdb" : [
        "custom.rspec"
    ]
            GOLD
          )

          test_raw_pe_conf_after_set_role(
            "compile_with_puppetdb",
            ["custom.rspec"],
            gold
          )
        end

        it "raises an error if role is not default or custom" do
          expect { modify.set_role("does_not_exist", ["node"]) }.to(
            raise_error(Puppet::Error, /Role.*does_not_exist.*is not a default role.*not defined.*custom_roles/)
          )
        end
      end
    end

    context "#add_to_role" do
      let(:layout) { 'add-test.tmp' }
      let(:hieradata) do
        <<-HIERADATA
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": ["compile.rspec"]
  }
}
        HIERADATA
      end

      around(:each) do |example|
        run_with_custom_hieradata(layout, hieradata, &example)
      end

      def test_add_to_role(role, elements, gold)
        expect(modify.add_to_role(role, elements)).to eq(true)
        expect(modify.get_in_pe_conf('node_roles')[role]).to eq(gold)
      end

      it "adds a cert to an existing role" do
        test_add_to_role(
          "pe_role::monolithic::compile_master",
          "compile2.rspec",
          ["compile.rspec", "compile2.rspec"]
        )
      end

      it "adds an array of certs to an existing role" do
        test_add_to_role(
          "pe_role::monolithic::compile_master",
          ["compile2.rspec", "compile3.rspec"],
          ["compile.rspec", "compile2.rspec", "compile3.rspec"]
        )
      end

      context "when there are duplicates" do
        it "does not duplicate certs" do
          test_add_to_role(
            "pe_role::monolithic::compile_master",
            "compile.rspec",
            ["compile.rspec"]
          )
        end

        it "adds an array without duplicating certs" do
          test_add_to_role(
            "pe_role::monolithic::compile_master",
            ["compile.rspec", "compile2.rspec", "compile3.rspec"],
            ["compile.rspec", "compile2.rspec", "compile3.rspec"]
          )
        end

        it "preserves original order when pruning duplicates" do
          test_add_to_role(
            "pe_role::monolithic::compile_master",
            ["compile2.rspec", "compile3.rspec", "compile.rspec"],
            ["compile.rspec", "compile2.rspec", "compile3.rspec"]
          )
        end
      end

      context "when no role exists" do
        it "creates and sets the role" do
          expect(modify.get_in_pe_conf('node_roles')["pe_role::monolithic::primary_master_replica"]).to eq(nil)
          test_add_to_role(
            "pe_role::monolithic::primary_master_replica",
            "replica.rspec",
            ["replica.rspec"]
          )
        end

        it "creates the role and sets to given array" do
          expect(modify.get_in_pe_conf('node_roles')["pe_role::monolithic::primary_master_replica"]).to eq(nil)
          test_add_to_role(
            "pe_role::monolithic::primary_master_replica",
            ["replica.rspec"],
            ["replica.rspec"]
          )
        end
      end

      it "raises an error if role is not default or custom" do
        expect { modify.add_to_role("does_not_exist", ['node']) }.to(
          raise_error(Puppet::Error, /Role.*does_not_exist.*is not a default role.*not defined.*custom_roles/)
        )
      end
    end

    context "#remove_from_role" do
      let(:layout) { 'add-test.tmp' }
      let(:hieradata) do
        <<-HIERADATA
{
  "node_roles": {
    "pe_role::monolithic::primary_master": ["master.rspec"]
    "pe_role::monolithic::compile_master": ["compile1.rspec", "compile2.rspec", "compile3.rspec"]
  }
}
        HIERADATA
      end

      around(:each) do |example|
        run_with_custom_hieradata(layout, hieradata, &example)
      end

      def test_remove_from_role(role, elements, gold)
        expect(modify.remove_from_role(role, elements)).to eq(true)
        expect(modify.get_in_pe_conf('node_roles')[role]).to eq(gold)
      end

      it "removes a cert from an existing role" do
        test_remove_from_role(
          "pe_role::monolithic::compile_master",
          "compile1.rspec",
          ["compile2.rspec", "compile3.rspec"]
        )
      end

      it "removes an array of certs from an existing role" do
        test_remove_from_role(
          "pe_role::monolithic::compile_master",
          ["compile2.rspec", "compile3.rspec"],
          ["compile1.rspec"]
        )
      end

      it "succeeds if certs are already absent" do
        test_remove_from_role(
          "pe_role::monolithic::compile_master",
          "compile_other.rspec",
          ["compile1.rspec", "compile2.rspec", "compile3.rspec"]
        )
      end

      it "succeeds if some certs are already absent" do
        test_remove_from_role(
          "pe_role::monolithic::compile_master",
          ["compile3.rspec", "compile_other.rspec"],
          ["compile1.rspec", "compile2.rspec"]
        )
      end

      it "succeeds if role is defined but absent and does not add an empty role" do
        test_remove_from_role(
          "pe_role::monolithic::primary_master_replica",
          "replica.rspec",
          nil
          # The fact that we get back nil when querying the role indicates the role
          # was not added to node_roles, since otherwise it would be an empty array.
        )
      end

      it "does not remove an empty role if role was present" do
        expect(modify.set_role("pe_role::monolithic::primary_master_replica", [])).to eq(true)
        test_remove_from_role(
          "pe_role::monolithic::primary_master_replica",
          "replica.rspec",
          []
          # The fact that we get back an empty array indicates role is still present
        )
      end

      it "raises an error if role is not default or custom" do
        expect { modify.remove_from_role("does_not_exist", ['node']) }.to(
          raise_error(Puppet::Error, /Role.*does_not_exist.*is not a default role.*not defined.*custom_roles/)
        )
      end
    end

    context "with contention" do
      let(:layout) { 'set-contention-test.tmp' }
      let(:hieradata) do
        <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
  "pe_role::monolithic::compile_master": ["compile.rspec"]
}
        HIERADATA
      end
      let(:timeout) { 0.5 }

      around(:each) do |example|
        begin
          PuppetX::Puppetlabs::Meep::Modify::SafeMutator.timeout = timeout
          run_with_custom_hieradata(layout, hieradata, &example)
        ensure
          PuppetX::Puppetlabs::Meep::Modify::SafeMutator.timeout = nil
        end
      end

      context "when the file is locked longer than the timeout" do
        around(:each) do |example|
          begin
            pe_conf = File.open(pe_conf_path, 'r')
            pe_conf.flock(::File::LOCK_EX)
            example.run
          ensure
            pe_conf.close
          end
        end

        def check_lock(&block)
          expect { block.call }.to(
            raise_error(Puppet::Error, /Unable to get a lock on .*set-contention-test\.tmp.*pe.conf within #{timeout}s/)
          )
        end

        it "raises an error attempting to set_in_pe_conf" do
          check_lock { modify.set_in_pe_conf("foo", "bar") }
        end

        it "raises an error attempting to set_role" do
          check_lock { modify.set_role("pe_role::monolithic::compile_master", ['compile1.rspec']) }
        end

        it "raises an error attempting to add_to_role" do
          check_lock { modify.add_to_role("pe_role::monolithic::compile_master", 'compile2.rspec') }
        end

        it "raises an error attempting to remove_from_role" do
          check_lock { modify.remove_from_role("pe_role::monolithic::compile_master", 'compile2.rspec') }
        end
      end

      def test_lock_timeout(gold, &action)
        t1 = Thread.new do
          File.open(pe_conf_path, 'r+') do |fh|
            fh.flock(::File::LOCK_EX | ::File::LOCK_NB)
            state = fh.read
            state = state.gsub(/"compile\.rspec"/, '"compile.rspec","other.rspec"')
            fh.rewind
            fh.puts state
            Thread.stop
          end
        end

        sleep 0.1

        t2 = Thread.new do
          expect {
            action.call
          }.to(
            raise_error(Puppet::Error, /Unable to get a lock/)
          )
        end

        # Since t1 is stopped, t2 times out
        t2.join

        # Let t1 complete
        t1.run
        t1.join

        expect(File.read(pe_conf_path)).to match(gold)
      end

      def test_concurrent_access(gold, &action)
        t1 = Thread.new do
          File.open(pe_conf_path, 'r+') do |fh|
            fh.flock(::File::LOCK_EX | ::File::LOCK_NB)
            state = fh.read
            state = state.gsub(/"compile\.rspec"/, '"compile.rspec","other.rspec"')
            fh.rewind
            fh.puts state
            # still within timeout
            sleep 0.3
          end
        end

        sleep 0.1

        t2 = Thread.new do
          expect(
            action.call
          ).to eq(true)
        end

        t2.join
        t1.join

        expect(File.read(pe_conf_path)).to match(gold)
      end

      context "#add_to_role" do
        it "raises on conflict if it times out, allowing initial caller to complete" do
          gold = hieradata.gsub(/"compile\.rspec"/, '"compile.rspec","other.rspec"')

          test_lock_timeout(gold) do
            modify.add_to_role("pe_role::monolithic::compile_master", "compile1.rspec")
          end
        end

        it "waits for an initial caller to complete" do
          gold = hieradata.gsub(
          /"compile\.rspec"\]\n/,
          <<-EOS
\n      "compile.rspec",
      "other.rspec",
      "compile_other_thread.rspec"
  ]
          EOS
          )

          test_concurrent_access(gold) do
            modify.add_to_role(
              "pe_role::monolithic::compile_master",
              "compile_other_thread.rspec"
            )
          end
        end
      end

      context "#remove_from_role" do
        it "raises on conflict if it times out, allowing initial caller to complete" do
          gold = hieradata.gsub(/"compile\.rspec"/, '"compile.rspec","other.rspec"')

          test_lock_timeout(gold) do
            modify.remove_from_role("pe_role::monolithic::compile_master", "compile.rspec")
          end
        end

        it "waits for an initial caller to complete" do
          gold = hieradata.gsub(
          /"compile\.rspec"\]\n/,
          <<-EOS
\n      "other.rspec"
  ]
          EOS
          )

          test_concurrent_access(gold) do
            modify.remove_from_role(
              "pe_role::monolithic::compile_master",
              "compile.rspec"
            )
          end
        end
      end
    end
  end
end
