require 'spec_helper'
require 'puppet_x/puppetlabs/meep/config'

module PuppetX::Puppetlabs::Meep
  describe 'MEEP Services interface' do
    include_context 'meep_config'
    include_context 'Config.local prep'

    let(:node) { 'master.rspec' }
    let(:config) { Config.local }

    def expected_service(service, certname, overrides = {})
      services = {
        'puppetserver' => {
          'display_name'  => 'Puppet Server',
          'status_key'    => 'pe-master',
          'status_port'   => 8140,
          'status_prefix' => 'status',
          'port'          => 8140,
          'prefix'        => '',
          'protocol'      => 'https',
          'type'          => 'master',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8140/status",
          'url'           => "https://#{certname}:8140/",
        },
        'file-sync-storage' => {
          'display_name'  => 'File Sync Storage Service',
          'status_key'    => 'file-sync-storage-service',
          'status_port'   => 8140,
          'status_prefix' => 'status',
          'port'          => 8140,
          'prefix'        => '',
          'protocol'      => 'https',
          'type'          => 'file-sync-storage',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8140/status",
          'url'           => "https://#{certname}:8140/",
        },
        'file-sync-client' => {
          'display_name'  => 'File Sync Client Service',
          'status_key'    => 'file-sync-client-service',
          'status_port'   => 8140,
          'status_prefix' => 'status',
          'port'          => 8140,
          'prefix'        => '',
          'protocol'      => 'https',
          'type'          => 'file-sync-client',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8140/status",
          'url'           => "https://#{certname}:8140/",
        },
        'puppetdb' => {
          'display_name'  => 'PuppetDB',
          'status_key'    => 'puppetdb-status',
          'status_port'   => 8081,
          'status_prefix' => 'status',
          'port'          => 8081,
          'prefix'        => 'pdb',
          'protocol'      => 'https',
          'type'          => 'puppetdb',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8081/status",
          'url'           => "https://#{certname}:8081/pdb",
        },
        'classifier' => {
          'display_name'  => 'Classifier',
          'status_key'    => 'classifier-service',
          'status_port'   => 4433,
          'status_prefix' => 'status',
          'port'          => 4433,
          'prefix'        => 'classifier-api',
          'protocol'      => 'https',
          'type'          => 'classifier',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:4433/status",
          'url'           => "https://#{certname}:4433/classifier-api",
        },
        'activity' => {
          'display_name'  => 'Activity Service',
          'status_key'    => 'activity-service',
          'status_port'   => 4433,
          'status_prefix' => 'status',
          'port'          => 4433,
          'prefix'        => 'activity-api',
          'protocol'      => 'https',
          'type'          => 'activity',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:4433/status",
          'url'           => "https://#{certname}:4433/activity-api",
        },
        'rbac' => {
          'display_name'  => 'RBAC',
          'status_key'    => 'rbac-service',
          'status_port'   => 4433,
          'status_prefix' => 'status',
          'port'          => 4433,
          'prefix'        => 'rbac-api',
          'protocol'      => 'https',
          'type'          => 'rbac',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:4433/status",
          'url'           => "https://#{certname}:4433/rbac-api",
        },
        'code-manager' => {
          'display_name'  => 'Code Manager',
          'status_key'    => 'code-manager-service',
          'status_port'   => 8140,
          'status_prefix' => 'status',
          'port'          => 8170,
          'prefix'        => '',
          'protocol'      => 'https',
          'type'          => 'code-manager',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8140/status",
          'url'           => "https://#{certname}:8170/",
        },
        'orchestrator' => {
          'display_name'  => 'Orchestrator',
          'status_key'    => 'orchestrator-service',
          'status_port'   => 8143,
          'status_prefix' => 'status',
          'port'          => 8143,
          'prefix'        => 'orchestrator',
          'protocol'      => 'https',
          'type'          => 'orchestrator',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8143/status",
          'url'           => "https://#{certname}:8143/orchestrator",
        },
        'pcp-broker' => {
          'display_name'  => 'PCP Broker',
          'status_key'    => 'broker-service',
          'status_port'   => 8143,
          'status_prefix' => 'status',
          'port'          => 8142,
          'prefix'        => 'pcp',
          'protocol'      => 'wss',
          'type'          => 'pcp-broker',
          'node_certname' => certname,
          'server'        => certname,
          'status_url'    => "https://#{certname}:8143/status",
          'url'           => "wss://#{certname}:8142/pcp",
        },
      }
      services[service].merge(overrides)
    end

    describe '#services_list' do
      subject { config.services_list }

      context 'monolithic' do
        let(:layout) { 'mono' }
        let(:default_mono_services) do
          {
            'primary' => {
              'puppetserver' => [expected_service('puppetserver', 'master.rspec')],
              'puppetdb'     => [expected_service('puppetdb', 'master.rspec')],
              'classifier'   => [expected_service('classifier', 'master.rspec')],
              'activity'     => [expected_service('activity', 'master.rspec')],
              'rbac'         => [expected_service('rbac', 'master.rspec')],
              'orchestrator' => [expected_service('orchestrator', 'master.rspec')],
              'pcp-broker'   => [expected_service('pcp-broker', 'master.rspec')],
            },
            'secondary' => {
              'puppetserver' => [expected_service('puppetserver', 'compile.rspec')],
            },
            'replica' => {}
          }
        end

        context 'with defaults' do
          it 'lists monolithic services' do
            is_expected.to match(default_mono_services)
          end
        end

        context 'with code manager enabled' do
          let(:layout) { 'mono-with-codemanager' }
          let(:hieradata) do
            <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
  "pe_role::monolithic::compile_master": ["compile.rspec"]
}
"puppet_enterprise::profile::master::code_manager_auto_configure": true
            HIERADATA
          end

          let(:mono_services) do
            {
              'primary' => default_mono_services['primary'].merge({
                'code-manager'      => [expected_service('code-manager', 'master.rspec')],
                'file-sync-storage' => [expected_service('file-sync-storage', 'master.rspec')],
                'file-sync-client'  => [expected_service('file-sync-client', 'master.rspec')],
              }),
              'secondary' => default_mono_services['secondary'].merge({
                'file-sync-client'  => [expected_service('file-sync-client', 'compile.rspec')],
              }),
              'replica' => {}
            }
          end

          it 'lists monolithic services' do
            run_with_custom_hieradata(layout, hieradata) do
              is_expected.to match(mono_services)
            end
          end
        end

        context 'with file sync enabled' do
          let(:layout) { 'mono-with-file-sync' }
          let(:hieradata) do
            <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
  "pe_role::monolithic::compile_master": ["compile.rspec"]
}
"puppet_enterprise::profile::master::file_sync_enabled": true
            HIERADATA
          end

          let(:mono_services) do
            {
              'primary' => default_mono_services['primary'].merge({
                'file-sync-storage' => [expected_service('file-sync-storage', 'master.rspec')],
                'file-sync-client'  => [expected_service('file-sync-client', 'master.rspec')],
              }),
              'secondary' => default_mono_services['secondary'].merge({
                'file-sync-client'  => [expected_service('file-sync-client', 'compile.rspec')],
              }),
              'replica' => {}
            }
          end

          it 'lists monolithic services' do
            run_with_custom_hieradata(layout, hieradata) do
              is_expected.to match(mono_services)
            end
          end
        end

        context 'with orchestrator disabled' do
          let(:layout) { 'mono-without-orch' }
          let(:hieradata) do
            <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
  "pe_role::monolithic::compile_master": ["compile.rspec"]
}
"puppet_enterprise::profile::orchestrator::run_service": false
            HIERADATA
          end

          let(:mono_services) do
            services = default_mono_services
            services['primary'].delete('orchestrator')
            services
          end

          it 'lists monolithic services' do
            run_with_custom_hieradata(layout, hieradata) do
              is_expected.to match(mono_services)
            end
          end
        end
      end

      context 'split' do
        let(:layout) { :overrides }
        let(:split_services) do

          puppetdb_overrides = {
            'status_port' => 5678,
            'status_url'  => 'https://puppetdb2.rspec:5678/status',
            'port'        => 5678,
            'url'         => 'https://puppetdb2.rspec:5678/pdb',
          }

          classifier_overrides = {
            'status_port' => 1234,
            'status_url'  => 'https://console.rspec:1234/status',
            'port'        => 1234,
            'url'         => 'https://console.rspec:1234/classifier-api',
          }

          activity_overrides = {
            'status_port' => 1234,
            'status_url'  => 'https://console.rspec:1234/status',
            'port'        => 1234,
            'url'         => 'https://console.rspec:1234/activity-api',
          }

          rbac_overrides = {
            'status_port' => 1234,
            'status_url'  => 'https://console.rspec:1234/status',
            'port'        => 1234,
            'url'         => 'https://console.rspec:1234/rbac-api',
          }


          {
            'primary' => {
              'puppetserver' => [expected_service('puppetserver', 'master.rspec')],
              'puppetdb'     => [
                expected_service('puppetdb', 'puppetdb1.rspec'),
                expected_service('puppetdb', 'puppetdb2.rspec', puppetdb_overrides)
              ],
              'classifier'   => [expected_service('classifier', 'console.rspec', classifier_overrides)],
              'activity'     => [expected_service('activity', 'console.rspec', activity_overrides)],
              'rbac'         => [expected_service('rbac', 'console.rspec', rbac_overrides)],
              'orchestrator' => [expected_service('orchestrator', 'master.rspec')],
              'pcp-broker'   => [expected_service('pcp-broker', 'master.rspec')],
            },
            'secondary' => {},
            'replica' => {}
          }
        end

        it 'lists split services with overrides' do
          services = config.services_list
          expect(services['primary'].sort.pretty_inspect).to(
            eq(split_services['primary'].sort.pretty_inspect)
          )
          is_expected.to match(split_services)
        end
      end

      context 'for replica' do
        let(:layout) { :replica }
        let(:replica_services) do
          {
            'primary' => {
              'puppetserver' => [expected_service('puppetserver', 'master.rspec')],
              'puppetdb'     => [expected_service('puppetdb', 'master.rspec')],
              'classifier'   => [expected_service('classifier', 'master.rspec')],
              'activity'     => [expected_service('activity', 'master.rspec')],
              'rbac'         => [expected_service('rbac', 'master.rspec')],
              'orchestrator' => [expected_service('orchestrator', 'master.rspec')],
              'pcp-broker'   => [expected_service('pcp-broker', 'master.rspec')],
            },
            'secondary' => {
              'puppetserver' => [expected_service('puppetserver', 'compile.rspec')],
            },
            'replica' => {
              'puppetserver' => [
                expected_service('puppetserver', 'provisioned_replica.rspec'),
                expected_service('puppetserver', 'enabled_replica.rspec')
              ],
              'puppetdb'     => [
                expected_service('puppetdb', 'provisioned_replica.rspec'),
                expected_service('puppetdb', 'enabled_replica.rspec')
              ],
              'classifier'   => [
                expected_service('classifier', 'provisioned_replica.rspec'),
                expected_service('classifier', 'enabled_replica.rspec')
              ],
              'activity'     => [
                expected_service('activity', 'provisioned_replica.rspec'),
                expected_service('activity', 'enabled_replica.rspec')
              ],
              'rbac'         => [
                expected_service('rbac', 'provisioned_replica.rspec'),
                expected_service('rbac', 'enabled_replica.rspec')
              ]
            }
          }
        end

        it 'lists replica services' do
          is_expected.to match(replica_services)
        end
      end
    end
  end
end
