require 'spec_helper'
require 'puppet_x/puppetlabs/meep/config'

module PuppetX::Puppetlabs::Meep

describe 'MEEP' do
  let(:node) { 'master.rspec' }

  describe "config interface defaults" do
    it 'initializes' do
      node = Puppet::Node.new('test')
      compiler = Puppet::Parser::Compiler.new(node)
      scope = Puppet::Parser::Scope.new(compiler)
      expect(scope).to receive(:lookupvar).and_return('test')
      expect(Config.new(scope)).to be
    end

    it { expect(Config.meep_config_path).to eq('/etc/puppetlabs/enterprise') }
  end

  describe "config interface" do
    let(:layout) { 'mono' }
    let(:config) { Config.local }

    include_context 'meep_config'
    include_context 'Config.local prep'

    context ".hiera" do
      it "returns a HieraAdapter instance" do
        expect(config.hiera).to be_kind_of(HieraAdapter)
      end
    end

    context "#hiera_lookup" do
      it "returns nil if parameter not present" do
        expect(config.hiera_lookup("nonexistent::key")).to be_nil
      end
    end

    context '.local_meep_run_in_progress!' do
      it { expect(Config.is_local_meep_run_in_progress?).to eq(false) }
      it do
        begin
          expect(Config.local_meep_run_in_progress!).to eq(true)
          expect(Config.is_local_meep_run_in_progress?).to eq(true)
        ensure
          PuppetX::Puppetlabs::Meep::Config.send(:remove_instance_variable, :@meep_run_in_progress)
        end
      end
    end

    context '.bootstrap_run_in_progress!' do
      it { expect(Config.get_bootstrap_version).to be_nil }

      it do
        begin
          expect(Config.bootstrap_run_in_progress!('2017.1.0')).to eq(true)
          expect(Config.get_bootstrap_version).to eq('2017.1.0')
        ensure
          PuppetX::Puppetlabs::Meep::Config.send(:remove_instance_variable, :@bootstrap_version)
        end
      end

      it 'expects a version string' do
        expect { Config.bootstrap_run_in_progress! }.to raise_error(ArgumentError)
      end
    end

    context ".local" do
      let(:node) { 'master.rspec' }
      let(:client_version_matcher) { match(/\d+\.\d+\.\d+/) }

      context "given nothing" do
        it "generates a config with a scope that has correct clientcert and trusted.cert" do
          config = Config.local

          expect(config.scope.to_hash).to match({
            "clientcert" => "master.rspec",
            "clientnoop" => false,
            "clientversion" => client_version_matcher,
            "environment" => "*root*",
            "facts" => {
              "operatingsystem" => "CentOS",
              "platform_tag" => "el-7-x86_64",
              "clientcert" => "master.rspec",
              "clientversion" => client_version_matcher,
              "clientnoop" => false,
            },
            "operatingsystem" => "CentOS",
            "platform_tag" => "el-7-x86_64",
            "server_facts" => {},
            "trusted" => {
              "certname" => "master.rspec",
              "extensions" => {"pp_auth_role" => "pe_role::monolithic::primary_master"},
            },
          })
        end

        it "can look things up" do
          config = Config.local
          expect(config.node_roles).to match({
            "pe_role::monolithic::primary_master" => ["master.rspec"],
            "pe_role::monolithic::compile_master" => ["compile.rspec"],
            "pe_role::monolithic::primary_master_replica" => [],
          })

          expect(config.hiera_lookup("console_admin_password")).to eq("")
        end
      end

      context "given a node" do
        let(:layout) { 'overrides' }
        let(:spec_env) do
          Puppet::Node::Environment.create(
            'spec',
            [File.join(Puppet[:environmentpath], 'spec', 'modules')]
          )
        end
        let(:other_node_name) { 'puppetdb2.rspec' }
        let(:node_instance) do
          node = Puppet::Node.new(other_node_name, :environment => spec_env)
          # Pulls in facts stored in around around() clause below
          node.fact_merge
          node
        end

        around(:each) do |example|
          begin
            Puppet::Node::Facts.indirection.terminus_class = :memory
            Puppet::Node::Facts.indirection.cache_class = :memory

            # Generate a set of facts appropriate to our 'other' node
            certname = Puppet[:certname]
            Puppet[:certname] = other_node_name
            node_facts = Puppet::Node::Facts.new(other_node_name, facts)
            node_facts.add_local_facts
            Puppet[:certname] = certname

            # And store them so they can be retrieved when :node_instance
            # is generated above.
            Puppet::Node::Facts.indirection.save(node_facts)

            example.run
          ensure
            Puppet::Node::Facts.indirection.reset_terminus_class
            Puppet::Node::Facts.indirection.cache_class = nil
          end
        end

        it "generates a config with a scope based on passed node" do
          config = Config.local(node_instance)
          expect(config.scope.to_hash).to match({
            "clientcert" => "puppetdb2.rspec",
            "clientnoop" => false,
            "clientversion" => client_version_matcher,
            "environment" => "spec",
            "facts" => {
              "operatingsystem" => "CentOS",
              "platform_tag" => "el-7-x86_64",
              "clientcert" => "puppetdb2.rspec",
              "clientversion" => client_version_matcher,
              "clientnoop" => false,
            },
            "operatingsystem" => "CentOS",
            "server_facts" => {},
            "platform_tag" => "el-7-x86_64",
            "trusted" => {
              "certname" => "puppetdb2.rspec",
              "extensions" => {"pp_auth_role" => "pdb_split_role"},
            },
          })
        end

        it "can lookup specialized parameters for this node" do
          config = Config.local(node_instance)
          expect(config.hiera_lookup('puppet_enterprise::profile::puppetdb::ssl_listen_port')).to eq(5678)
        end
      end
    end

    context "#hiera_lookup" do
      it do
        expect(config.hiera_lookup("node_roles")).to eq({
          "pe_role::monolithic::primary_master"         => ["master.rspec"],
          "pe_role::monolithic::primary_master_replica"        => [],
          "pe_role::monolithic::compile_master" => ["compile.rspec"],
        })
      end

      context "with overides" do
        let(:layout) { 'overrides' }

        it 'is the base value in pe.conf when no node or role file applies' do
          expect(config.hiera_lookup("puppet_enterprise::profile::puppetdb::java_args")).to eq(
            { 'Xmx' => '512m' }
          )
        end

        context "with a node file" do
          let(:node) { 'puppetdb2.rspec' }

          it 'looks up from node file first' do
            expect(config.hiera_lookup(
              "puppet_enterprise::profile::puppetdb::java_args")).to eq(
              { 'Xmx' => '128m' }
            )
          end
        end

        context "with a role file" do
          let(:node) { 'puppetdb1.rspec' }

          it 'retrieves from node role.conf if no node file is set' do
            expect(config.hiera_lookup(
              "puppet_enterprise::profile::puppetdb::java_args")).to eq(
              { 'Xmx' => '256m' }
            )
          end
        end
      end
    end

    context "#hiera_lookup_for_node" do
      let(:node) { 'master.rspec' }
      let(:layout) { 'overrides' }

      context "when node files exist" do
        it { expect(config.hiera_lookup_for_node("puppetdb2.rspec", "puppet_enterprise::profile::puppetdb::ssl_listen_port")).to eq(5678) }

        it 'can switch context' do
          expect(config.hiera_lookup_for_node("puppetdb1.rspec", "puppet_enterprise::profile::console::console_services_api_ssl_listen_port")).to eq(1234)
          expect(config.hiera_lookup_for_node("puppetdb2.rspec", "puppet_enterprise::profile::puppetdb::ssl_listen_port")).to eq(5678)
        end
      end

      context "when role files exist" do
        it 'finds a parameter set in a role' do
          expect(
            config.hiera_lookup_for_node(
              "puppetdb1.rspec",
              "puppet_enterprise::profile::puppetdb::java_args"
            )
          ).to(
            eq({ 'Xmx' => '256m' })
          )
        end

        it 'can switch context' do
          expect(
            config.hiera_lookup_for_node(
              "puppetdb2.rspec",
              "puppet_enterprise::profile::puppetdb::ssl_listen_port"
            )
          ).to eq(5678)
          expect(
            config.hiera_lookup_for_node(
              "puppetdb1.rspec",
              "puppet_enterprise::profile::puppetdb::java_args"
            )
          ).to eq({ 'Xmx' => '256m' })
        end
      end
    end

    # These methods work with the node_roles hash which only exists in the new format
    context "#get_role_for" do
      it { expect(config.get_role_for('master.rspec')).to eq('pe_role::monolithic::primary_master') }
      it { expect(config.get_role_for('unknown.node')).to be_nil }
    end

    context 'profiles' do
      let(:primary_master_profiles) do
        [
          'puppet_enterprise',
          'pe_install',
          'puppet_enterprise::profile::database',
          'puppet_enterprise::profile::primary_master',
          'puppet_enterprise::profile::certificate_authority',
          'puppet_enterprise::profile::orchestrator',
          'puppet_enterprise::profile::puppetdb',
          'puppet_enterprise::profile::console',
          'pe_infrastructure::enterprise::agent',
        ]
      end
      let(:custom_compile_master_profiles) do
        [
          'puppet_enterprise',
          'pe_install',
          'puppet_enterprise::profile::compile_master',
          'puppet_enterprise::profile::puppetdb',
          'pe_infrastructure::enterprise::agent',
        ]
      end
      let(:custom_compile_master_pupptedb_profiles) do
        [
          'puppet_enterprise',
          'pe_install',
          'puppet_enterprise::profile::compile_master',
          'puppet_enterprise::profile::puppetdb',
          'pe_infrastructure::enterprise::agent',
        ]
      end

      context "#get_role_profiles" do
        it do
          expect(config.get_role_profiles('pe_role::monolithic::primary_master')).to(
            eq(primary_master_profiles)
          )
        end

        it "returns an empty array if role not found" do
          expect(config.get_role_profiles('role_that_does_not_exist')).to eq([])
        end

        context 'with custom roles' do
          let(:layout) { 'custom-roles' }

          it 'handles a set of profiles' do
            expect(config.get_role_profiles('custom_compile_master')).to match_array(
              custom_compile_master_profiles
            )
          end

          it 'handles a mix of profiles and roles' do
            expect(config.get_role_profiles('custom_compile_master_puppetdb')).to match_array(
              custom_compile_master_pupptedb_profiles
            )
          end
        end
      end

      context "#get_node_profiles" do
        let(:node) { 'master.rspec' }
        let(:layout) { 'mono' }

        before(:each) do
          primary_master_profiles << 'pe_repo::platform::el_7_x86_64'
        end

        it do
          expect(config.get_node_profiles(node)).to eq(primary_master_profiles)
        end

        it 'returns an empty array if node is not mapped' do
          expect(config.get_node_profiles('nonexistent.node')).to eq([])
        end

        context 'with custom roles' do
          let(:layout) { 'custom-roles' }

          it 'finds profiles' do
            expect(config.get_node_profiles('compile.rspec')).to eq(
              custom_compile_master_profiles
            )
          end
        end

        context 'with external services' do
          context 'in a split layout' do
            let(:layout) { 'external-services' }

            it 'does not include database profile in split puppetdb node' do
              expect(config.get_node_profiles('puppetdb.rspec')).to eq(
                [
                  'puppet_enterprise',
                  'pe_install',
                  # 'puppet_enterprise::profile::database',
                  'puppet_enterprise::profile::puppetdb',
                  'pe_infrastructure::enterprise::agent',
                ]
              )
            end

            it 'does not include certificate authority profile in primary master' do
              expect(config.get_node_profiles('master.rspec')).to eq(
                [
                  'puppet_enterprise',
                  'pe_install',
                  'puppet_enterprise::profile::primary_master',
                  # 'puppet_enterprise::profile::certificate_authority',
                  'puppet_enterprise::profile::orchestrator',
                  'pe_infrastructure::enterprise::agent',
                ]
              )
            end
          end

          context 'in a monolithic layout' do
            let(:layout) { 'external-mono.tmp' }
            let(:hieradata) do
              <<-HIERADATA
"puppet_enterprise::database_host": "external.database.rspec"
"puppet_enterprise::certificate_authority_host": "external.ca.rspec"
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
"agent_platforms": [
  "el-7-x86_64"
]
              HIERADATA
            end

            it 'includes neither database nor ca profiles in primary master' do
              run_with_custom_hieradata(layout, hieradata) do
                expect(config.get_node_profiles('master.rspec')).to eq(
                  primary_master_profiles.reject do |r|
                    [
                      "puppet_enterprise::profile::database",
                      "puppet_enterprise::profile::certificate_authority",
                    ].include?(r)
                  end
                )
              end
            end

          end

          context 'without agent_platforms specified' do
            let(:layout) { 'no-agent-mono.tmp' }
            let(:hieradata) do
              <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
              HIERADATA
            end

            it 'includes no agent profile classes' do
              run_with_custom_hieradata(layout, hieradata) do
                expect(config.get_node_profiles('master.rspec')).to eq(
                  primary_master_profiles.reject do |r|
                    [
                      "pe_repo::platform::el_7_x86_64",
                    ].include?(r)
                  end
                )
              end
            end

          end
        end
      end
    end

    context "#has_external_db?" do
      it { expect(config.has_external_db?).to eq(false) }

      context "with database_host set" do
        let(:layout) { 'external-db.tmp' }
        let(:hieradata) do
          <<-HIERADATA
"puppet_enterprise::database_host": "database.rspec"
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
          HIERADATA
        end

        it "is an external db" do
          run_with_custom_hieradata(layout, hieradata) do
            expect(config.has_external_db?).to eq(true)
          end
        end
      end

      context "with database_host set to an existing node" do
        let(:layout) { 'explicit-db.tmp' }
        let(:hieradata) do
          <<-HIERADATA
"puppet_enterprise::database_host": "puppetdb.rspec"
"node_roles": {
  "pe_role::split::primary_master": ["master.rspec"]
  "pe_role::split::puppetdb": ["puppetdb.rspec"]
  "pe_role::split::console": ["console.rspec"]
}
          HIERADATA
        end

        it "is not an external db" do
          run_with_custom_hieradata(layout, hieradata) do
            expect(config.has_external_db?).to eq(false)
          end
        end
      end
    end

    context "#has_external_ca" do
      it { expect(config.has_external_ca?).to eq(false) }

      context "with certificate_authority_host set" do
        let(:layout) { 'external-ca.tmp' }
        let(:hieradata) do
          <<-HIERADATA
"puppet_enterprise::certificate_authority_host": "ca.rspec"
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
          HIERADATA
        end

        it "is an external ca" do
          run_with_custom_hieradata(layout, hieradata) do
            expect(config.has_external_ca?).to eq(true)
          end
        end
      end

      context "with certificate_authority_host set to an existing node" do
        let(:layout) { 'explicit-ca.tmp' }
        let(:hieradata) do
          <<-HIERADATA
"puppet_enterprise::certificate_authority_host": "master.rspec"
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
          HIERADATA
        end

        it "is not an external ca" do
          run_with_custom_hieradata(layout, hieradata) do
            expect(config.has_external_ca?).to eq(false)
          end
        end
      end
    end

    context "#is_node_a" do
      shared_examples 'verify components' do |expectations|
        expectations.each do |node, node_components|
          Config.pe_components.each do |component|
            expected = node_components.include?(component)

            it "the #{component} component #{expected ? 'is' : 'is not'} present in the #{node} node" do
              expect(config.is_node_a(component, node)).to(eq(expected))
            end
          end
        end
      end

      context 'with monolithic layout:' do
        let(:layout) { 'mono' }

        # These hashes drive the testing in the 'verify components' shared_examples.
        # This is unspooled to create one spec per node x pe_components()
        # combination, where any component in the associated array is
        # expected to return true in an #is_node_a(component) call, and any
        # component not listed is expected to return false.
        expected_components = {
          'master.rspec' => [
            'primary_master',
            'certificate_authority',
            'orchestrator',
            'puppetdb',
            'database',
            'console',
          ],
          'compile.rspec' => [
            'compile_master',
          ],
        }

        include_examples 'verify components', expected_components
      end

      context 'with split layout:' do
        let(:layout) { 'split' }
        expected_components = {
          'master.rspec' => [
            'primary_master',
            'certificate_authority',
            'orchestrator',
          ],
          'puppetdb1.rspec' => [
            'puppetdb',
            'database',
          ],
          'puppetdb2.rspec' => [
            'puppetdb',
            'database',
          ],
          'console.rspec' => [
            'console',
          ],
          'compile.rspec' => [
            'compile_master',
          ],
        }

        include_examples 'verify components', expected_components
      end

      context 'with replica layout:' do
        let(:layout) { 'replica' }
        expected_components = {
          'master.rspec' => [
            'primary_master',
            'certificate_authority',
            'orchestrator',
            'puppetdb',
            'database',
            'console',
          ],
          'provisioned_replica.rspec' => [
            'primary_master_replica',
          ],
          'enabled_replica.rspec' => [
            'primary_master_replica',
            'enabled_primary_master_replica',
          ],
          'compile.rspec' => [
            'compile_master',
          ],
        }

        include_examples 'verify components', expected_components
      end

      context 'with external-services layout:' do
        let(:layout) { 'external-services' }
        expected_components = {
          'master.rspec' => [
            'primary_master',
            'orchestrator',
          ],
          'puppetdb.rspec' => [
            'puppetdb',
          ],
          'console.rspec' => [
            'console',
          ],
          'compile.rspec' => [
            'compile_master',
          ],
        }

        include_examples 'verify components', expected_components
      end

      context 'with custom-roles layout:' do
        let(:layout) { 'custom-roles' }
        expected_components = {
          'master.rspec' => [
            'primary_master',
            'certificate_authority',
            'orchestrator',
            'puppetdb',
            'database',
            'console',
          ],
          'compile.rspec' => [
            'compile_master',
            'puppetdb',
          ],
        }

        include_examples 'verify components', expected_components
      end
    end

    context "#list_all_infrastructure" do
      let(:layout) { 'mono' }

      it do
        expect(config.list_all_infrastructure).to match_array([
          "master.rspec",
          "compile.rspec",
        ])
      end

      context "for a replica layout" do
        let(:layout) { 'replica' }

        it do
          expect(config.list_all_infrastructure).to match_array([
            "master.rspec",
            "compile.rspec",
            "provisioned_replica.rspec",
            "enabled_replica.rspec",
          ])
        end
      end

      context "for custom roles" do
        let(:layout) { 'custom-roles' }

        it do
          expect(config.list_all_infrastructure).to match_array([
            "master.rspec",
            "compile.rspec",
            "compile2.rspec",
          ])
        end
      end
    end

    context "#is_infrastructure" do
      let(:layout) { 'mono' }

      it "is true for nodes listed in node_roles" do
        expect(config.is_infrastructure("master.rspec")).to eq(true)
        expect(config.is_infrastructure("compile.rspec")).to eq(true)
      end

      it "is false for nodes not listed in node roles" do
        expect(config.is_infrastructure("not-in-pe-conf.net")).to eq(false)
      end

      it "treats nil as false" do
        expect(config.is_infrastructure(nil)).to eq(false)
      end
    end

    context "#list_nodes" do
      it { expect { config.list_nodes(:unknown_component_symbol) }.to raise_error(ArgumentError, /Undefined component role/) }

      it "lists master nodes" do
        expect(config.list_nodes(:primary_master)).to eq(['master.rspec'])
      end

      it "lists console nodes" do
        expect(config.list_nodes(:console)).to eq(['master.rspec'])
      end

      it "lists puppetdb nodes" do
        expect(config.list_nodes(:puppetdb)).to eq(['master.rspec'])
      end

      it "lists database nodes" do
        expect(config.list_nodes(:database)).to eq(['master.rspec'])
      end

      it "lists an empty provisioned replica set" do
        expect(config.list_nodes(:primary_master_replica)).to be_empty
      end

      it "lists an empty enabled replica set" do
        expect(config.list_nodes(:enabled_primary_master_replica)).to be_empty
      end

      it "lists compile master nodes" do
        expect(config.list_nodes(:compile_master)).to eq(['compile.rspec'])
      end

      context "for a split layout" do
        let(:layout) { 'split' }

        it "lists master nodes" do
          expect(config.list_nodes(:primary_master)).to eq(['master.rspec'])
        end

        it "lists console nodes" do
          expect(config.list_nodes(:console)).to eq(['console.rspec'])
        end

        it "lists puppetdb nodes" do
          expect(config.list_nodes(:puppetdb)).to eq(['puppetdb1.rspec', 'puppetdb2.rspec'])
        end

        it "lists database nodes" do
          expect(config.list_nodes(:database)).to eq(['puppetdb1.rspec', 'puppetdb2.rspec'])
        end
      end

      context "for a replica layout" do
        let(:layout) { 'replica' }

        it "lists master nodes" do
          expect(config.list_nodes(:primary_master)).to eq(['master.rspec'])
        end

        it "lists console nodes" do
          expect(config.list_nodes(:console)).to eq(['master.rspec'])
        end

        it "lists puppetdb nodes" do
          expect(config.list_nodes(:puppetdb)).to eq(['master.rspec'])
        end

        it "lists database nodes" do
          expect(config.list_nodes(:database)).to eq(['master.rspec'])
        end

        it "lists provisioned replica nodes" do
          expect(config.list_nodes(:primary_master_replica)).to eq(['provisioned_replica.rspec', 'enabled_replica.rspec'])
        end

        it "lists enabled replica nodes" do
          expect(config.list_nodes(:enabled_primary_master_replica)).to eq(['enabled_replica.rspec'])
        end
      end

      context "for custom roles" do
        let(:layout) { 'custom-roles' }

        it "lists master nodes" do
          expect(config.list_nodes(:primary_master)).to eq(['master.rspec'])
        end

        it "lists custom compile master nodes" do
          expect(config.list_nodes(:compile_master)).to eq([
            'compile.rspec',
            'compile2.rspec',
          ])
        end
      end

      context "with external services" do
        let(:layout) { 'external-services' }

        it "does not consider primary master to be database" do
          expect(config.list_nodes(:database)).to eq([])
        end

        it "does not consider primary master to be certificate_authority" do
          expect(config.list_nodes(:certificate_authority)).to eq([])
        end
      end
    end
  end
end
end
