require 'spec_helper'
require 'puppet_x/puppetlabs/meep/config'

module PuppetX::Puppetlabs::Meep

describe 'MEEP/Hiera integration' do
  describe "hiera interface" do
    include_context 'meep_config'
    include_context 'Config.local prep'

    let(:node) { 'master.rspec' }
    let(:layout) { 'mono' }
    let(:hiera) { HieraAdapter.new("#{meep_config_path}/hiera.yaml") }
    let(:scope) { Config.local.scope }

    it { expect(hiera).to be }

    it "provides a means to lookup a hiera parameter" do
      expect(hiera.lookup("puppet_enterprise::ha_enabled_replicas", scope)).to eq([])
    end

    context "with qualified variables" do
      it 'finds correct data' do
        expect(hiera.lookup("agent_platforms", scope)).to eq(['el-7-x86_64'])
      end
    end

    context "with non-qualified variables" do
      it 'finds correct data' do
        expect(hiera.lookup('agent_platforms', scope)).to eq(['el-7-x86_64'])
      end
    end

    context "with node overrides" do
      let(:layout) { 'overrides' }
      let(:node) { 'puppetdb2.rspec' }

      it "finds data in node files as well" do
        expect(hiera.lookup("puppet_enterprise::profile::console::console_services_api_ssl_listen_port", scope)).to eq(1234)
        expect(hiera.lookup("puppet_enterprise::profile::puppetdb::ssl_listen_port", scope)).to eq(5678)
      end
    end

    context "loaders" do
      it "uses compiler loaders" do
        # No loaders in the Puppet Context
        expect(Puppet.lookup(:loaders) { nil }).to be_nil

        # Generate scope before we set expectations on Lookup, since we're using
        # Config.local to generate a scope, and Config#initialize references lookup
        compiler_loaders = scope.compiler.loaders.__id__
        loaders_used_during_lookup = nil

        expect(Puppet::Pops::Lookup).to receive(:lookup) do
          loaders_used_during_lookup = Puppet.lookup(:loaders).__id__
          nil
        end

        hiera.lookup('foo', scope)

        expect(loaders_used_during_lookup).to equal(compiler_loaders)
      end

      # I'm not positive that this case is relevant.
      # If we're querying meep from within a catalog compilation, env will be the same,
      # and so will the loaders.
      # If we're querying meep outside of a compiler, I don't think there will
      # be any loaders on the context stack.
      # I'm not positive about that last statement, though, and it doesn't hurt to validate
      # that in this case, HieraAdapter#lookup correctly uses its scope's compiler's loaders.
      it "uses compiler loaders when pre-existing loaders set (as would be the case when calling Config#hiera_lookup_for_node during catalog compilation)" do
        other_env = Puppet::Node::Environment.create('other', [File.join(Puppet[:environmentpath], 'other', 'modules')])
        original_compiler = Puppet::Parser::Compiler.new(Puppet::Node.new('some.node', :environment => other_env))
        original_compiler_loaders = original_compiler.loaders

        # Generate scope before we set expectations on Lookup, since we're using
        # Config.local to generate a scope, and Config#initialize references lookup
        compiler_loaders = scope.compiler.loaders.__id__

        expect(original_compiler_loaders).to_not equal(compiler_loaders)

        loaders_used_during_lookup = nil
        expect(Puppet::Pops::Lookup).to receive(:lookup) do
          loaders_used_during_lookup = Puppet.lookup(:loaders).__id__
          nil
        end

        Puppet.override({:loaders => original_compiler.loaders}, 'original compiler loaders') do
          hiera.lookup('foo', scope)
          expect(loaders_used_during_lookup).to eq(compiler_loaders)
        end
      end
    end

    context "without an enterprise dir" do
      let(:hiera) { HieraAdapter.new("/dev/null/hiera.yaml") }

      context "during an agent run" do

        it "does not raise an error" do
          expect(hiera.lookup("foo", scope)).to be_nil
        end

        it "logs a warning" do
          expect(hiera.lookup("foo", scope)).to be_nil
          expect(@logs.map {|l| l.message}).to include(/Puppet Enterprise was not able to find.*hiera\.yaml/)
        end
      end

      context "during a meep run" do
        around(:each) do |example|
          begin
            Config.local_meep_run_in_progress!
            example.run
          ensure
            PuppetX::Puppetlabs::Meep::Config.send(:remove_instance_variable, :@meep_run_in_progress)
          end
        end

        it "raises an error" do
          expect { hiera.lookup("foo", scope) }.to(
            raise_error(RuntimeError, /Puppet Enterprise was not able to find.*hiera\.yaml/)
          )
        end
      end
    end

    context "two seperate hiera contexts" do
      let(:layout) { 'master-meep' }
      let(:master_hiera_dir) { 'master-hiera' }
      let(:master_hiera_yaml) do
        <<-YAML
---
:backends:
  - yaml
:hierarchy:
  - "common"
:yaml:
    :datadir: "spec/fixtures/hiera/#{master_hiera_dir}/hieradata"
        YAML
      end
      let(:master_common) do
        <<-YAML
context: master
        YAML
      end
      let(:pe_conf) do
        <<-HOCON
"context": "meep"
"node_roles": {
  "pe_role::monolithic::primary_master": ["master.rspec"]
}
        HOCON
      end
      let(:hiera_config) { "spec/fixtures/hiera/#{master_hiera_dir}/hiera.yaml" }
      let(:meep_hiera_adapter) { HieraAdapter.new("spec/fixtures/hiera/#{layout}/hiera.yaml") }
      let(:master_lookup_invocation) { Puppet::Pops::Lookup::Invocation.new(scope, Puppet::Pops::EMPTY_HASH, Puppet::Pops::EMPTY_HASH, true) }

      def master_lookup(key)
        _lookup = lambda { Puppet::Pops::Lookup.lookup(key, nil, nil, true, nil, master_lookup_invocation) }
        # if the test is being run by itself, need to register loaders
        if (Puppet.lookup(:loaders) { nil } != scope.compiler.loaders)
          Puppet.override({:loaders => scope.compiler.loaders}, 'for master lookup') do
            _lookup.call
          end
        else
          _lookup.call
        end
      end

      around(:each) do |example|
        customize_hiera(
          :fixture_directory => master_hiera_dir,
          :hiera_yaml        => master_hiera_yaml,
          :data              => {
            "hieradata"      => {
              "common.yaml"    => master_common
            }
          }
        ) do

          run_with_custom_hieradata(layout, pe_conf, &example)
        end
      end
      
      context 'independently', :type => :class do
        it { expect(meep_hiera_adapter.lookup('context', scope)).to eq('meep') }
        it { expect(master_lookup('context')).to eq('master') }
      end

      # These were validating some ordering issues with Hiera related to
      # cacheing of backends based on order of invocation. They seem generally
      # applicable to our case of multiple lookup contexts, however.
      context 'validating order of lookup', :type => :class do
        it 'is order independent when searching master context first' do
          # first two calls establish configuration
          expect(master_lookup('context')).to eq('master')
          expect(meep_hiera_adapter.lookup('context', scope)).to eq('meep')

          # after that calls with their cached handles should continue to
          # retrieve correct information
          expect(meep_hiera_adapter.lookup('context', scope)).to eq('meep')
          expect(master_lookup('context')).to eq('master')
        end

        it 'is order independent when searching meep context first' do
          # first two calls establish configuration
          expect(meep_hiera_adapter.lookup('context', scope)).to eq('meep')
          expect(master_lookup('context')).to eq('master')

          # after that calls with their cached handles should continue to
          # retrieve correct information
          expect(master_lookup('context')).to eq('master')
          expect(meep_hiera_adapter.lookup('context', scope)).to eq('meep')
        end
      end
    end
  end
end
end
