require 'spec_helper'
require 'puppet_x/puppetlabs/meep/validate'

RSpec.shared_examples "check errors and warnings" do
  it "has the expected error messages" do
    expected_errors.each do |error|
      expect(subject.errors.to_s).to(match(error))
    end
  end

  it "has the expected warning messages" do
    expected_warnings.each do |warning|
      expect(subject.warnings.to_s).to(match(warning))
    end
  end

  it "has only the expected number of errors" do
    expect(subject.errors.length).to(eq(expected_errors.length))
  end

  it "has only the expected number of warnings" do
    expect(subject.warnings.length).to(eq(expected_warnings.length))
  end

end

describe PuppetX::Puppetlabs::Meep::Validate do
  let(:subject) {described_class.pe_conf(pe_conf_path, options)}
  let(:options) { {} }
  let(:pe_conf_path) { '/etc/puppetlabs/enterprise/conf.d/pe.conf' }
  let(:expected_errors) { [] }
  let(:expected_warnings) { [] }

  describe '#pe_conf while pe.conf does not exist' do
    let(:expected_errors) { [/Unable to read HOCON config/] }

    it 'fails validation' do
      expect(subject.valid?).to(be false)
    end

    include_examples "check errors and warnings"
  end

  describe '#pe_conf' do
    before(:each) do
      allow(File).to receive(:open).with(pe_conf_path).and_return(StringIO.new(conf.to_json))
    end

    describe 'malformed json/hocon' do

      describe 'missing closing curly brace' do
        let(:conf) { "{ 'console_admin_password': 'foo' " }
        let(:expected_errors) { [/Unable to parse HOCON config/] }

        it 'fails validation' do
          expect(subject.valid?).to(be false)
        end

        include_examples "check errors and warnings"
      end
    end

    describe 'valid json/hocon' do
      describe 'invalid MEEP schema version' do
        let(:conf) { { 'meep_schema_version' => '0.01' } }
        let(:expected_errors) { [
          /Unknown MEEP version: #{conf['meep_schema_version']}/,
          /schema version #{conf['meep_schema_version']}.*expected.*2\.0/] }

        it 'fails validation' do
          expect(subject.valid?).to(be false)
        end

        include_examples "check errors and warnings"

        it 'prints errors when output_warnings is called' do
          expect(Puppet).to receive(:warning).exactly(3).times
          subject.output_warnings
        end
      end

      describe 'pre-2.0 MEEP' do
        let(:options) { {:is_install => true, :expected_version => '1.0'} }
        # pre-2.0 we did not specify a meep version
        let(:conf) { {} }

        describe 'minimal valid pre-2.0 monolithic conf' do
          describe 'with all required parameters' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "puppet_enterprise::puppet_master_host" => "master.rspec",
            } }

            it 'passes validation' do
              expect(subject.valid?).to(be true)
            end

            include_examples "check errors and warnings"
          end

          describe 'missing console_admin_password' do
            let(:conf) { {
              "console_admin_password" => "",
              "puppet_enterprise::puppet_master_host" => "master.rspec",
            } }
            let(:expected_errors) { [/.*Required parameters are empty.*console_admin_password.*/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end
        end
      end

      describe '2.0 MEEP' do
        # pre-2.0 we did not specify a meep version
        let(:conf) { {} }
        let(:options) { {:is_install => true, :expected_version => '2.0'} }
        let(:meep_schema_version) { '2.0' }

        describe 'monolithic conf' do
          describe 'with all required parameters' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }

            it 'passes validation' do
              expect(subject.valid?).to(be true)
            end

            include_examples "check errors and warnings"
          end

          describe 'using trusted certname' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["%{::trusted.certname}"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/contains.*::trusted\.certname/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'missing console_admin_password' do
            let(:conf) { {
              "console_admin_password" => "",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/.*Required parameters are empty.*console_admin_password.*/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'missing pe_role master' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => {  },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [
              /No architecture detected/,
              /Required parameters are empty.*node_roles/
              ] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'invalid architecture' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::mono::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/Unknown architecture mono/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'invalid host' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => [{"master.rspec" => {"params" => "arenotallowed"}}] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/contains.*arenotallowed.*hosts must be provided as strings/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'invalid role' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => {"master.rspec" => {"params" => "arenotallowed"}} },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/is not an array/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'invalid role' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => {
                "pe_role::monolithic::primary_master" => ["master.rspec"],
                "pe_role::split::puppetdb" => ["pdb.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/Multiple architectures/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'node_roles not a hash' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => [ {"pe_role::monolithic::primary_master" => ["master.rspec"]} ],
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
            } }
            let(:expected_errors) { [/node_roles must be a hash/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'valid custom_roles' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
              "custom_roles" => {"compiler" => ["profile1", "profile2"]}
            } }

            it 'passes validation' do
              expect(subject.valid?).to(be true)
            end

            include_examples "check errors and warnings"
          end

          describe 'custom_roles not a hash' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => ["el-7-x86_64"],
              "custom_roles" => [{"compiler" => ["profile1", "profile2"]}]
            } }
            let(:expected_errors) { [/custom_roles must be a hash/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'agent_platforms is not an array' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => { "el-7-x86_64" => {"parameters" => "arenotallowed"} },
            } }
            let(:expected_errors) { [/agent_platforms must be an array/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'agent_platforms contains non-strings' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => { "pe_role::monolithic::primary_master" => ["master.rspec"] },
              "meep_schema_version" => meep_schema_version,
              "agent_platforms" => [{ "el-7-x86_64" => {"parameters" => "arenotallowed"}}],
            } }
            let(:expected_errors) { [/agent_platforms contains.*arenotallowed/] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end

          describe 'no required_params' do
            let(:conf) { {
              "meep_schema_version" => meep_schema_version,
            } }
            let(:expected_errors) { [
              /Missing required parameters: node_roles, agent_platforms, console_admin_password/
              ] }

            it 'fails validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end
        end

        describe 'split conf' do
          describe 'with all required parameters' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => {
                "pe_role::split::primary_master" => [
                  "cs3bf7hkck5n7bs.delivery.puppetlabs.net"
                ],
                "pe_role::split::puppetdb" => [
                  "tzx61utk0vwlmhi.delivery.puppetlabs.net"
                ],
                "pe_role::split::console" => [
                  "o7gml0hhsc1ti9j.delivery.puppetlabs.net"
                ] },
                "agent_platforms" => [
                  "el-7-x86_64"
                ],
                "meep_schema_version" => meep_schema_version
            } }

            it 'passes validation' do
              expect(subject.valid?).to(be true)
            end

            include_examples "check errors and warnings"
          end

          describe 'with duplicate hosts' do
            let(:conf) { {
              "console_admin_password" => "puppetlabs",
              "node_roles" => {
                "pe_role::split::primary_master" => [
                  "cs3bf7hkck5n7bs.delivery.puppetlabs.net"
                ],
                "pe_role::split::puppetdb" => [
                  "cs3bf7hkck5n7bs.delivery.puppetlabs.net"
                ],
                "pe_role::split::console" => [
                  "cs3bf7hkck5n7bs.delivery.puppetlabs.net"
                ] },
                "agent_platforms" => [
                  "el-7-x86_64"
                ],
                "meep_schema_version" => meep_schema_version
            } }
            let(:expected_errors) { [/hosts with multiple roles.*cs3bf7hkck5n7bs.delivery.puppetlabs.net/] }

            it 'passes validation' do
              expect(subject.valid?).to(be false)
            end

            include_examples "check errors and warnings"
          end
        end
      end
    end
  end
end
