require 'spec_helper'
require 'puppet'

provider_class = Puppet::Type.type(:pe_ini_subsetting).provider(:ruby)
describe provider_class do
  include PuppetlabsSpec::Files

  let(:tmpfile) { tmpfilename("pe_ini_setting_test") }

  def validate_file(expected_content, tmpfile)
    expect(File.read(tmpfile)).to eq expected_content
  end


  before :each do
    File.open(tmpfile, 'w') do |fh|
      fh.write(orig_content)
    end
  end

  context "when ensuring that a subsetting is present" do
    let(:common_params) { {
        :title    => 'pe_ini_setting_ensure_present_test',
        :path     => tmpfile,
        :section  => '',
        :key_val_separator => '=',
        :setting => 'JAVA_ARGS',
    } }

    let(:orig_content) {
      <<-EOS
JAVA_ARGS="-Xmx192m -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/var/log/pe-puppetdb/puppetdb-oom.hprof"
      EOS
    }

    expected_content_one = <<-EOS
JAVA_ARGS="-Xmx192m -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/var/log/pe-puppetdb/puppetdb-oom.hprof -Xms128m"
      EOS

    it "should add a missing subsetting" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
         :subsetting => '-Xms', :value => '128m'))
      provider = described_class.new(resource)
      provider.exists?.should be_nil
      provider.create
      validate_file(expected_content_one, tmpfile)
    end

    expected_content_two = <<-EOS
JAVA_ARGS="-XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/var/log/pe-puppetdb/puppetdb-oom.hprof"
      EOS

    it "should remove an existing subsetting" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
          :subsetting => '-Xmx'))
      provider = described_class.new(resource)
      provider.exists?.should == "192m"
      provider.destroy
      validate_file(expected_content_two, tmpfile)
    end

    expected_content_three = <<-EOS
JAVA_ARGS="-Xmx256m -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/var/log/pe-puppetdb/puppetdb-oom.hprof"
      EOS

    it "should modify an existing subsetting" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
          :subsetting => '-Xmx', :value => '256m'))
      provider = described_class.new(resource)
      provider.exists?.should == "192m"
      provider.value=('256m')
      validate_file(expected_content_three, tmpfile)
    end
  end

  context "when working with subsettings in files with unquoted settings values" do
    let(:common_params) { {
        :title    => 'pe_ini_setting_ensure_present_test',
        :path     => tmpfile,
        :section  => 'master',
        :setting => 'reports',
    } }

    let(:orig_content) {
      <<-EOS
[master]

reports = http,foo
      EOS
    }

     expected_content_four = <<-EOS
[master]

reports = foo
      EOS

    it "should remove an existing subsetting" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
          :subsetting => 'http', :subsetting_separator => ','))
      provider = described_class.new(resource)
      provider.exists?.should == ""
      provider.destroy
      validate_file(expected_content_four, tmpfile)
    end

    expected_content_five = <<-EOS
[master]

reports = http,foo,puppetdb
      EOS

    it "should add a new subsetting when the 'parent' setting already exists" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
          :subsetting => 'puppetdb', :subsetting_separator => ','))
      provider = described_class.new(resource)
      provider.exists?.should be_nil
      provider.value=('')
      validate_file(expected_content_five, tmpfile)
    end

    expected_content_six = <<-EOS
[master]

reports = http,foo
somenewsetting = puppetdb
      EOS

    it "should add a new subsetting when the 'parent' setting does not already exist" do
      resource = Puppet::Type::Pe_ini_subsetting.new(common_params.merge(
          :setting => 'somenewsetting',
          :subsetting => 'puppetdb',
          :subsetting_separator => ','))
      provider = described_class.new(resource)
      provider.exists?.should be_nil
      provider.value=('')
      validate_file(expected_content_six, tmpfile)
    end
  end
end
