require 'spec_helper'

RSpec.shared_examples 'common node groups' do

  it 'PE Certificate Authority node group' do
    is_expected.to contain_pe_node_group('PE Certificate Authority')
    .with_parent('PE Infrastructure')
    .with_pinned([node])
    .with_classes({'puppet_enterprise::profile::certificate_authority' => {}})
    .with_environment(environment)
  end

  it 'PE Orchestrator node group' do
    is_expected.to contain_pe_node_group('PE Orchestrator')
    .with_parent('PE Infrastructure')
    .with_pinned([node])
    .with_classes({'puppet_enterprise::profile::orchestrator' => {}})
    .with_environment(environment)
  end

  it 'PE PuppetDB node group' do
    is_expected.to contain_pe_node_group('PE PuppetDB')
    .with_parent('PE Infrastructure')
    .with_pinned([node])
    .with_classes({'puppet_enterprise::profile::puppetdb' => {}})
    .with_environment(environment)
  end

  it 'PE Console node group' do
    is_expected.to contain_pe_node_group('PE Console')
    .with_parent('PE Infrastructure')
    .with_pinned([node])
    .with_classes({
      'puppet_enterprise::profile::console' => {},
      'puppet_enterprise::license' => {},
    })
    .with_environment(environment)
  end

  it 'PE Agent node group' do
    is_expected.to contain_pe_node_group('PE Agent')
    .with_parent('PE Infrastructure')
    .with_rule(['and',
                ['~', ['fact', 'aio_agent_version'], '.+'],
               ])
    .with_classes({'puppet_enterprise::profile::agent' => {}})
    .with_environment(environment)
  end

  it 'PE Infrastructure Agent node group' do
    is_expected.to contain_pe_node_group('PE Infrastructure Agent')
    .with_parent('PE Agent')
    .with_rule(['and',
                ['~', ['fact', 'pe_server_version'], '.+'],
               ])
    .with_classes({'puppet_enterprise::profile::agent' => {}})
    .with_require('Pe_node_group[PE Agent]')
    .with_environment(environment)
  end
end

describe 'pe_install::install::classification' do
  let(:environment) { 'production' }
  # Number of node groups to install by default, includes environment groups,
  # but not mco groups.
  let(:install_group_count) { 13 }
  # Number of node groups for an upgrade/puppet-infra configure run (no mco).
  let(:post_install_group_count) { 9 }
  # Number of node groups to install with mco enabled, include environment groups.
  let(:install_group_count_with_mco) { 15 }

  context 'supported operating systems' do
    on_filtered_os(:redhat, :debian, :sles) do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)

        let(:pe_version) { '2018.1.7' }
        let(:pre_condition) do
          <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare {}
class pe_install::install {}
class { pe_install:
  bootstrap_to => '#{pe_version}'
}
          PE_INSTALL_STUB
        end

        context 'default monolithic install' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono' }
          let(:base_parameter_hash) do
            {
              'certificate_authority_host'   => 'master.rspec',
              'puppet_master_host'           => 'master.rspec',
              'console_host'                 => 'master.rspec',
              'puppetdb_host'                => 'master.rspec',
              'database_host'                => 'master.rspec',
              'pcp_broker_host'              => 'master.rspec',
            }
          end

          include_examples 'common node groups'

          it 'PE Infrastructure node group' do
            is_expected.to contain_pe_node_group('PE Infrastructure')
            .with_classes({'puppet_enterprise' => base_parameter_hash})
          end

          it 'PE Master node group' do
            is_expected.to contain_pe_node_group('PE Master')
            .with_parent('PE Infrastructure')
            .with_pinned(['master.rspec'])
            .with_classes({
              'pe_repo' => {},
              "pe_repo::platform::#{platform_class}" => {},
              'pe_repo::platform::windows_x86_64' => {},
              'puppet_enterprise::profile::master' => { 'replication_mode' => 'none' },
            })
          end

          it 'PE Database node group' do
            is_expected.to contain_pe_node_group('PE Database')
            .with_parent('PE Infrastructure')
            .with_pinned([node])
            .with_classes({'puppet_enterprise::profile::database' => {}})
            .with_environment(environment)
          end

          it { is_expected.to have_pe_node_group_resource_count(install_group_count) }

          it 'Environment node groups' do
            is_expected.to contain_pe_node_group('All Environments')
            .with_classes({})
            .with_environment(environment)
            .with_environment_trumps(true)
            .with_create_only(true)

            is_expected.to contain_pe_node_group('Production environment')
            .with_classes({})
            .with_environment(environment)
            .with_environment_trumps(true)
            .with_create_only(true)

            is_expected.to contain_pe_node_group('Development environment')
            .with_classes({})
            .with_environment('development')
            .with_environment_trumps(true)
            .with_create_only(true)

            is_expected.to contain_pe_node_group('Development one-time run exception')
            .with_classes({})
            .with_environment('agent-specified')
            .with_environment_trumps(true)
            .with_create_only(true)
          end
        end

        context 'default monolithic upgrade' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono' }
          let(:base_parameter_hash) do
            {
              'certificate_authority_host'   => 'master.rspec',
              'puppet_master_host'           => 'master.rspec',
              'console_host'                 => 'master.rspec',
              'puppetdb_host'                => 'master.rspec',
              'database_host'                => 'master.rspec',
              'mcollective'                  => nil,
              'mcollective_middleware_hosts' => nil,
              'mcollective_middleware_port'  => nil,
              'mcollective_middleware_user'  => nil,
              'mcollective_middleware_password' => nil,
              'pcp_broker_host'              => 'master.rspec',
            }
          end

          let(:pre_condition) do
            <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare {}
class pe_install::install {}
class { pe_install:
  upgrade_from => '2018.1.3'
}
            PE_INSTALL_STUB
          end

          include_examples 'common node groups'

          it 'PE Infrastructure node group' do
            is_expected.to contain_pe_node_group('PE Infrastructure')
            .with_classes({'puppet_enterprise' => base_parameter_hash})
          end

          it 'PE Master node group' do
            is_expected.to contain_pe_node_group('PE Master')
            .with_parent('PE Infrastructure')
            .with_pinned(['master.rspec'])
            .with_classes({
              'pe_repo' => {},
              "pe_repo::platform::#{platform_class}" => {},
              'pe_repo::platform::windows_x86_64' => {},
              'puppet_enterprise::profile::master' => { 'replication_mode' => 'none' },
              'puppet_enterprise::profile::master::mcollective' => nil,
              'puppet_enterprise::profile::mcollective::peadmin' => nil,
            })
          end

          it { is_expected.to have_pe_node_group_resource_count(post_install_group_count) }

          it 'Environment node groups' do
            is_expected.not_to contain_pe_node_group('All Environments')
            is_expected.not_to contain_pe_node_group('Production environment')
            is_expected.not_to contain_pe_node_group('Development environment')
            is_expected.not_to contain_pe_node_group('Development one-time run exception')
          end
        end

        context 'running puppet-infra configure to remediate configuration post install' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono' }
          let(:pe_version) { nil }

          it { is_expected.to have_pe_node_group_resource_count(post_install_group_count) }

          it 'Environment node groups' do
            is_expected.not_to contain_pe_node_group('All Environments')
            is_expected.not_to contain_pe_node_group('Production environment')
            is_expected.not_to contain_pe_node_group('Development environment')
            is_expected.not_to contain_pe_node_group('Development one-time run exception')
          end
        end

        context 'PuppetDB as an array of hosts' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono-additional-puppetdb' }

          it { is_expected.to have_pe_node_group_resource_count(install_group_count) }

          it 'PE PuppetDB node group' do
            is_expected.to contain_pe_node_group('PE PuppetDB')
            .with_parent('PE Infrastructure')
            .with_pinned(['master.rspec','additional_pdb'])
            .with_classes({'puppet_enterprise::profile::puppetdb' => {}})
            .with_environment(environment)
          end
        end

        context 'Mulitple database_hosts' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'multiple-database-hosts' }

          it { is_expected.to have_pe_node_group_resource_count(install_group_count) }

          it 'PE Database node group' do
            is_expected.to contain_pe_node_group('PE Database')
            .with_parent('PE Infrastructure')
            .with_pinned(['pe-postgres.rspec','console.rspec','orch-db.rspec'])
            .with_classes({'puppet_enterprise::profile::database' => {}})
            .with_environment(environment)
          end
        end

        context 'Mulitple database_hosts and manage_pe_database_pinned_nodes is false' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'multiple-database-hosts' }
          let(:params) do
            { :manage_pe_database_pinned_nodes => false }
          end

          it { is_expected.to have_pe_node_group_resource_count(install_group_count) }

          it 'PE Database node group' do
            is_expected.to contain_pe_node_group('PE Database')
            .with_parent('PE Infrastructure')
            .with_pinned([])
            .with_classes({'puppet_enterprise::profile::database' => {}})
            .with_environment(environment)
          end
        end

        context 'monolithic install with hiera overrides' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono-classification-overrides' }
          let(:base_parameter_hash) do
            {
              'certificate_authority_host'   => 'master.rspec',
              'puppet_master_host'           => 'master.rspec',
              'console_host'                 => 'master.rspec',
              'puppetdb_host'                => 'master.rspec',
              'database_host'                => 'master.rspec',
              'pcp_broker_host'              => 'spoke.rspec',
            }
          end
          let(:environment) { 'master' }

          include_examples 'common node groups'

          it { is_expected.to have_pe_node_group_resource_count(install_group_count) }

          it 'PE Infrastructure node group' do
            is_expected.to contain_pe_node_group('PE Infrastructure')
            .with_classes({
              'puppet_enterprise' =>
              base_parameter_hash.merge({
                'activity_database_name'     => 'activity-db-name',
                'activity_service_migration_db_user'   => 'activity-db-user',
                'activity_service_regular_db_user'     => 'activity-db-user',
                'classifier_database_name'   => 'classifier-db-name',
                'classifier_service_migration_db_user'   => 'classifier-db-user',
                'classifier_service_regular_db_user'     => 'classifier-db-user',
                'orchestrator_database_name' => 'orchestrator-db-name',
                'orchestrator_service_migration_db_user' => 'orchestrator-db-user',
                'orchestrator_service_regular_db_user'   => 'orchestrator-db-user',
                'puppetdb_database_name'     => 'puppetdb-db-name',
                'puppetdb_database_user'     => 'puppetdb-db-user',
                'rbac_database_name'         => 'rbac-db-name',
                'rbac_service_migration_db_user'     => 'rbac-db-user',
                'rbac_service_regular_db_user'       => 'rbac-db-user',
                'database_ssl'               => false,
                'database_cert_auth'         => false,
                'console_port'               => 123,
                'database_port'              => 456,
                'puppetdb_port'              => 789,
              })})
          end

          it 'PE Master node group' do
            is_expected.to contain_pe_node_group('PE Master')
            .with_parent('PE Infrastructure')
            .with_pinned(['master.rspec'])
            .with_classes({
              'pe_repo' => {
                'base_path' => 'fake_tarball_server',
              },
              "pe_repo::platform::#{platform_class}" => {},
              'pe_repo::platform::windows_x86_64' => {},
              'puppet_enterprise::profile::master' => {
                'check_for_updates'           => false,
                'code_manager_auto_configure' => false,
                'file_sync_enabled'           => false,
                'r10k_remote'                 => 'fake_remote',
                'r10k_private_key'            => 'fake_path',
                'replication_mode'            => 'none',
                'use_legacy_auth_conf'        => false,
              },
            })
          end
        end
      end
    end
  end
end
