require 'spec_helper'

describe 'pe_install::prepare::certificates' do
  let(:hiera_config) { 'spec/fixtures/hiera/hiera.yaml' }

  context 'supported operating systems' do
    on_supported_os.each do |os, facts|
      context "on #{os}", :os => os do
        let(:facts) do
          facts
        end

        context 'with a pe_install stub' do
          def pre_condition
            <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare {}
class pe_install::install {}
include pe_install

# These dependencies must be explicit with Puppet 5
service { 'pe-puppetserver': }
service { 'puppet': }
service { 'pxp-agent': }
package { 'pe-puppetserver': }
            PE_INSTALL_STUB
          end

          let(:node) { 'master.rspec' }
          let(:layout) { 'split' }
          let(:facts) {
            facts.merge(
              :layout => layout
            )
          }
          let(:formatted_date_regex) {
            /(\+|-)\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}\s(\+|-)\d{4}/
          }

          it 'ensures ca setup that has puppet as a dnsaltname by default' do
            is_expected.to contain_exec('generate ca cert')
              .with_command(/\/opt\/puppetlabs\/bin\/puppetserver ca setup --certname #{node} --ca-name \'Puppet Enterprise CA generated on #{node} at #{formatted_date_regex}\' --subject-alt-names \'puppet\'/)
              .with_creates("/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem")
              .with_before('Exec[generate node cert]')
              .with_require('Package[pe-puppetserver]')
          end

          it 'ensures node cert' do
            is_expected.to contain_exec('generate node cert')
              .with_command(%{/opt/puppetlabs/bin/puppetserver ca generate --certname master.rspec --ca-client --subject-alt-names 'puppet'})
              .with_creates("/etc/puppetlabs/puppet/ssl/certs/#{node}.pem")
              .with_require([
                'Package[pe-puppetserver]',
                'Pe_install::Stop_service[stop pe-puppetserver before cert generation]'
              ])
          end

          context 'with a pre-generated CA bundle' do
            let(:node) { 'masterwithpregeneratedca.rspec' }
            let(:dnsaltnames_entry) { '' }
            let(:pe_conf) do
              <<-HIERA
"puppet_enterprise::certificate_authority_host": "#{node}"
"puppet_enterprise::puppet_master_host": "#{node}"
"pe_install::signing_ca": {
  "bundle": "/root/ca/int_ca_bundle"
  "crl_chain": "/root/ca/int_crl_chain"
  "private_key": "/root/ca/int_key"
}
#{dnsaltnames_entry}
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end

            it 'imports the ca' do
              is_expected.to contain_exec('import provided signing ca')
                .with_command(%r{/opt/puppetlabs/bin/puppetserver ca import --cert-bundle /root/ca/int_ca_bundle --crl-chain /root/ca/int_crl_chain --private-key /root/ca/int_key --subject-alt-names 'puppet'})
                .with_creates("/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem")
                .with_before('Exec[generate node cert]')
                .with_require('Package[pe-puppetserver]')
            end

            it 'does not attempt to generate a CA' do
              is_expected.not_to contain_exec('generate ca cert')
            end

            it 'ensures node cert' do
              is_expected.to contain_exec('generate node cert')
                .with_command(%{/opt/puppetlabs/bin/puppetserver ca generate --certname #{node} --ca-client --subject-alt-names 'puppet'})
                .with_creates("/etc/puppetlabs/puppet/ssl/certs/#{node}.pem")
            end

            context 'and dnsaltnames' do
              let(:node) { 'masterwithpregeneratedcaandaltnames.rspec' }
              let(:dnsaltnames) { %Q{["some","other.net","#{node}"]} }
              let(:dnsaltnames_entry) do
                %Q{"pe_install::puppet_master_dnsaltnames": #{dnsaltnames}}
              end

              it 'imports with given SANs' do
                is_expected.to contain_exec('import provided signing ca')
                  .with_command(/\/opt\/puppetlabs\/bin\/puppetserver ca import.* --subject-alt-names 'some,other.net,#{node}'/)
              end
            end
          end

          context 'with an empty signing_ca' do
            let(:node) { 'masterwithanemptysigningca.rspec' }
            let(:pe_conf) do
              <<-HIERA
"puppet_enterprise::certificate_authority_host": "#{node}"
"puppet_enterprise::puppet_master_host": "#{node}"
"pe_install::signing_ca": {
}
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end

            it 'raises Type error' do
              expect { catalogue }.to raise_error(Puppet::PreformattedError, /expects size to be 3, got 0/)
            end
          end

          context 'allow multiple puppetdbs' do
            let(:node) { 'mult_pdb_master.rspec' }
            let(:pe_conf) do
              <<-HIERA
"puppet_enterprise::certificate_authority_host": "#{node}"
"puppet_enterprise::puppet_master_host": "#{node}"
"puppet_enterprise::console_host": "mult_pdb_console.rspec"
"puppet_enterprise::puppetdb_host": ["puppetdb1.rspec", "puppetdb2.rspec"]
"puppet_enterprise::database_host": "puppetdb1.rspec"
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end

            it 'includes correct autosign entries' do
              is_expected.to contain_pe_file_line('autosign mult_pdb_console.rspec')
              is_expected.to contain_pe_file_line('autosign puppetdb1.rspec')
              is_expected.to contain_pe_file_line('autosign puppetdb2.rspec')
              is_expected.not_to contain_pe_file_line("autosign #{node}")
            end
          end

          context 'allow different console and puppetdb database_hosts' do
            let(:node) { 'mult_database_hosts_master.rspec' }
            let(:pe_conf) do
              <<-HIERA
"puppet_enterprise::certificate_authority_host": "#{node}"
"puppet_enterprise::puppet_master_host": "#{node}"
"puppet_enterprise::console_host": "console.rspec"
"puppet_enterprise::puppetdb_host": ["puppetdb1.rspec", "puppetdb2.rspec"]
"puppet_enterprise::database_host": "external_console_db_host.rspec"
"puppet_enterprise::puppetdb_database_host": "external_puppetdb_db_host.rspec"
"puppet_enterprise::orchestrator_database_host": "external_orch_db_host.rspec"
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end

            it 'includes correct autosign entries' do
              is_expected.to contain_pe_file_line('autosign console.rspec')
              is_expected.to contain_pe_file_line('autosign puppetdb1.rspec')
              is_expected.to contain_pe_file_line('autosign puppetdb2.rspec')
              is_expected.to contain_pe_file_line('autosign external_console_db_host.rspec')
              is_expected.to contain_pe_file_line('autosign external_puppetdb_db_host.rspec')
              is_expected.to contain_pe_file_line('autosign external_orch_db_host.rspec')
              is_expected.not_to contain_pe_file_line("autosign #{node}")
            end
          end

          context 'set dnsaltnames' do
            let(:node) { 'masterwithalt.rspec' }
            let(:dnsaltnames) { %Q{["some","other.net","#{node}"]} }
            let(:dnsaltnames_entry) do
              %Q{"pe_install::puppet_master_dnsaltnames": #{dnsaltnames}}
            end
            let(:pe_conf) do
              <<-HIERA
{
"puppet_enterprise::certificate_authority_host": "#{node}"
"puppet_enterprise::puppet_master_host": "#{node}"
"puppet_enterprise::console_host": "console.rspec"
"puppet_enterprise::puppetdb_host": "puppetdb.rspec"
"puppet_enterprise::database_host": "puppetdb.rspec"
"puppet_enterprise::pcp_broker_host": "#{node}"
#{dnsaltnames_entry}
}
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end

            it 'includes dnsaltnames if passed' do
              is_expected.to contain_exec('generate ca cert')
                .with_command(/\/opt\/puppetlabs\/bin\/puppetserver ca setup --certname #{node} --ca-name \'Puppet Enterprise CA generated on #{node} at #{formatted_date_regex}\' --subject-alt-names \'some,other\.net,masterwithalt\.rspec\'/)
                .with_creates("/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem")
            end

            it 'includes correct autosign entries' do
              is_expected.to contain_pe_file_line('autosign console.rspec')
              is_expected.to contain_pe_file_line('autosign puppetdb.rspec')
              is_expected.not_to contain_pe_file_line("autosign #{node}")
            end

            context 'as an empty list' do
              let(:node) { 'masterwithemptyalt.rspec' }
              let(:dnsaltnames) { "[]" }

              it 'skips dnsaltnames arg' do
                is_expected.to contain_exec('generate ca cert')
                  .with_command(/\/opt\/puppetlabs\/bin\/puppetserver ca setup --certname #{node} --ca-name \'Puppet Enterprise CA generated on #{node} at #{formatted_date_regex}\'/)
                  .with_creates("/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem")
              end
            end

            context 'when not set' do
              let(:node) { 'masterwithnoaltvalue.rspec' }
              let(:dnsaltnames_entry) { }

              it 'defaults to puppet' do

                is_expected.to contain_exec('generate ca cert')
                  .with_command(/\/opt\/puppetlabs\/bin\/puppetserver ca setup --certname #{node} --ca-name \'Puppet Enterprise CA generated on #{node} at #{formatted_date_regex}\' --subject-alt-names \'puppet\'/)
                  .with_creates("/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem")
              end
            end

            context 'with an empty string' do
              let(:node) { 'masterwithanemptystringalt.rspec' }
              let(:dnsaltnames) { '""' }

              it 'raises Type error' do
                expect { catalogue }.to raise_error(Puppet::PreformattedError, /expects an Array value, got String/)
              end
            end
          end

          context 'on a non-CA node' do
            let(:node) { 'puppetdb.rspec' }

            let(:pe_conf) do
              <<-HIERA
"puppet_enterprise::certificate_authority_host": "master.rspec"
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::console_host": "console.rspec"
"puppet_enterprise::puppetdb_host": "puppetdb.rspec"
"puppet_enterprise::database_host": "puppetdb.rspec"
              HIERA
            end

            around(:each) do |example|
              run_with_custom_pe_conf(node, pe_conf, example)
            end


            it 'retrieves certificates from the CA node' do
              is_expected.to contain_exec("retrieve cert for #{node}")
                .with_command("/opt/puppetlabs/puppet/bin/puppet ssl bootstrap --certname #{node} --ca_server master.rspec")
                .with_creates("/etc/puppetlabs/puppet/ssl/certs/#{node}.pem")
            end
          end
        end
      end
    end
  end
end
