require 'spec_helper'
require 'webmock/rspec'
require File.expand_path('../../../files/set_console_admin_password.rb', __FILE__)

WebMock.disable_net_connect!(allow_localhost: true)

describe SetConsoleAdminPassword do
  before(:each) do
    allow(File).to receive(:read).and_return('fake_file')
    allow(OpenSSL::PKey::RSA).to receive(:new)
    allow(OpenSSL::X509::Certificate).to receive(:new)
    stub_request(:get, /\/rbac-api\/v1\/users/).to_return(status: 200, body: '[{"id": 5, "login": "admin"}]', headers: {})
    stub_request(:post, /\/rbac-api\/v1\/users\/5\/password\/reset/).to_return(status: 200, body: 'fake_password', headers: {})
    stub_request(:post, /\/rbac-api\/v1\/auth\/reset/).to_return(status: 200, body: 'fake_password', headers: {})
  end

  it 'when url doesn\'t exist, get response returns raises error' do
    stub_request(:get, /\/rbac-api\/v1\/fake-endpoint/).to_return(status: 404, body: '', headers: {})

    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect{rp.send(:get_response, '/fake-endpoint')}.to raise_error(RuntimeError)
  end

  it 'when url exists, get response returns successfully' do
    stub_request(:get, /\/rbac-api\/v1\/fake-endpoint/).to_return(status: 200, body: 'fake-data', headers: {})

    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.send(:get_response, '/fake-endpoint').body).to eq('fake-data')
  end

  it 'when url doesn\'t exist, post response returns raises error' do
    stub_request(:post, /\/rbac-api\/v1\/fake-endpoint/).to_return(status: 404, body: '', headers: {})

    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect{rp.send(:post_response, '/fake-endpoint', 'fake-data')}.to raise_error(RuntimeError)
  end

  it 'when url exists, post response returns successfully' do
    stub_request(:post, /\/rbac-api\/v1\/fake-endpoint/).to_return(status: 200, body: 'fake-data', headers: {})

    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.send(:post_response, '/fake-endpoint', 'fake-data').body).to eq('fake-data')
  end

  it 'console_reset_url returns a URL with a reset token' do
    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.console_reset_url('fake_token')).to end_with('token=fake_token')
  end

  it 'reset_token_for returns token' do
    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.console_reset_url('fake_token')).to end_with('token=fake_token')
  end

  it 'reset_password returns password response' do
    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.reset_password.body).to eq('fake_password')
  end

  it 'get_user on existing user finds user' do
    rp = SetConsoleAdminPassword.new('admin', 'fake')

    expect(rp.get_user).to eq({'login' => 'admin', 'id' => 5})
  end

  it 'get_user on missing user doesn\'t return a user' do
    rp = SetConsoleAdminPassword.new('missing', 'fake')

    expect(rp.get_user).to be_nil
  end

end
