require 'spec_helper'

describe 'razor host' do
  context 'supported operating systems' do
    on_filtered_os(:redhat) do |os, base_facts|
      let(:node) { 'razor.rspec' }
      include_context('hosts', os, base_facts)
      let(:layout) { 'none' }
      let(:pe_postgresql_info) { nil }
      let(:context_facts) do
        {
          :pe_razor_server_version => pe_razor_server_version,
          :pe_postgresql_info => pe_postgresql_info
        }
      end
      let(:pe_version) { '2016.2.1' }

      before :each do
         _pe_version = pe_version
         Puppet::Parser::Functions.newfunction(:pe_build_version, :type => :rvalue) do |args|
          _pe_version
        end
      end

      RSpec.shared_examples('of a razor install') do
        it { is_expected.to compile.with_all_deps }
        it { is_expected.to contain_package('pe-java') }
        it { is_expected.to contain_package('pe-razor-server') }
        it { is_expected.to contain_package('postgresql-client') }
        it { is_expected.to contain_package('postgresql-server') }
        it { is_expected.to contain_package('postgresql-contrib') }
        it { is_expected.to_not contain_class('puppet_enterprise::repo::config') }
        it { is_expected.to_not contain_package('pe-postgresql-pglogical') }

        it do
          is_expected.to contain_pe_razor__server__repo('razor')
            .with_target("/opt/puppetlabs/server/data/razor-server/packages/#{pe_version}/razor-repo")
            .with_pe_tarball_base_url('https://pm.puppetlabs.com/puppet-enterprise')
        end

        it 'gets a tarball only if it has not yet unpacked a tarball for the current version of pe' do
          is_expected.to contain_exec('unpack the razor repo')
            .with_creates("/opt/puppetlabs/server/data/razor-server/packages/#{pe_version}/razor-repo/repodata/repomd.xml.asc")
        end

        it 'gets the correct pe/platform tarball' do
          is_expected.to contain_exec('unpack the razor repo')
            .with_command(%r{curl .* ".*/puppet-enterprise-#{pe_version}-#{platform_tag}.tar.gz"})
        end

        it 'unpacks the tarball into the correct versioned package directory' do
          is_expected.to contain_exec('unpack the razor repo')
            .with_command(%r{tar xC "/opt/puppetlabs/server/data/razor-server/packages/#{pe_version}/razor-repo"})
        end

        it 'configures yum to seek packages in the current pe versioned package directory that we unpacked packages into' do
          is_expected.to contain_yumrepo('pe-razor')
            .with_baseurl("file:///opt/puppetlabs/server/data/razor-server/packages/#{pe_version}/razor-repo")
        end

        it { is_expected.to contain_pe_razor__server__config('razor') }
        it { is_expected.to contain_pe_razor__server__database('razor') }
        it { is_expected.to contain_pe_razor__server__torquebox('razor') }
        it { is_expected.to contain_exec('migrate the razor database') }
        it { is_expected.to contain_exec('unpack the microkernel') }
        it { is_expected.to contain_service('pe-razor-server') }
      end

      context 'install' do
        let(:pe_razor_server_version) { 'package pe-razor-server is not installed' }
        include_examples 'of a razor install'
      end

      context 'upgrade from < 2017.3.0 with postgres migration' do
        let(:pe_version) { '2017.3.3' }
        let(:pe_razor_server_version) { '1.4.0.1' }
        let(:pe_postgresql_info) do
          {
            'installed_server_version' => '9.4',
            'installed_packages' => {
              'pe-postgresql' => '9.4.10',
              # ...
            },
          }
        end

        include_examples 'of a razor install'
        it { is_expected.to contain_class('pe_install::upgrade::postgres') }
        it do
          is_expected.to contain_service('Shutdown pe-razor-server before [Class[Pe_install::Upgrade::Postgres]]')
            .with_name('pe-razor-server')
            .with_before(['Class[Pe_install::Upgrade::Postgres]'])
        end
        it do
          is_expected.to(
            have_a_resource('Exec[migrate to new postgres]')
              .that_is_a_dependency_for_a_downstream('Class[Pe_postgresql::Server::Config]')
              .that_requires_an_upstream('Class[Pe_postgresql::Server::Initdb]')
          )
        end
        it do
          is_expected.to(
            have_a_resource('Exec[copy postgresql.conf from old install location to new install location]')
              .that_requires_an_upstream('Class[Pe_postgresql::Server::Initdb]')
          )
        end
      end

      context 'upgrade from >= 2017.3.0' do
        let(:pe_version) { '2018.1.0' }
        let(:pe_razor_server_version) { '1.7.0.1' }
        let(:pe_postgresql_info) do
          {
            'installed_server_version' => '9.6',
            'installed_packages' => {
              'pe-postgresql' => '9.6.12',
              # ...
            },
          }
        end

        include_examples 'of a razor install'
        it { is_expected.to_not contain_class('pe_install::upgrade::postgres') }
      end
    end
  end
end
