require 'spec_helper'
require 'facter/pe_postgresql_info'

describe 'pe_postgresql_info fact' do
  let(:tempdir) { Dir.mktmpdir("postgres-fact") }
  let(:info) { PEPostgresqlInfo.new }
  let(:empty_dir_bytes) do
    FileUtils.mkdir_p("#{tempdir}/empty")
    File.write("#{tempdir}/empty/foo", "database")
    bytes = info.get_bytes_used("#{tempdir}/empty")
    expect(bytes).to satisfy { |v| v > 0 }
    bytes
  end
  let(:server_version) { :absent }
  let(:client_version) { server_version }
  let(:pglogical_version) { :absent }
  let(:multiversion_package) { '' }
  let(:package_versions) do
    {
      "pe-postgresql#{multiversion_package}-pglogical" => pglogical_version,
      "pe-postgresql#{multiversion_package}-server" => server_version,
      "pe-postgresql#{multiversion_package}" => client_version,
    }
  end

  # If we have an rspec-puppet test in an earlier spec which sets the pe_postgresql_info
  # fact, rspec-puppet's support.rb will stub_facts!() and set this stub on Facter itself.
  # Facter will then never reload the real fact that we are testing here, unless we force
  # a clear first. See rambling in before(:each) for why we are not always using Facter.clear.
  #
  # The initial value call for pe_postgresql_info ensures that Facter
  # Kernel.load()'s a fresh copy of lib/facter/pe_postgresql_info, so that
  # mocking in our before(:each) doesn't get wiped out. Without this, the first spec
  # run does so without mocks, although subsequent specs will function as expected.
  before(:all) do
    Facter.clear
    Facter.value('pe_postgresql_info')
  end

  before(:each) do
    # Facter.reset causes the lib/facter/pe_postgresql_info.rb file to be reloaded
    # which wipes out the mocking of pe_server_dir. (Facter uses Kernel.load() in its
    # loader...)
    Facter.flush
    allow(PEPostgresqlInfo).to receive(:pe_server_dir).and_return(tempdir)
    allow_any_instance_of(PEPostgresqlInfo).to receive(:packages).and_return(package_versions)
  end

  after(:each) do
    FileUtils.rm_rf tempdir, :secure => true
  end

  def create_package(name, version)
    package_type = Puppet::Type.type(:package)
    package_type.new(:title => name, :ensure => version)
  end

  shared_context('mock postgres package') do
    before(:each) do
      Facter.add('pe_server_version') do
        setcode { '2019.1.0' }
      end
    end
  end

  shared_context('a 9.4 install') do
    let(:pglogical_version) { '2017.2.1.2.1-1.pe.el6' }
    let(:postgres_pglogical_version) { '1.2.1-1' }
    let(:server_version) { '2017.2.9.4.12-1.pe.el6' }
    let(:postgres_version) { '9.4.12-1' }

    include_context('a 9.4 data dir')
  end

  shared_context('a 9.4 data dir') do
    before(:each) do
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity/PG_9.4_201409291")
      File.write("#{tempdir}/data/postgresql/activity/PG_9.4_201409291/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/classifier/PG_9.4_201409291")
      File.write("#{tempdir}/data/postgresql/classifier/PG_9.4_201409291/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291")
      File.write("#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291")
      File.write("#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/rbac/PG_9.4_201409291")
      File.write("#{tempdir}/data/postgresql/rbac/PG_9.4_201409291/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/9.4")
      File.write("#{tempdir}/data/postgresql/9.4/database", "database")
    end
  end

  shared_context('a 9.6 install') do
    let(:pglogical_version) { '2019.1.2.2.1-5.pe.el6' }
    let(:postgres_pglogical_version) { '2.2.1-5' }
    let(:server_version) { '2019.1.9.6.12-1.pe.el7' }
    let(:postgres_version) { '9.6.12-1' }
    let(:multiversion_package) { '96' }

    include_context('a 9.4 data dir')

    before(:each) do
      FileUtils.mkdir_p("#{tempdir}/apps/postgresql/9.6")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity/PG_9.6_201608131")
      File.write("#{tempdir}/data/postgresql/activity/PG_9.6_201608131/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/classifier/PG_9.6_201608131")
      File.write("#{tempdir}/data/postgresql/classifier/PG_9.6_201608131/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/orchestrator/PG_9.6_201608131")
      File.write("#{tempdir}/data/postgresql/orchestrator/PG_9.6_201608131/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/puppetdb/PG_9.6_201608131")
      File.write("#{tempdir}/data/postgresql/puppetdb/PG_9.6_201608131/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/rbac/PG_9.6_201608131")
      File.write("#{tempdir}/data/postgresql/rbac/PG_9.6_201608131/tables", "tables")
      FileUtils.mkdir_p("#{tempdir}/data/postgresql/9.6")
      File.write("#{tempdir}/data/postgresql/9.6/database", "database")

      FileUtils.mkdir_p("#{tempdir}/apps/postgresql/9.4")
      File.write("#{tempdir}/data/postgresql/9.4/database", "database")
    end
  end

  shared_context('an 11 install') do
    let(:pglogical_version) { '2019.2.2.2.1-6.pe.el7' }
    let(:postgres_pglogical_version) { '2.2.1-6' }
    let(:server_version) { '2019.1.11.2-1.pe.el7' }
    let(:postgres_version) { '11.2-1' }
    let(:multiversion_package) { '11' }
  end

  context 'on a pe server node' do
    let(:pg_9_4_facts) do
      {
        'installed_server_version'       => '9.4',
        'installed_packages' => {
          'pe-postgresql-pglogical' => postgres_pglogical_version,
          'pe-postgresql-server' => postgres_version,
          'pe-postgresql' => postgres_version,
        },
        'data_partition_size_bytes'      => satisfy { |v| v > 0 },
        'data_partition_available_bytes' => satisfy { |v| v > 0 },
        'versions'         => {
          '9.4'            => {
            'used_bytes'   => empty_dir_bytes * 6,
            'app_dir'      => "#{tempdir}/apps/postgresql",
            'data_dir'     => "#{tempdir}/data/postgresql/9.4",
            'tablespaces'  => [
              "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/classifier/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/rbac/PG_9.4_201409291",
            ],
          },
        },
      }
    end
    let(:pg_9_6_facts) do
      {
        'installed_server_version'       => '9.6',
        'installed_packages' => {
          'pe-postgresql96-pglogical' => postgres_pglogical_version,
          'pe-postgresql96-server' => postgres_version,
          'pe-postgresql96' => postgres_version,
        },
        'data_partition_size_bytes'      => satisfy { |v| v > 0},
        'data_partition_available_bytes' => satisfy { |v| v > 0},
        'versions'         => {
          '9.4'            => {
            'used_bytes'   => empty_dir_bytes * 6,
            'app_dir'      => "#{tempdir}/apps/postgresql/9.4",
            'data_dir'     => "#{tempdir}/data/postgresql/9.4",
            'tablespaces'  => [
              "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/classifier/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291",
              "#{tempdir}/data/postgresql/rbac/PG_9.4_201409291",
            ],
          },
          '9.6'            => {
            'used_bytes'   => empty_dir_bytes * 6,
            'app_dir'      => "#{tempdir}/apps/postgresql/9.6",
            'data_dir'     => "#{tempdir}/data/postgresql/9.6",
            'tablespaces'  => [
              "#{tempdir}/data/postgresql/activity/PG_9.6_201608131",
              "#{tempdir}/data/postgresql/classifier/PG_9.6_201608131",
              "#{tempdir}/data/postgresql/orchestrator/PG_9.6_201608131",
              "#{tempdir}/data/postgresql/puppetdb/PG_9.6_201608131",
              "#{tempdir}/data/postgresql/rbac/PG_9.6_201608131",
            ],
          },
        },
      }
    end

    include_context('mock postgres package')

    context 'without pe-postgresql-server installed' do
      let(:server_version) { :absent }
      let(:client_version) {  '2019.1.9.6.12-1.pe.el7' }

      it do
        expect(Facter.value('pe_postgresql_info')).to eq({
          'installed_server_version' => nil,
          'data_partition_size_bytes' => 0,
          'data_partition_available_bytes' => 0,
          'installed_packages' => {
            'pe-postgresql' => '9.6.12-1',
          },
          'versions' => {},
        })
      end
    end

    context 'without any pe-postgresql packages info' do
      let(:package_versions) { {} }
      it 'has nothing' do
        expect(Facter.value('pe_postgresql_info')).to match(
          {
            'data_partition_available_bytes' => 0,
            'data_partition_size_bytes' => 0,
            'installed_server_version' => nil,
            'installed_packages' => {},
            'versions' => {},
          }
        )
      end
    end

    context 'with pe-postgresql-server installed' do
      context 'before migration', :broken_on_osx do
        include_context 'a 9.4 install'
        let(:server_version) { '2016.4.9.4.12-1.pe.el7' }

        it do
          expect(Facter.value('pe_postgresql_info')).to match(pg_9_4_facts)
        end
      end

      context 'after migration', :broken_on_osx do
        include_context 'a 9.6 install'
        let(:server_version) { '2019.1.9.6.12-1.pe.el7' }

        it do
          expect(Facter.value('pe_postgresql_info')).to match(pg_9_6_facts)
        end
      end

      context 'with older version format (<2016.5.0)', :broken_on_osx do
        include_context 'a 9.4 install'
        let(:pglogical_version) { 'absent' }
        let(:postgres_pglogical_version) { 'absent' }
        let(:server_version) { '9.4.9-1.pe.el7' }
        let(:postgres_version) { '9.4.9-1' }

        it do
          expect(Facter.value('pe_postgresql_info')).to match(
            pg_9_4_facts.merge(
              'installed_packages' => {
                'pe-postgresql-server' => postgres_version,
                'pe-postgresql' => postgres_version,
              }
            )
          )
        end
      end

      context 'different version formats' do
        include_context 'a 9.6 install'

        context 'sles', :broken_on_osx do
          let(:pglogical_version) { '2019.1.2.2.1-5.pe.sles12' }
          let(:server_version) { '2019.1.9.6.12-1.pe.sles12' }

          it 'extracts PG version from a sles version' do
            expect(Facter.value('pe_postgresql_info')).to match(pg_9_6_facts)
          end
        end

        context 'ubuntu', :broken_on_osx do
          let(:pglogical_version) { '2019.1.2.2.1-5xenial' }
          let(:server_version) { '2019.1.9.6.12-1xenial' }

          it 'extracts PG version from an ubuntu version' do
            expect(Facter.value('pe_postgresql_info')).to match(pg_9_6_facts)
          end
        end

        context 'postgres 11', :broken_on_osx do
          let(:server_version) { '2019.2.11.1-1.pe.el6' }

          it 'will handle Postgres 10' do
            expect(Facter.value('pe_postgresql_info')).to include({
              'installed_server_version' => '11'
            })
          end
        end
      end
    end
  end

  context 'PEPostgresqlInfo' do
    context '#tablespace' do
      it { expect(info.tablespace(nil)).to be_nil }
      it { expect(info.tablespace('9.4')).to be_nil }

      context 'with an activity dir' do
        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity")
        end

        it { expect(info.tablespace('9.4')).to be_nil }

        context 'and a tablespace dir' do
          before(:each) do
            FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity/PG_9.4_201608131")
          end

          it { expect(info.tablespace('9.4')).to eq('PG_9.4_201608131') }
          it { expect(info.tablespace('9.6')).to be_nil }
        end
      end
    end

    context '#versions' do
      it { expect(info.versions).to eq([]) }

      context 'with a postgres data dir' do
        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/data/postgresql")
        end

        it { expect(info.versions).to eq([]) }

        context 'with some data versions' do
          before(:each) do
            FileUtils.mkdir_p("#{tempdir}/data/postgresql/9.4")
          end

          it { expect(info.versions).to eq(['9.4']) }
          it do
            FileUtils.mkdir_p("#{tempdir}/data/postgresql/9.6")
            FileUtils.mkdir_p("#{tempdir}/data/postgresql/11.0")
            expect(info.versions).to eq(['9.4','9.6','11.0'])
          end
        end
      end
    end

    context '#app_dir' do
      include_context('mock postgres package')
      let(:server_version) { 'absent' }

      it { expect(info.app_dir(nil)).to be_nil }
      it { expect(info.app_dir('9.4')).to be_nil }

      context 'when installed server matches' do
        let(:server_version) { '2017.2.9.4.12-1.pe.el6' }

        it 'returns default appdir' do
          expect(info.app_dir('9.4')).to eq("#{tempdir}/apps/postgresql")
        end

        context 'else nothing if directory does not exist' do
          let(:server_version) { '2019.1.9.6.12-2.pe.el6'}

          it { expect(info.app_dir('9.4')).to be_nil }
        end
      end

      context 'with app_dir' do
        let(:server_version) { '2019.1.9.6.12-2.pe.el6'}

        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/apps/postgresql/9.6")
          FileUtils.mkdir_p("#{tempdir}/apps/postgresql/9.4")
        end

        it { expect(info.app_dir('9.6')).to eq("#{tempdir}/apps/postgresql/9.6") }
        it { expect(info.app_dir('9.4')).to eq("#{tempdir}/apps/postgresql/9.4") }
        it { expect(info.app_dir('9.2')).to be_nil }
      end
    end

    context '#data_dir' do
      it { expect(info.data_dir(nil)).to be_nil }
      it { expect(info.data_dir('9.4')).to be_nil }

      context 'with data_dir' do
        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/9.4")
        end

        it { expect(info.data_dir('9.4')).to eq("#{tempdir}/data/postgresql/9.4") }
        it { expect(info.data_dir('9.6')).to be_nil }
      end
    end

    context '#tablespace_dirs' do
      it { expect(info.tablespace_dirs(nil)).to eq([]) }
      it { expect(info.tablespace_dirs('9.4')).to eq([]) }

      context 'with a tablespace' do
        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity/PG_9.4_201409291")
        end

        it do
          expect(info.tablespace_dirs('9.4')).to eq([
            "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
          ])
        end
        it { expect(info.tablespace_dirs('9.6')).to eq([]) }
      end

      context 'with all tablespaces' do
        before(:each) do
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/activity/PG_9.4_201409291")
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/classifier/PG_9.4_201409291")
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291")
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291")
          FileUtils.mkdir_p("#{tempdir}/data/postgresql/rbac/PG_9.4_201409291")
        end

        it do
          expect(info.tablespace_dirs('9.4')).to eq([
            "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
            "#{tempdir}/data/postgresql/classifier/PG_9.4_201409291",
            "#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291",
            "#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291",
            "#{tempdir}/data/postgresql/rbac/PG_9.4_201409291",
          ])
        end
        it { expect(info.tablespace_dirs('9.6')).to eq([]) }
      end
    end

    context '#files' do
      include_context('mock postgres package')
      let(:server_version) { 'absent' }

      let(:empty_files) do
        {
          'app_dir' => nil,
          'data_dir' => nil,
          'tablespaces' => [],
        }
      end

      it { expect(info.files(nil)).to eq(empty_files) }
      it { expect(info.files('9.4')).to eq(empty_files) }

      context 'with a 9.4 install' do
        include_context 'a 9.4 install'

        it 'matches the default install structure' do
          expect(info.files('9.4')).to match(
            {
              'app_dir' => "#{tempdir}/apps/postgresql",
              'data_dir' => "#{tempdir}/data/postgresql/9.4",
              'tablespaces' => [
                "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/classifier/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/rbac/PG_9.4_201409291",
              ],
            }
          )
        end

        it { expect(info.files('9.6')).to eq(empty_files) }
      end
    end

    context '#get_bytes_used', :broken_on_osx do
      before(:each) do
        FileUtils.mkdir("#{tempdir}/bar")
        File.write("#{tempdir}/bar/foo", "database")
      end
      it '0 bytes if no directory is specified' do
        expect(info.get_bytes_used(nil)).to eq 0
      end
      it "0 bytes if the directory doesn't exist" do
        expect(Dir).to receive(:exists?).once.with('/fake').and_return(false)
        expect(info).not_to receive(:`)
        expect(info.get_bytes_used('/fake')).to eq 0
      end
      it 'Kilobytes of directory size if the directory does exist' do
        expect(Dir).to receive(:exists?).once.with('/fake').and_return(true)
        allow_any_instance_of(Process::Status).to receive(:exitstatus).and_return(0)
        expect(info).to receive(:`).once.with('du --block-size=1024 -s /fake 2>&1').and_return('5\t/fake')
        expect(info.get_bytes_used('/fake')).to eq 5120
      end
      it 'Kilobytes of the files found if there is an error' do
        # Postgres may create temporary files that are reaped before du completes.
        # We want to determine the size of the directory for all files that were found
        expect(Dir).to receive(:exists?).once.with('/fake').and_return(true)
        allow_any_instance_of(Process::Status).to receive(:exitstatus).and_return(1)
        expect(info).to receive(:`).once.with('du --block-size=1024 -s /fake 2>&1').and_return("du: cannot access `/opt/puppetlabs/server/data/postgresql/9.6/data/pg_stat_tmp/global.tmp': No such file or directory\n5\t/fake")
        expect(Facter).to receive(:warn)
        expect(info.get_bytes_used('/fake')).to eq 5120
      end
      it "0 bytes if the directory doesn't exist", :integration_test do
        expect(info.get_bytes_used('/fake')).to eq 0
      end
      it 'Kilybotes of directory size if the directory does exist', :integration_test do
        expect(info.get_bytes_used("#{tempdir}/bar")).to eq(empty_dir_bytes)
      end
    end

    context '#used_bytes' do
      include_context('mock postgres package')
      let(:server_version) { 'absent' }

      it { expect(info.used_bytes(nil)).to eq(0) }
      it { expect(info.used_bytes('9.4')).to eq(0) }

      context 'with a 9.4 install', :broken_on_osx do
        include_context 'a 9.4 install'

        it 'is six empty directories of space' do
          expect(info.used_bytes('9.4')).to eq(empty_dir_bytes * 6)
        end
      end
    end

    context '#version_data' do
      include_context('mock postgres package')
      let(:server_version) { 'absent' }
      let(:empty_version_data) do
        {
          'app_dir' => nil,
          'data_dir' => nil,
          'tablespaces' => [],
          'used_bytes' => 0,
        }
      end

      it { expect(info.version_data(nil)).to eq(empty_version_data) }
      it { expect(info.version_data('9.4')).to eq(empty_version_data) }

      context 'with a 9.4 install', :broken_on_osx do
        include_context 'a 9.4 install'

        it 'returns file and used space data for 9.4' do
          expect(info.version_data('9.4')).to eq(
            {
              'app_dir' => "#{tempdir}/apps/postgresql",
              'data_dir' => "#{tempdir}/data/postgresql/9.4",
              'tablespaces' => [
                "#{tempdir}/data/postgresql/activity/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/classifier/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/orchestrator/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/puppetdb/PG_9.4_201409291",
                "#{tempdir}/data/postgresql/rbac/PG_9.4_201409291",
              ],
              'used_bytes' => empty_dir_bytes * 6
            }
          )
        end
      end
    end

    context '#data_partition_size' do
      context 'without postgres' do
        around(:each) do |example|
          current_log_level = Puppet[:log_level]
          Puppet[:log_level] = :debug
          example.run
          Puppet[:log_level] = current_log_level
        end

        it do
          expect(info.data_partition_size).to eq({
            :size_bytes => 0,
            :used_bytes => 0,
            :available_bytes => 0,
          })
          expect(@logs.map { |l| "#{l.level}:#{l.message}" }).to include(
            match(/^debug:Failed disk free check/)
          )
        end
      end

      context 'when data directory exists', :broken_on_osx do
        include_context 'a 9.4 install'

        it do
          expect(psb = info.data_partition_size).to match({
            :size_bytes => satisfy { |v| v > 0 },
            :used_bytes => satisfy { |v| v > 0 },
            :available_bytes => satisfy { |v| v > 0 },
          })
          expect((psb[:used_bytes] + psb[:available_bytes]).to_f/psb[:size_bytes].to_f).to be_within(0.051).of(0.95)
        end
      end
    end

    context '#installed_server_version' do
      context 'old packages' do
        include_context 'a 9.4 install'

        it { expect(info.installed_server_version).to eq('9.4') }
      end
      context 'multiversion 9.6' do
        include_context 'a 9.6 install'

        it { expect(info.installed_server_version).to eq('9.6') }
      end
      context 'multiversion 11' do
        include_context 'an 11 install'

        it { expect(info.installed_server_version).to eq('11') }

        context 'and multiversion 9.6' do
          before(:each) do
            package_versions.merge!({
              "pe-postgresql96-pglogical" => '2019.1.2.2.1-2',
              "pe-postgresql96-server" => '2019.1.9.6.12-1',
              "pe-postgresql96" => '2019.1.9.6.12-1',
            })
          end
         
          it do
            expect(info.installed_packages).to eq({
              "pe-postgresql11" => "11.2-1",
              "pe-postgresql11-pglogical" => "2.2.1-6",
              "pe-postgresql11-server" => "11.2-1",
              "pe-postgresql96" => "9.6.12-1",
              "pe-postgresql96-pglogical" => "2.2.1-2",
              "pe-postgresql96-server" => "9.6.12-1",
            })
          end
          it { expect(info.installed_server_version).to eq('11') }
        end
      end
    end
  end

  context 'on an agent node' do
    it 'will always return nil because we are confined' do
      expect(Puppet::Resource).to_not receive(:indirection)
      Facter.reset
      expect(Facter.value('pe_postgresql_info')).to be_nil
    end
  end
end
