require 'puppet/indirector/face'
require 'puppet/application/apply'
require 'puppet/util/command_line'
require 'puppet/agent'
require 'puppet/error'
require 'puppet/util/pe_conf'
require 'puppet_x/util/stringformatter'
require 'puppet_x/puppetlabs/meep/configure/validator'
require 'puppet_x/puppetlabs/meep/configure/mco_status'
require 'puppet_x/puppetlabs/meep/configure/psql'
require 'puppet_x/puppetlabs/meep/config/modify'
require 'puppet_x/puppetlabs/meep/hiera_adapter'
require 'puppet_x/puppetlabs/meep/puppet_context'
require 'puppet_x/util/password'
require 'puppet_x/util/analytics'

# The initial version of the face is based of Reid Vandewiele's nimbus
# module's latest version, which is 0.7.0.
#
# https://github.com/puppetlabs/tse-module-nimbus/releases/tag/0.7.0
Puppet::Face.define(:infrastructure, '1.0.0') do
  # Remove check after Puppet 5 is released.
  if defined?(Puppet::Util::Locker)
    extend Puppet::Util::Locker
  else
    extend Puppet::Agent::Locker
  end
  extend PuppetX::Puppetlabs::Meep::PuppetContext

  # This is the major.minor of the Postgresql in this branch of PE.
  # A migration is required whenever this version changes (not the patch
  # version)
  CURRENT_POSTGRES_VERSION = '9.6'

  # This is the major.minor of the Postgresql we are upgrading from.
  # This is a simplification that assumes we won't be upgrading across two
  # releases of Postgresql.
  PREVIOUS_POSTGRES_VERSION = '9.4'

  # This file is used to mark the version we are upgrading from if the
  # --upgrade-from flag has been passed in.  It persists until we have a
  # successful run of the configure face.
  UPGRADE_IN_PROCESS_FILE = '/opt/puppetlabs/server/upgrade_in_process'

  # Location of MEEP's pe.conf hieradata directory.
  ENTERPRISE_CONF_DIR = '/etc/puppetlabs/enterprise'

  action :configure do

    summary _('Configure the local system using given Puppet all-in-one configuration file(s).')
    description PuppetX::Util::String::Formatter.join_and_indent([
      _("Configure the local system using given Puppet all-in-one configuration file(s)."),
      "\n",
      _("Will exit (17) if another puppet run is already in progress.")])
    when_invoked do |options|
      install_method = options[:install_method]
      @is_upgrade = options.include?(:upgrade_from) && !options[:upgrade_from].nil?
      @is_install = !(install_method.nil? || install_method.empty?) && !@is_upgrade
      action = @is_upgrade ? "upgrade" : "install"
      begin
        lock do
          begin
            attempt_to_recover_configuration(options)
            update_pe_conf_to_disable_analytics
          ensure
            puppet_infrastructure_context do
              send_analytics_event("pe_installer","pe_version",options[:install])

              if @is_upgrade
                save_upgrade_in_process(options)
                send_analytics_event("pe_installer","upgrade_start","upgrade_start")
                send_analytics_event("pe_installer","upgrade_from",options[:upgrade_from])
              end
              send_analytics_event("pe_installer","install_start",install_method) if @is_install

              ensure_meep_load_path(Puppet.lookup(:current_environment))
              validate_current_installation(options)
              update_pe_conf_with_database_encoding_if_migrating(options)

              argv = default_arguments(options)
              argv << '--detailed-exitcodes' if options[:detailed_exitcodes]
              command_line = Puppet::Util::CommandLine.new('puppet', argv)
              apply = Puppet::Application::Apply.new(command_line)
              apply.parse_options
              Puppet::Util::Log.close(:console)
              apply.run_command
            end
          end
        end
      rescue PuppetX::Puppetlabs::Meep::Configure::Validator::InstallationInvalid
        send_analytics_event("pe_installer","#{action}_finish","failed")
        send_analytics_event("pe_installer","#{action}_fail_type","validator.installationinvalid")
        exit(27)
      rescue Puppet::LockError
        send_analytics_event("pe_installer","#{action}_finish","failed")
        send_analytics_event("pe_installer","#{action}_fail_type","puppet_lock_error")
        Puppet.notice _("Puppet run already in progress") + "; " + _("aborting  (%{lockfile_path} exists)") % { lockfile_path: lockfile_path }
        exit(17)
      rescue SystemExit => e
        if [0,2].include?(e.status)
          send_analytics_event("pe_installer","#{action}_finish","succeeded")
          inform_on_successful_postgres_migration(options)
          remove_upgrade_in_process
          if options.include?(:install) && !options[:upgrade_from] && password_omitted? && console_node?
            revoke_admin_access
            notify_reset_url
          end
          if options.include?(:upgrade_from) && !options[:upgrade_from].nil? && console_node?
            revoked = check_for_default_password_and_revoke
            notify_reset_url if revoked
          end
        else
          send_analytics_event("pe_installer","#{action}_finish","failed")
          send_analytics_event("pe_installer","#{action}_fail_type","exit_code_not_0_or_2")
        end
        raise e
      end
    end

    option('--detailed-exitcodes') do
      summary _('Provide extra information about the run via exit codes')
      description PuppetX::Util::String::Formatter.join_and_indent([
        _("Provide extra information about the run via exit codes.") + " " + _("If enabled, 'puppet infrastructure configure' will use the following exit codes:"),
        "\n",
        _("0: The run succeeded with no changes or failures; the system was already in the desired state."),
        "\n",
        _("1: The run failed."),
        "\n",
        _("2: The run succeeded, and some resources were changed."),
        "\n",
        _("4: The run succeeded, and some resources failed."),
        "\n",
        _("6: The run succeeded, and included both changes and failures.")])
    end

    option('--install <version>') do
      summary _('Install version submitted by the puppet-enterprise-installer shim')
      description PuppetX::Util::String::Formatter.indent(_("The installer shim includes this flag to distinguish this as a bootstrap run of puppet-infrastructure intended to install or upgrade the specified version."))
    end

    option('--install-method <method>') do
      summary _('Method of install chosen by the user, submitted by the installer shim')
      description <<-EOT
        The installer shim will supply this flag to tell the configure action what method
        of install the user chose (e.g. express, text, web). This is used for sending
        install analytics, unless puppet_enterprise::send_analytics_data = false in 
        pe.conf.
      EOT
    end

    option('--upgrade-from <version>') do
      summary _('The currently installed version of PE that we are upgrading from')
      description <<-EOT
        The installer shim will supply this flag if it detects a previous install
        of PE that is different from the version we are currently installing. This
        affects the installation process; notably, cached catalogs are cleared, and
        all services are stopped before packages are installed.

        The fact that we are upgrading is preserved in the file #{UPGRADE_IN_PROCESS_FILE},
        which is automatically cleared after a successful configure run. If the
        configure is not successful, the value in this file is pulled in as the default
        for --upgrade-from in subsequent executions of configure.

        If that behavior is not desired, the file can be deleted.
      EOT

      default_to do
        configure_action = @face
        infrastructure_interface = configure_action.face
        upgrade_from = infrastructure_interface.upgrade_in_process
        unless upgrade_from.nil?
          message = _("Setting --upgrade-from=%{upgrade_from} based on presence of %{upgrade_in_process_file}.") % { upgrade_from: upgrade_from, upgrade_in_process_file: UPGRADE_IN_PROCESS_FILE }
          Puppet.notice(message)
        end
        upgrade_from
      end
    end

    option('--[no-]postgres-migration') do
      summary _('Override the automatic inclusion/exclusion of the postgres migration class')
      description <<-EOT
        Whether to include puppet code to migrate from an older version of
        Postgresql to the current Puppet Enterprise version is determined
        automatically based on the currently installed pe-postgresql-server
        package.

        If the configure run is interrupted or fails after the new pe-postgresql-server
        is installed, but before pg_upgrade is called to perform the data
        migration, then you can include --postgres-migration to force the migration.
      EOT
      default_to do
        configure_action = @face
        infrastructure_interface = configure_action.face

        postgres_server_version = infrastructure_interface.postgres_server_version
        cmp = infrastructure_interface.version_cmp(postgres_server_version, CURRENT_POSTGRES_VERSION)
        postgres_migration = (cmp == -1)

        if postgres_migration
          message = _("Including pe_install::upgrade::postgres migration class based on current pe-postgresql-server %{postgres_server_version} being older than the version being installed (%{current_postgres_version}).") % { postgres_server_version: postgres_server_version, current_postgres_version: CURRENT_POSTGRES_VERSION }
          Puppet.notice(message)
        end
        postgres_migration
      end
    end

    option('--force') do
      summary _('Force the installation even if validation fails.')
      description <<-EOT
        Normally if a step validating the state of the current installation fails,
        the install is halted. If you are certain the validation does not apply,
        you may force the install to proceed by supplying this flag.

        This is not recommended.
      EOT
      default_to { false }
    end

    option('--[no-]recover') do
      summary _('Attempt to recover PE configuration')
      default_to { nil }
    end

    option('--pe-environmentpath PATH') do
      summary _('The environmentpath to be used when recovering configuration.')
      description <<-EOT
        The puppet-infrastructure face loads from an isolated enterprise
        environmentpath under /opt/puppetlabs/server. But it must recover hiera
        configuration from PE's environmentpath which defaults to
        /etc/puppetlabs/code/environments. If this value has been changed in
        puppet.conf it can be set here to ensure proper configuration recovery.
      EOT
    end

    option('--pe-environment ENVIRONMENT') do
      summary _('The environment to be used when recovering configuration.')
      description <<-EOT
        The puppet-infrastructure face load from an isolated enterprise
        environment under /opt/puppetlabs/server. But it recovers hiera configuration
        from PE's environment as specified in the PE Node Groups. Commonly this
        is the 'production' environment, but if PE infrastructure has been configured
        in a different environment it may be specified here.
      EOT
    end
  end

  def enterprise_conf_dir
    ENTERPRISE_CONF_DIR
  end

  def send_analytics_event(category,action,label)
    return if ENV['DISABLE_ANALYTICS']
    PuppetX::Util::Analytics.send_analytics_event(category,action,label) if @is_install || @is_upgrade
  end

  def update_pe_conf_to_disable_analytics
    if ENV['DISABLE_ANALYTICS']
      Puppet.notice(_("Detected DISABLE_ANALYTICS environment variable. Setting puppet_enterprise::send_analytics_data = false in pe.conf."))
      pe_conf = PuppetX::Puppetlabs::Meep::Modify.new(enterprise_conf_dir)
      pe_conf.set_in_pe_conf("puppet_enterprise::send_analytics_data",false)
      PuppetX::Util::Analytics.check_analytics_setting
    end
  end

  def inform_on_successful_postgres_migration(options)
    if options[:postgres_migration]
      old_datastore = pe_postgresql_info['versions'][PREVIOUS_POSTGRES_VERSION] || {}
      old_data_dir = old_datastore['data_dir']
      old_tablespaces = old_datastore['tablespaces']

      directories_to_remove = [old_data_dir, old_tablespaces].flatten.compact

      message = [
        _("Your Postgresql datastore has been migrated from %{previous_postgres_version} to %{current_postgres_version}.") % { previous_postgres_version: PREVIOUS_POSTGRES_VERSION, current_postgres_version: CURRENT_POSTGRES_VERSION },
        _("The %{previous_postgres_version} datastore has been left on disk as a backup.") % { previous_postgres_version: PREVIOUS_POSTGRES_VERSION },
        _("When you have verified that the %{current_postgres_version} datastore is up to date, you may wish to remove the following %{previous_postgres_version} directories to free up space: %{directories_to_remove}") % { current_postgres_version: CURRENT_POSTGRES_VERSION, previous_postgres_version: PREVIOUS_POSTGRES_VERSION, directories_to_remove: directories_to_remove.join(", ") }
      ]
      Puppet.notice(message.join("\n"))
    end
  end

  def update_pe_conf_with_database_encoding_if_migrating(options)
    if options[:postgres_migration]
      psql_opts = {}
      psql_opts[:uid] = pe_postgres_uid

      psql = PuppetX::Puppetlabs::Meep::Configure::PSQL.new(psql_opts)
      encoding = nil
      begin
        encoding = psql.encoding_data
      rescue Puppet::ExecutionFailure => e
        message = [
          _("Failed to obtain current Postgresql server encoding, lc_type and lc_collate."),
          _("Will proceed with defaults (UTF8, en_US.UTF8, en_US.UTF8)."),
          _("Original error: %{error}") % { error: e.message },
        ]
        Puppet.warning(message.join("\n"))
        return false
      end

      pe_conf = PuppetX::Puppetlabs::Meep::Modify.new(enterprise_conf_dir)
      [
        ["server_encoding","encoding"],
        # The Postgresql initdb command takes a --locale flag which sets all of
        # the lc_* settings for Postgres which are in turn stored in the
        # database. We can only query out the lc_* values.  The only settings
        # we can pass to the postgresql module for initdb are locale, ctype and
        # collate, so we have to fudge and use ctype for locale.
        # https://github.com/puppetlabs/puppetlabs-pe_postgresql/blob/2016.5.0/manifests/server/initdb.pp#L45-L47
        ["lc_ctype", "locale"],
        ["lc_ctype", "ctype"],
        ["lc_collate", "collate"],
      ].each do |postgres_key,module_parameter|
        pe_conf.set_in_pe_conf("puppet_enterprise::profile::database::#{module_parameter}", encoding[postgres_key])
      end

      return true
    end
  end

  def validate_current_installation(options)
    state = {
      :force => options[:force],
      :upgrade_from => options[:upgrade_from],
    }
    rules = []

    if options[:postgres_migration]
      state.merge!({
        :postgres_migration => options[:postgres_migration],
        :pe_postgresql_info => pe_postgresql_info,
        :previous_postgres_version => PREVIOUS_POSTGRES_VERSION,
        # CURRENT_POSTGRES_VERSION is the version of Postgress we're upgrading to in this release.
        :new_postgres_version => CURRENT_POSTGRES_VERSION,
      })
      rules << PuppetX::Puppetlabs::Meep::Configure::PostgresMigrationSpace
    end

    if !options[:upgrade_from].nil? && !options[:upgrade_from].empty?
      state.merge!({
          :jruby_9k_enabled => puppet_lookup('puppet_enterprise::master::puppetserver::jruby_9k_enabled')
      })
      rules << PuppetX::Puppetlabs::Meep::Configure::ParameterValidation

      if separate_database_node? && master? && local_postgresql_client_is_behind?
        state.merge!({
          :pe_postgresql_info => pe_postgresql_info,
          :separate_database_node => separate_database_node?,
          :database_host => database_host_parameter,
          :external_database_version => external_database_version,
          :external_database_behind => version_cmp(external_database_version, CURRENT_POSTGRES_VERSION) == -1,
        })
        rules << PuppetX::Puppetlabs::Meep::Configure::ExternalPostgres
      end

      if master?
        state.merge!({
          :mco_all_nodes => mco_status.nodes,
          :mco_infrastructure_nodes => mco_status.infrastructure,
          :mco_agent_nodes => mco_status.agents,
          :mco_query_error => mco_status.error,
        })
        rules << PuppetX::Puppetlabs::Meep::Configure::McollectiveRemoval
      end
    end

    validator = PuppetX::Puppetlabs::Meep::Configure::Validator.new(state)
    validator.register(rules)

    unless validator.valid?
      validator.log_and_fail!
    end
    validator.valid?
  end

  def local_postgresql_client_is_behind?
    installed_packages = pe_postgresql_info['installed_packages'] || {}
    postgresql_version = installed_packages['pe-postgresql']
    postgresql_version.nil? ?
      false :
      version_cmp(postgresql_version, CURRENT_POSTGRES_VERSION) == -1
  end

  # If the database host is not one of the three primary hosts, this is an
  # indication that it may be an externally managed database.
  #
  # @return [Boolean] true if this installation is configured with a
  #   database that is not one of the primary, puppetdb, or console nodes.
  def separate_database_node?
    cached(:separate_database_node) do
      database_host = database_host_parameter

      if database_host.nil?
        false
      else
        master_host = puppet_lookup('puppet_enterprise::puppet_master_host')
        console_host = puppet_lookup('puppet_enterprise::console_host')
        puppetdb_host = puppet_lookup('puppet_enterprise::puppetdb_host')
        primary_infrastructure = [master_host, console_host, puppetdb_host].compact

        !primary_infrastructure.include?(database_host)
      end
    end
  end

  def database_host_parameter
    cached(:database_host) do
      puppet_lookup('puppet_enterprise::database_host')
    end
  end

  # @return [Hash] A set of postgres options that should allow a DB connection
  def get_psql_options()
    database_host = database_host_parameter
    database_port = puppet_lookup('puppet_enterprise::database_port')
    default_pe_user = 'pe-postgres'

    master_host = puppet_lookup('puppet_enterprise::puppet_master_host')
    orch_ssldir = '/etc/puppetlabs/orchestration-services/ssl'
    orch_creds = {
      cert: "#{orch_ssldir}/#{master_host}.cert.pem",
      key: "#{orch_ssldir}/#{master_host}.private_key.pem",
      user: puppet_lookup('puppet_enterprise::orchestrator_database_super_user') || puppet_lookup('puppet_enterprise::orchestrator_database_user') || 'pe-orchestrator',
      database: puppet_lookup('puppet_enterprise::orchestrator_database_name') || 'pe-orchestrator',
      password: puppet_lookup('puppet_enterprise::orchestrator_database_password')
    }

    console_host = puppet_lookup('puppet_enterprise::console_host')
    console_ssldir = '/opt/puppetlabs/server/data/console-services/certs'
    console_creds = {
      cert: "#{console_ssldir}/#{console_host}.cert.pem",
      key: "#{console_ssldir}/#{console_host}.private_key.pem",
      user: puppet_lookup('puppet_enterprise::classifier_database_super_user') || puppet_lookup('puppet_enterprise::classifier_database_user') || 'pe-classifier',
      database: puppet_lookup('puppet_enterprise::classifier_database_name') || 'pe-classifier',
      password: puppet_lookup('puppet_enterprise::classifier_database_password')
    }

    puppetdb_host = puppet_lookup('puppet_enterprise::puppetdb_host')
    puppetdb_ssldir = '/etc/puppetlabs/puppetdb/ssl'
    puppetdb_creds = {
      cert: "#{puppetdb_ssldir}/#{puppetdb_host}.cert.pem",
      key: "#{puppetdb_ssldir}/#{puppetdb_host}.private_key.pem",
      user: puppet_lookup('puppet_enterprise::puppetdb_database_super_user') || puppet_lookup('puppet_enterprise::puppetdb_database_user') || 'pe-puppetdb',
      database: puppet_lookup('puppet_enterprise::puppetdb_database_name') || 'pe-puppetdb',
      password: puppet_lookup('puppet_enterprise::puppetdb_database_password')
    }

    ca = '/etc/puppetlabs/puppet/ssl/certs/ca.pem'

    options = {
      :host => database_host,
    }
    options[:port] = database_port if !database_port.nil?

    available_ssl = [orch_creds, puppetdb_creds, console_creds].select { |creds| File.readable?(creds[:cert]) && File.readable?(creds[:key]) }
    found_ssl = available_ssl[0] if available_ssl.any?
    # assume cert auth unless overridden
    puppet_enterprise_database_ssl = puppet_lookup('puppet_enterprise::database_ssl')
    database_ssl = !puppet_enterprise_database_ssl.nil? ? puppet_enterprise_database_ssl : true
    puppet_enterprise_database_cert_auth = puppet_lookup('puppet_enterprise::database_cert_auth')
    database_cert_auth = !puppet_enterprise_database_cert_auth.nil? ? puppet_enterprise_database_cert_auth : true

    if found_ssl
      options[:user] = found_ssl[:user]
      options[:database] = found_ssl[:database]
      # User has cert auth disabled, but the SSL certs exist
      if !database_cert_auth
        options[:password] = found_ssl[:password]
      end
    # SSL certs aren't available and local postgres user exists
    elsif system("id -u #{default_pe_user}", :out => File::NULL, :err => File::NULL)
      options[:uid] = default_pe_user
      options[:user] = default_pe_user
      options[:database] = 'postgres'
      options[:host] = 'localhost'
    else
      options[:user] = puppetdb_creds[:user]
      options[:database] = puppetdb_creds[:database]
      # Not sure the condition is necessary, since no certs were found, we can assume the user has to use password auth?
      if !database_cert_auth
        options[:password] = puppetdb_creds[:password]
      end
    end

    if database_ssl
      options[:sslmode] = 'verify-full' if database_ssl
      if File.readable?(ca)
        options[:sslcacert] = ca
      else
        Puppet.warning(_("We found that database SSL is enabled, but no CA certificate could be found at %{ca}" % {ca: ca}))
      end
    end

    if database_cert_auth && options[:host] != 'localhost'
      if found_ssl
        options[:sslcert] = found_ssl[:cert]
        options[:sslkey] = found_ssl[:key]
      else
        Puppet.warning(_("We found that database SSL certificate-based authentication is enabled, but no certificates could be found."))
      end
    end

    return options
  end

  # @return [String] the major.minor of the external postgresql database,
  #   or nil if we manage postgresql on primary or puppetdb node.
  def external_database_version
    return nil if !separate_database_node?

    cached(:external_database_version) do
      psql_opts = get_psql_options()
      psql = PuppetX::Puppetlabs::Meep::Configure::PSQL.new(psql_opts)
      version = nil
      begin
        version = psql.server_version
      rescue Puppet::ExecutionFailure => e
        messages = [
          _("It appears you have an externally managed Postgresql server '%{host}'.") % psql_opts,
          _("However, we were unable to look up the version of the Postgresql server on '%{host}'.") % psql_opts,
          _("Puppet::ExecutionFailure: %{error}") % { error: e.message },
        ]
        Puppet.warning(messages.join("\n"))
      end

      version
    end
  end

  # @return [McoStatus] returns a cached, queried McoStatus instance which can be
  #   used to determine whether there are any nodes which still have MCollective
  #   or Activemq managed on them, according to Puppetdb.
  def mco_status
    cached(:mco_status) do
      mco = PuppetX::Puppetlabs::Meep::Configure::McoStatus.new
      mco.query_puppetdb(300)
      mco
    end
  end

  def upgrade_in_process_file
    UPGRADE_IN_PROCESS_FILE
  end

  def upgrade_in_process
    File.read(upgrade_in_process_file).chomp if File.exist?(upgrade_in_process_file)
  end

  def save_upgrade_in_process(options)
    upgrade_from = options[:upgrade_from]
    return if upgrade_from.nil? || upgrade_from.empty?
    File.write(upgrade_in_process_file, upgrade_from)
  end

  def remove_upgrade_in_process
    FileUtils.rm(upgrade_in_process_file) if File.exist?(upgrade_in_process_file)
  end

  def enterprise_hiera_yaml
    '/etc/puppetlabs/enterprise/hiera.yaml'
  end

  def puppet_infrastructure_context(user_arguments = [], &block)
    Puppet::Node.indirection.reset_terminus_class

    settings_to_override = {
      :splay                 => false,
      :node_terminus         => 'plain',
      :data_binding_terminus => 'hiera',
      :default_file_terminus => 'file_server',
      :hiera_config          => enterprise_hiera_yaml,
      :user                  => 'pe-puppet',
      :group                 => 'pe-puppet',
    }

    # Puppet.settings.use(:main) was already called in Puppet::Application::Infrastructure#setup()
    # Clear the Puppet::Settings @service_user_available so that future calls to
    # Puppet.settings.use() bubbling up from the Indirector during the catalog run
    # (lib/puppet/indirector/ssl_file.rb, lib/puppet/indirector/report/processor.rb)
    # can act appropriately if pe-puppet has been created during the execution
    # of the catalog.
    Puppet.settings.send(:remove_instance_variable, :@service_user_available) if Puppet.settings.send(:instance_variable_defined?, :@service_user_available)
    puppet_environment_context(default_environmentpath, default_environment, '', settings_to_override, &block)
  end

  def attempt_recovery?(recover = :auto)
    case recover
    when true then true
    when false then false
    # :auto (default) case -- only attempt recovery if we are upgrading the
    # master node, as that is the only node that has the codedir for hiera
    # overrides in addition to being able to query puppetdb/classifier...
    # TODO: This should be replaced with a classification lookup in Flanders
    else master? && installed?
    end
  end

  def attempt_to_recover_configuration(options)
    if attempt_recovery?(options[:recover])
      recover_options = options.select { |k,v| [:pe_environmentpath, :pe_environment].include?(k) }
      conf = Puppet::Util::Pe_conf.recover_and_save_pe_configuration(recover_options)
      pp_pe_conf = JSON.pretty_generate(conf.pe_conf)
      Puppet.debug(_("Generated the following pe.conf: %{conf}") % { conf: pp_pe_conf })

      if conf.nodes_conf.size > 0
        pp_nodes_conf = JSON.pretty_generate(conf.nodes_conf)
        Puppet.debug(_("Generated the following node specific overrides: %{conf}") % { conf: pp_nodes_conf })
      end
      send_analytics_event("pe_installer","pe_conf_recovery","succeeded")
      return true
    else
      return false
    end
  rescue Exception => e
    Puppet.notice(_("Unable to recover PE configuration: %{error}") % { error: e })
    if e.respond_to?(:backtrace)
      Puppet.debug("#{e}:#{e.backtrace.join("\n")}")
    end
    send_analytics_event("pe_installer","pe_conf_recovery","failed")
    return false
  end

  # A handle for puppet lookup set to the enterprise hiera configuration.
  # This let's us interogate pe.conf as the current node.
  def enterprise_lookup_handle
    cached(:enterprise_lookup_handle) do
      PuppetX::Puppetlabs::Meep::HieraAdapter.new(enterprise_hiera_yaml)
    end
  end

  # Lookup a parameter in the Hiera hierarchy pointed to by the current setting
  # of Puppet[:hiera_config]. This is intended to lookup current parameter
  # values set in pe.conf, but this function must be called within the context
  # of a puppet_infrastructure_context() block in order for hiera_config to have
  # been set properly.
  #
  # @param parameter [String] the parameter key to lookup.
  # @return [String] the found value or nil if the parameter was not found.
  def puppet_lookup(parameter)
    node = cached(:puppet_lookup_node) do
      PuppetX::Puppetlabs::Meep::HieraAdapter.get_node
    end
    scope = cached(:puppet_lookup_scope) do
      PuppetX::Puppetlabs::Meep::HieraAdapter.generate_scope(node)
    end
    enterprise_lookup_handle.lookup(parameter, scope)
  end

  def resource_lookup(resource, id)
    resource_face = Puppet::Face[:resource, :current]
    resource_face.find("#{resource}/#{id}")
  end

  def pe_postgres_uid
    pe_postgres_user = resource_lookup('user', 'pe-postgres')
    pe_postgres_user[:uid] if pe_postgres_user[:ensure] == :present
  end

  def package_version(package_name)
    package_resource = resource_lookup('package', package_name)
    package_resource[:ensure]
  end

  def package_present?(package_name)
    case package_version(package_name)
    when :purged, :absent then false
    else true
    end
  end

  def master?
    cached(:pe_puppetserver_package) do
      package_present?('pe-puppetserver')
    end
  end

  def pe_postgresql_info
    cached(:pe_postgresql_info) do
      Facter.value('pe_postgresql_info') || {}
    end
  end

  # @return the major.minor of the currently installed pe-postgresql-server
  #   package or nil if not installed.
  def postgres_server_version
    version = pe_postgresql_info['installed_server_version']
    version == '' ? nil : version
  end

  def pe_server_version
    cached(:pe_server_version) do
      Facter.value('pe_server_version')
    end
  end

  def installed?
    !pe_server_version.nil?
  end

  # Expects dot separated version strings such as "2017.3.0"
  #
  # Short circuits returning nil if either argument is nil or an empty string.
  #
  # @return -1, 0 or 1 based on <=> comparison of the Gem::Version
  #   where -1 indicates a < b
  #          0 indicates a = b
  #          1 indicates a > b
  def version_cmp(a, b)
    return nil if a.nil? || b.nil? || a.empty? || b.empty?
    v1 = Gem::Version.create(a)
    v2 = Gem::Version.create(b)
    v1 <=> v2
  end

  def classes_to_include(options)
    classes = []

    installing = options.include?(:install) && !options[:install].nil?
    upgrading  = options.include?(:upgrade_from) && !options[:upgrade_from].nil?

    pe_install_params = []
    pe_install_params << "bootstrap_to => '#{options[:install]}'," if installing
    pe_install_params << "upgrade_from => '#{options[:upgrade_from]}'," if upgrading

    pe_install_class = <<-EOS
class { pe_install:
  #{pe_install_params.join("\n  ")}
}
    EOS

    if installing || upgrading
      classes.push(
        <<-EOS
class { puppet_enterprise::packages:
  installing => true,
}
        EOS
      )
    end
    classes.push(pe_install_class)

    if options[:postgres_migration]
      classes.push(
        <<-EOS
class { pe_install::upgrade::postgres:
  old_postgres_version => '#{PREVIOUS_POSTGRES_VERSION}',
  new_postgres_version => '#{CURRENT_POSTGRES_VERSION}',
}
        EOS
      )
    end
    classes.join("\n")
  end

  def default_arguments(options)
    [
      'apply',
      '--execute', classes_to_include(options),
      '--logdest', 'timestamped_console',
    ]
  end

  def default_environmentpath
    '/opt/puppetlabs/server/data/environments'
  end

  def default_environment
    'enterprise'
  end

  # This method just ensures we have a working LOAD_PATH just in case the initial
  # modulepath pointed to a directory which contained pe_manager, but not the rest
  # of the pe modules needed for meep to function.
  def ensure_meep_load_path(env)
    env.each_plugin_directory do |dir|
      $LOAD_PATH << dir unless $LOAD_PATH.include?(dir)
    end
  end

  # This is a test convenience
  # @api private
  def clear_cache
    @configure_cache = {}
  end

  # @api private
  def cache
    @configure_cache ||= {}
  end

  # @api private
  def cached(key)
    if !cache.include?(key)
      cache[key] = yield
    end
    cache[key]
  end

  def notify_reset_url
    begin
      output = PuppetX::Util::PasswordUtils.console_reset_admin_url
      write_to_reset_url_file(output)
    rescue StandardError => e
      Puppet.err(_("A problem was encountered while attempting to generate a Reset Token - %{error_message}") % {error_message: e.message})
    end
  end

  def write_to_reset_url_file(url)
    reset_url_file = '/opt/puppetlabs/server/share/installer/reset-url'
    reset_dir = File.dirname(reset_url_file)
    FileUtils.mkdir_p(reset_dir)
    File.open(reset_url_file, 'w', 0600) do |file|
      file.write(url)
    end
  end

  def console_node?
    puppet_master_host = puppet_lookup("puppet_enterprise::puppet_master_host")
    console_host = puppet_lookup('puppet_enterprise::console_host')
    certname = Puppet[:certname]

    console_host.nil? ?
      puppet_master_host == certname :
      console_host == certname
  end

  def password_omitted?
    console_admin_password = puppet_lookup("console_admin_password")

    # At the time of this change, console_admin_password can not be empty
    # The empty check is in place to prevent future changes that may allow it to be empty from
    # creating a bug.
    console_admin_password.nil? || console_admin_password.empty?
  end

  def revoke_admin_access
    begin
      PuppetX::Util::PasswordUtils.revoke_admin_access
    rescue StandardError => e
      Puppet.err(_("A problem was encountered when ensuring the admin account is inaccessible until a password is set - %{error_message}") % {error_message: e.message})
    end
  end

  def check_for_default_password_and_revoke
    begin
      revoked = PuppetX::Util::PasswordUtils.check_for_default_password_and_revoke
    rescue StandardError => e
      Puppet.err(_("A problem was encountered when ensuring the admin account is inaccessible until a password is set - %{error_message}") % {error_message: e.message}) 
    end
  end
end
