require 'puppet/indirector/face'
require 'puppet/application/apply'
require 'puppet/util/command_line'
require 'puppet/agent'
require 'puppet/error'
require 'puppet/util/pe_node_groups'
require 'puppet_x/util/service_status'
require 'puppet_x/util/rbac'
require 'puppet_x/util/classification'
require 'puppet_x/util/infrastructure_error'
require 'puppet_x/util/orchestrator'
require 'puppet/network/http_pool'
require 'puppet_x/util/interview'

Puppet::Face.define(:infrastructure, '1.0.0') do

  action :enable do
    summary _("Enable a replica node")
    description PuppetX::Util::String::Formatter.join_and_indent([
        _("After a node has been fully provisioned, it needs to be enabled."),
        "\n",
        "puppet infrastructure enable replica <NODE NAME>",
        "\n",
        "replica role:"],
        PuppetX::Util::String::Formatter.indent_level(1)) + "\n" +
      PuppetX::Util::String::Formatter.join_and_indent([
        _("When provisioning a primary master replica, some data needs to be copied to it before it's ready to serve requests."),
        _("After the data has been copied (which you can verify using `puppet infrastructure status`), enabling it will allow it to serve requests from the rest of your PE infrastructure when the master is not available.")],
        PuppetX::Util::String::Formatter.indent_level(2))

    arguments "<role> <replica_certname>"

    option("--token-file <token_path>") do
      summary _("Path to a token file")
      description PuppetX::Util::String::Formatter.indent(
        _("Path to an RBAC token to use for authentication."),
        PuppetX::Util::String::Formatter.indent_level(1))
    end

    option('--print-config') do
      summary _("Print the current configuration")
      description PuppetX::Util::String::Formatter.indent(
        _("Shows the current configuration of HA related settings."),
        PuppetX::Util::String::Formatter.indent_level(1))
    end

    option('-y', '--yes') do
      summary _("Answer yes to all confirmations.")
      description PuppetX::Util::String::Formatter.indent(
        _("Answer yes to all confirmations."),
        PuppetX::Util::String::Formatter.indent_level(1))
    end

    option('--topology <topology>') do
      summary _("The topology of your PE installation")
      description PuppetX::Util::String::Formatter.indent(
          _("The topology of your pe installation:"
        ), PuppetX::Util::String::Formatter.indent_level(1)) + "\n" +
        PuppetX::Util::String::Formatter.join_and_indent([
          _("mono - All PE services run on the primary master and all agents connect to that master."),
          _("mono-with-compile - All PE services run on the primary master but agents connect to compile masters.")
        ], PuppetX::Util::String::Formatter.indent_level(2))
    end

    option('--skip-agent-config') do
      summary _("Do not change agent settings")
      default_to {nil}
      description PuppetX::Util::String::Formatter.indent(
        _("Do not change agent settings when enabling replica. Agent failover settings must be configured by the user."))
    end

    option("--force") do
      summary _("Skip all pre-enable validation.")
      default_to {false}
      description <<-EOT
        Skip all pre-enable validation. Skipping checks will mean that
        prerequisite services may not be configured or running and some
        enable steps may fail or need to be manually run.
      EOT
    end

    option('--pcp-brokers <uris>') do
      summary _("Hostnames and ports of your PCP Brokers")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    option('--infra-pcp-brokers <uris>') do
      summary _("Hostnames and ports of your PCP Brokers for PE infrastructure")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    option('--puppetdb-termini <uris>') do
      summary _("Hostnames and ports of your PuppetDBs")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    option('--classifier-termini <uris>') do
      summary _("Hostnames and ports of your Node Classifiers")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    option('--agent-server-urls <uris>') do
      summary _("Hostnames and ports of your Puppet Masters to run your Puppet Agents against")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    option('--infra-agent-server-urls <uris>') do
      summary _("Hostnames and ports your Puppet Masters to run your Puppet Infrastructure Agents against")
      description <<-EOT
        hostname:port,hostname:port
      EOT
    end

    when_invoked do |role, replica_certname, options|
      if role != 'replica'
        raise PuppetX::Util::InfrastructureUnknownRoleError.new(role, ["replica"]) # rubocop:disable GetText/DecorateFunctionMessage
      end

      master_certname = Puppet[:certname]
      if options[:print_config]
        # for print_config we only really need the classifier
        required_services = PuppetX::Util::ServiceStatus.replica_services().select {|service, _| service == :classifier}
      else
        required_services = PuppetX::Util::ServiceStatus.primary_services()
      end

      config = PuppetX::Util::ServiceStatus.load_services_config()
      nodes_config = PuppetX::Util::ServiceStatus.load_nodes_config()
      services = PuppetX::Util::ServiceStatus.validate_and_select_services_from_config(config, 'enable', master_certname, required_services)

      nc_service = services[:classifier]
      nc = Puppet::Util::Pe_node_groups.new(nc_service[:server], nc_service[:port].to_i, "/#{nc_service[:prefix]}")

      all_groups = nc.get_groups()

      if options[:print_config]
        puts _('Current configuration:')
        puts PuppetX::Util::Classification.print_config(all_groups)
        return
      end

      # check that the thing we're enabling really is a replica
      unless options[:force]
        unless PuppetX::Util::ServiceStatus.node_role(nodes_config, replica_certname) == 'primary_master_replica'
          Puppet.err(_("The node %{replica_certname} does not appear to be provisioned as a replica.") % { replica_certname: replica_certname } + " " + _("Make sure that it has been provisioned with the 'puppet infrastructure provision replica' command and run 'puppet agent -t' on the master to update service configuration."))
          exit(1)
        end
      end

      configure_agent = !options[:skip_agent_config]

      if PuppetX::Util::Interview.any_custom_flags?(options)
        PuppetX::Util::Interview.validate_infra_flags('enable', options)
        infra_conf = PuppetX::Util::Interview.parse_infra_flags(options)
      else
        class8n = PuppetX::Util::Classification.get_config(all_groups)
        suggestions = PuppetX::Util::Classification.suggest_enable_config(class8n, master_certname, replica_certname, configure_agent)
        # TODO: We actually manipulate these in the interview now
        infra_conf = PuppetX::Util::Interview.enable_interview(class8n, suggestions, options)
      end

      orch = services[:orchestrator]

      unless options[:force]
        rbac_token = PuppetX::Util::RBAC.load_token(options[:token_file])
        unless PuppetX::Util::Orchestrator.node_in_orch_inventory(orch, rbac_token, replica_certname)
          Puppet.err(_("The node %{replica_certname} is not connected to the master via PCP.") % { replica_certname: replica_certname } + " " + _("Make sure that orchestration services are enabled on the master and that the PXP agent is running on %{replica_certname}") % { replica_certname: replica_certname })
          exit(1)
        end
      end

      PuppetX::Util::Classification.enable_replica(nc, all_groups, master_certname, replica_certname, infra_conf)

      puts _("Updated classification")

      rbac_token ||= PuppetX::Util::RBAC.load_token(options[:token_file])
      PuppetX::Util::Orchestrator.run_puppet(services[:orchestrator], master_certname, {:nodes => [master_certname]}, rbac_token)
      PuppetX::Util::Orchestrator.run_puppet(services[:orchestrator], replica_certname, {:nodes => [replica_certname]}, rbac_token)

      PuppetX::Util::Orchestrator.run_puppet(services[:orchestrator],
                                             'all other master nodes',
                                             {:query => ['from', 'resources',
                                                         ['extract', ['certname'],
                                                          ['and',
                                                           ['=', 'type', 'Class'],
                                                           ['=', 'title', 'Puppet_enterprise::Profile::Master'],
                                                           ['not', ['=', 'certname', master_certname]],
                                                           ['not', ['=', 'certname', replica_certname]],
                                                           ['=', ['node', 'active'], true]]]]},
                                             rbac_token,
                                             :allow_empty => true)

      puts
      if configure_agent
        puts <<-EOT
#{replica_certname} has been enabled for your Puppet Enterprise
infrastructure. Agent configuration is now managed with Puppet, and will
be updated the next time each agent runs to enable replica failover.

If you wish to immediately run Puppet on all your agents, you can do so
with this command:

    puppet job run --no-enforce-environment --query 'nodes {deactivated is null and expired is null}'
EOT
      end
    end
  end
end
