require 'puppet/indirector/face'
require 'puppet/application/apply'
require 'puppet/util/command_line'
require 'puppet/agent'
require 'puppet/error'
require 'puppet/util/pe_node_groups'
require 'puppet_x/util/service_status'
require 'puppet_x/util/rbac'
require 'puppet_x/util/classifier'
require 'puppet_x/util/infrastructure_error'
require 'puppet_x/util/orchestrator'
require 'puppet_x/util/code_manager'
require 'puppet_x/util/stringformatter'
require 'puppet/network/http_pool'

Puppet::Face.define(:infrastructure, '1.0.0') do

  action :provision do
    summary _("Provision a PE node")
    description <<-EOT
      Provision a PE node with a specific role.
      Known roles:
        replica
    EOT

    arguments "<role> <replica_certname>"

    option("--token-file PATH") do
      summary _("Path to a token file")
      description <<-EOT
        Path to an RBAC token to use for authentication.
      EOT
    end

    option("--force") do
      summary _("Skip all pre-provision validation.")
      default_to {false}
      description <<-EOT
        Skip all pre-provision validation. Skipping checks will mean that
        prerequisite services may not be configured or running and some
        provisioning steps may fail or need to be manually run.
      EOT
    end

    when_invoked do |role, replica_certname, options|
      if role != 'replica'
        raise PuppetX::Util::InfrastructureUnknownRoleError.new(role, ["replica"]) # rubocop:disable GetText/DecorateFunctionMessage
      end

      master_certname = Puppet.settings[:certname].strip

      config = PuppetX::Util::ServiceStatus.load_services_config()
      nodes_config = PuppetX::Util::ServiceStatus.load_nodes_config()

      services = PuppetX::Util::ServiceStatus.validate_and_select_services_from_config(config, 'provision', master_certname)
      orch = services[:orchestrator]
      unless options[:force]
        services_statuses = PuppetX::Util::ServiceStatus.services_for(master_certname, nil, config, 5)
        PuppetX::Util::ServiceStatus.ensure_all_services_running(services_statuses)
        fs_status = services_statuses.find {|s| s[:service] == "file-sync-storage-service" }
        if fs_status[:status]["repos"]["puppet-code"]["latest_commit"].nil?
          Puppet.err(_("No commits have been made to puppet-code."))
          Puppet.err(_("Deploy with code-manager before provisioning by running `puppet code deploy <environment>`."))
          exit(1)
        end

        rbac_token = PuppetX::Util::RBAC.load_token(options[:token_file])

        if replica_certname == master_certname
          Puppet.err(_("%{certname} is already provisioned as the primary master.") % { certname: replica_certname })
          exit(1)
        end

        existing_role = PuppetX::Util::ServiceStatus.node_role(nodes_config, replica_certname)
        if existing_role && existing_role != 'primary_master_replica'
          Puppet.err(_("%{certname} is already provisioned with the role %{role}.") % { certname: replica_certname, role: existing_role })
          exit(1)
        end

        unless PuppetX::Util::Orchestrator.node_in_orch_inventory(orch, rbac_token, replica_certname)
          Puppet.err(_("The node %{certname} is not connected to the master via PCP.") % { certname: replica_certname } + " " +
                     _("Check that you are using the certname of the replica node, which may be different from the hostname.") + " " +
                     _("Also verify that orchestration services are enabled on the master and that the PXP agent is running on %{certname}.") % { certname: replica_certname })
          exit(1)
        end
      end

      nc_service = services[:classifier]
      nc = Puppet::Util::Pe_node_groups.new(nc_service[:server], nc_service[:port].to_i, "/#{nc_service[:prefix]}")

      PuppetX::Util::Classification.provision_replica(nc, master_certname, replica_certname)

      puts _("Updated classification")

      rbac_token ||= PuppetX::Util::RBAC.load_token(options[:token_file])
      PuppetX::Util::Orchestrator.run_puppet(orch, master_certname, {:nodes => [master_certname]}, rbac_token)
      PuppetX::Util::CodeManager.kick_file_sync()

      begin
        PuppetX::Util::Orchestrator.run_puppet(orch, replica_certname, {:nodes => [replica_certname]}, rbac_token)
      ensure
        PuppetX::Util::Orchestrator.run_puppet(orch, master_certname, {:nodes => [master_certname]}, rbac_token)
      end

      version = PuppetX::Util::ServiceStatus.pe_version
      pdb_sync_link = "https://docs.puppet.com/pe/#{version}/ha_configure.html#manually-copy-puppetdb-to-speed-replication"

      puts PuppetX::Util::String::Formatter.join_and_wrap_no_indent([
        _("A primary master replica has been provisioned. Services are syncing.") + " " + _("If you have a large PuppetDB instance, consult the documentation at %{link} to learn approaches for speeding up the process.") % { link: pdb_sync_link },
        "\n",
        _("To track sync progress, enter the following command:") + "\n",
        "`puppet infrastructure status --host %{certname}`" % { certname: replica_certname },
        "\n",
        _("Once the sync is complete, enter `puppet infrastructure enable replica %{certname}` on the primary master so that it will be ready to handle a failover from the primary master.") % { certname: replica_certname },
        _("If you run this command before the primary master replica has fully synchronized, the replica will not have all the necessary information to replace the primary master during a failover.")])
    end
  end
end
