require 'puppet/indirector/face'
require 'puppet/util/pe_node_groups'
require 'puppet_x/util/apply'
require 'puppet_x/util/classification'
require 'puppet_x/util/classifier'
require 'puppet_x/util/interview'
require 'puppet_x/util/service_status'
require 'puppet_x/util/shell'
require 'puppet_x/util/stringformatter'

ReinitializeStatusTimeoutSeconds = 5

Puppet::Face.define(:infrastructure, '1.0.0') do
  action :reinitialize do
    summary _("Reinitialize replication")
    description _("Clear and re-synchronize replicated databases on the replica")

    arguments "<role>"

    option('-y', '--yes') do
      summary _("Always proceed without asking the user for confirmation")
      default_to {nil}
    end

    option '--db DATABASE' do
      summary 'Reinitialize only this database'
      default_to { nil }
    end

    when_invoked do |role, options|
      if role != 'replica'
        msg = <<-HEREDOC
Usage:
  puppet infrastructure reinitialize replica
        HEREDOC
        raise msg
        exit(1)
      end

      replica_certname = Puppet[:certname]

      config = PuppetX::Util::ServiceStatus.load_services_config()
      nc_service = config.find { |svc| svc[:type] == 'classifier' }

      # Bail if nc_service is nil
      if nc_service.nil?
        msg = PuppetX::Util::String::Formatter.join_and_wrap_no_indent([
_("Could not find configuration for the Classifier terminus."),
_("Has this node been provisioned as a replica?"),
_("This command must only be run on a replica!")])
        raise msg
        exit(1)
      end

      nc = Puppet::Util::Pe_node_groups.new(nc_service[:server], nc_service[:port].to_i, "/#{nc_service[:prefix]}")
      all_groups = nc.get_groups()

      # Is the current node even a replica?
      if !PuppetX::Util::Classifier.node_is_pinned_as_replica?(all_groups, replica_certname)
        raise _("This command must only be run on a replica!")
        exit(1)
      end

      # Warn the user if the primary is down
      primary = PuppetX::Util::Classification.find_primary_master(all_groups)
      primary_status = PuppetX::Util::ServiceStatus.services_for(primary, nil, config, ReinitializeStatusTimeoutSeconds)
      unless PuppetX::Util::ServiceStatus.all_services_running?(primary_status)
        not_running = PuppetX::Util::ServiceStatus.services_not_running(primary_status)
                        .map{|service| service[:display_name]}
                        .join(", ")
        services_warning = _("The following services on the primary are not running: %{services}") % {services: not_running}
        if options[:yes].nil?
          unless PuppetX::Util::Interview.confirm(PuppetX::Util::String::Formatter.join_and_wrap_no_indent([
services_warning,
_("Are you sure you want to proceed?") + " [Y/n]:"]))
          end
        else
          Puppet.warn(_("%{warning}, continuing anyways due to `--yes` flag") % { warning: services_warning })
        end
      end

      # Get confirmation
      if options[:yes].nil?
        unless PuppetX::Util::Interview.confirm(PuppetX::Util::String::Formatter.join_and_wrap_no_indent([
_("Reinitializing is a destructive operation: the existing databases will be dropped and re-created."),
_("Are you sure you want to proceed?") + " [y/N]:"]), false)
          exit(1)
        end
      end

      pe_infrastructure_group = PuppetX::Util::Classifier.find_group(all_groups, 'PE Infrastructure')
      puppet_enterprise = pe_infrastructure_group['classes']['puppet_enterprise']

      # Is the target_database_name specified, and is it the puppetdb_database_name?
      if options[:db] && (options[:db] == puppet_enterprise['puppetdb_database_name'])
        raise _("The %{database} database cannot be reinitialized via this command.") % {database: target_database_name}
        exit(1)
      end

      puts _("Destroying replicated databases on %{certname}...") % {certname: replica_certname}

      manifest = <<-EOS
      class { 'puppet_enterprise':
        puppet_master_host => '#{replica_certname}',
      }
      class { 'pe_manager::reinitialize_replication':
        target_database_name       => '#{options[:db]}',
        classifier_database_name   => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['classifier_database_name'])},
        rbac_database_name         => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['rbac_database_name'])},
        activity_database_name     => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['activity_database_name'])},
        orchestrator_database_name => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['orchestrator_database_name'])},
        inventory_database_name    => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['inventory_database_name'])},
      }
      EOS

      # Run the reinitialize_replication.pp manifest
      PuppetX::Util::Apply.run_puppet_apply(manifest)
      puts _("Replicated databases have been destroyed")

      # Run puppet normally
      puts _("Running puppet to recreate replicated databases on %{certname}...") % {certname: replica_certname}
      PuppetX::Util::Shell.run_and_echo_out('puppet agent --onetime --no-daemonize --detailed-exitcodes --no-splay --no-usecacheonfailure --show_diff')
      reinitialization_status = $?.exitstatus

      ## we expect Puppet to exit with its code for a successful run with changes
      if reinitialization_status == 2
        puts _("Replica successfully reinitialized")
        exit 0
      else
        Puppet.err _("Replica reinitialization failed")
        exit(reinitialization_status)
      end
    end
  end
end
