# coding: utf-8
require 'puppet/indirector/face'
require 'pty'
require 'puppet_x/util/stringformatter'

Puppet::Face.define(:infrastructure, '1.0.0') do
  def command_mappings 
    {
      'regenerate_agent_certificate' => 'enterprise_tasks::agent_cert_regen',
      'enable_ha_failover' => 'enterprise_tasks::enable_ha_failover',
      'regenerate_master_certificate' => 'enterprise_tasks::master_cert_regen',
      'rebuild_certificate_authority' => 'enterprise_tasks::rebuild_ca',
      'migrate_split_to_mono' => 'enterprise_tasks::migrate_split_to_mono',
    }
  end

  def run_bolt(bolt_action)
    output = ''
    process_status = nil
    PTY.spawn("/opt/puppetlabs/installer/bin/bolt --boltdir /opt/puppetlabs/installer/share/boltdir #{bolt_action}") do |read, write, pid|
      write.close
      while process_status == nil
        begin
          line = read.gets
          line = line.gsub("bolt plan run", "puppet infrastructure run")
          command_mappings.each do |k, v|
            line = line.gsub(v, k)
          end
          puts line
          output += line
        rescue EOFError, Errno::EIO
          # GNU/Linux raises EIO on read operation when pty slave is closed - see pty.rb docs
          # Ensure the child process finishes and then pass through to ensure block below 
          # to get the child status
          nil
        rescue IO::WaitReadable, IO::WaitWritable
          retry
        ensure
          # Get Process::Status so we can report the exit code
          process_status ||= PTY.check(pid, raise=false)
        end
      end

      Process.waitall
      # Ensure we have captured the Process::Status
      process_status ||= PTY.check(pid, raise=false)
    end

    return process_status, output
  end

  def args_options_to_string(args, options)
    args_str = args.join(' ')
    opts_str = options.collect do |k, v|
      if v.is_a?(TrueClass)
        "--#{k}"
      elsif v.is_a?(FalseClass)
        "--no-#{k}"
      else
        "--#{k}='#{v}'"
      end
    end.join(' ')

    return "#{args_str} #{opts_str}"
  end

  ### Puppet face action
  action :run do
    summary _("Run PE maintenance commands")
    description(
      PuppetX::Util::String::Formatter.wrap_no_indent(_('Run PE maintenance commands')))
    arguments "\nValid commands:\n" + @face.command_mappings.keys.join("\n")

    # Provide a sub-set of bolt options to be passed in
    option('-h', '--help') do
      summary _('Displays this help text')
    end
    option('--[no-]host-key-check') do
      summary _('Enables or disables automatic host key checking')
    end
    option('--connect-timeout TIMEOUT') do
      summary _('Sets a connection timeout value')
    end  
    option('--inventoryfile FILE') do
      summary _('Sets a custom path to your inventory file')
    end
    option('--sudo-password PASSWORD') do
      summary _('Pass your superuser password through to your run command')
    end
    option('--run-as USER') do
      summary _('Run your command as the specified user')
    end
    option('--private-key KEY') do
      summary _('Pass your private key through to your run command')
    end
    # -u/--user doesn't need to be added because it is a Puppet default
    option('--password PASSWORD') do
      summary _('Pass your password through to your run command')
    end
    option('--format FORMAT') do
      summary _('Outputs status information as a given format')
    end

    when_invoked do |command, *args|
      options = args.pop

      if command == 'boltymcboltface'
        if args.size > 0
          bolt_action = 'plan run boltymcboltface ' + args.join(' ')
        else
          puts 'USAGE: puppet infra run boltymcboltface -- <any bolt command or flag to be passed directly to something that is definitely not bolt>'
          puts
          puts '⚠️  DO NOT USE THIS WITH A PRODUCTION/CUSTOMER SYSTEM  ⚠️'
          puts '⚠️  IF YOU NEED BOLT, INSTALL IT NORMALLY: https://puppet.com/docs/bolt/latest/bolt_installing.html  ⚠️'
          exit 1
        end
      elsif command_mappings.keys.include?(command)
        plan = command_mappings[command]
        if options[:help]
          bolt_action = "plan show #{plan}"
        else
          bolt_action = "plan run #{plan} #{args_options_to_string(args, options)}"
        end
      elsif command.nil? || command.empty? || command == "help" || (command.is_a?(Hash) && command[:help])
        puts Puppet::Face[:help, :current].help('infrastructure',:run)
        return
      else
        puts _('Expected subcommand to be one of the available plans:'), command_mappings.keys
        return
      end
      
      bolt_status, bolt_output = run_bolt(bolt_action.strip)

      exit bolt_status.exitstatus
    end
  end
end
