require 'puppet'
require 'puppet/util/pe_conf/recover'
require 'puppet_x/puppetlabs/meep/puppet_context'
require 'erb'
require 'json'

class Puppet::Util::Pe_conf
  attr_accessor :pe_conf, :nodes_conf, :recovery_engine

  include PuppetX::Puppetlabs::Meep::PuppetContext

  ENTERPRISE_DIR = '/etc/puppetlabs/enterprise'
  CONF_DIR = '/etc/puppetlabs/enterprise/conf.d'
  PE_CONF_PATH = "#{CONF_DIR}/pe.conf"
  NODE_CONF_PATH = "#{CONF_DIR}/nodes"

  def initialize
    @pe_conf = {}
    @nodes_conf = Hash.new{|h,k| h[k] = {}}
    @recovery_engine = Puppet::Util::Pe_conf::Recover.new
  end

  def deep_merge(this_hash, other_hash)
    other_hash.each_pair do |current_key, other_value|
      this_value = this_hash[current_key]

      this_hash[current_key] =
        this_value.is_a?(Hash) && other_value.is_a?(Hash) ? deep_merge(this_value, other_value) : other_value
    end

    this_hash
  end

  # Creates a new pe.conf and associated nodes/certname.conf files
  # based on information from PuppetDB, the PE Classifier, and Hiera.
  #
  # << How it works
  #
  # << Limitations
  #
  # @param [Hash] opts Options to alter how information is gathered
  # @option opts [String] :pe_environmentpath The environmentpath to query against.
  # @option opts [String] :pe_environment The puppet environment to query against.
  def recover(opts = {})
    overrides = Hash.new{|h,k| h[k] = {}}
    env = opts[:pe_environment] || 'production'
    environmentpath = opts[:pe_environmentpath] || '/etc/puppetlabs/code/environments'

    puppet_environment_context(environmentpath, env, Puppet[:basemodulepath]) do
      _recover(env, opts[:target_dir], overrides)
    end
  end

  def _recover(env, target_dir, overrides)
    using_nc = recovery_engine.using_node_classifier?
    core_infra_nodes = recovery_engine.get_core_infra_nodes(env)
    pe_conf = recovery_engine.pe_conf_from_puppetdb(env)
    @pe_conf = recovery_engine.extract_required_pe_conf_params(pe_conf)
    node_terminus = recovery_engine.get_node_terminus

    core_infra_nodes.each do |node_name|
      # Facts are needed for both classifier and hiera lookups
      facts = recovery_engine.facts_for_node(node_name, env)

      pe_module_resources = recovery_engine.resources_applied_to_node(node_name, 'Puppet_enterprise', env)
      pe_module_hiera_overrides = recovery_engine.find_hiera_overrides(node_name, pe_module_resources.keys, facts, env, node_terminus)
      pe_module_hiera_overrides.each do |k,v|
        overrides[k][node_name] = v
      end

      pe_repo_resources = recovery_engine.resources_applied_to_node(node_name, 'Pe_repo', env)
      pe_repo_hiera_overrides = recovery_engine.find_hiera_overrides(node_name, pe_repo_resources.keys, facts, env, node_terminus)
      pe_repo_hiera_overrides.each do |k,v|
        overrides[k][node_name] = v
      end

      classifier_data_backend = recovery_engine.find_hiera_overrides(node_name, ['puppet_enterprise_classifier_data_backend_present'], facts, env, node_terminus)
      classifier_data_backend.each do |k,v|
        overrides[k][node_name] = v
      end

      # Grab any overrides that are specified in the classifier
      # (if the user is using it)
      if using_nc
        classifier_overrides = recovery_engine.classifier_overrides_for_node(node_name, facts, facts['::trusted'])
        classifier_overrides.each do |k,v|
          overrides[k][node_name] = v
        end
      end
    end

    overrides.each do |param, nodes|
      # Create a global pe.conf override, if and only if, the value is the same
      # across all nodes **and** the override exists across all PE nodes.
      #
      # If you do not check that the override is on **all** nodes, then you
      # run the risk of adding a setting that is only actually provided on
      # one host becoming a global default. For an example, see PE-22999.
      #
      # Otherwise, the value should only be set for a specific node, so a
      # node-specific override should be set.
      if nodes.size == core_infra_nodes.size && nodes.values.uniq.length == 1
        @pe_conf[param] = nodes.values[0]
      else
        nodes.each do |node,value|
          @nodes_conf[node][param] = value
        end
      end
    end
  end
  private :_recover

  def render_conf_template(conf)
    template_path = File.join(File.expand_path(File.dirname(__FILE__)), 'pe_conf', 'templates', 'pe.conf.erb')
    template = ERB.new(File.read(template_path))
    template.result(binding)
  end

  def hocon_merge_if_exists(original, updates)
    if original
      deep_merge(original, updates)
    else
      updates
    end
  end

  def hocon_load_if_exists(conf_path)
    Hocon.load(conf_path) if File.exists?(conf_path)
  end

  def save(dir)
    FileUtils.mkdir_p(dir)

    # Create a new user_data.conf with recovered values
    File.open("#{dir}/user_data.conf", 'w', 0600) do |file|
        file.write(render_conf_template(@pe_conf))
        FileUtils.chown('pe-puppet',nil,file.path)
    end

    if @nodes_conf.size > 0
      FileUtils.mkdir_p("#{dir}/nodes")
      @nodes_conf.each do |node, overrides|
        node_conf_path = "#{dir}/nodes/#{node}.conf"
        # Load existing node overrides from nodes/
        orig_node_conf = hocon_load_if_exists(node_conf_path)
        # Merge in updated values
        overrides = hocon_merge_if_exists(orig_node_conf, overrides)
        # Write updated node overrides
        File.open(node_conf_path, 'w', 0600) do |file|
          file.write(render_conf_template(overrides))
          FileUtils.chown('pe-puppet',nil,file.path)
        end
      end
    end
  end

  # Used by the configure action when attempting recovery during upgrades.
  #
  # @param opts [Hash]
  # @option opts [String] :target_dir Target directory to save configuration
  #   to; defaults to CONF_DIR.
  # @option opts [String] :pe_environmentpath The environmentpath to recover from.
  # @option opts [String] :pe_environment The environment to recover from.
  # @return [Puppet::Util::Pe_conf] recovered configuration
  def self.recover_and_save_pe_configuration(opts = {})
    target_dir = opts[:target_dir] || CONF_DIR
    conf = self.new()
    conf.recover(opts.select { |k,v| [:pe_environmentpath, :pe_environment].include?(k) })
    conf.save(target_dir)
    return conf
  end
end
