module PuppetX
module Puppetlabs
module Meep
module Configure
  class McoStatus
    attr_accessor :classes, :infrastructure, :agents, :error

    def initialize
      self.classes = []
      self.infrastructure = {}
      self.agents = []
    end

    # Swallows all errors, and stores them in {McoStatus#error} so that the
    # Validator can log the problem, and choose whether or not to die based on
    # the Configure action's :force flag.
    # 
    # The timeout variable is used to override the server_url_timeout setting in puppetdb.conf.
    def query_puppetdb(timeout = nil)
      two_hours_in_seconds = 2 * 60 * 60
      threshold = Time.now().utc - two_hours_in_seconds
      query = mco_query(threshold)

      begin
        self.classes = self.class.query(query,timeout)
      rescue StandardError => e
        Puppet.debug(_("PuppetX::Puppetlabs::Meep::Configure::McoStatus.query(): %{exception}") % { :exception => e })
        self.error = Puppet::Error.new(_("PuppetX::Puppetlabs::Meep::Configure::McoStatus.query(): Unexpected exception querying puppetdb. query:\n%{puppetdb_query}") % { :puppetdb_query => query }, e)
        self.classes = []
      end

      # First pass identifies infrastructure and sets up a hash keyed by certname listing profiles
      classes.each do |r|
        node = r['certname']
        profile = r['title']
        if profile !~ /::Agent\Z/
          infrastructure[node] ||= []
          infrastructure[node] << profile
        end
      end

      # Second pass identifies agents
      classes.each do |r|
        node = r['certname']
        profile = r['title']
        if profile =~ /::Agent\Z/
          infrastructure.include?(node) ?
            infrastructure[node] << profile :
            agents << node
        end
      end

      classes
    end

    def nodes
      self.agents + self.infrastructure.keys
    end

    def mco_query(threshold)
      <<-EOS
        resources[title,certname] {
          type = "Class" and (
             title ~ "Puppet_enterprise::Profile::.*Mcollective" or
             title ~ "Puppet_enterprise::Profile::Amq"
          ) and
          ! nodes {
             resources {
               type = "Class" and
               title ~ "Puppet_enterprise::Mcollective::Purge.*"
             }
          } and
          nodes {
            deactivated is null and
            expired is null and
            catalog_timestamp > "#{threshold.strftime('%Y-%m-%d %H:%M:%S')}"
          }
        }
      EOS
    end

    # Private wrapper around Puppetdb.query_puppetdb due to
    # the code being installed by the puppetdb package.
    #
    # If this was a require at the top of the file, then it would be
    # auto loaded / required as part of the install process, before
    # the puppetdb package was installed, producing an error.
    #
    # The timeout variable is used to override the server_url_timeout setting in puppetdb.conf
    def self.query(pql, timeout = nil)
      require 'puppet/util/puppetdb'
      orig_timeout = Puppet::Util::Puppetdb.config.config[:server_url_timeout]
      begin
        Puppet::Util::Puppetdb.config.config[:server_url_timeout] = timeout unless timeout.nil?
        result = Puppet::Util::Puppetdb.query_puppetdb(pql)
      ensure
        Puppet::Util::Puppetdb.config.config[:server_url_timeout] = orig_timeout unless timeout.nil?
      end
      result
    end
  end
end
end
end
end
