require 'puppet/util/execution'
require 'puppet/face'

module PuppetX
module Puppetlabs
module Meep
module Configure
  # Utility for making very simple PostgreSQL queries through the local
  # /opt/puppetlabs/server/bin/psql tool. This is being used to obtain a few
  # values from SHOW and version() necessary for upgrades.  It is not intended
  # as a general purpose tool for parsing row output from Select queries.
  #
  # Relies on Puppet::Util::Execution to do the lifting.
  class PSQL

    ENCODING_QUERIES = %w{
      server_encoding
      lc_ctype
      lc_collate
    }.freeze

    attr_accessor :uid, :user, :password, :sslcert, :sslkey, :sslmode, :sslcacert, :database, :host, :port

    def initialize(options = {})
      options.each do |k,v|
        attr  = "#{k}="
        self.send(attr, v) if self.respond_to?(attr)
      end
    end

    def command(query)
      cmd = []
      # The provided uid may not have access to the pwd.
      cmd << %Q{cd /; } if uid
      # Reference: https://www.postgresql.org/docs/9.6/static/app-psql.html
      cmd << %Q{/opt/puppetlabs/server/bin/psql}
      # -q/--quiet Specifies that psql should do its work quietly.
      cmd << %Q{-q}
      # In tuples-only mode, only actual table data is shown.
      cmd << %Q{-P}
      cmd << %Q{tuples_only=on}
      # -X, --no-psqlrc Do not read the start-up file (neither the system-wide psqlrc file nor the user's ~/.psqlrc file).
      cmd << %Q{-X}
      cmd << %Q{--username='#{user}'} if user
      cmd << %Q{--host='#{host}'} if host
      cmd << %Q{--port='#{port}'} if port
      cmd << %Q{--dbname='#{database}'} if database
      cmd << %Q{--command='#{query}'}

      cmd.join(' ')
    end

    # @return [Hash] of the server_encoding, lc_ctype and lc_collate values
    #   from the Postgresql server
    def encoding_data
      ENCODING_QUERIES.inject({}) do |hash,key|
        hash[key] = run("SHOW #{key}")
        hash
      end
    end

    # @return the x.y.z of the Postgres server
    def server_version
      # PostgreSQL 9.6.3 on x86_64-pc-linux-gnu, compiled by gcc (GCC) 4.4.7 20120313 (Red Hat 4.4.7-4), 64-bit
      output = run("select version();")
      md = output.match(/PostgreSQL (\d+\.\d+\.\d+)/)
      md[1] if md
    end

    # @param query [String] a Postgresql query expected to return parseable output.
    # @return [String] the result of the query.
    # @raise Puppet::ExecutionFailure if the command fails to execute.
    def run(query)
      psql_command = command(query)

      options = {
        :failonfail => true
      }
      options[:uid] = uid if uid

      options[:custom_environment] = {}

      # Reduce the logging level so we don't get DEBUG, NOTICE or WARNING messages
      # Reference: https://www.postgresql.org/docs/9.6/static/runtime-config-logging.html
      options[:custom_environment]["PGOPTIONS"] = '--client-min-messages=error'

      options[:custom_environment]["PGPASSWORD"] = password if password

      # Cert based auth requires the client cert and key, as well as the CA cert
      options[:custom_environment]["PGSSLCERT"] = sslcert if sslcert
      options[:custom_environment]["PGSSLKEY"] = sslkey if sslkey
      options[:custom_environment]["PGSSLMODE"] = sslmode if sslmode
      options[:custom_environment]["PGSSLROOTCERT"] = sslcacert if sslcacert

      result = Puppet::Util::Execution.execute(psql_command, options)
      result.strip
    end
  end
end
end
end
end
