module PuppetX
module Puppetlabs
module Meep
module Configure
  # These helpers rely on the format of the puppetlabs-pe_install pe_postgresql_info
  # structured fact, which is returned as a hash of values detailing the installed
  # server version, data directories, partition and used disk space.
  module PostgresHelpers
    def pe_postgresql_info
      state[:pe_postgresql_info] || {}
    end

    def available_bytes
      pe_postgresql_info.fetch('data_partition_available_bytes', 0).to_i
    end

    # @param version [String] Expects a Postgresql major.minor version string
    #   (EX 9.4)
    def used_bytes_for(version)
      the_version(version).fetch('used_bytes', 0).to_i
    end


    # give ourselves a 110% buffer for other processes and additional PE
    # files that may be installed/copied during the upgrade
    def required_bytes_for(version)
      (used_bytes_for(version) * 1.10).round
    end

    def the_version(version)
      versions = pe_postgresql_info['versions'] || {}
      versions[version] || {}
    end

    def pg_data_dir(version)
      the_version(version)['data_dir']
    end

    def postgresql_not_installed?
      version = pe_postgresql_info['installed_server_version']
      version.nil? || version.empty?
    end

    def postgresql_installed?
      !postgresql_not_installed?
    end
  end

  class ValidationMessage
    attr_accessor :severity, :message
    def initialize(severity, message)
      @severity = severity
      @message = message
    end
  end

  class ValidationRule
    attr_accessor :state

    def initialize(state)
      self.state = state
    end

    def valid?
      raise(NotImplemented, _('ValidationRule must implement valid?().'))
    end

    def messages
      messages = []
      if !valid?
        messages = _messages
      end
      messages
    end

    def _messages
      raise(NotImplemented, _('ValidationRule must implement _messages().'))
    end
  end

  class PostgresMigrationSpace < ValidationRule
    include PostgresHelpers

    def valid?
      if !state[:postgres_migration] || postgresql_not_installed?
        return true
      end

      previous_version = state[:previous_postgres_version]

      (available_bytes - required_bytes_for(previous_version)) >= 0
    end

    def _messages
      previous_version = state[:previous_postgres_version]

      m = [
        _("Insufficient space to migrate the Postgresql database in %{data_dir}.") % {
          :data_dir => pg_data_dir(previous_version),
        },
        _("Available bytes: %{available_bytes}. Required bytes: %{required_bytes}.") % {
          :available_bytes => available_bytes,
          :required_bytes => required_bytes_for(previous_version),
        },
        _("Because we keep a backup of your %{previous_version} database, we need as much free space as your %{previous_version} database currently uses to continue.") % {
          :previous_version => previous_version,
        },
        _("If you wish to force the upgrade to proceed, you may re-run the original puppet-infrastructure configure command with the --force flag, but pg_upgrade will fail if there is insufficient space."),
      ]

      [ValidationMessage.new(:fatal, m.join("\n"))]
    end
  end

  class ExternalPostgres < ValidationRule
    include PostgresHelpers

    def valid?
      if !state[:separate_database_node] || postgresql_installed? || !state[:upgrade_from]
        return true
      end

      external_version = state[:external_database_version]
      version_is_behind = state[:external_database_behind]

      !(external_version.nil? || version_is_behind)
    end

    def _messages
      database_host = state[:database_host]
      external_version = state[:external_database_version]

      m = [
        _("Your installation is managing the Postgresql database on a separate host (%{database_host}).") % { :database_host => database_host },
      ]

      if external_version.nil?
        m << _("We were unable to confirm the version of Postgresql on %{database_host}.") % { :database_host => database_host }
        m << _("Please review the logs for connectivity issues.")
        m << _("If you wish to force the upgrade to proceed, you may re-run the original puppet-infrastructure configure command with the --force flag, but Puppet Enterprise will experience issues if Postgresql is not the correct version.")
      else
        m << _("We have detected that %{database_host} is running Postgresql %{database_version}.") % { :database_host => database_host, :database_version => external_version }
      end

      m << _("Puppet Enterprise requires Postgresql %{new_version}, please upgrade your %{database_host} host's Postgresql instance before continuing with the rest of the upgrade of Puppet Enterprise.") % { :new_version => state[:new_postgres_version], :database_host => database_host }

      [ValidationMessage.new(:fatal, m.join("\n"))]
    end
  end

  class McollectiveRemoval < ValidationRule
    def valid?
      encountered_error_querying_puppetdb = state[:mco_query_error]
      any_nodes_found_with_mco_or_amq = state[:mco_all_nodes]

      encountered_error_querying_puppetdb.nil? && any_nodes_found_with_mco_or_amq.empty?
    end

    def _messages
      messages = []

      encountered_error_querying_puppetdb = state[:mco_query_error]
      agent_nodes = state[:mco_agent_nodes]
      infrastructure_nodes = state[:mco_infrastructure_nodes]

      messages << ValidationMessage.new(:warning, _("MCollective and Activemq are no longer part of Puppet Enterprise."))

      if encountered_error_querying_puppetdb.nil?
        infrastructure_node_count = infrastructure_nodes.size

        if infrastructure_node_count > 0
          im = [
            _("The following %{count} Puppet Enterprise server nodes had these MCollective or Activemq profiles applied:") % { :count => infrastructure_node_count }
          ]
          infrastructure_nodes.each do |node,profiles|
            im << "  #{node}: #{profiles.sort}"
          end

          messages << ValidationMessage.new(:fatal, im.join("\n"))
        end

        agent_node_count = agent_nodes.size

        if agent_node_count > 0
          node_list = agent_node_count > 10 ?
            agent_nodes[0..9] + ["...(truncated)"] :
            agent_nodes

          am = [
             _("The following %{count} agent nodes had the Puppet_enterprise::Profile::Mcollective::Agent class still applied:") %{ :count => agent_node_count, }
          ]
          dm = [_("Full agent list:")]

          if Puppet[:log_level] != 'debug'
            node_list.each do |a|
              am << "  #{a}"
            end
            am << _("(To see the full list of nodes, run with --debug)") if agent_node_count > 10
          else
            agent_nodes.each do |a|
              dm << "  #{a}"
            end
          end

          messages << ValidationMessage.new(:fatal, am.join("\n"))
          messages << ValidationMessage.new(:debug, dm.join("\n")) if Puppet[:log_level] == 'debug'
        end

        cm = _("Before upgrading, please ensure that all MCollective and Activemq configuration is purged by setting the puppet_enterprise::mcollective parameter to 'absent'. (See Puppet Enterprise docs for details on purging MCollective)")
        messages << ValidationMessage.new(:warning, cm)
      else
        em = [
          _("But we were unable to query puppetdb to validate that these components are no longer being managed: %{mco_query_error}") % { :mco_query_error => encountered_error_querying_puppetdb }
        ]
        em << _("Because: %{wrapped_exception}") % { :wrapped_exception => encountered_error_querying_puppetdb.original } if !encountered_error_querying_puppetdb.original.nil?

        messages << ValidationMessage.new(:fatal, em.join("\n"))
      end

      messages
    end
  end

  class ParameterValidation < ValidationRule
    def valid?
        if state[:jruby_9k_enabled] != false
            return true
        end
    end
    def _messages
      m = [
          _("One or more currently defined parameters are invalid.")
      ]
      if !state[:jruby_9k_enabled]
          m << _("puppet_enterprise::master::puppetserver::jruby_9k_enabled is set to false. You can no longer use JRuby 1.7. Please update your PE install to use JRuby 9k.")
      end
      [ValidationMessage.new(:fatal, m.join("\n"))]
    end
  end

  # The Validator is a simple class that is initialized with a Hash of state.
  # After initialization {Validator.register} creates some set of
  # ValidationRules.
  #
  # The only shared state is the initial {Validator#state} Hash.
  #
  # All expensive operations should be performed before the Validator is
  # initialized, and refined data passed in through {Validator#state}. Given
  # that ValidationRule tests are inexpensive, no additional cacheing is necessary.
  #
  # The Validator will log messages through Puppet, but only if
  # {Validator.log_and_fail!} is called. An {InstallationInvalid} error will
  # only be raised if some state is invalid, and state[:force] is not true.
  class Validator 
    class InstallationInvalid < RuntimeError; end

    attr_accessor :state, :rules

    # @param state [Hash] Expects the following entries:
    #
    #   Postgresql:
    #
    #   :postgres_migration [Boolean] true if performing the postgres migration
    #   :pe_postgresql_info [Hash] the pe_postgresql_info fact
    #   :previous_postgres_version [String] the major.minor of Postgresql that
    #     we are migrating from
    #   :new_postgres_version [String] the major.minor of Postgresql that we are
    #     migrating to
    #   :separate_database_node [Boolean] true if database host is not one of the
    #     primary master, console, puppetdb nodes
    #   :database_host [String] certname of the database host, may be nil
    #   :external_database_version [String] the major.minor.patch of Postgresql that
    #     was detected on the separate database node
    #   :external_database_behind [Boolean] true if the version of Postgresql
    #     on the sepaarate database node is less than new_postgres_version
    #
    #   MCollective:
    #
    #   :mco_all_nodes [Array] list of any nodes that still have mco/amq profiles applied
    #   :mco_infrastructure_nodes [Hash<String,Array>] hash keyed by certname of infrastructure
    #     nodes listing the mco/amq profiles applied to them
    #   :mco_agent_nodes [Array] list of agent nodes which just have
    #     puppet_enterprise::profile::mcollective::agent applied to them
    #   :mco_query_error [Puppet::Error] a Puppet::Error instance if we were unable to
    #     query puppetdb for mco status, otherwise nil
    #
    #   General:
    #
    #   :force [Boolean] true if we should ignore fatal validation errors
    #   :upgrade_from [String] version of pe we're upgrading from
    def initialize(state)
      @state = state
      @rules = []
    end

    def register(rules)
      rules.each do |r|
        raise(RuntimeError, _("Expected a ValidationRule, got '%{class}'") % { class: r }) unless r.kind_of?(Class) && r <= (ValidationRule)
        self.rules << r.new(state)
      end
    end

    def valid?
      rules.all? do |r|
        r.valid?
      end
    end

    def messages
      rules.collect do |r|
        r.messages
      end.flatten
    end

    # Log all messages at the appropriate level and then raise a Validator::Error
    # if any were fatal.
    #
    # @note Does not raise an exception if the state[:force] is true.
    # @raise PuppetX::Puppetlabs::Meep::Validator::InstallationInvalid if there are
    #   any fatal messages logged.
    def log_and_fail!
      die = false
      messages.each do |m|
        case m.severity
        when :fatal
          die = true
          Puppet.err(m.message)
        when :warning
          Puppet.warning(m.message)
        when :debug
          Puppet.debug(m.message)
        else
          Puppet.notice(m.message)
        end
      end
      raise(InstallationInvalid, _('Failed to validate the state of the current installation.')) if die && !state[:force]
    end
  end
end
end
end
end
