require 'puppet/network/http_pool'
require 'puppet/util/colors'
require 'hocon'
require 'json'
require 'puppet_x/util/orchestrator_error'

module PuppetX
  module Util
    class Orchestrator

      def self.orch_connection(orch_service)
        Puppet::Network::HttpPool.http_instance(orch_service[:server], orch_service[:port])
      end

      def self.orch_request_headers(rbac_token)
        {'Content-Type' => 'application/json',
         'X-Authentication' => rbac_token }
      end

      def self.check_orch_response(response, expected_http_status)
        if response.code.to_i != expected_http_status
          raise PuppetX::Util::OrchestratorResponseError.new(response.code.to_i, response.body)
        elsif response.body.nil?
          raise PuppetX::Util::OrchestratorEmptyResponseError.new()
        end
      end

      def self.node_in_orch_inventory(orch_service, rbac_token, node_certname)
        url = "#{orch_service[:url]}/v1/inventory/#{node_certname}"
        response = self.orch_connection(orch_service).get(url, orch_request_headers(rbac_token))
        self.check_orch_response(response, 200)
        body = JSON.parse(response.body)
        return body['connected']
      end

      def self.start_puppet_run_with_orch(orch_service, scope, rbac_token, allow_empty: false)
        url = "#{orch_service[:url]}/v1/command/deploy"
        payload = {:scope => scope, :environment => '', :enforce_environment => false}

        response = self.orch_connection(orch_service).post(url, payload.to_json, orch_request_headers(rbac_token))

        if allow_empty and
            response.code.to_i == 400 and
            response.body and
            JSON.parse(response.body)['kind'] == 'puppetlabs.orchestrator/empty-target'
          return :empty
        end

        self.check_orch_response(response, 202)

        body = JSON.parse(response.body)

        # Yes, this is right. The numerical id of a job comes back as its name,
        # whereas the url is called 'id' in the response.
        {
          :id => body['job']['name'],
          :url => body['job']['id'],
        }
      end

      def self.get_orch_job_status(orch_service, job_url, rbac_token)
        response = self.orch_connection(orch_service).get(job_url, orch_request_headers(rbac_token))

        if response.body.nil?
          raise PuppetX::Util::OrchestratorEmptyResponseError.new()
        end

        # don't check the response code here; the calling code has logic to deal with errors itself
        response.body = JSON.parse(response.body)
        response
      end

      def self.get_finished_job_details(orch_service, job_url, rbac_token)
        url = "#{job_url}/nodes"

        response = self.orch_connection(orch_service).get(url, orch_request_headers(rbac_token))
        self.check_orch_response(response, 200)

        first_item = JSON.parse(response.body)['items'].first()
        details = first_item['details']

        {:message => details['message'],
         :report_url => details['report-url']}
      end

      def self.wait_for_orch_job(orch_service, job_id, job_url, display_scope, rbac_token)
        puts
        puts _("Running puppet on %{display_scope}.") % { display_scope: display_scope } + " " + _("Run 'puppet job show %{job_id}' to monitor detailed progress.") % { job_id: job_id }

        error_state_start_time = nil
        while true
          # Bail out if there's been something wrong with the orchestrator for 5
          # min; we expect transient errors due to console services restarting on
          # the master
          if error_state_start_time and (Time.now - error_state_start_time) > 300
            raise PuppetX::Util::OrchestratorJobTimeoutError.new(300, job_id)
          end

          response = self.get_orch_job_status(orch_service, job_url, rbac_token)

          if response.code.to_i != 200
            unless error_state_start_time
              error_state_start_time = Time.now
            end

            sleep(1)
            next
          end

          error_state_start_time = nil
          last_state = response.body['status'].last()['state']

          print "\r" + _("Overall job status: %{last_state}...") % { last_state: last_state }
          STDOUT.flush

          if last_state == 'finished'
            puts
            return true
          elsif last_state == 'stopped'
            puts
            raise PuppetX::Util::OrchestratorJobStoppedError.new(job_id)
          elsif last_state == 'failed'
            details = self.get_finished_job_details(orch_service, job_url, rbac_token)
            raise PuppetX::Util::OrchestratorJobFailedError.new(details[:message], details[:report_url])
          end

          sleep(1)
        end
      end

      def self.run_puppet(orch_service, display_scope, scope, rbac_token, allow_empty: false)
        job = self.start_puppet_run_with_orch(orch_service, scope, rbac_token, allow_empty: allow_empty)
        if job == :empty
          puts _("Skipping puppet run on %{display_scope}.") % { display_scope: display_scope } + " " + _("(nothing to do)")
        else
          self.wait_for_orch_job(orch_service, job[:id], job[:url], display_scope, rbac_token)
        end
      end
    end
  end
end
