require 'puppet/network/http_pool'
require 'puppet/util/colors'
require 'puppet_x/util/stringformatter'
require 'hocon'
require 'json'

module PuppetX
  module Util
    class ServiceStatus
      extend Puppet::Util::Colors

      def self.config_path()
        "/etc/puppetlabs/client-tools/services.conf"
      end

      # Utility method to filter out services that aren't required on the replica from the list
      def self.replica_services()
        services = required_services
        [:orchestrator, :code_manager, :file_sync_storage, :pcp_broker].each do |svc|
          services.delete(svc)
        end
        services
      end

      def self.primary_services()
        required_services
      end

      # Using a dynamic version means these links may need occasional updates to avoid breakage.
      def self.pe_version()
        if ver = Facter.value('pe_server_version')
          ver.split('.')[0..1].join('.')
        else
          'latest'
        end
      end

      # Docs links below indicate something that may not be enabled out of the box.
      # Services without docs links are core to PE and cannot be disabled.
      def self.required_services()
        version = self.pe_version()
        {
          :classifier => {
            :type => 'classifier',
          },
          :master => {
            :type => 'master',
          },
          :orchestrator => {
            :type => 'orchestrator',
            :docs => "https://puppet.com/docs/pe/#{version}/configuring/config_orchestration.html#configure-the-orchestrator-and-pe-orchestration-services",
          },
          :pcp_broker => {
            :type => 'pcp-broker',
          },
          :code_manager => {
            :type => 'code-manager',
            :docs => "https://puppet.com/docs/pe/#{version}/code_management/code_mgr_config.html#task-1695",
          },
          :file_sync_storage => {
            :type => 'file-sync-storage',
            :docs => "https://puppet.com/docs/pe/#{version}/code_management/filesync_about.html#enabling-or-disabling-file-sync",
          },
          :file_sync_client => {
            :type => 'file-sync-client',
            :docs => "https://puppet.com/docs/pe/#{version}/code_management/filesync_about.html#enabling-or-disabling-file-sync",
          },
        }
      end

      def self.symbolize_keys (h)
        if h.kind_of?(Array)
          h.map { |elem| symbolize_keys(elem) }
        elsif h.kind_of?(Hash)
          h.map { |k,v| [k.to_sym, symbolize_keys(v)] }.to_h
        else
          h
        end
      end


      ### service status functions

      # A 'status hash' looks like this:
      # { :service => 'service name',
      #   :state => :error, :unknown, :unreachable, :starting, :stopping, or :running,
      #   :status => status information,
      #   :server => some-server.infra.net,
      #   :type "master",
      #   :url => https://some-server.ifra.net:1234/some-service
      #   :alerts => [{:severity => :error, :warning, or :info,
      #               :message => "help!"}]}

      # A service config hash is read from services.conf, and looks like this:
      # { :node_certname : "mom.dev",
      #   :port 8140,
      #   :prefix "",
      #   :primary true,
      #   :server "mom.dev",
      #   :status_prefix "status",
      #   :status_url "https://mom.dev:8140/status",
      #   :type "master",
      #   :url "https://mom.dev:8140/",
      #   :status_key 'key-in-status-response-map'

      def self.unreachable_status_for_service(svc)
        { :service => svc[:type],
          :display_name => svc[:display_name],
          :state => :unreachable,
          :status => nil,
          :server => svc[:server],
          :url => svc[:url],
          :type => svc[:type],
          :alerts => [] }
      end

      # Call the status api at status_service_url and return an array of status hashes.
      # Cross-check with expected_service_names to determine which services
      # were unreachable.
      def self.try_get_status(status_service_url, expected_service_configs, timeout_seconds)
        begin
          uri = URI.parse(status_service_url)
          server = uri.host
          port = uri.port
          connection = Puppet::Network::HttpPool.http_instance(server, port)
          response = connection.get("#{status_service_url}/v1/services?timeout=#{timeout_seconds}")
          response_hash = JSON.parse(response.body)

          expected_service_configs.map do |svc|
            type = svc[:type]
            status_key = svc[:status_key]
            status = response_hash[status_key]
            if status
              { :service => status_key,
                :state => status['state'].to_sym,
                :status => status['status'],
                :display_name => svc[:display_name],
                :server => server,
                :url => svc[:url],
                :type => type,
                :alerts => (status['active_alerts'] || []).map { |a| { :severity => a['severity'].to_sym,
                                                                       :message => a['message']}}}
            else
              unreachable_status_for_service(svc)
            end
          end
            # It would be awesome to be more explicit here but I'm not positive
            # what exceptions HttpPool raises. Maybe we can add exceptions until
            # we have an exhaustive list.
        rescue JSON::ParserError, Exception => e
          Puppet.debug _("Exception raised while trying to fetch status for %{url}") % { url: status_service_url }
          Puppet.debug "#{e.class}: #{e.message}"
          expected_service_configs.map { |svc| unreachable_status_for_service(svc)}
        end
      end

      def self.services_for(host, service, config_services, timeout_seconds)
        config_services.select { |svc| host ? svc[:server] == host : true }
            .select { |svc| service ? svc[:type].downcase == service.downcase : true }
            .group_by { |svc| svc[:status_url] }
            .flat_map { |status_service_url, configured_services| try_get_status(status_service_url, configured_services, timeout_seconds) }

      end

      def self.node_role(nodes_config, certname)
        return nil unless nodes_config

        node_record = nodes_config.find { |node| node[:certname] == certname }
        node_record[:role] if node_record
      end

      def self.load_config(path_override)
        path = path_override || self.config_path()
        unless File.file?(path)
          raise PuppetX::Util::String::Formatter.wrap_no_indent(_("No configuration file found at %{path}.") % { path: path } + " " +
            _("This file is installed automatically on Puppet Server nodes.") + " " +
            _("Make sure you are running the command on a primary master, primary master replica, or compile master."))
        end
        config = Hocon.load(path)
        symbolize_keys(config)
      end

      def self.load_services_config(path_override=nil)
        self.load_config(path_override)[:services]
      end

      def self.load_nodes_config(path_override=nil)
        self.load_config(path_override)[:nodes]
      end

      # Filters the config hash for only services defined on the node the command
      # was run on (assumption being that no commands need to directly contact any
      # services not on the current host), checks that the config hash has the
      # entries for all key services (defined in the SERVICES constant), pulls them
      # out, and returns them in a map.
      def self.validate_and_select_services_from_config(config, action, certname, services = self.required_services())
        errors = []
        failed_services = []
        service_hash = services.each_with_object({}) do |(svc_name, svc_hash), out_hash|
          cur_config = config.find {|cur_service| cur_service[:node_certname] == certname and cur_service[:type] == svc_hash[:type]}
          if cur_config
            out_hash[svc_name] = cur_config
          else
            if svc_hash[:docs]
              failed_services << svc_hash[:type]
              errors << _("To enable %{type}, consult the documentation at %{docs}.") % { type: svc_hash[:type], docs: svc_hash[:docs] }
            else
              # This failure case should be hard to get to in the real world.
              failed_services << svc_hash[:type]
              errors << _("Missing %{type} service.") % { type: svc_hash[:type] } + " " + _("This probably means this command hasn't been run on the Primary Master node.")
            end
          end
        end

        if errors.empty?
          service_hash
        else
          # Ideally this message would be handled outside of this validation
          # functions but as a temporary measure to prevent an erroroneous
          # message from being printed we check which action we're dealing with
          # to print the correct message
          command_string = action == 'promote' ? "#{action} replica" : "#{action} replica HOSTNAME"
          error_string = _("Puppet cannot provision a primary master replica until %{failed} is/are enabled.") % { failed: failed_services.join(", ") }
          error_string << "\n" << errors.join(" ") << "\n"
          error_string << _("Then, enter `puppet infrastructure %{command_string}` to %{action} a replica.") % { command_string: command_string, action: action }
          raise error_string
        end
      end

      def self.services_running(status)
        status.select {|service| service[:state] == :running}
      end

      def self.services_not_running(status)
        status.select {|service| service[:state] != :running}
      end

      def self.all_services_running?(status)
        self.services_running(status).count == status.count
      end

      def self.ensure_all_services_running(status)
        if self.all_services_running?(status)
          return true
        else
          errors = []
          self.services_not_running(status).each do |service|
            errors << _("%{display_name} is not running") % { display_name: service[:display_name] }
          end

          error_string = errors.join("\n")
          error_string << "\n" + _("Puppet cannot provision a primary master replica until all required services are running.")
          raise error_string
        end
      end

      ### Status formatting

      def self.service_has_changing_state(status_hash)
        [:starting, :stopping].include? status_hash[:state]
      end

      def self.service_has_error_state(status_hash)
        [:error, :unknown, :unreachable].include? status_hash[:state]
      end

      def self.service_has_important_alert(status_hash)
        alert_severities = status_hash[:alerts].map { |a| a[:severity] }
        (alert_severities & [:error, :warning]).count > 0  # & is set intersection
      end

      def self.format_summary_message(service_statuses_count, error_service_statuses_count)
        operational_service_count = service_statuses_count - error_service_statuses_count
        color = if error_service_statuses_count == 0
                  :green
                else
                  :yellow
                end
        colorized_summary = colorize(color, _("%{operational_service_count} of %{service_statuses_count} services are fully operational.") % { operational_service_count: operational_service_count, service_statuses_count: service_statuses_count })
        "#{Time.now}\n#{colorized_summary}\n"
      end

      def self.format_status(nodes_config, status)
        server = status[:server]
        node = nodes_config.find { |node| node[:certname] == status[:server] }
        display_name_summary = node ? " on #{node[:display_name]}" : ''
        alerts_text = status[:alerts]
                          .map { |alert| "    #{alert[:severity].to_s.capitalize}: #{alert[:message]}\n" }
                          .join("")
        colored_summary = "#{status[:display_name]}: #{status[:state].to_s.capitalize}"
        if service_has_error_state(status)
          colored_summary = colorize(:red, colored_summary)
        elsif service_has_changing_state(status)
          colored_summary = colorize(:yellow, colored_summary)
        elsif :running == status[:state]
          colored_summary = colorize(:green, colored_summary)
        end
        "#{colored_summary}#{display_name_summary}, #{status[:url]}\n#{alerts_text}"
      end
    end
  end
end
