require 'spec_helper'
require 'puppet/agent'
require 'puppet/face'
require 'puppet/util/pe_conf'
require 'puppet/util/puppetdb'
require 'webmock/rspec'

# Cribbed from Puppet spec/lib/puppet_spec/matchers.rb
RSpec::Matchers.define :exit_with do |expected|
  actual = nil
  match do |block|
    begin
      block.call
    rescue SystemExit => e
      actual = e.status
    end
    actual and actual == expected
  end

  supports_block_expectations

  failure_message do |block|
    "expected exit with code #{expected} but " +
      (actual.nil? ? " exit was not called" : "we exited with #{actual} instead")
  end

  failure_message_when_negated do |block|
    "expected that exit would not be called with #{expected}"
  end

  description do
    "expect exit with #{expected}"
  end
end

describe "Puppet::Face[:infrastructure, :current]" do
  let(:megabyte) { 10**6 }
  let(:infrastructure) { Puppet::Face[:infrastructure, :current] }
  let(:resource_face) { Puppet::Face[:resource, :current] }
  let(:tmpdir) { Dir.mktmpdir("tmp") }
  let(:state_dir) { "#{tmpdir}/var/state" }
  let(:mock_conf) { double('conf', :pe_conf => {'parameter' => 'value'}, :nodes_conf => [{'override' => 'other'}]) }

  class Config
    def config
      {:server_url_timeout => 30}
    end
  end
  before(:each) do
    if Module.const_defined?('WebMock')
      WebMock.allow_net_connect!
      ENV["DISABLE_ANALYTICS"] = "true"
    end
    allow(Puppet::Util::Puppetdb).to receive(:config).and_return(Config.new)
  end

  after(:each) do
    FileUtils.rm_rf tmpdir, :secure => true
    infrastructure.clear_cache
  end

  def expect_log(log_level, pattern)
    all_logs = []
    Puppet::Util::Log.with_destination(Puppet::Test::LogCollector.new(all_logs)) do
      yield
    end
    logs = all_logs.select { |l| l.level == log_level }.map(&:message)
    expect(logs).to_not be_empty
    expect(logs).to include(pattern)
  end

  RSpec.shared_context "puppet lookup" do
    let(:test_conf_dir) { "#{tmpdir}/etc/puppetlabs/enterprise" }
    let(:test_pe_conf_path) { "#{test_conf_dir}/conf.d/pe.conf" }
    let(:hiera_config) { "#{test_conf_dir}/hiera.yaml" }
    let(:default_pe_conf) do
      <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
}
      EOF
    end
    let(:default_hiera) do
      <<-EOF
---
version: 5
defaults:  # Used for any hierarchy level that omits these keys.
  datadir: conf.d        # This path is relative to hiera.yaml's directory.
  data_hash: hocon_data  # Use the built-in hocon backend.

hierarchy:
  - name: "Puppet Enterprise configuration"
    path: "pe.conf"
      EOF
    end

    before(:each) do
      FileUtils.mkdir_p("#{test_conf_dir}/conf.d")
      File.write(test_pe_conf_path, default_pe_conf)
      File.write(hiera_config, default_hiera)
      allow(infrastructure).to receive(:enterprise_hiera_yaml).and_return(hiera_config)
    end
  end

  context "configure action" do
    let(:execute) { ['--execute', 'notice("ran")'] }
    let(:test_process_file) { "#{tmpdir}/test-in-process" }
    let(:environment) { "enterprise#{(Time.now.to_r * 1000).to_i}" }

    include_context('puppet_context environmentpath')

    before(:each) do
      FileUtils.mkdir_p(state_dir)
      FileUtils.mkdir_p("#{tmpdir}/code")
      FileUtils.mkdir_p("#{tmpdir}/etc")
      FileUtils.mkdir_p("#{tmpdir}/run")
      FileUtils.mkdir_p("#{tmpdir}/log")
      Puppet[:vardir] = "#{tmpdir}/var"
      Puppet[:codedir] = "#{tmpdir}/code"
      Puppet[:confdir] = "#{tmpdir}/etc"
      Puppet[:rundir] = "#{tmpdir}/run"
      Puppet[:logdir] = "#{tmpdir}/log"
      # Cleans up log errors around Puppet managing the above basedirs (vardir, codedir, etc.)
      Puppet[:user] = ENV['USER']
      Puppet::Parser::Functions.reset
      allow(infrastructure).to receive(:upgrade_in_process_file).and_return(test_process_file)
      allow(infrastructure).to receive(:default_environmentpath).and_return(environmentpath)
      allow(infrastructure).to receive(:default_environment).and_return(environment)
    end

    after(:each) do
      # lockfile is an instance variable on the Puppet::Face[:infrastructure,
      # :current] class instance which is cached in a
      # Puppet::Interface::FaceCollection and only loaded once.  There isn't an
      # explicit clear or unload for this collection, and the lockfile_path
      # will alter between tests, so we need to clear these manually.
      infrastructure.instance_variable_set(:@lockfile_path, nil)
      infrastructure.instance_variable_set(:@lockfile, nil)
    end

    def expect_configure_with(exitcode, log_messages, args = {})
      # do not include 'pe_install' class so we are just testing the face, not the modules
      allow(infrastructure).to receive(:default_arguments).and_return(execute)

      logs = []
      # Provide 'production' as current_environment so that configurer doesn't
      # switch from 'production' to '*root*' since we are jumping into the face
      # without a full initialization
      env = Puppet::Node::Environment.create(:production, [], '')
      Puppet.override(:current_environment => env) do
        Puppet::Util::Log.with_destination(Puppet::Test::LogCollector.new(logs)) do
          expect { infrastructure.configure(args) }.to exit_with(exitcode)
        end
      end
      Array(log_messages).each do |m|
        expect(logs.map { |l| l.message }).to include(m)
      end

      return logs
    end

    it 'splay setting is overridden to false in enterprise context' do
      Puppet[:splay] = true

      infrastructure.puppet_infrastructure_context do
        expect( Puppet[:splay] ).to be false
      end
    end

    context 'with help flag' do
      let(:action) { 'configure' }

      include_examples 'help'
    end

    context 'when performing a new install' do
      before(:each) do
        allow(resource_face).to receive(:find).with('package/pe-puppetserver')
          .and_return({ ensure: :purged })
        # Other parts of puppet will call Facter during the apply...
        allow(Facter).to receive(:value).and_call_original
        # But we care about mocking pe_server_version
        allow(Facter).to receive(:value).with('pe_server_version').and_return(nil)
      end

      it 'configures' do
        expect_configure_with(0, 'ran')
      end

      context 'analytics' do
        include_context 'facter stubs'
        before(:each) do
          WebMock.disable_net_connect!
          stub_request(:post, "www.google-analytics.com/collect").to_return(status: 200, body: "OK")
          ENV["DISABLE_ANALYTICS"] = nil
          allow(Hocon).to receive(:load).with("/etc/puppetlabs/enterprise/conf.d/pe.conf").and_return({"parameter"=>"value"})
          PuppetX::Util::Analytics.check_analytics_setting
        end

        after(:each) do
          WebMock.allow_net_connect!
        end

        it 'attempts to send analytics when --install and --install-method are defined' do
          expect_configure_with(0, 'ran', {:install=>"2019.0.2", :install_method=>"spec"})
          WebMock.should have_requested(:post, "www.google-analytics.com/collect").times(3)
        end

        it 'attempts to send analytics when --upgrade-from is defined' do
          expect_configure_with(0, 'ran', {:install=>"2019.0.2", :upgrade_from=>"2018.1.5"})
          WebMock.should have_requested(:post, "www.google-analytics.com/collect").times(4)
        end

        it 'does not attempt to send analytics when not installing or upgrading' do
          expect_configure_with(0, 'ran')
          WebMock.should_not have_requested(:post, "www.google-analytics.com/collect")
        end

        it 'does not attempt to send analytics when disabled in pe.conf' do
          allow(Hocon).to receive(:load).with("/etc/puppetlabs/enterprise/conf.d/pe.conf").and_return({"puppet_enterprise::send_analytics_data"=>false})
          PuppetX::Util::Analytics.check_analytics_setting
          expect_configure_with(0, 'ran', {:install=>"2019.0.2", :install_method=>"spec"})
          WebMock.should_not have_requested(:post, "www.google-analytics.com/collect")
        end
      end


      context 'reset_url' do
        include_context 'puppet lookup'

        let(:console_admin_password_parameter) { '' }
        let(:monolithic_pe_conf) do
          <<-EOF
{
  #{console_admin_password_parameter}
  "puppet_enterprise::puppet_master_host": "master.rspec"
}
          EOF
        end
        let(:split_pe_conf) do
          <<-EOF
{
  #{console_admin_password_parameter}
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::puppetdb_host": "puppetdb.rspec"
  "puppet_enterprise::console_host": "console.rspec"
}
          EOF
        end

        before(:each) do
          Puppet[:certname] = node
        end

        context 'with password' do
          let(:console_admin_password_parameter) { '"console_admin_password": "password"' }

          context 'monolithic' do
            let(:node) { 'master.rspec'}
            let(:default_pe_conf) { monolithic_pe_conf }

            it "without --install flag, doesn't provide console admin reset url and doesn't revoke admin access" do
              expect(infrastructure).not_to receive(:revoke_admin_access)
              expect(infrastructure).not_to receive(:notify_reset_url)
              expect_configure_with(0, 'ran')
            end

            it "with --install flag, still doesn't provide console admin reset url or revoke admin access because password is set" do
              expect(infrastructure).not_to receive(:revoke_admin_access)
              expect(infrastructure).not_to receive(:notify_reset_url)
              expect_configure_with(0, 'ran', install: '2019.0.0')
            end
          end

          context 'split' do
            let(:node) { 'console.rspec' }
            let(:default_pe_conf) { split_pe_conf }

            it "with --install flag, still doesn't provide console admin reset url or revoke admin access because password is set" do
              expect(infrastructure).not_to receive(:revoke_admin_access)
              expect(infrastructure).not_to receive(:notify_reset_url)
              expect_configure_with(0, 'ran', install: '2019.0.0')
            end
          end
        end

        context 'without password' do
          context 'monolithic' do
            let(:node) { 'master.rspec'}
            let(:default_pe_conf) { monolithic_pe_conf }

            it "without --install flag, doesn't provide console admin reset url or revoke admin access" do
              expect(infrastructure).not_to receive(:revoke_admin_access)
              expect(infrastructure).not_to receive(:notify_reset_url)
              expect_configure_with(0, 'ran')
            end

            it "with --install flag, does provide console admin reset url and revokes admin access" do
              expect(infrastructure).to receive(:revoke_admin_access)
              expect(infrastructure).to receive(:notify_reset_url)
              expect_configure_with(0, 'ran', install: '2019.0.0')
            end

            it "with --upgrade-from flag and default password, revokes admin access and provides reset url" do
              expect(infrastructure).to receive(:check_for_default_password_and_revoke).and_return(true)
              expect(infrastructure).to receive(:notify_reset_url)
              expect_configure_with(0, 'ran', upgrade_from: '2018.1.2')
            end

            it "with --upgrade-from flag and not default password, does not revoke admin access or provide reset url" do
              expect(infrastructure).to receive(:check_for_default_password_and_revoke).and_return(false)
              expect(infrastructure).not_to receive(:notify_reset_url)
              expect_configure_with(0, 'ran', upgrade_from: '2018.1.2')
            end

            context 'with empty string password' do
              let(:console_admin_password_parameter) { '"console_admin_password": ""' }

              it "with --install flag, does provide console admin reset url" do
                expect(infrastructure).to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', install: '2019.0.0')
              end

              it "with --upgrade-from flag, doesn't provide console admin reset url" do
                expect(infrastructure).to receive(:check_for_default_password_and_revoke).and_return(false)
                expect(infrastructure).not_to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', upgrade_from: '2019.0.0')
               end
            end
          end

          context 'split' do
            let(:default_pe_conf) { split_pe_conf }

            context 'not on console host' do
              let(:node) { 'master.rspec' }

              it "with --install flag, does not provide console admin reset url or revoke admin access" do
                expect(infrastructure).not_to receive(:revoke_admin_access)
                expect(infrastructure).not_to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', install: '2019.0.0')
              end
            end

            context 'on console host' do
              let(:node) { 'console.rspec' }

              it "with --install flag, does provide console admin reset url and revokes admin access" do
                expect(infrastructure).to receive(:revoke_admin_access)
                expect(infrastructure).to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', install: '2019.0.0')
              end

              it "with --upgrade-from flag and default password, revokes admin access and provides reset url" do
                expect(infrastructure).to receive(:check_for_default_password_and_revoke).and_return(true)
                expect(infrastructure).to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', upgrade_from: '2018.1.2')
              end

              it "with --upgrade-from flag and not default password, does not revoke admin access or provide reset url" do
                expect(infrastructure).to receive(:check_for_default_password_and_revoke).and_return(false)
                expect(infrastructure).not_to receive(:notify_reset_url)
                expect_configure_with(0, 'ran', upgrade_from: '2018.1.2')
              end
            end
          end
        end
      end

      it 'exits 17 if puppet lockfile is in place' do
        lockfile = instance_double(Puppet::Util::Pidlock)
        expect(lockfile).to receive(:lock).and_raise(Puppet::LockError, 'locked')
        allow(infrastructure).to receive(:lockfile) { lockfile }

        expect_configure_with(17, %r{Puppet run already in progress.*#{Puppet[:agent_catalog_run_lockfile]}})
      end

      context 'with --debug flag' do
        let(:execute) { ['--execute', 'notify { "ran debug": loglevel => debug }'] }

        before(:each) do
          # simulate the --debug flag being caught by appication opt parsing:
          # https://github.com/puppetlabs/puppet/blob/4.4.0/lib/puppet/application/face_base.rb#L8
          Puppet.debug = true
        end

        it 'configures and emits debug logs' do
          expect_configure_with(0, 'ran debug')
        end
      end

      context 'with --detailed-exitcodes flag' do
        let(:execute) { ['--execute', 'notify { "should exit 2": }'] }

        it 'configures and returns 2 for changes' do
          expect_configure_with(2, 'should exit 2', detailed_exitcodes: true)
        end
      end

      context 'with --upgrade-from flag' do
        context 'and a successful apply' do
          it 'configures and returns 0' do
            expect_configure_with(0, 'ran', upgrade_from: '2016.4.3')
            expect(File.exist?(test_process_file)).to eq(false)
          end
        end

        context 'and a failed apply' do
          let(:execute) { ['--execute', 'fail("something went wrong")'] }

          it 'fails and returns 1, leaving upgrade file in place ' do
            expect_configure_with(1, %r{something went wrong}, upgrade_from: '2016.4.3')
            expect(File.exist?(test_process_file)).to eq(true)
          end
        end

        context 'picking up after a failed apply' do
          before(:each) do
            File.write(test_process_file, '2016.5.0')
          end

          it 'configures and returns 0' do
            expect_configure_with(0, [
              'ran',
              %r{Setting --upgrade-from=2016\.5\.0 based on presence of},
            ])
            expect(File.exist?(test_process_file)).to eq(false)
          end

          it 'uses explicit --upgrade-from even after failed apply' do
            logs = expect_configure_with(0, 'ran', upgrade_from: '2017.2.1')
            expect(logs).to_not include(%r{Setting --upgrade-from=})
            expect(File.exist?(test_process_file)).to eq(false)
          end
        end
      end

      RSpec.shared_context('postgres-migration-encoding') do
        let(:pe_postgres_user_hash) do
          {
            :ensure           => :present,
            :uid              => 495,
          }
        end
        let(:test_pe_conf_path) { "#{test_conf_dir}/conf.d/pe.conf" }
        let(:test_conf_dir) { "#{tmpdir}/etc/puppetlabs/enterprise" }
        let(:default_pe_conf) do
          <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
}
          EOF
        end
        let(:encoding) { 'UTF8' }
        let(:collate) { 'en_US.UTF-8' }
        let(:ctype) { 'en_US.UTF-8' }

        before(:each) do
          expect(resource_face).to receive(:find).with('user/pe-postgres').and_return(pe_postgres_user_hash)
          allow(Puppet::Util::Execution).to receive(:execute).and_call_original
          expect(Puppet::Util::Execution).to receive(:execute).with(match(/server_encoding/), anything).and_return(encoding)
          expect(Puppet::Util::Execution).to receive(:execute).with(match(/lc_ctype/), anything).and_return(collate)
          expect(Puppet::Util::Execution).to receive(:execute).with(match(/lc_collate/), anything).and_return(ctype)
          FileUtils.mkdir_p("#{test_conf_dir}/conf.d")
          File.write(test_pe_conf_path, default_pe_conf)
          # May get this more than once due to DISABLE_ANALYTICS being set for spec tests and writing to pe.conf
          expect(infrastructure).to receive(:enterprise_conf_dir).and_return(test_conf_dir).at_least(:once)
        end
      end

      context 'with --postgres-migration flag' do
        context 'set true' do
          let(:pe_postgresql_info) do
            {
              'installed_server_version' => '9.4',
              'data_partition_available_bytes' => 50 * megabyte,
              'versions' => {
                '9.4' => {
                  'used_bytes' => used_bytes,
                  'data_dir' => '/opt/puppetlabs/server/data/postgresql/9.4',
                  'tablespaces' => [
                    '/opt/puppetlabs/server/data/postgresql/activity/PG_9.4_201409291',
                    '/opt/puppetlabs/server/data/postgresql/classifier/PG_9.4_201409291',
                    '/opt/puppetlabs/server/data/postgresql/orchestrator/PG_9.4_201409291',
                    '/opt/puppetlabs/server/data/postgresql/puppetdb/PG_9.4_201409291',
                    '/opt/puppetlabs/server/data/postgresql/rbac/PG_9.4_201409291'
                  ]
                }
              }
            }
          end

          before(:each) do
            expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(pe_postgresql_info)
          end

          context 'failing disk space validation' do
            let(:used_bytes) { 50 * megabyte}

            it 'errors out before running with an Error message' do
              logs = expect_configure_with(27, %r{Insufficient space to migrate the Postgresql database}, postgres_migration: true)
              expect(logs).to_not include('ran')
            end

            context 'but --forced' do
              include_context 'postgres-migration-encoding'

              it 'proceeds' do
                expect_configure_with(
                  0,
                  [
                    %r{Insufficient space to migrate the Postgresql database},
                    'ran',
                  ],
                  {
                    postgres_migration: true,
                    force: true,
                  }
                )
              end
            end
          end

          context 'passing disk space validation' do
            let(:used_bytes) { 40 * megabyte }

            include_context 'postgres-migration-encoding'

            it 'configures and returns 0' do
              expect_configure_with(
                0,
                [
                  %r{Your Postgresql datastore.*from 9\.4 to 9\.6},
                  %r{The 9\.4 datastore has been left on disk},
                  %r{When you have verified that the 9\.6 datastore is up to date.*remove.*9\.4 directories.*data/postgresql/9\.4.*PG_9\.4},
                  'ran',
                ],
                postgres_migration: true
              )
            end
          end
        end

        context 'set false' do
          it 'configures and returns 0' do
            expect_configure_with(0, 'ran', postgres_migration: false)
          end
        end
      end

      context 'upgrading' do
        context 'external-postgres' do
          context 'failing external postgres validation' do
            include_context 'puppet lookup'
            let(:pe_postgresql_info) do
              {
                'installed_server_version' => nil,
                'data_partition_size_bytes' => 0,
                'data_partition_available_bytes' => 0,
                'versions' => {},
                'installed_packages' => {
                  'pe-postgresql' => '9.4.3-1',
                }
              }
            end
            let(:default_pe_conf) do
              <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::database_host": "database.rspec"
}
              EOF
            end

            before(:each) do
              expect(infrastructure).to receive(:enterprise_hiera_yaml).and_return(hiera_config)
              expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(pe_postgresql_info)
              allow(resource_face).to receive(:find).with('package/pe-puppetserver')
                .and_return({ ensure: 'x.y.z' })
            end

            it 'errors out before running with an Error message' do
              logs = expect_configure_with(
                27,
                [
                  %r{It appears you have an externally managed Postgresql server.*database\.rspec.*unable to look up}m,
                  %r{Your installation is managing the Postgresql database on a separate host.*database\.rspec.*We were unable to confirm the version}m,
                ],
                upgrade_from: "2017.2.3",
              )
              expect(logs).to_not include('ran')
            end

            context 'but --forced' do
              it 'proceeds' do
                expect_configure_with(
                  0,
                  [
                    %r{It appears you have an externally managed Postgresql server.*database\.rspec.*unable to look up}m,
                    %r{Your installation is managing the Postgresql database on a separate host.*database\.rspec.*We were unable to confirm the version}m,
                    'ran',
                  ],
                  {
                    force: true,
                    upgrade_from: "2017.2.3",
                  }
                )
              end
            end
          end
        end

        context 'after jruby_9k_enabled parameter update' do
         include_context 'puppet lookup'

         let(:default_pe_conf) do
           <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::master::puppetserver::jruby_9k_enabled": #{jruby_9k_enabled}
}
           EOF
         end

         context 'when JRuby 9k is enabled' do
            let(:jruby_9k_enabled) { true }
            it 'proceeds with the upgrade' do
              expect_configure_with(0, 'ran', upgrade_from: "2018.1.3")
            end
          end

          context 'when JRuby 9k is disabled' do
            let(:jruby_9k_enabled) { false }
            it 'fails the upgrade early' do
              expect_configure_with(
                27,
                [
                  %r{One or more currently defined parameters are invalid.},
                  %r{puppet_enterprise::master::puppetserver::jruby_9k_enabled is set to false. You can no longer use JRuby 1.7. Please update your PE install to use JRuby 9k.},
                ],
                upgrade_from: "2018.1.3",
              )
            end
          end
        end

        context 'mco removal' do
          let(:mco_classes) { [] }
          before(:each) do
            allow(resource_face).to receive(:find).with('package/pe-puppetserver')
              .and_return({ ensure: 'x.y.z' })
            MockPuppetdb.mock(
              %w{resources type Class title Mcollective},
              mco_classes
            )
          end

          it 'proceeds when no mco classes found' do
            expect_configure_with(0, 'ran', upgrade_from: "2018.1.3")
          end

          context 'mco present' do
            let(:mco_classes) do
              [
                {
                  'title'    => 'Puppet_enterprise::Profile::Mcollective::Agent',
                  'certname' => 'some.node',
                }
              ]
            end

            it 'proceeds when no mco classes found' do
              expect_configure_with(
                27,
                [
                  %r{MCollective and Activemq are no longer part of Puppet Enterprise},
                  %r{The following 1 agent nodes had},
                ],
                upgrade_from: "2018.1.3"
              )
            end
          end
        end
      end
    end

    context 're-running partial install or upgrading' do
      context 'on the master' do
        before(:each) do
          allow(resource_face).to receive(:find).with('package/pe-puppetserver')
            .and_return({ ensure: 'x.y.z' })
          allow(Facter).to receive(:value).and_call_original
          allow(Facter).to receive(:value).with('pe_server_version').and_return('1.2.3')
        end

        it 'by default it configures with recover' do
          expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
            .and_return(mock_conf)
          expect_configure_with(0, 'ran')
        end

        context 'puppetdb inaccessible' do
          before(:each) do
            expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
              .and_raise(
                Puppet::Error,
                "Failed to execute '/pdb/query/v4' on at least 1 of the following 'server_urls': https://dummyhostname:8081"
              )
          end

          it 'still configures' do
            expect_configure_with(0, 'ran')
          end
        end
      end

      context 'on non-master infra node' do
        before(:each) do
          allow(resource_face).to receive(:find).with('package/pe-puppetserver')
            .and_return({ ensure: :purged })
          allow(Facter).to receive(:value).and_call_original
          allow(Facter).to receive(:value).with('pe_server_version').and_return('1.2.3')
        end

        it 'by default it configures without recover' do
          expect(Puppet::Util::Pe_conf).not_to receive(:recover_and_save_pe_configuration)
          expect_configure_with(0, 'ran')
        end
      end
    end
  end

  context "#attempt_to_recover_configuration" do
    def with_logs
      logs = []
      Puppet::Util::Log.with_destination(Puppet::Test::LogCollector.new(logs)) do
        yield
      end
      logs.map { |l| l.message }
    end

    it "returns true for successful recovery" do
      expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
        .and_return(mock_conf)
      expect(infrastructure.attempt_to_recover_configuration(:recover => true)).to eq(true)
    end

    it "returns false if no recovery attempted" do
      expect(infrastructure.attempt_to_recover_configuration(:recover => false)).to eq(false)
    end

    it "logs errors thrown during recovery and returns false" do
      expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
        .and_raise(RuntimeError, 'file not found')
      logs = with_logs do
        expect(infrastructure.attempt_to_recover_configuration(:recover => true)).to eq(false)
      end
      expect(logs).to include(/file not found/)
    end

    context "with --debug logging" do
      before(:each) do
        Puppet[:log_level] = :debug
      end

      it "also logs generated conf during successful recovery" do
        conf = double('conf',
                      :pe_conf => {'parameter' => 'value'},
                      :nodes_conf => [{'override' => 'other'}])

        expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
          .and_return(conf)

        logs = with_logs do
          expect(infrastructure.attempt_to_recover_configuration(:recover => true)).to eq(true)
        end
        expect(logs).to include(/parameter.*value/)
        expect(logs).to include(/override.*other/)
      end

      it "also logs backtrace when an error is thrown" do
        expect(Puppet::Util::Pe_conf).to receive(:recover_and_save_pe_configuration)
          .and_raise(RuntimeError, 'file not found')

        logs = with_logs do
          expect(infrastructure.attempt_to_recover_configuration(:recover => true)).to eq(false)
        end
        expect(logs).to include(/file not found/)
        expect(logs).to include(/file not found:.*expectation.*and_raise/)
      end
    end
  end

  context "#attempt_recovery?" do
    RSpec.shared_examples 'attempt_recovery_query' do
      context "with --recover" do
        it { expect(infrastructure.attempt_recovery?(true)).to eq(forced_recover) }
      end

      context "with --no-recover" do
        it { expect(infrastructure.attempt_recovery?(false)).to eq(forced_no_recover) }
      end

      context "by default" do
        it { expect(infrastructure.attempt_recovery?).to eq(default_recover) }
      end
    end

    let(:forced_recover) { true }
    let(:forced_no_recover) { false }

    context "when performing a fresh install" do
      before(:each) do
        allow(resource_face).to receive(:find).and_return({ :ensure => :purged })
        allow(Facter).to receive(:value).with('pe_server_version').and_return(nil)
      end

      let(:default_recover) { false }

      include_examples 'attempt_recovery_query'
    end

    context "when re-installing or upgrading" do
      before(:each) do
        allow(Facter).to receive(:value).with('pe_server_version').and_return('1.2.3')
      end

      context "on the master node" do
        before(:each) do
          allow(resource_face).to receive(:find).and_return({ :ensure => :installed })
        end

        # This is the only case where we recover by default...
        let(:default_recover) { true }

        include_examples 'attempt_recovery_query'
      end

      context "on other pe-infra node" do
        before(:each) do
          allow(resource_face).to receive(:find).and_return({ :ensure => :purged })
        end

        let(:default_recover) { false }

        include_examples 'attempt_recovery_query'
      end
    end
  end

  context '#package_version' do
    it do
      expect(resource_face).to receive(:find).and_return({ :ensure => '2017.3.9.6.3-1.pe.el6' })
      expect(infrastructure.package_version('pe-postgresql-server')).to eq('2017.3.9.6.3-1.pe.el6')
    end
  end

  context '#postgres_server_version' do
    it do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => '9.6'
      )
      expect(infrastructure.postgres_server_version).to eq('9.6')
    end

    it 'returns nil if there is no fact' do
      expect(infrastructure.postgres_server_version).to be_nil
    end

    it 'returns nil if installed_server_version empty' do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => ''
      )
      expect(infrastructure.postgres_server_version).to be_nil
    end

    it 'returns nil if installed_server_version nil' do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => nil
      )
      expect(infrastructure.postgres_server_version).to be_nil
    end
  end

  context "#master?" do
    it "returns false if pe-puppetserver is in a purged state" do
      expect(resource_face).to receive(:find).and_return({ :ensure => :purged })
      expect(infrastructure.master?).to eq(false)
    end

    it "returns false if pe-puppetserver is in an absent state" do
      expect(resource_face).to receive(:find).and_return({ :ensure => :absent })
      expect(infrastructure.master?).to eq(false)
    end

    it "returns true for other states" do
      expect(resource_face).to receive(:find).and_return({ :ensure => :foo })
      expect(infrastructure.master?).to eq(true)
    end
  end

  context "#installed?" do
    it "returns true if pe_server_version fact is present" do
      expect(Facter).to receive(:value).with('pe_server_version').and_return('1.2.3')
      expect(infrastructure.installed?).to eq(true)
    end

    it "returns false if pe_server_version fact is absent" do
      expect(Facter).to receive(:value).with('pe_server_version').and_return(nil)
      expect(infrastructure.installed?).to eq(false)
    end
  end

  context "--postgres-migration defaults" do
    let(:action) { infrastructure.get_action(:configure) }
    let(:option) { action.get_option(:postgres_migration) }

    it "is true if pe-postgresql-server is older than current" do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => '9.4'
      )
      expect(option.default).to eq(true)
      expect(@logs.find { |m| m.message.start_with?('Including pe_install')}.message).to eq("Including pe_install::upgrade::postgres migration class based on current pe-postgresql-server 9.4 being older than the version being installed (9.6).")
    end

    it "is false if pe-postgresql-server is the current version" do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => '9.6'
      )
      expect(option.default).to eq(false)
    end

    it "is false if pe-postgresql-server package is not present" do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => nil
      )
      expect(option.default).to eq(false)
    end

    it "is false if pe-postgresql-server package is an empty string" do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(
        'installed_server_version' => ''
      )
      expect(option.default).to eq(false)
    end
  end

  context "#pe_server_version" do
    it 'returns the pe_server_version fact' do
      expect(Facter).to receive(:value).with('pe_server_version').and_return('2017.3.0')
      expect(infrastructure.pe_server_version).to eq('2017.3.0')
    end
  end

  context "#version_cmp" do
    it do
      expect(infrastructure.version_cmp('2016.3.0','2017.3.0')).to eq(-1)
      expect(infrastructure.version_cmp('2017.2.0','2017.3.0')).to eq(-1)
      expect(infrastructure.version_cmp('2017.3.0','2017.3.1')).to eq(-1)
    end
    it { expect(infrastructure.version_cmp('2017.3.0','2017.3.0')).to eq(0) }
    it do
      expect(infrastructure.version_cmp('2018.3.0','2017.3.0')).to eq(1)
      expect(infrastructure.version_cmp('2017.4.0','2017.3.0')).to eq(1)
      expect(infrastructure.version_cmp('2017.3.1','2017.3.0')).to eq(1)
    end

    context 'with a dev version' do
      it do
        expect(infrastructure.version_cmp('2017.3.0.0.0.2','2017.3.0.0.1')).to eq(-1)
        expect(infrastructure.version_cmp('2017.3.0.6.52','2017.3.0.7.12')).to eq(-1)
        expect(infrastructure.version_cmp('2017.3.0.7.11','2017.3.0.7.12')).to eq(-1)
        # But keep in mind that the final tag 2017.3.0 will be 2017.3.0.0.0 and
        # will be less than a dev build...
        expect(infrastructure.version_cmp('2017.3.0.0.0','2017.3.0.7.12')).to eq(-1)
      end
      it do
        expect(infrastructure.version_cmp('2017.3.0.7.12','2017.3.0.7.12')).to eq(0)
      end
      it do
        expect(infrastructure.version_cmp('2017.3.0.7.12','2017.3.0.6.52')).to eq(1)
        expect(infrastructure.version_cmp('2017.3.0.6.53','2017.3.0.6.52')).to eq(1)
      end
    end

    context 'returns nil' do
      it 'when both args nil' do
        expect(infrastructure.version_cmp(nil,nil)).to be_nil
      end
      it 'when either arg nil' do
        expect(infrastructure.version_cmp(nil,'2017.3.0')).to be_nil
        expect(infrastructure.version_cmp('2017.4.0',nil)).to be_nil
      end
      it 'when both args empty' do
        expect(infrastructure.version_cmp('','')).to be_nil
      end
      it 'when either arg empty' do
        expect(infrastructure.version_cmp('','2017.3.0')).to be_nil
        expect(infrastructure.version_cmp('2017.4.0','')).to be_nil
      end
    end

    context 'postgres versions' do
      it 'correctly compares x.y to x.y.z versions' do
        expect(infrastructure.version_cmp('9.6','9.4.12')).to eq(1)
        expect(infrastructure.version_cmp('9.6.3','9.4')).to eq(1)
        expect(infrastructure.version_cmp('9.6.3','9.6')).to eq(1)

        expect(infrastructure.version_cmp('9.4','9.6.3')).to eq(-1)
        expect(infrastructure.version_cmp('9.4.12','9.6')).to eq(-1)
      end
    end
  end

  context '#classes_to_include' do
    it 'includes just pe_install with no flags' do
      expect(infrastructure.classes_to_include({})).to eq(
        <<-EOS
class { pe_install:
\s\s
}
        EOS
      )
    end

    it 'includes pe_install with empty upgrade flags' do
      expect(infrastructure.classes_to_include({:postgres_migration => false, :upgrade_from => nil})).to eq(
        <<-EOS
class { pe_install:
\s\s
}
        EOS
      )
    end

    it 'ignores a nil install flag' do
      expect(infrastructure.classes_to_include({:install => nil})).to eq(
        <<-EOS
class { pe_install:
\s\s
}
        EOS
      )
    end

    it 'includes boostrap_to' do
      expect(infrastructure.classes_to_include({:install => '2018.1.7', :postgres_migration => false})).to eq(
        <<-EOS
class { puppet_enterprise::packages:
  installing => true,
}

class { pe_install:
  bootstrap_to => '2018.1.7',
}
        EOS
      )
    end

    it 'adds upgrade_from' do
      expect(infrastructure.classes_to_include({:postgres_migration => false, :upgrade_from => '2017.2.3'})).to eq(
        <<-EOS
class { puppet_enterprise::packages:
  installing => true,
}

class { pe_install:
  upgrade_from => '2017.2.3',
}
        EOS
      )
    end

    it 'adds bootstrap_to and upgrade-from' do
      expect(infrastructure.classes_to_include({:install => '2018.1.7', :upgrade_from => '2017.2.3'})).to eq(
        <<-EOS
class { puppet_enterprise::packages:
  installing => true,
}

class { pe_install:
  bootstrap_to => '2018.1.7',
  upgrade_from => '2017.2.3',
}
        EOS
      )
    end

    it 'adds postgres' do
      expect(infrastructure.classes_to_include({:postgres_migration => true, :install => '2018.1.7', :upgrade_from => '2017.2.3'})).to eq(
        <<-EOS
class { puppet_enterprise::packages:
  installing => true,
}

class { pe_install:
  bootstrap_to => '2018.1.7',
  upgrade_from => '2017.2.3',
}

class { pe_install::upgrade::postgres:
  old_postgres_version => '9.4',
  new_postgres_version => '9.6',
}
        EOS
      )
    end
  end

  context 'upgrade_in_process' do
    let(:test_process_file) { "#{tmpdir}/test-in-process" }

    before(:each) do
      allow(infrastructure).to receive(:upgrade_in_process_file).and_return(test_process_file)
    end

    it do
      expect(infrastructure.upgrade_in_process).to be_nil
    end

    it 'does nothing when no upgrade_from is set' do
      expect(infrastructure.save_upgrade_in_process({})).to be_nil
      expect(infrastructure.upgrade_in_process).to be_nil
      expect(infrastructure.remove_upgrade_in_process).to be_nil
      expect(File.exist?(test_process_file)).to eq(false)
    end

    it 'does nothing when upgrade_from is nil' do
      expect(infrastructure.save_upgrade_in_process(:upgrade_from => '')).to be_nil
      expect(infrastructure.upgrade_in_process).to be_nil
      expect(infrastructure.remove_upgrade_in_process).to be_nil
      expect(File.exist?(test_process_file)).to eq(false)
    end

    it 'does nothing when upgrade_from is empty' do
      expect(infrastructure.save_upgrade_in_process(:upgrade_from => nil)).to be_nil
      expect(infrastructure.upgrade_in_process).to be_nil
      expect(infrastructure.remove_upgrade_in_process).to be_nil
      expect(File.exist?(test_process_file)).to eq(false)
    end

    it 'stores the upgrade version' do
      expect(infrastructure.save_upgrade_in_process(:upgrade_from => '2017.3.0')).to eq(8) #bytes
      expect(infrastructure.upgrade_in_process).to eq('2017.3.0')
      expect(infrastructure.remove_upgrade_in_process).to eq([test_process_file])
      expect(File.exist?(test_process_file)).to eq(false)
    end

    it 'handles a line ending' do
      expect(infrastructure.save_upgrade_in_process(:upgrade_from => "2017.3.0\n")).to eq(9) #bytes
      expect(infrastructure.upgrade_in_process).to eq('2017.3.0')
      expect(infrastructure.remove_upgrade_in_process).to eq([test_process_file])
      expect(File.exist?(test_process_file)).to eq(false)
    end
  end

  context '#pe_postgres_uid' do
    context 'no pe-postgres user' do
      it { expect(infrastructure.pe_postgres_uid).to be_nil }
    end

    context 'pe-postgres user exists' do
      it do
        expect(resource_face).to receive(:find).with('user/pe-postgres')
          .and_return({
            :ensure           => :present,
            :gid              => 496,
            :home             => '/opt/puppetlabs/server/data/postgresql',
            :password         => '!!',
            :password_max_age => -1,
            :password_min_age => -1,
            :shell            => '/sbin/nologin',
            :uid              => 495,
          })
        expect(infrastructure.pe_postgres_uid).to eq(495)
      end
    end
  end

  context '#update_pe_conf_with_database_encoding_if_migrating' do
    let(:args) { {} }

    it { expect(infrastructure.update_pe_conf_with_database_encoding_if_migrating(args)).to be_nil }

    context 'when upgrading postgres' do
      let(:args) { { :postgres_migration => true } }

      it 'warns on failure' do
        message = /Failed to obtain current Postgresql server encoding.*Execution of.*psql.*returned 1/m
        expect_log(:warning, message) do
          expect(infrastructure.update_pe_conf_with_database_encoding_if_migrating(args)).to eq(false)
        end
      end

      context 'and able to make psql queries' do
        include_context 'postgres-migration-encoding'

        context 'that return default encoding values' do
          let(:encoding) { 'UTF8' }
          let(:collate) { 'en_US.UTF-8' }
          let(:ctype) { 'en_US.UTF-8' }

          it 'adds the default values (decided to be explicit)' do
            expect(infrastructure.update_pe_conf_with_database_encoding_if_migrating(:postgres_migration => true)).to eq(true)
            expect(File.read(test_pe_conf_path)).to eq(
              <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::profile::database::encoding" : "UTF8"
  "puppet_enterprise::profile::database::locale" : "en_US.UTF-8"
  "puppet_enterprise::profile::database::ctype" : "en_US.UTF-8"
  "puppet_enterprise::profile::database::collate" : "en_US.UTF-8"
}
              EOF
            )
          end
        end

        context 'that return non-default encoding values' do
          let(:encoding) { 'EUC_KR' }
          let(:collate) { 'ko_KR.euckr' }
          let(:ctype) { 'ko_KR.euckr' }

          it 'updates pe.conf with the discovered values' do
            expect(infrastructure.update_pe_conf_with_database_encoding_if_migrating(:postgres_migration => true)).to eq(true)
            expect(File.read(test_pe_conf_path)).to eq(
              <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::profile::database::encoding" : "EUC_KR"
  "puppet_enterprise::profile::database::locale" : "ko_KR.euckr"
  "puppet_enterprise::profile::database::ctype" : "ko_KR.euckr"
  "puppet_enterprise::profile::database::collate" : "ko_KR.euckr"
}
              EOF
            )
          end
        end
      end
    end
  end

  context '#local_postgresql_client_is_behind?' do
    let(:pe_postgresql_info) { nil }
    before(:each) do
      expect(Facter).to receive(:value).with('pe_postgresql_info').and_return(pe_postgresql_info)
    end

    it 'is false when no fact is found' do
      expect(infrastructure.local_postgresql_client_is_behind?).to eq(false)
    end

    context 'no installed_packages hash' do
      let(:pe_postgresql_info) { {} }

      it { expect(infrastructure.local_postgresql_client_is_behind?).to eq(false) }
    end

    context 'no pe-postgresql version info' do
      let(:pe_postgresql_info) { { 'installed_packages' => {} } }

      it { expect(infrastructure.local_postgresql_client_is_behind?).to eq(false) }
    end

    context 'with pe-postgresql version info' do
      let(:pe_postgresql_info) do
        {
          'installed_packages' => {
            'pe-postgresql' => postgres_version,
          }
        }
      end

      context 'equal to current postgres major.minor' do
        let(:postgres_version) { '9.6.10-2' }

        it { expect(infrastructure.local_postgresql_client_is_behind?).to eq(false) }
      end

      context 'greater than current postgres major.minor' do
        let(:postgres_version) { '10.0.0-1' }

        it { expect(infrastructure.local_postgresql_client_is_behind?).to eq(false) }
      end

      context 'less than current postgres major.minor' do
        let(:postgres_version) { '9.4.3-1' }

        it { expect(infrastructure.local_postgresql_client_is_behind?).to eq(true) }
      end
    end
  end

  context 'external postgres functions' do
    include_context 'puppet lookup'
    before(:each) do

    end

    context  '#puppet_lookup' do
      it do
        expect(infrastructure.puppet_lookup('puppet_enterprise::puppet_master_host')).to eq('master.rspec')
      end

      it do
        expect(infrastructure.puppet_lookup('not_present')).to be_nil
      end
    end

    context 'managed postgres' do
      it { expect(infrastructure.separate_database_node?).to eq(false) }
      it { expect(infrastructure.external_database_version).to be_nil }
    end

    context 'external postgres' do
      let(:default_pe_conf) do
        <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::database_host": "external-db.rspec"
  "puppet_enterprise::database_port": 1234
  "puppet_enterprise::puppetdb_database_name": "pe-puppetdb"
  "puppet_enterprise::puppetdb_database_user": "pe-puppetdb"
  "puppet_enterprise::puppetdb_database_password": "puppetdb-PASSWORD"
}
        EOF
      end

      it { expect(infrastructure.separate_database_node?).to eq(true) }

      it "warns if psql fails" do
        message = %r{externally managed Postgresql.*external-db\.rspec.*unable to look up the version.*Puppet::ExecutionFailure.*psql.*external-db\.rspec}m
        expect_log(:warning, message ) do
          expect(infrastructure.external_database_version).to be_nil
        end
      end

      it "obtains a version" do
        expect(Puppet::Util::Execution).to receive(:execute)
          .with(match(/psql.*--username='pe-puppetdb' --host='external-db\.rspec'.*--dbname='pe-puppetdb'.*select version/), anything)
          .and_return('PostgreSQL 9.6.3')
        expect(infrastructure.external_database_version).to eq('9.6.3')
      end
    end

    context 'external postgres with cert auth' do
      let(:default_pe_conf) do
        <<-EOF
{
  "console_admin_password": "password"
  "puppet_enterprise::puppet_master_host": "master.rspec"
  "puppet_enterprise::database_host": "external-db.rspec"
  "puppet_enterprise::database_port": 1234
  "puppet_enterprise::database_ssl": true
  "puppet_enterprise::database_cert_auth": true
  "puppet_enterprise::orchestrator_database_name": "pe-orchestrator"
  "puppet_enterprise::orchestrator_database_user": "pe-orchestrator"
  "puppet_enterprise::orchestrator_database_password": "orch-PASSWORD"
  "puppet_enterprise::puppetdb_host": "puppetdb.rspec"
  "puppet_enterprise::puppetdb_database_name": "pe-puppetdb"
  "puppet_enterprise::puppetdb_database_user": "pe-puppetdb"
  "puppet_enterprise::puppetdb_database_password": "puppetdb-PASSWORD"
  "puppet_enterprise::console_host": "console.rspec"
  "puppet_enterprise::classifier_database_name": "pe-classifier"
  "puppet_enterprise::classifier_database_user": "pe-classifier"
  "puppet_enterprise::classifier_database_password": "console-PASSWORD"
}
        EOF
      end
      let(:orch_cert) { '/etc/puppetlabs/orchestration-services/ssl/master.rspec.cert.pem' }
      let(:orch_key) { '/etc/puppetlabs/orchestration-services/ssl/master.rspec.private_key.pem' }
      let(:puppetdb_cert) { '/etc/puppetlabs/puppetdb/ssl/puppetdb.rspec.cert.pem' }
      let(:puppetdb_key) { '/etc/puppetlabs/puppetdb/ssl/puppetdb.rspec.private_key.pem' }
      let(:classifier_cert) { '/opt/puppetlabs/server/data/console-services/certs/console.rspec.cert.pem' }
      let(:classifier_key) { '/opt/puppetlabs/server/data/console-services/certs/console.rspec.private_key.pem' }
      let(:ca_cert) { '/etc/puppetlabs/puppet/ssl/certs/ca.pem' }

      it { expect(infrastructure.separate_database_node?).to eq(true) }

      it "warns if psql fails" do
        message = %r{externally managed Postgresql.*external-db\.rspec.*unable to look up the version.*Puppet::ExecutionFailure.*psql.*external-db\.rspec}m
        expect_log(:warning, message ) do
          expect(infrastructure.external_database_version).to be_nil
        end
      end

      it "obtains a version on orchestration node" do
        expect(File).to receive(:readable?).with(orch_cert).and_return(true)
        expect(File).to receive(:readable?).with(orch_key).and_return(true)
        expect(File).to receive(:readable?).with(ca_cert).and_return(true)
        expect(File).to receive(:readable?).with(anything).and_return(false).at_least(:once)
        expect(Puppet::Util::Execution).to receive(:execute)
          .with(match(/psql.*--username='pe-orchestrator' --host='external-db\.rspec'.*--dbname='pe-orchestrator'.*select version/), include({
            :custom_environment => a_hash_including({
              'PGSSLCERT' => orch_cert,
              'PGSSLKEY' => orch_key,
              'PGSSLMODE' => 'verify-full',
              'PGSSLROOTCERT' => ca_cert
            })
          })).and_return('PostgreSQL 9.6.3')
        expect(infrastructure.external_database_version).to eq('9.6.3')
      end

      it "obtains a version on puppetdb node" do
        expect(File).to receive(:readable?).with(puppetdb_cert).and_return(true)
        expect(File).to receive(:readable?).with(puppetdb_key).and_return(true)
        expect(File).to receive(:readable?).with(ca_cert).and_return(true)
        expect(File).to receive(:readable?).with(anything).and_return(false).at_least(:once)
        expect(Puppet::Util::Execution).to receive(:execute)
          .with(match(/psql.*--username='pe-puppetdb' --host='external-db\.rspec'.*--dbname='pe-puppetdb'.*select version/), include({
            :custom_environment => a_hash_including({
              'PGSSLCERT' => puppetdb_cert,
              'PGSSLKEY' => puppetdb_key,
              'PGSSLMODE' => 'verify-full',
              'PGSSLROOTCERT' => ca_cert
            })
          })).and_return('PostgreSQL 9.6.3')
        expect(infrastructure.external_database_version).to eq('9.6.3')
      end

      it "obtains a version on classifier node" do
        expect(File).to receive(:readable?).with(classifier_cert).and_return(true)
        expect(File).to receive(:readable?).with(classifier_key).and_return(true)
        expect(File).to receive(:readable?).with(ca_cert).and_return(true)
        expect(File).to receive(:readable?).with(anything).and_return(false).at_least(:once)
        expect(Puppet::Util::Execution).to receive(:execute)
          .with(match(/psql.*--username='pe-classifier' --host='external-db\.rspec'.*--dbname='pe-classifier'.*select version/), include({
            :custom_environment => a_hash_including({
              'PGSSLCERT' => classifier_cert,
              'PGSSLKEY' => classifier_key,
              'PGSSLMODE' => 'verify-full',
              'PGSSLROOTCERT' => ca_cert
            })
          })).and_return('PostgreSQL 9.6.3')
        expect(infrastructure.external_database_version).to eq('9.6.3')
      end
    end
  end
end
