require 'spec_helper'
require 'puppet/face'
require 'puppet/network/http_pool'
require 'hocon'
require 'webmock/rspec'
require 'services_helpers'

describe Puppet::Face[:infrastructure, :current] do
  let(:action) { subject.get_action(:enable) }
  let(:config_file_path) { '/etc/puppetlabs/client-tools/services.conf' }

  context "enable action" do
    include_context 'SSL stubs'
    include_context 'facter stubs'

    before(:each) do
      WebMock.disable_net_connect!

      # This is required so the correct elements are selected from the service config
      Puppet.settings[:certname] = 'mono.example.vm'

      allow(PuppetX::Util::ServiceStatus).to receive(:load_config).and_return(
          PuppetX::Util::ServiceStatus.symbolize_keys(
            {
              'services' => [
                  puppetdb_service,
                  pe_master_service,
                  classifier_service,
                  file_sync_client,
                  orch_service,
                  code_manager,
                  pcp_broker,
                  file_sync_storage,
              ],
              'nodes' => [
                { 'certname' => 'mono.example.vm',
                  'display_name' => 'Primary Master',
                  'role' => 'primary_master' },
                { 'certname' => 'buzz',
                  'display_name' => 'Primary Master Replica',
                  'role' => 'primary_master_replica' },
              ]
            }
          )
      )
      Puppet::Util::Log.level = :info
    end

    after(:each) do
      WebMock.allow_net_connect!
    end

    let(:puppetdb_service) { ServicesTestHelpers.gen_service_hash('puppetdb', 8081, 'puppetdb.example.vm', "PuppetDB")}
    let(:pe_master_service) { ServicesTestHelpers.gen_service_hash('master', 8140, 'mono.example.vm', "Pe_master")}
    let(:file_sync_client) { ServicesTestHelpers.gen_service_hash('file-sync-client', 8140, 'mono.example.vm', "File Sync Client")}
    let(:file_sync_storage) { ServicesTestHelpers.gen_service_hash('file-sync-storage', 8140, 'mono.example.vm', "File Sync Storage")}
    let(:code_manager) { ServicesTestHelpers.gen_service_hash('code-manager', 8170, 'mono.example.vm', "Code Manager")}
    let(:pcp_broker) { ServicesTestHelpers.gen_service_hash('pcp-broker', 8142, 'mono.example.vm', "PCP Broker")}
    let(:classifier_service) { ServicesTestHelpers.gen_service_hash('classifier', 4433, 'mono.example.vm', "Classifier")}
    let(:rbac_service) { ServicesTestHelpers.gen_service_hash('rbac', 4433, 'mono.example.vm', "RBAC")}
    let(:activity_service) { ServicesTestHelpers.gen_service_hash('activity', 4433, 'mono.example.vm', "Activity")}
    let(:orch_service) { ServicesTestHelpers.gen_service_hash('orchestrator', 4433, 'mono.example.vm', "Orchestrator")}
    let(:no_params_groups_response) {
      [{'name' => 'PE Infrastructure Agent',
        'classes' =>
            {'puppet_enterprise::profile::agent' =>
                 {},
            },
       },
       {'name' => 'PE Infrastructure',
        'classes' =>
            {'puppet_enterprise' =>
                 {},
            },
       },
       {'name' => 'PE Master',
        'classes' =>
            {'puppet_enterprise::profile::master' =>
                 {}
            },
       },
       {'name' => 'PE Agent',
        'classes' =>
            {'puppet_enterprise::profile::agent' =>
                 {},
            },
       }
      ]
    }
    let(:all_groups_response) {
      [{'name' => 'PE Infrastructure Agent',
        'classes' =>
            {'puppet_enterprise::profile::agent' =>
                 {'server_list' => ['blitz:8140', 'baz:8140'],
                  'pcp_broker_ws_uris' => ['wss://blitz:8142/pcp/', 'wss://baz:8142/pcp/']},
            },
       },
       {'name' => 'PE Infrastructure',
        'classes' =>
        {'puppet_enterprise' =>
         {'ha_enabled_replicas' => ['foo', 'bar']},
        },
       },
       {'name' => 'PE Master',
        'classes' =>
            {'puppet_enterprise::profile::master' =>
                 {'puppetdb_host' => ['foo', 'bar'],
                  'puppetdb_port' => ['8080', '8081'],
                  'classifier_host' => ['blitz', 'baz'],
                  'classifier_port' => ['4433', '4433'],
                 }

            }},
       {'name' => 'PE Agent',
        'classes' =>
            {'puppet_enterprise::profile::agent' =>
                 {'server_list' => ['foo:8140', 'bar:8140'],
                  'pcp_broker_ws_uris' => ['wss://foo:8142/pcp/', 'wss://bar:8142/pcp/']},
            },
       }
      ]
    }

    context "--print-config" do
      it 'it prints the expected config with groups populated' do
        stub_request(:get, "https://mono.example.vm:4433/fake_prefix/v1/groups").
            to_return(:status => 200, :body => all_groups_response.to_json, :headers => {})

        expect {subject.enable("replica", "buzz", {:print_config => true})}.to output(<<-HEREEEEEEEEE
Current configuration:
  agent-server-urls       foo:8140, bar:8140
  infra-agent-server-urls blitz:8140, baz:8140
  pcp-brokers             foo:8142, bar:8142
  infra-pcp-brokers       blitz:8142, baz:8142
  puppetdb-termini        foo:8080, bar:8081
  classifier-termini      blitz:4433, baz:4433
HEREEEEEEEEE
).to_stdout
      end

      it 'it prints the expected config with groups empty' do
        stub_request(:get, "https://mono.example.vm:4433/fake_prefix/v1/groups").
            to_return(:status => 200, :body => no_params_groups_response.to_json, :headers => {})

        expect {subject.enable("replica", "buzz", {:print_config => true})}.to output(<<-HEREEEEEEEEE
Current configuration:
  agent-server-urls       not configured for HA
  infra-agent-server-urls not configured for HA
  pcp-brokers             not configured for HA
  infra-pcp-brokers       not configured for HA
  puppetdb-termini        not configured for HA
  classifier-termini      not configured for HA
                                                                               HEREEEEEEEEE
                                                                               ).to_stdout
      end

      it 'it prints the expected config when a replica is already enabled' do
        stub_request(:get, "https://mono.example.vm:4433/fake_prefix/v1/groups").
          to_return(:status => 200, :body => all_groups_response.to_json, :headers => {})

        expect {subject.enable("replica", "foo", {:print_config => true})}.to output(<<-HEREEEEEEEEE
Current configuration:
  agent-server-urls       foo:8140, bar:8140
  infra-agent-server-urls blitz:8140, baz:8140
  pcp-brokers             foo:8142, bar:8142
  infra-pcp-brokers       blitz:8142, baz:8142
  puppetdb-termini        foo:8080, bar:8081
  classifier-termini      blitz:4433, baz:4433
HEREEEEEEEEE
).to_stdout
      end
    end

    context "interview" do
      before(:each) do
        expect(STDIN).to receive(:tty?).and_return(false)
        stub_request(:get, "https://mono.example.vm:4433/fake_prefix/v1/groups").
            to_return(:status => 200, :body => all_groups_response.to_json, :headers => {})
      end

      context 'fails with no tty' do
        it 'only topology is set' do
          expect {subject.enable("replica", "buzz", {:topology => 'mono'})}.to \
            raise_error(SystemExit).and \
            output(/interview requires a tty to proceed/).to_stdout
        end

        it 'yes is set, topology is set to mono-with-compile and agent-server-urls is not set' do
          expect {subject.enable("replica", "buzz", {:topology => 'mono-with-compile', :yes => true})}.to \
            raise_error(SystemExit).and \
            output(/interview requires a tty to proceed/).to_stdout
        end

        it 'no flags passed' do
          expect {subject.enable("replica", "buzz")}.to \
            raise_error(SystemExit).and \
            output(/interview requires a tty to proceed/).to_stdout
        end
      end

      context 'proceeds with no tty' do
        # Without these this gets super noisy
        before(:each) do
          allow($stdout).to receive(:puts)
        end

        # these checks are unfortunate, but the measure of success here is that we get
        # to the next failure checkpoint in enable, which is looking for an RBAC token
        # that won't exist
        it 'yes and topology is mono' do
          expect { subject.enable("replica", "buzz", {:topology => 'mono', :yes => true}) }.to \
            raise_error(RuntimeError, /RBAC token/)
        end

        it 'yes and topology is mono-with-compile and agent-server-urls is passed' do
          expect { subject.enable("replica", "buzz", {:topology => 'mono-with-compile', :yes => true, :agent_server_urls => 'foo:8140', :pcp_brokers => "foo:8142"}) }.to \
            raise_error(RuntimeError, /RBAC token/)
        end

        it 'yes and topology is mono-with-compile and skip-agent-config is passed' do
          expect { subject.enable("replica", "buzz", {:topology => 'mono-with-compile', :yes => true, :skip_agent_config => true}) }. to \
            raise_error(RuntimeError, /RBAC token/)
        end
      end
    end
  end

  context 'with help example flag' do
    let(:action) { 'enable' }

    include_examples 'help'
  end
end
