require 'spec_helper'
require 'puppet/face'
require 'hocon'
RSpec::Matchers.define :exit_with do |expected|
  actual = nil
  match do |block|
    begin
      block.call
    rescue SystemExit => e
      actual = e.status
    end
    actual and actual == expected
  end

  supports_block_expectations

  failure_message do |block|
    "expected exit with code #{expected} but " +
      (actual.nil? ? " exit was not called" : "we exited with #{actual} instead")
  end

  failure_message_when_negated do |block|
    "expected that exit would not be called with #{expected}"
  end

  description do
    "expect exit with #{expected}"
  end
end

RSpec.shared_examples "outputs help" do
  it 'does not raise an error' do
    expect { subject.run(*args) }.not_to raise_error
  end

  it 'outputs usage' do
    expect { subject.run(*args) }.to output(/.*regenerate_agent_certificate.*rebuild_certificate_authority/m).to_stdout
  end
end

describe Puppet::Face[:infrastructure, :current] do
  let(:command) { [*args] }
  let(:action) { subject.get_action(:run) }
  let(:options) { {} }
  let(:args) { [ options ] }
  context "#args_options_to_string" do
    it 'returns an empty string' do
      expect(subject.args_options_to_string([], {})).to eq(' ')
    end

    it 'joins args' do
      expect(subject.args_options_to_string(['foo', 'bar'], {})).to eq('foo bar ')
    end

    it 'handles boolean true flags' do
      expect(subject.args_options_to_string([], { help: true })).to eq(' --help')
    end

    it 'handles boolean false flags' do
      expect(subject.args_options_to_string([], { help: false })).to eq(' --no-help')
    end

    it 'handles string flags' do
      expect(subject.args_options_to_string([], { thing: 'wow' })).to eq(' --thing=\'wow\'')
    end

    it 'handles args boolean and strings all at once' do
      expect(subject.args_options_to_string(['foo', 'bar'], { thing: 'wow' })).to eq('foo bar --thing=\'wow\'')
    end
  end

  context "run action" do
    before(:each) do
      allow($stdout).to receive(:puts)
    end

    context 'base action' do
      include_examples "outputs help"
    end

    context 'help option' do
      let(:options) { { help: true } }
      include_examples "outputs help"
    end

    context 'help subcommand' do
      let(:args) { [ 'help' ] }
      include_examples "outputs help"
    end

    context 'boltymcboltface command with no args' do
      let(:args) { [ 'boltymcboltface' ] }
      let(:command) {'plan show boltymcboltface'}
      before(:each) do
        expect(Puppet::Face[:infrastructure, :current]).not_to receive(:run_bolt).with(command)
      end

      it 'sends usage info' do
        expect {subject.run(*args)}.to exit_with(1).and output(/USAGE\: puppet infra run boltymcboltface/m).to_stdout
      end  
    end

    context 'boltymcboltface with junk args' do
      let(:args) { [ 'boltymcboltface', 'extra', 'arguments'] }
      let(:command) { 'plan run boltymcboltface extra arguments' }
      before(:each) do
        process_status = double()
        allow(process_status).to(receive(:exitstatus)).and_return(1)
        expect(Puppet::Face[:infrastructure, :current]).to receive(:run_bolt).with(command).and_return(process_status, nil).once
      end

      it 'raises error' do
        expect {subject.run(*args)}.to exit_with(1)
      end  
    end

    context 'when bolt is expected to be called and exit 0' do
      before(:each) do
        process_status = double()
        allow(process_status).to(receive(:exitstatus)).and_return(0)
        expect(Puppet::Face[:infrastructure, :current]).to receive(:run_bolt).with(command).and_return(process_status, nil).once
      end

      context 'mapped argument with no flags' do
        let(:args) { [ 'regenerate_agent_certificate' ] }
        let(:command) { 'plan run enterprise_tasks::agent_cert_regen' }
        it 'exits successfully' do
          expect {subject.run(*args)}.to exit_with(0)
        end
      end

      context 'mapped argument with help flag' do
        let(:options) { { help: true } }
        let(:args) { [ 'regenerate_agent_certificate', options ] }
        let(:command) { 'plan show enterprise_tasks::agent_cert_regen' }
        it 'exits successfully' do
          expect {subject.run(*args)}.to exit_with(0)
        end
      end
    end
  end
end
