require 'spec_helper'
require 'puppet/face'
require 'puppet/network/http_pool'
require 'hocon'
require 'webmock/rspec'
require 'services_helpers'
require 'puppet_x/util/service_status'
describe Puppet::Face[:infrastructure, :current] do
  let(:action) { subject.get_action(:status) }
  let(:config_file_path) { '/etc/puppetlabs/client-tools/services.conf' }

  context "status action" do
    include_context 'SSL stubs'
    before(:each) do
      WebMock.disable_net_connect!

      allow(PuppetX::Util::ServiceStatus).to receive(:load_config).and_return(
        PuppetX::Util::ServiceStatus.symbolize_keys(
          {
            'services' => [
              puppetdb_service,
              pe_master_service,
              orchestrator_service,
            ],
            'nodes' => [
              { 'certname' => 'mono.example.vm',
                'role' => 'primary_master',
                'display_name' => 'Primary Master'}
            ]
          }
        )
      )
      Puppet::Util::Log.level = :info
    end

    after(:each) do
      WebMock.allow_net_connect!
    end

    def stub_status_unreachable(url)
      stub_request(:get, url).to_raise(Errno::ECONNREFUSED)
    end

    def stub_status_running(url, service_name)
      response = {
        "#{service_name}-status" => {
          "state" => 'running',
          "status" => {},
        },
        "active_alerts" => []
      }

      stub_request(:get, url).to_return(:status => 200, :body => response.to_json)
    end

    def stub_status_error(url, service_name)
      response = {
        "#{service_name}-status" => {
          'service_status_version' => 1,
          'state' => 'error',
          'status' => {
            'db_up' => false,
          },
          'active_alerts' => []
        },
      }
      stub_request(:get, url).to_return(:status => 503, :body => response.to_json)
    end

    def status_url(service)
      "#{service['status_url']}/v1/services?timeout=5"
    end

    def console_output(options={})
      result = subject.status(options)
      action.when_rendering(:console).call(result, options)
    end

    def log_output(level, options={})
      logs = []
      Puppet::Util::Log.with_destination(Puppet::Test::LogCollector.new(logs)) do
        console_output(options)
      end
      if level
        logs.select {|l| l.level == level}
      else
        logs
      end
    end

    let(:puppetdb_service) { ServicesTestHelpers.gen_service_hash('puppetdb', 8081, 'puppetdb.example.vm', "PuppetDB")}
    let(:pe_master_service) { ServicesTestHelpers.gen_service_hash('pe_master', 8140, 'mono.example.vm', "Pe_master")}
    let(:orchestrator_service) { ServicesTestHelpers.gen_service_hash('orchestrator', 8143, 'mono.example.vm', "Orchestrator")}
    let(:classifier_service) { ServicesTestHelpers.gen_service_hash('classifier', 4433, 'mono.example.vm', "Classifier")}
    let(:rbac_service) { ServicesTestHelpers.gen_service_hash('rbac', 4433, 'mono.example.vm', "RBAC")}
    let(:activity_service) { ServicesTestHelpers.gen_service_hash('activity', 4433, 'mono.example.vm', "Activity")}
    let(:puppetdb_url) { status_url(puppetdb_service) }
    let(:pe_master_url) { status_url(pe_master_service) }
    let(:orchestrator_url) { status_url(orchestrator_service) }

    context 'flags' do
      context '--host' do
        before(:each) do
          stub_status_running(puppetdb_url, 'puppetdb')
        end

        it 'only checks requested host' do
          status = subject.status({:host => 'puppetdb.example.vm'})
          expect(status.count).to be(1)
        end

        it 'lists no services in error state' do
          opts = {:host => 'puppetdb.example.vm'}
          expect(console_output(opts)).to match(/1 of 1 services are fully operational/)
        end
      end

      context '--format json' do
        before(:each) do
          stub_status_running(puppetdb_url, 'puppetdb')
          stub_status_running(pe_master_url, 'pe_master')
          stub_status_running(orchestrator_url, 'orchestrator')
        end

        it 'renders a json parseable response' do
          opts = {:format => 'json'}
          output = console_output(opts)
          expect { JSON.parse(output) }.to_not raise_error
        end
      end

      context '--service' do
        before(:each) do
          stub_status_running(puppetdb_url, 'puppetdb')
        end

        it 'only checks requested service and is case insensitive' do
          status = subject.status({:service => 'PuppetDB'})
          expect(status.count).to be(1)
        end
      end
    end

    context 'service states' do
      context 'puppetdb in error state' do
        before(:each) do
          stub_status_error(puppetdb_url, 'puppetdb')
          stub_status_running(pe_master_url, 'pe_master')
          stub_status_running(orchestrator_url, 'orchestrator')
        end

        it 'parses the response properly' do
          status = subject.status
          expect(status[0][:state]).to eq(:error)
          expect(status[1][:state]).to eq(:running)
          expect(status[2][:state]).to eq(:running)
        end

        it 'lists 1 service in error state' do
          expect(console_output).to match(/2 of 3 services are fully operational./)
        end

        it 'marks puppetdb as error' do
          expect(console_output).to match(/PuppetDB: Error/)
        end

        it 'logs running services individual service status' do
          expect(console_output).to match(/Pe_master: Running/)
          expect(console_output).to match(/Orchestrator: Running/)
        end
      end

      context 'puppetdb is unreachable' do
        before(:each) do
          stub_status_unreachable(puppetdb_url)
          stub_status_running(pe_master_url, 'pe_master')
          stub_status_running(orchestrator_url, 'orchestrator')
        end

        it 'parses the response properly' do
          status = subject.status
          expect(status[0][:state]).to eq(:unreachable)
          expect(status[1][:state]).to eq(:running)
          expect(status[2][:state]).to eq(:running)
        end

        it 'lists 1 service in error state' do
          expect(console_output).to match(/2 of 3 services are fully operational./)
        end

        it 'marks puppetdb as unreachable' do
          expect(console_output).to match(/PuppetDB: Unreachable/)
        end
      end

      context 'puppetdb in running state' do
        before(:each) do
          stub_status_running(puppetdb_url, 'puppetdb')
          stub_status_running(pe_master_url, 'pe_master')
          stub_status_running(orchestrator_url, 'orchestrator')
        end

        it 'parses the response properly' do
          status = subject.status
          expect(status[0][:state]).to eq(:running)
          expect(status[1][:state]).to eq(:running)
          expect(status[2][:state]).to eq(:running)
        end

        it 'lists no services in error state' do
          expect(console_output).to match(/3 of 3 services are fully operational/)
        end
      end

      context 'with help example flag' do
        let(:action) { 'status' }

        include_examples 'help'
      end
    end
  end
end
