require 'spec_helper'
require 'puppet/util/pe_conf/recover'
require 'puppet/util/pe_node_groups'

describe Puppet::Util::Pe_conf::Recover do
  let(:subject) { described_class.new }

  describe '#pe_conf_from_puppetdb' do
    it 'returns a hash of parameters' do
      resources = [
        {
          'certname' => 'mono.example.vm',
          'parameters' => {
            'api_port' => '4433',
            'console_host' => 'mono.example.vm',
          }
        }
      ]
      expect(Puppet::Util::Pe_conf::Recover::Puppetdb).to receive(:query)
        .and_return(resources)

      expect(subject.pe_conf_from_puppetdb('production')).to eq({
        'puppet_enterprise::api_port' => '4433',
        'puppet_enterprise::console_host' => 'mono.example.vm'
      })
    end
    it 'does not fail when puppetdb returns an empty array' do
      resources = []

      expect(Puppet::Util::Pe_conf::Recover::Puppetdb).to receive(:query)
        .and_return(resources)

      expect(subject.pe_conf_from_puppetdb('production')).to eq({})
    end
  end

  describe '#get_core_infra_nodes' do
    it 'returns only 1 certname in a mono install' do
      pdb_response = [
        { 'certname' => 'mono.rspec' },
        { 'certname' => 'mono.rspec' }
      ]

      expect(Puppet::Util::Pe_conf::Recover::Puppetdb).to receive(:query)
        .and_return(pdb_response)
      expect(subject.get_core_infra_nodes('production')).to eq(['mono.rspec'])
    end

    it 'returns 3 certnames in a split install' do
      pdb_response = [
        { 'certname' => 'master.rspec' },
        { 'certname' => 'puppetdb.rspec' },
        { 'certname' => 'console.rspec' }
      ]

      expect(Puppet::Util::Pe_conf::Recover::Puppetdb).to receive(:query)
        .and_return(pdb_response)

      expect(subject.get_core_infra_nodes('production')).to eq(['master.rspec', 'puppetdb.rspec', 'console.rspec'])
    end
  end

  describe '#puppet_conf' do
    context 'parsing a good puppet.conf' do
      let(:conf) { subject.send(:puppet_conf, 'spec/fixtures/puppet.conf') }

      it 'returns a hash with 3 sections' do
        expect(conf.keys.size).to eq(3)
      end

      it 'has node_terminus set on the master run mode' do
        expect(conf[:master][:node_terminus]).to eq('classifier')
      end

      it 'has server set to localhost in the agent run mode' do
        expect(conf[:agent][:server]).to eq('localhost')
      end
    end

    context 'parsing a non-existent file' do
      let(:conf) { subject.send(:puppet_conf, 'spec/fixtures/puppet.conf.nope') }

      it 'returns the empty hash' do
        expect(conf).to be_empty
        expect(conf).to eq({})
      end
    end
  end

  describe '#find_hiera_overrides' do
    let(:layout) { 'pe-production-default' }
    let(:environment) { 'production' }

    context 'with classifier_data hiera layer' do
      let(:pe_production_hiera_yaml) do
        <<-EOS
---
# Hiera 5 Global configuration file

version: 5

# defaults:
#   data_hash: yaml_data
# hierarchy:
#  - name: Common
#    data_hash: yaml_data
hierarchy:
 - name: Classifier Configuration Data
   data_hash: classifier_data
        EOS
      end

      around(:each) do |example|
        customize_hiera(
          :fixture_directory => layout,
          :hiera_yaml        => pe_production_hiera_yaml,
          &example
        )
      end

      before(:each) do
        Puppet[:modulepath] = "#{fixtures_path}/modules"
        Puppet[:hiera_config] = "#{fixtures_path}/hiera/#{layout}/hiera.yaml"
      end

      it "finds data from the classifier's hiera using our scope (PE-22444)" do
        expect(
          subject.find_hiera_overrides(
            'master.rspec',
            ['node_name','scope_class'],
            {},
            environment,
            'plain'
          )
        ).to(
          match({
            'node_name'   => 'master.rspec',
            'scope_class' => 'Puppet::Parser::Scope',
          })
        )
      end
    end
  end
end
