require 'spec_helper'
require 'puppet/util/pe_conf'
require 'puppet/util/pe_conf/recover'

RSpec.shared_examples('(PE-23818) recovery when env has manifests') do
  let(:manifest_name) { 'site-bad.pp' }

  context 'that are unparseable' do
    let(:manifest_code) do
      <<-EOS
        bad syntax _$quirr3l_ divide by zero pc load letter
      EOS
    end


    it 'can call hiera without raising an error' do
      expect(subject.pe_conf['puppet_enterprise::puppet_master_host']).to eq(master_node)
    end
  end

  context 'that are unevaluatable' do
    let(:manifest_code) do
      <<-EOS
        include this_manifest::does_not_exis
      EOS
    end

    it 'can call hiera without raising an error' do
      expect(subject.pe_conf['puppet_enterprise::puppet_master_host']).to eq(master_node)
    end
  end
end

RSpec.shared_examples 'pe.conf node mapping' do
  include_examples '(PE-23818) recovery when env has manifests'

  it 'creates the right params to host mapping' do
    expect(subject.pe_conf['puppet_enterprise::puppet_master_host']).to eq(master_node)
    expect(subject.pe_conf['puppet_enterprise::console_host']).to eq(console_node)
    expect(subject.pe_conf['puppet_enterprise::puppetdb_host']).to eq(puppetdb_node)
    expect(subject.pe_conf['puppet_enterprise::database_host']).to eq(database_node)
  end
end

RSpec.shared_examples 'classifier data backend' do |hiera_version|
  it 'checks that puppet_enterprise_classifier_data_backend_present is recovered for the classifier hiera data layer (PE-24106)', :if => (hiera_version == 5) do
    expect(subject.pe_conf['puppet_enterprise_classifier_data_backend_present']).to eq(true)
  end
end

HIERA_VERSIONS = [3, 5]

describe Puppet::Util::Pe_conf do
  let(:subject) { described_class.new() }

  HIERA_VERSIONS.each do |hiera_version|
    describe "Using hiera global config version #{hiera_version}" do
      describe '#recover' do
        context 'topology' do
          context 'monolithic' do
            include_context('pe.conf recover context', hiera_version)

            context 'with a custom enc' do
              let(:using_nc) { false }

              include_examples 'pe.conf node mapping'

              context 'hiera' do
                it 'finds global overrides' do
                  expect(subject.pe_conf[
                             'puppet_enterprise::master::puppetserver::jruby_max_active_instances']).to eq(10001)
                end

                it 'finds node specific overrides' do
                  expect(subject.pe_conf['puppet_enterprise::manage_symlinks']).to eq(false)
                end
              end
            end

            context 'pe_repo config data being used' do
              let(:using_pe_repo_config_data) {true}

              it 'has pe_repo::compile_master_pool_address recovered into the pe.conf' do
                expect(subject.pe_conf['pe_repo::compile_master_pool_address']).to eq('test_server')
              end
            end

            context 'with the PE Node Classifier' do
              let(:using_nc) { true }

              include_examples 'pe.conf node mapping'
              include_examples('classifier data backend', hiera_version)

              context 'hiera' do
                it 'finds global overrides' do
                  expect(subject.pe_conf[
                             'puppet_enterprise::master::puppetserver::jruby_max_active_instances']).to eq(10001)
                end

                it 'finds node specific overrides' do
                  expect(subject.pe_conf['puppet_enterprise::manage_symlinks']).to eq(false)
                end
              end

              context 'classifier' do
                it 'finds global overrides' do
                  expect(subject.pe_conf['puppet_enterprise::send_analytics_data']).to eq(false)
                end
              end
            end

            context 'with external postgres' do
              let(:database_node) { 'database.rspec' }

              include_examples 'pe.conf node mapping'
            end
          end

          context 'split' do
            include_context('pe.conf recover context', hiera_version)

            let(:master_node) { 'master.rspec' }
            let(:console_node) { 'console.rspec' }
            let(:puppetdb_node) { ['puppetdb.rspec'] }
            let(:database_node) { 'puppetdb.rspec' }
            let(:scenario) { 'split' }

            context 'with a custom enc' do
              let(:using_nc) { false }

              include_examples 'pe.conf node mapping'

              context 'hiera' do
                it 'finds global overrides' do
                  expect(subject.pe_conf[
                             'puppet_enterprise::master::puppetserver::jruby_max_active_instances']).to eq(10001)
                end

                it 'finds node specific overrides' do
                  expect(subject.nodes_conf[master_node]['puppet_enterprise::manage_symlinks']).to eq(true)
                  expect(subject.nodes_conf[console_node]['puppet_enterprise::manage_symlinks']).to eq(false)
                  expect(subject.nodes_conf[puppetdb_node]['puppet_enterprise::manage_symlinks']).to eq(nil)
                end
              end
            end

            context 'with the PE Node Classifier' do
              let(:using_nc) { true }

              include_examples 'pe.conf node mapping'
              include_examples('classifier data backend', hiera_version)

              context 'hiera' do
                it 'finds global overrides' do
                  expect(subject.pe_conf[
                             'puppet_enterprise::master::puppetserver::jruby_max_active_instances']).to eq(10001)
                end

                it 'finds node specific overrides' do
                  expect(subject.nodes_conf[master_node]['puppet_enterprise::manage_symlinks']).to eq(true)
                  expect(subject.nodes_conf[console_node]['puppet_enterprise::manage_symlinks']).to eq(false)
                  expect(subject.nodes_conf[puppetdb_node]['puppet_enterprise::manage_symlinks']).to eq(nil)
                end
              end

              context 'classifier' do
                it 'finds global overrides' do
                  expect(subject.pe_conf['puppet_enterprise::send_analytics_data']).to eq(false)
                end
              end
            end
          end
        end
      end
    end
  end
end
