require 'spec_helper'
require 'puppet_x/util/stringformatter'

describe PuppetX::Util::String::Formatter do
  let(:subject) { described_class }

  context "#format" do
    let(:joined_input) { input.join("\n") }

    it { expect(subject.format('foo', 20, 0)).to eq('foo') }

    it { expect(subject.format("foo\nfoo", 20, 0)).to eq("foo\nfoo") }

    context "multiple lines" do
      let(:input) { [
        "this string is longer than the given width",
        "this is short",
        "this line is also longer than the given width"
      ] }

      context "with indentation=4" do
        let(:expected_output) { %Q(    this string is
    longer than the
    given width
    this is short
    this line is
    also longer than
    the given width)
        }
        it { expect(subject.format(joined_input, 20, 4)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 20, 4)).to eq(expected_output) }
      end

      context "with indentation=4 width=80" do
        let(:expected_output) { %Q(    this string is longer than the given width
    this is short
    this line is also longer than the given width)
        }
        it { expect(subject.format(joined_input, 80, 4)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 80, 4)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.indent(joined_input, 4)).to eq(expected_output) }
        it { expect(subject.join_and_indent(input, 4)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH and DEFAULT_INDENT
        it { expect(subject.indent(joined_input)).to eq(expected_output) }
        it { expect(subject.join_and_indent(input)).to eq(expected_output) }
      end

      context "with indentation=0" do
        let(:expected_output) { %Q(this string is
longer than the
given width
this is short
this line is also
longer than the
given width)
        }

        it { expect(subject.format(joined_input, 20, 0)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 20, 0)).to eq(expected_output) }

        it { expect(subject.wrap_no_indent(joined_input, 20)).to eq(expected_output) }
        it { expect(subject.join_and_wrap_no_indent(input, 20)).to eq(expected_output) }
      end

      context "with indentation=0 width=80" do
        let(:expected_output) { %Q(this string is longer than the given width
this is short
this line is also longer than the given width)
        }

        it { expect(subject.format(joined_input, 80, 0)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 80, 0)).to eq(expected_output) }

        # Relies on DEFAULT_WIDTH
        it { expect(subject.wrap_no_indent(joined_input)).to eq(expected_output) }
        it { expect(subject.wrap_no_indent(joined_input, 80)).to eq(expected_output) }
        it { expect(subject.join_and_wrap_no_indent(input, 80)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.join_and_indent(input, 0)).to eq(expected_output) }
      end
    end

    context "multiple lines with empty lines with no indentation" do
      let(:input) { ["this string is longer",
                    "",
                    "this is short",
                    "",
                    "this line is also longer"] }

      context "with indentation=0 width=80" do
        let(:expected_output) { %Q(this string is longer

this is short

this line is also longer)
        }
        it { expect(subject.format(joined_input, 80, 0)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 80, 0)).to eq(expected_output) }

        # Relies on DEFAULT_WIDTH
        it { expect(subject.wrap_no_indent(joined_input)).to eq(expected_output) }
        it { expect(subject.wrap_no_indent(joined_input, 80)).to eq(expected_output) }
        it { expect(subject.join_and_wrap_no_indent(input, 80)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.join_and_indent(input, 0)).to eq(expected_output) }
      end

      context "with indentation=0 width=20" do
        let(:expected_output) { %Q(this string is
longer

this is short

this line is also
longer)
        }
        it { expect(subject.format(joined_input, 20, 0)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 20, 0)).to eq(expected_output) }
        it { expect(subject.join_and_wrap_no_indent(input, 20)).to eq(expected_output) }
      end

      context "with indentation=0 width=80" do
        it { expect(subject.format(joined_input, 80, 0)).to eq(joined_input) }
        it { expect(subject.join_and_format(input, 80, 0)).to eq(joined_input) }
        it { expect(subject.join_and_wrap_no_indent(input, 80)).to eq(joined_input) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.join_and_indent(input, 0)).to eq(joined_input) }
      end
    end

    context "multiple JA lines with 4 indentation" do
      let(:input) { ["売くる日巨ドずうを議査ソ化間トキウチ権健コムス鋭森却ナヌ相82援7車ナ携義サヌニク要白ぶほ見約づぴきれ乱俣めほにぐ。",
                    "時もぽく兼26異よ同始別ネ台台ルナニレ減道なべ要個ひ気聞まよだづ木整メツ約助ッき台本がてど見害ぞをる演文移つ。"] }

      context "with indentation=4, width=20" do
        let(:expected_output) { %Q(    売くる日巨ドずうを議査ソ化間トキウチ権健コムス鋭森却ナヌ相82援7車ナ携義サヌニク要白ぶほ見約づぴきれ乱俣めほにぐ。
    時もぽく兼26異よ同始別ネ台台ルナニレ減道なべ要個ひ気聞まよだづ木整メツ約助ッき台本がてど見害ぞをる演文移つ。)
        }

        it { expect(subject.format(joined_input, 20, 4)).to eq(expected_output) }
        it { expect(subject.join_and_format(input, 20, 4)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.join_and_indent(input, 4)).to eq(expected_output) }
        # Relies on DEFAULT_WIDTH and DEFAULT_INDENT
        it { expect(subject.join_and_indent(input)).to eq(expected_output) }
      end

      context "with indentation=0, width=20" do
        it { expect(subject.format(joined_input, 20, 0)).to eq(joined_input) }
        it { expect(subject.join_and_format(input, 20, 0)).to eq(joined_input) }

        # Relies on DEFAULT_WIDTH
        it { expect(subject.wrap_no_indent(joined_input)).to eq(joined_input) }
        it { expect(subject.join_and_wrap_no_indent(input, 0)).to eq(joined_input) }
        # Relies on DEFAULT_WIDTH
        it { expect(subject.join_and_indent(input, 0)).to eq(joined_input) }
      end
    end
  end
end
