require 'spec_helper'
require 'puppet_x/puppetlabs/meep/configure/mco_status'
require 'puppet/util/puppetdb'
require 'benchmark'

describe 'PuppetX::Puppetlabs::Meep::Configure::McoStatus' do
  let(:mco) { PuppetX::Puppetlabs::Meep::Configure::McoStatus.new() }
  class Config
    def config
      {:server_url_timeout => 30}
    end
  end
  before(:each) do
    allow(Puppet::Util::Puppetdb).to receive(:config).and_return(Config.new)
  end

  context 'mco validation' do
    before(:each) do
      MockPuppetdb.mock(
        %w{resources type Class title Mcollective},
        mco_classes
      )
      mco.query_puppetdb
    end

    context 'when mco is purged' do
      let(:mco_classes) { [] }

      it { expect(mco.classes).to eq([]) }
      it { expect(mco.nodes).to eq([]) }
      it { expect(mco.infrastructure).to eq({}) }
      it { expect(mco.agents).to eq([]) }
    end

    context 'when mco is not purged' do
      let(:mco_classes) do
        [
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'certname' => 'compile.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Master::Mcollective',
            'certname' => 'compile.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'compile.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'certname' => 'replica.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'certname' => 'primary.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Master::Mcollective',
            'certname' => 'primary.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'primary.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Amq::Broker',
            'certname' => 'primary.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'agent1.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'agent2.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'agent3.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Master::Mcollective',
            'certname' => 'replica.rspec',
          },
          {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => 'replica.rspec',
          }
        ]
      end

      it "provides PQL a timestamp of form YYYY-MM-DD HH:MM:SS" do
        expect(mco.mco_query(Time.now)).to match(/catalog_timestamp > "\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}"/)
      end

      it do
        expect(mco.classes).to match_array(mco_classes)
      end

      it do
        expect(mco.nodes).to match_array([
          'agent1.rspec',
          'agent2.rspec',
          'agent3.rspec',
          'replica.rspec',
          'compile.rspec',
          'primary.rspec',
        ])
      end

      it do
        expect(mco.infrastructure).to match({
          'replica.rspec' => match_array([
            'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'Puppet_enterprise::Profile::Master::Mcollective',
            'Puppet_enterprise::Profile::Mcollective::Agent',
          ]),
          'compile.rspec' => match_array([
            'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'Puppet_enterprise::Profile::Master::Mcollective',
            'Puppet_enterprise::Profile::Mcollective::Agent',
          ]),
          'primary.rspec' => match_array([
            'Puppet_enterprise::Profile::Amq::Broker',
            'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'Puppet_enterprise::Profile::Master::Mcollective',
            'Puppet_enterprise::Profile::Mcollective::Agent',
          ]),
        })
      end

      it do
        expect(mco.agents).to match_array([
          'agent1.rspec',
          'agent2.rspec',
          'agent3.rspec',
        ])
      end
    end
  end

  context 'benchmarking' do
    before(:each) do
      MockPuppetdb.mock(
        %w{resources type Class title Mcollective},
        mco_classes
      )
    end

    context 'when there are thousands of agent nodes' do
      let(:mco_classes) do
        mco_classes = []
        100000.times do |i|
          mco_classes << {
            'title' => 'Puppet_enterprise::Profile::Mcollective::Agent',
            'certname' => "agent.#{i}.rspec",
          }
        end
        mco_classes
      end

      it do
        mco_classes
        time = Benchmark.realtime do
          mco.query_puppetdb
        end
        expect(mco.nodes.size).to eq(100000)
        expect(time).to be_within(0.5).of(0.5)
      end
    end
  end

  context 'when unable to query puppetdb' do
    before(:each) do
      expect(Puppet::Util::Puppetdb).to receive(:query_puppetdb).and_raise(RuntimeError, 'foo')
    end

    it do
      expect(mco.query_puppetdb).to eq([])
      expect(mco.error).to(
        be_kind_of(Puppet::Error).and(
          have_attributes(
            :message => /Unexpected exception/,
            :original => be_kind_of(RuntimeError),
          )
        )
      )
    end

    context 'debugging' do
      before(:each) do
        Puppet[:log_level] = 'debug'
      end

      it do
        expect(mco.query_puppetdb).to eq([])
        expect(@logs.map(&:message)).to include(/McoStatus.query.*:.*foo/)
      end
    end
  end
end
