require 'puppet_x/puppetlabs/meep/configure/validator'
require 'spec_helper'

RSpec::Matchers.define :include_message do |expected|
  match do |actual|
    @actual = actual.map { |m| m.message }
    @actual = @actual.first if @actual.size == 1

    result = case expected
    when String
      @actual.kind_of?(Array) ?
        @actual.any? { |m| m == expected } :
        @actual == expected
    when Regexp
      @actual.kind_of?(Array) ?
        @actual.any? { |m| m.match(expected) } :
        @actual.match(expected)
    else raise("Unknown type of expected: #{expected}")
    end

    result
  end

  diffable
end

RSpec::Matchers.define :include_log do |level,message|
  match do |actual|
    actual.any? do |l|
      message_matched = case message
      when String
        l.message == message
      when Regexp
        l.message =~ message
      end
      l.level == level && message_matched
    end
  end
end

RSpec.shared_context('postgresql state') do
  let(:megabyte) { 10**6 }
  let(:state) do
    {
      :postgres_migration => postgres_migration,
      :pe_postgresql_info => pe_postgresql_info,
      :previous_postgres_version => '9.4',
      :new_postgres_version => '9.6',
      :separate_database_node => separate_database_node,
      :database_host => database_host,
      :upgrade_from => upgrade_from,
      :external_database_version => external_database_version,
      :external_database_behind => external_database_behind,
    }
  end
  let(:pe_postgresql_info) do
    {
      'installed_server_version' => installed_server_version,
      'data_partition_available_bytes' => 50 * megabyte,
      'versions' => {
        installed_server_version => {
          'used_bytes' => used_bytes,
          'data_dir' => '/opt/puppetlabs/server/data/postgresql/9.4'
        }
      }
    }
  end
  let(:used_bytes) { 40 * megabyte }
  let(:postgres_migration) { false }
  let(:separate_database_node) { false }
  let(:external_database_version) { nil }
  let(:external_database_behind) { nil }
  let(:installed_server_version) { '9.6' }
  let(:database_host) { nil }
  let(:upgrade_from) { nil }
end

C = PuppetX::Puppetlabs::Meep::Configure

describe "PuppetX::Puppetlabs::Meep::Configure::Validator" do
  let(:state) { {} }
  let(:validator) { C::Validator.new(state) }

  class ARule < C::ValidationRule
    def valid?
      state[:foo] == :bar
    end

    def _messages
      [C::ValidationMessage.new(:fatal, ":foo == #{state[:foo]}")]
    end
  end

  context '#register' do
    let(:state) { { :foo => :bar } }

    it 'expects a class' do
      expect { validator.register(['foo']) }.to(
        raise_error(RuntimeError, "Expected a ValidationRule, got 'foo'")
      )
    end

    it 'expects a ValidationRule' do
      expect { validator.register([String]) }.to(
        raise_error(RuntimeError, "Expected a ValidationRule, got 'String'")
      )
    end

    it do
      validator.register([ARule])
      expect(validator.rules).to match_array(kind_of(C::ValidationRule))
    end

    it do
      validator.register([ARule])
      expect(validator.rules.first.state).to eq(state)
    end
  end

  context '#valid?' do
    it "is true by default if no validation is performed" do
      expect(validator).to be_valid
    end

    context 'with rules' do
      let(:state) { { :foo => :bar } }

      before(:each) do
        validator.register([ARule])
      end

      it { expect(validator).to be_valid }

      context 'when invalid' do
        let(:state) { { :foo => :invalid } }

        it { expect(validator).to_not be_valid }
      end
    end
  end

  context '#messages' do
    it "are empty if no validation is performed" do
      expect(validator.messages).to be_empty
    end

    context 'with rules' do
      let(:state) { { :foo => :bar } }

      before(:each) do
        validator.register([ARule])
      end

      it { expect(validator.messages).to be_empty }

      context 'when invalid' do
        let(:state) { { :foo => :invalid } }

        it do
          expect(validator.messages).to(
            include_message(":foo == invalid")
          )
        end
      end
    end
  end

  context '#log_and_fail!' do
    let(:messages) { [] }

    before(:each) do
      expect(validator).to receive(:messages).and_return(messages)
    end

    it { expect(validator.log_and_fail!).to be_nil }

    it 'logs messages' do
      messages << C::ValidationMessage.new(:warning, 'foo')
      expect(validator.log_and_fail!).to be_nil
      expect(@logs).to include_log(:warning, 'foo')
    end

    it 'raises an error for fatal messages' do
      messages << C::ValidationMessage.new(:fatal, 'oops')
      expect { validator.log_and_fail! }.to raise_exception(C::Validator::InstallationInvalid, /Failed to validate/)
      expect(@logs).to include_log(:err, 'oops')
    end

    it 'skips error for fatal if forced' do
      validator.state[:force] = true
      messages << C::ValidationMessage.new(:fatal, 'oops')
      expect(validator.log_and_fail!).to be_nil
      expect(@logs).to include_log(:err, 'oops')
    end

    it 'logs debug messages' do
      Puppet[:log_level] = 'debug'
      messages << C::ValidationMessage.new(:debug, 'debugging')
      expect(validator.log_and_fail!).to be_nil
      expect(@logs).to include_log(:debug, 'debugging')
    end
  end
end

describe 'ValidationRules' do
  include_context('postgresql state')

  context 'PostgresMigrationSpace' do
    let(:rule) { C::PostgresMigrationSpace.new(state) }

    it do
      r = C::PostgresMigrationSpace.new({})
      expect(r).to be_valid
      expect(r.messages).to be_empty
    end

    context 'from a 9.4 install' do
      let(:postgres_migration) { true }
      let(:installed_server_version) { '9.4' }

      context 'with enough space' do
        let(:used_bytes) { 40 * megabyte }

        it do
          expect(rule).to be_valid
          expect(rule.messages).to be_empty
        end
      end

      context 'without enough space' do
        let(:used_bytes) { 50 * megabyte }

        it do
          expect(rule).to_not be_valid
        end

        it "Generates the correct error message" do
          expect(rule.messages).to include_message(%r{Insufficient space})
        end
      end
    end

    context 'from a 9.6 install' do
      let(:postgres_migration) { false }
      let(:installed_server_version) { '9.6' }

      it 'never tests because postgres is already 9.6' do
        expect(rule).to be_valid
        expect(rule.messages).to be_empty
      end
    end
  end

  context 'ExternalPostgres' do
    let(:postgres_migration) { true }
    let(:separate_database_node) { true }
    let(:database_host) { 'database.rspec' }
    let(:installed_server_version) { '' }
    let(:pe_postgresql_info) do
      {
        'installed_server_version' => installed_server_version,
        'data_partition_size_bytes' => 0,
        'data_partition_available_bytes' => 0,
        'versions' => {}
      }
    end
    let(:rule) { C::ExternalPostgres.new(state) }

    it do
      r = C::ExternalPostgres.new({})
      expect(r).to be_valid
      expect(r.messages).to be_empty
    end

    context 'at version 9.4' do
      let(:external_database_version) { '9.4.12' }
      let(:external_database_behind) { true }
      let(:upgrade_from) { '2017.2.3' }

      it { expect(rule).to_not be_valid  }

      it 'generates an error message for being behind' do
        expect(rule.messages).to include_message(
"Your installation is managing the Postgresql database on a separate host (database.rspec).
We have detected that database.rspec is running Postgresql 9.4.12.
Puppet Enterprise requires Postgresql 9.6, please upgrade your database.rspec host's Postgresql instance before continuing with the rest of the upgrade of Puppet Enterprise."
        )
      end

      context 'without version data' do
        let(:external_database_version) { nil }
        let(:external_database_behind) { false }
        let(:upgrade_from) { '2017.2.3' }

        it 'generates an error message about being unable to check' do
          expect(rule.messages).to include_message(
"Your installation is managing the Postgresql database on a separate host (database.rspec).
We were unable to confirm the version of Postgresql on database.rspec.
Please review the logs for connectivity issues.
If you wish to force the upgrade to proceed, you may re-run the original puppet-infrastructure configure command with the --force flag, but Puppet Enterprise will experience issues if Postgresql is not the correct version.
Puppet Enterprise requires Postgresql 9.6, please upgrade your database.rspec host's Postgresql instance before continuing with the rest of the upgrade of Puppet Enterprise."
          )
        end
      end
    end

    context 'at version 9.6' do
      let(:external_database_version) { '9.6.3' }
      let(:external_database_behind) { false }
      it { expect(rule).to be_valid }
    end

    context 'at version 9.4, locally installed on a separate managed database.rpsec node' do
      let(:installed_server_version) { '9.4' }
      let(:external_database_version) { '9.4.12' }
      let(:external_database_behind) { true }
      it { expect(rule).to be_valid }
    end
  end

  context 'Parameter Validation' do
    let(:state) do
      {
        :jruby_9k_enabled => jruby_9k_enabled 
      }
    end

    let(:rule) { C::ParameterValidation.new(state) }

    context 'when JRuby 9k is enabled' do
      let(:jruby_9k_enabled) { true }

      it { expect(rule).to be_valid }
      it { expect(rule.messages).to be_empty }
    end

    context 'when JRuby 9k is disabled' do
      let(:jruby_9k_enabled) { false }
        
      it { expect(rule).to_not be_valid }
      it 'generates an error message about updating JRuby' do
          expect(rule.messages).to include_message(
"One or more currently defined parameters are invalid.
puppet_enterprise::master::puppetserver::jruby_9k_enabled is set to false. You can no longer use JRuby 1.7. Please update your PE install to use JRuby 9k."
          )
      end
    end

    context 'when JRuby 9k is nil' do
        let(:jruby_9k_enabled) { nil }

        it { expect(rule).to be_valid }
        it { expect(rule.messages).to be_empty }
    end
  end

  context 'McollectiveRemoval' do
    let(:state) do
      {
        :mco_all_nodes => mco_nodes,
        :mco_agent_nodes => mco_agents,
        :mco_infrastructure_nodes => mco_infrastructure,
        :mco_query_error => mco_error,
        :upgrade_from => upgrade_from,
      }
    end
    let(:mco_agents) { [] }
    let(:mco_infrastructure) { {} }
    let(:mco_nodes) { mco_agents + mco_infrastructure.keys }
    let(:mco_error) { nil }
    let(:upgrade_from) { '2018.1.0' }

    let(:rule) { C::McollectiveRemoval.new(state) }

    context 'when mco is gone' do
      it { expect(rule).to be_valid }
      it { expect(rule.messages).to be_empty }
    end

    context 'when mco is still present' do
      let(:mco_infrastructure) do
        {
          'master.node' => [
            'Puppet_enterprise::Profile::Amq::Broker',
            'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'Puppet_enterprise::Profile::Master::Mcollective',
            'Puppet_enterprise::Profile::Mcollective::Agent',
          ],
          'compile1.node' => [
            'Puppet_enterprise::Profile::Mcollective::Peadmin',
            'Puppet_enterprise::Profile::Master::Mcollective',
            'Puppet_enterprise::Profile::Mcollective::Agent',
          ],
        }
      end
      let(:mco_agents) do
        [
          'agent1.mco.node',
          'agent2.mco.node',
          'agent3.mco.node',
        ]
      end

      it { expect(rule).to_not be_valid }
      it 'includes a general warning' do
        expect(rule.messages).to include_message("MCollective and Activemq are no longer part of Puppet Enterprise.")
      end

      it 'logs the infrastructure' do
        expect(rule.messages).to include_message(
%q{The following 2 Puppet Enterprise server nodes had these MCollective or Activemq profiles applied:
  master.node: ["Puppet_enterprise::Profile::Amq::Broker", "Puppet_enterprise::Profile::Master::Mcollective", "Puppet_enterprise::Profile::Mcollective::Agent", "Puppet_enterprise::Profile::Mcollective::Peadmin"]
  compile1.node: ["Puppet_enterprise::Profile::Master::Mcollective", "Puppet_enterprise::Profile::Mcollective::Agent", "Puppet_enterprise::Profile::Mcollective::Peadmin"]}
        )
      end

      it 'logs the agents' do
        expect(rule.messages).to include_message(
"The following 3 agent nodes had the Puppet_enterprise::Profile::Mcollective::Agent class still applied:
  agent1.mco.node
  agent2.mco.node
  agent3.mco.node"
        )
      end

      context 'just agent nodes' do
        let(:mco_infrastructure) { {} }

        it 'only shows agent nodes message' do
          expect(rule.messages).to_not include_message(/The following .* Puppet Enterprise server nodes/)
          expect(rule.messages).to include_message(/The following 3 agent nodes/)
        end
      end


      context 'just infrastructure nodes' do
        let(:mco_agents) { [] }

        it 'only shows infra nodes message' do
          expect(rule.messages).to include_message(/The following 2 Puppet Enterprise server nodes/)
          expect(rule.messages).to_not include_message(/The following .* agent nodes/)
        end
      end

      context 'and node list is long' do
        let(:mco_agents) do
          agents = []
          999.times { |n| agents << "agent#{n}.mco.node" }
          agents
        end

        it do
          expect(rule.messages).to include_message(
"The following 999 agent nodes had the Puppet_enterprise::Profile::Mcollective::Agent class still applied:
  agent0.mco.node
  agent1.mco.node
  agent2.mco.node
  agent3.mco.node
  agent4.mco.node
  agent5.mco.node
  agent6.mco.node
  agent7.mco.node
  agent8.mco.node
  agent9.mco.node
  ...(truncated)
(To see the full list of nodes, run with --debug)"
          )
        end

        context 'debugging' do
          before(:each) do
            Puppet[:log_level] = 'debug'
          end

          it 'logs all agents...' do
            expect(rule.messages).to include_message(/Full agent list:.*agent998.mco.node/m)
          end
        end
      end
    end

    context "when mco can't be queried" do
      let(:mco_error) { Puppet::Error.new('puppetdbquery failed', RuntimeError.new('wrapped exception')) }

      it { expect(rule).to_not be_valid }
      it do
        expect(rule.messages).to include_message(
"But we were unable to query puppetdb to validate that these components are no longer being managed: puppetdbquery failed
Because: wrapped exception"
        )
      end
    end
  end
end
