require 'spec_helper'
require 'puppet/network/http_pool'
require 'puppet/util/colors'
require 'puppet_x/util/classification'
require 'hocon'
require 'set'
require 'testutil'

describe PuppetX::Util::Classification do
  let(:all_groups) { [{'name' => 'Test Group',
                       'id' => '42',
                       'classes' =>
                       {'test::class' =>
                        {'param' => 'val'}}}] }

  let(:group_name) { 'Test Group' }
  let(:empty_conf)  do
    { agent_list: [],
      infra_agent_list: [],
      pcp_broker_list: [],
      infra_pcp_broker_list: [],
      puppetdb_list: [],
      classifier_list: [],
      agent_port: 8140,
      infra_agent_port: 8140,
      pcp_broker_port: 8142,
      infra_pcp_broker_port: 8142,
      puppetdb_port: 8081,
      classifier_port: 4433,
    }
  end

  let(:full_conf) do
      list = ['puppet.master', 'puppet.replica']
      conf = {
        agent_port: 8140,
        infra_agent_port: 8140,
        pcp_broker_port: 8142,
        infra_pcp_broker_port: 8142,
        puppetdb_port: 8081,
        classifier_port: 4433,
      }
      conf[:agent_list] = list.map {|i| "#{i}:8140"}
      conf[:infra_agent_list] = list.map {|i| "#{i}:8140"}
      conf[:pcp_broker_list] = list.map {|i| "#{i}:8142"}
      conf[:infra_pcp_broker_list] = list.map {|i| "#{i}:8142"}
      conf[:puppetdb_list] = list.map {|i| "#{i}:8081"}
      conf[:classifier_list] = list.map {|i| "#{i}:4433"}
      conf
  end

  describe 'find_and_check_node_group' do
    let(:required_classes) { [] }
    subject { lambda { PuppetX::Util::Classification.find_and_check_node_group(all_groups, group_name, required_classes: required_classes) }}
    it { is_expected.not_to raise_error }

    context 'when group is missing' do
      let(:group_name) { 'Bogus Group' }
      it { is_expected.to raise_error(PuppetX::Util::NodeGroupNotFoundError) }
    end

    context 'when the required class' do
      context 'is present' do
        let(:required_classes) { ['test::class'] }
        it { is_expected.not_to raise_error }
      end

      context 'is missing' do
        let(:required_classes) { ['missing::class'] }
        it { is_expected.to raise_error(PuppetX::Util::MissingClassError) }
      end
    end
  end

  describe 'find_and_check_node_group_id' do
    subject { lambda { PuppetX::Util::Classification.find_and_check_node_group_id(all_groups, group_name) }}
    it { is_expected.not_to raise_error }

    context 'when the group is missing' do
      let(:group_name) { 'Bogus Group' }
      it { is_expected.to raise_error(PuppetX::Util::NodeGroupNotFoundError) }
    end
  end

  describe 'update_ha_enabled_replicas' do
    let(:nc) { double('nc') }
    before(:each) { allow(nc).to receive(:update_group) }

    let(:pe_infra_group) { {'name' => 'PE Infrastructure','id' => '512'} }
    let(:ha_enabled_replicas) { ['replica.dev'] }
    subject { lambda { PuppetX::Util::Classification.update_ha_enabled_replicas(nc, pe_infra_group, ha_enabled_replicas) }}
    it { is_expected.not_to raise_error }

    context 'when the group cannot be updated' do
      before(:each) { allow(nc).to receive(:update_group).and_raise(RuntimeError) }
      it { is_expected.to raise_error(PuppetX::Util::UpdateClassificationParametersError) }
    end

  end

  describe 'get_conf' do
    it 'prefers pcp_broker_ws_uri' do
      conf = PuppetX::Util::Classification.get_config(PuppetX::TestUtil.pe_all_groups)
      expect(conf[:infra_pcp_broker_list]).to eq(['mom.dev:8142','replica.dev:8142'])
      expect(conf[:pcp_broker_list]).to eq(['lb1.dev:8142','lb2.dev:8142'])
    end

    it 'falls back to pcp_broker_list' do
      groups = PuppetX::TestUtil.pe_all_groups
      agent_prof = groups.find { |g| g['name'] == "PE Agent" }["classes"]["puppet_enterprise::profile::agent"]
      agent_prof.delete("pcp_broker_ws_uris")
      infra_agent_prof = groups.find { |g| g['name'] == "PE Infrastructure Agent" }["classes"]["puppet_enterprise::profile::agent"]
      infra_agent_prof.delete("pcp_broker_ws_uris")
      conf = PuppetX::Util::Classification.get_config(groups)

      expect(conf[:infra_pcp_broker_list]).to eq(['listmom.dev:8142','listreplica.dev:8142'])
      expect(conf[:pcp_broker_list]).to eq(['listlb1.dev:8142','listlb2.dev:8142'])
    end

    it 'defaults to empty pcp_broker_lists' do
      groups = PuppetX::TestUtil.pe_all_groups
      agent_prof = groups.find { |g| g['name'] == "PE Agent" }["classes"]["puppet_enterprise::profile::agent"]
      agent_prof.delete("pcp_broker_ws_uris")
      agent_prof.delete("pcp_broker_list")
      infra_agent_prof = groups.find { |g| g['name'] == "PE Infrastructure Agent" }["classes"]["puppet_enterprise::profile::agent"]
      infra_agent_prof.delete("pcp_broker_ws_uris")
      infra_agent_prof.delete("pcp_broker_list")
      conf = PuppetX::Util::Classification.get_config(groups)

      expect(conf[:infra_pcp_broker_list]).to eq([])
      expect(conf[:pcp_broker_list]).to eq([])
    end

  end

  describe 'suggest_enable_config' do
    it 'creates master+replica for all suggestions' do
      master = 'puppet.master'
      rep    = 'puppet.replica'
      suggestions = PuppetX::Util::Classification.suggest_enable_config(empty_conf, master, rep, true)
      list = [master,rep]
      expect(suggestions[:agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:infra_agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:infra_pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:puppetdb_list]).to eq(list.map {|i| "#{i}:8081"})
      expect(suggestions[:classifier_list]).to eq(list.map {|i| "#{i}:4433"})
    end

    it 'wont add a replica twice' do
      master = 'puppet.master'
      rep    = 'puppet.replica'
      suggestions = PuppetX::Util::Classification.suggest_enable_config(full_conf, master, rep, true)
      list = [master,rep]
      expect(suggestions[:agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:infra_agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:infra_pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:puppetdb_list]).to eq(list.map {|i| "#{i}:8081"})
      expect(suggestions[:classifier_list]).to eq(list.map {|i| "#{i}:4433"})
    end

    it "doesn't suggest values for agent_list or pcp_broker_list with skip-agent-configure"
  end

  describe 'suggest_promote_config' do
    it 'removes master for all suggestions in mono' do
      master = 'puppet.master'
      rep    = 'puppet.replica'
      suggestions = PuppetX::Util::Classification.suggest_promote_config(full_conf, :a, master, rep, true)
      list = [rep]
      expect(suggestions[:agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:infra_agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:infra_pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:puppetdb_list]).to eq(list.map {|i| "#{i}:8081"})
      expect(suggestions[:classifier_list]).to eq(list.map {|i| "#{i}:4433"})
    end

    it 'doesnt add replica to agent_list with mono-with-compile' do
      master = 'puppet.master'
      rep    = 'puppet.replica'
      full_conf[:agent_list] = ['puppet.master:8140', 'puppet.compile:8140']
      suggestions = PuppetX::Util::Classification.suggest_promote_config(full_conf, :b, master, rep, true)
      list = [rep]
      expect(suggestions[:agent_list]).to eq(['puppet.compile:8140'])
      expect(suggestions[:infra_agent_list]).to eq(list.map {|i| "#{i}:8140"})
      expect(suggestions[:pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:infra_pcp_broker_list]).to eq(list.map {|i| "#{i}:8142"})
      expect(suggestions[:puppetdb_list]).to eq(list.map {|i| "#{i}:8081"})
      expect(suggestions[:classifier_list]).to eq(list.map {|i| "#{i}:4433"})
    end
  end

  describe "configure_ha_master_node_group_to_allow_replica" do
    it "does stuff"
  end

  describe "enable_replica" do
    it "does stuff"
  end

  describe 'remove_server' do
    let(:servers) { ['mom.dev', 'replica.dev', 'replica.dev:8080', 'replica.devo'] }

    it 'removes only the named server, even if it has a port' do
      output = PuppetX::Util::Classification.remove_server(servers, 'replica.dev')
      expect(output).to eq(['mom.dev', 'replica.devo'])
    end

    it 'changes nothing if the server is not in the list' do
      output = PuppetX::Util::Classification.remove_server(servers, 'bogus.co.uk')
      expect(output).to eq(servers)
    end
  end

  describe 'disable_replica' do
    let(:nc) { double('nc') }
    let(:pe_all_groups) { PuppetX::TestUtil.pe_all_groups }
    let(:master_certname) { 'mom.dev' }
    let(:replica_certname) { 'replica.dev' }
    let(:classification) { PuppetX::Util::Classification }
    subject { lambda { PuppetX::Util::Classification.disable_replica(nc, pe_all_groups, master_certname, replica_certname) } }

    context "doesn't raise an exception when given the right parameters" do
      before(:each) {
        allow(nc).to receive(:update_group).and_return(nil)
        expect(nc).to receive(:update_group).exactly(5).times
      }
      it { is_expected.not_to raise_error }
    end
  end

  describe 'find_primary_master_group_ids' do
    let(:nc) { double('nc') }
    let(:master_certname) { 'master.dev' }
    subject { lambda { PuppetX::Util::Classification.find_primary_master_group_ids(nc, all_groups, master_certname) }}

    context "when classification doesn't work" do
      before(:each) { allow(nc).to receive(:get_classification).and_raise(RuntimeError) }
      it { is_expected.to raise_error(PuppetX::Util::GetClassificationError)}
    end
  end

  describe 'update_pins_for_replica_during_promotion' do
    let(:nc) { double('nc') }
    let(:replica_certname) { 'replica.dev' }
    let(:primary_master_group_ids) { ['1'] }
    let(:groups_during_promotion) { [{'name' => 'PE HA Replica', 'id' => '1', 'classes' => {}}] }
    subject { lambda { PuppetX::Util::Classification.update_pins_for_replica_during_promotion(
                         nc, groups_during_promotion, replica_certname, primary_master_group_ids) }}

    context "when unpinning doesn't work" do
      before(:each) { allow(nc).to receive(:unpin_nodes_from_group).and_raise(RuntimeError) }
      it { is_expected.to raise_error(PuppetX::Util::UnpinNodeError)}
    end

    context "when unpinning works, but pinning doesn't" do
      before(:each) { allow(nc).to receive(:unpin_nodes_from_group) }
      before(:each) { allow(nc).to receive(:pin_nodes_to_group).and_raise(RuntimeError) }
      it { is_expected.to raise_error(PuppetX::Util::PinNodeError)}
    end
  end
end
