require 'spec_helper'
require 'puppet_x/util/classifier'

describe PuppetX::Util::Classifier do
  let(:groups) { [{"name" => "Puppet Master",
                   "id" => "1"},
                  {"name" => "Puppet Console",
                   "id" => "2"},
                  {"name" => "Puppet Database",
                   "id" => "3"}] }
  let(:ha_master_group) { {"name" => "PE HA Master",
                           "id" => "4"} }
  let(:ha_replica_group) { {"name" => "PE HA Replica",
                            "id" => "5"} }

  let(:nc) { instance_double('nc') }

  describe "get_groups" do
    context "request error" do
      before(:each) { allow(nc).to receive(:get_groups).and_throw(RuntimeError) }

      it "raises the expected error" do
        expect {
          described_class.get_groups(nc)
        }.to raise_error(PuppetX::Util::GetNodeGroupsError)
      end
    end
  end

  describe "pin_node_to_group" do
    context "request error" do
      before(:each) { allow(nc).to receive(:pin_nodes_to_group).and_throw(RuntimeError) }

      it "raises the expected error" do
        expect {
          described_class.pin_node_to_group(nc, 'some group', 42, 'certname')
        }.to raise_error(PuppetX::Util::PinNodeError)
      end
    end
  end

  describe "find_group" do
    context "given a list of groups" do
      it "returns one with a given name" do
        result = described_class.find_group(groups, "Puppet Console")
        expect(result).to include("id" => "2", "name" => "Puppet Console")
      end
    end
  end

  describe "find_group_id" do
    context "given a list of groups" do
      it "returns the ID of a group with a given name" do
        result = described_class.find_group_id(groups, "Puppet Console")
        expect(result).to eq("2")
      end
      it "returns nil if a group with the given name does not exist" do
        result = described_class.find_group(groups, "Blargh")
        expect(result).to eq(nil)
      end
    end
  end

  describe "create_ha_master_node_group" do
    context "if the group does not exist" do
      context "creates a group that" do
        it "has no rule, has the correct classes and params, has the correct name, has PE Infrastructure for a parent" do
          # TODO the qualities of the group are too hard to write as an assertion. should make as a helper method that returns the expected group.
          expect(nc).to receive(:create_group)
          described_class.create_ha_master_node_group(nc, groups, "1")
        end
      end

      context "and NC request fails" do
        before(:each) { allow(nc).to receive(:create_group).and_throw(RuntimeError) }

        it "raises the expected error" do
          expect {
            PuppetX::Util::Classifier.create_ha_master_node_group(nc, {}, 42)
          }.to raise_error(PuppetX::Util::CreateNodeGroupError)
        end
      end
    end

    context "if the group does exist" do
      it "does not make changes" do
        groups << ha_master_group
        expect(nc).not_to receive(:create_group)
        described_class.create_ha_master_node_group(nc, groups, "1")
      end
    end
  end

  describe "create_ha_replica_node_group" do
    context "if the group does not exist" do
      context "and NC request fails" do
        before(:each) { allow(nc).to receive(:create_group).and_throw(RuntimeError) }

        it "raises the expected error" do
          expect {
            PuppetX::Util::Classifier.create_ha_replica_node_group(nc, {}, 42)
          }.to raise_error(PuppetX::Util::CreateNodeGroupError)
        end
      end
      context "constructs a group that" do
        # TODO fix these assertions like above
        it "has no rule"
        it "has the correct classes and params"
        it "has the correct name"
        it "has PE Infrastructure for a parent"
      end
      it "sends the group to be created" do
        expect(nc).to receive(:create_group)
        described_class.create_ha_master_node_group(nc, groups, "1")
      end
    end

    context "if the group does exist" do
      it "does not make changes" do
        groups << ha_replica_group
        expect(nc).not_to receive(:create_group)
        described_class.create_ha_replica_node_group(nc, groups, "1")
      end
    end
  end
end
