require 'spec_helper'
require 'puppet_x/util/orchestrator'
require 'puppet/network/http_pool'

describe PuppetX::Util::Orchestrator do
  let(:svc) { {:server => "puppetmaster.net",
               :port => 49281,
               :url => "puppetmaster.net:49281"} }
  let(:token) { "I'm a token!" }

  context "orch_connection" do
    it "takes a hash with :server and :port and returns a connection object" do
      result = described_class.orch_connection(svc)
      expect(result).to be_instance_of(Puppet::Network::HTTP::Connection)
      expect(result.address).to eq(svc[:server])
      expect(result.port).to eq(svc[:port])
    end
  end

  context "orch_request_headers" do
    it "returns a hash of headers" do
      result = described_class.orch_request_headers(token)
      expect(result).to include("Content-Type" => "application/json", "X-Authentication" => token)
    end
  end

  context "check_orch_response" do

    let(:expected) { 200 }
    let(:response) { double("response") }

    context "if response code does not match expected" do
      it "returns an error" do
        allow(response).to receive(:code).and_return("400")
        allow(response).to receive(:body).and_return("Bad thing!")
        expect { described_class.check_orch_response(response, expected)}.to raise_error(PuppetX::Util::OrchestratorResponseError, /Bad thing/)
      end
    end

    context "if response code matches expected" do
      context "and body is not empty" do
        it "returns nil" do
          allow(response).to receive(:code).and_return("200")
          allow(response).to receive(:body).and_return("Everything's good")
          result = described_class.check_orch_response(response, expected)
          expect(result).to eq(nil)
        end
      end

      context "and body is empty" do
        it "returns an error" do
          allow(response).to receive(:code).and_return("200")
          allow(response).to receive(:body).and_return(nil)
          expect { described_class.check_orch_response(response, expected)}.to raise_error(PuppetX::Util::OrchestratorEmptyResponseError)
        end
      end
    end
  end

  context "start_puppet_run_with_orch" do
    # TODO What is even a valid scope
    let(:scope) { "Valid?" }
    let(:request_url) { "#{svc[:url]}/v1/command/deploy" }
    let(:payload) { {:scope => scope, :environment => '', :enforce_environment => false}.to_json }
    let(:body) { {'id' => 200,
                  'url' => "http://test/job",
                  'job' => {'name' => 42, 'id' => 'really_a_guid'}}.to_json }
    let(:response_object) { double("response with code", :code => "202", :body => body) }
    let(:connection) { double("connection", :post => response_object) }

    it "makes a request to the orchestrator" do
      expect(described_class).to receive(:orch_connection).and_return(connection)
      expect(connection).to receive(:post).with(request_url, payload, anything)
      described_class.start_puppet_run_with_orch(svc, scope, token)
    end

    it "handles error responses" do
    end

    it "returns the job  id and url" do
    end

    context "if allow_empty is true" do
      it "and response is 400 with a body with kind empty-target, returns :empty" do
      end

      it "and response is not 400 or body is empty or not with kind empty-target, proceeds as normal" do
      end
    end
  end

  context "get_orch_job_status" do
    it "makes a request to the orchestator"
    it "handles error responses"
    it "returns the response with parsed body"
  end

  context "get_finished_job_details" do
    it "makes a request to the orchestrator"
    it "handles error responses"
    it "returns a message and report url in a hash"
  end

  context "wait_for_orch_job" do
    it "logs that puppet is running"
    context "timeout loop" do
      it "times out after 5 minutes"
      it "requests job status"
      it "handles error responses"
      it "terminates once a final status is reported"
    end
  end

  context "run_puppet" do
    it "starts a puppet run"
    it "logs if there are no nodes to run on"
    it "waits for the job to complete"
  end
end
