require 'spec_helper'
require 'puppet_x/util/password'
require 'webmock/rspec'
require 'puppet'

describe PuppetX::Util::PasswordUtils do
  let(:get_user_response) { '[{"login":"admin","id":"ID-for-admin"},{"login":"anotheruser","id":"ID-for-other-user"}]' }
  let(:user_expected) { {"login"=>"admin", "id"=>"ID-for-admin"} }
  let(:admin_user_hash) { {"id" => "ID-for-admin", "login" => "admin", "email" => "admin@example.com", "display_name" => "admin", "role_ids" => [1, 2, 3], "is_group" => false, "is_remote" => false, "is_superuser" => false, "is_revoked" => true, "last_login" => "2014-05-04T02:32:00Z"} }
  let(:https) { 
    https = Net::HTTP.new("mono.example.vm",4433,nil)
    https.use_ssl = true
    https
  }
  before(:each) do
    WebMock.disable_net_connect!
    allow(described_class).to receive(:build_auth).and_return(https)
    stub_request(:get, "https://mono.example.vm:4433/rbac-api/v1/users").with(headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 200, body: get_user_response, headers: {})
    stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/users/ID-for-admin/password/reset") \
        .with(headers: {"Content-Type" => "application/json"}).to_return(status: 201, body: "token", headers: {})
    stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/auth/reset") \
        .with(body: '{"token":"token","password":"newpassword"}', headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 200, body: "OK", headers: {})
    stub_request(:put, "https://mono.example.vm:4433/rbac-api/v1/users/ID-for-admin") \
        .with(body: '{"login":"admin","id":"ID-for-admin","is_revoked":true}', headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 200, body: "OK", headers: {})
  end

  after(:each) do
    WebMock.allow_net_connect!
  end

  context "get_user" do
    it "gets the admin user" do
      expect(described_class.get_user("admin")).to eq(user_expected)
    end

    it "gets a user that doesn't exist" do
      expect(described_class.get_user("neo")).to be_nil
    end
  end

  context "reset_password" do
    it "resets password passing in hash for user" do
      expect(described_class.reset_password(user_expected,"newpassword").code).to eq('200')
    end

    it "reset password passing in string for user" do
      expect(described_class.reset_password("admin","newpassword").code).to eq('200')
    end

    it "fails when trying to reset password on nonexistent user" do
      expect{described_class.reset_password("neo","thereisnospoon")}.to raise_error(RuntimeError, "User neo does not exist")
    end

    it "fails when passing in a user hash with invalid ID" do
      stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/users/theone/password/reset") \
        .with(headers: {"Content-Type" => "application/json"}).to_return(status: 404, body: "", headers: {})
      expect { described_class.reset_password({"login" => "neo", "id" => "theone" },"thereisnospoon") }.to raise_error(RuntimeError)
    end

    it "fails when receiving a non-200 response from the server" do
      stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/auth/reset") \
        .with(body: '{"token":"token","password":"newpassword"}', headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 502, body: "Bad Gateway", headers: {})
      expect { described_class.reset_password(user_expected, "newpassword") }.to raise_error(RuntimeError)
    end
  end

  context "revoke_admin_access" do
    it "revokes admin access correctly" do
      expect(described_class.revoke_admin_access.code).to eq('200')
    end

    it "checks for default password and correctly revokes admin access" do
      stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/auth/token") \
        .with(body: '{"login":"admin","password":"admin"}', headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 200, body: "", headers: {})
      expect(described_class.check_for_default_password_and_revoke).to eq(true)
    end

    it "checks for default password and correctly does not revoke admin access" do
      stub_request(:post, "https://mono.example.vm:4433/rbac-api/v1/auth/token") \
        .with(body: '{"login":"admin","password":"admin"}', headers: {"Content-Type" => "application/json"}) \
        .to_return(status: 401, body: "", headers: {})
      expect(described_class.check_for_default_password_and_revoke).to eq(false)
    end
  end
end
