require 'spec_helper'
require 'puppet_x/util/service_status'
require 'puppet/network/http_pool'
require 'puppet/util/colors'
require 'hocon'
require 'json'

describe PuppetX::Util::ServiceStatus do
  let(:orch_svc) { {:type => "orchestrator",
                    :display_name => "Orchestrator",
                    :server => "puppetmaster.net",
                    :url => "puppetmaster.net:41982/orch",
                    :status_key => :orchestrator} }
  let(:status_response) { {:orchestrator => {:state => "running",
                                             :alerts => []}}.to_json }
  let(:status_url) { "http://foo.net:5902/status-prefix" }

  ### Helpers
  context 'load_config' do
    it 'raises error when config file doesn\'t exist' do
      expect do
        PuppetX::Util::ServiceStatus.load_config(
          '/blah/foo1232421/services.conf'
        )
      end.to raise_error(%r{No configuration file found at /blah/foo})
    end
  end

  context "config_path" do
    it "loads expected config path" do
      expect(described_class.config_path).to eq("/etc/puppetlabs/client-tools/services.conf")
    end
  end

  context "pe_version" do
    it "returns latest if not defined" do
      expect(Facter).to receive(:value).and_return(nil)
      result = described_class.pe_version
      expect(result).to eq("latest")
    end

    it "gets value from facter and returns x.y version" do
      expect(Facter).to receive(:value).and_return("10.11.12.13")
      result = described_class.pe_version
      expect(result).to eq("10.11")
    end
  end

  context "required_services" do
    it "returns a hash with expected keys and values" do
      result = described_class.required_services
      expect(result).to include(:classifier, :master, :orchestrator, :pcp_broker, :code_manager, :file_sync_storage, :file_sync_client)
    end
  end

  context "symbolize_keys" do
    it "does not change individual elements" do
      result = described_class.symbolize_keys("foo")
      expect(result).to eq("foo")
    end

    it "raises errors for non-symbolizable types" do
      result = described_class.symbolize_keys(1)
      expect(result).to be(1)
    end

    it "turns keys into a hash into symbols" do
      hash = {:foo => :a,
              "bar" => "b",
              "baz" => "c"}
      result = described_class.symbolize_keys(hash)
      expect(result).to include(:foo => :a, :bar => "b", :baz => "c")
      expect(result).not_to include("bar", "baz")
    end

    it "works on nested data structures" do
      array = ["foo", {"bar" => {"baz" => "quux"}}]
      result = described_class.symbolize_keys(array)
      expect(result).to eq(["foo", {:bar => {:baz => "quux"}}])
    end
  end

  ### Status Querying

  context "unreachable_status_for_service" do
    it "represents a service object as unreachable" do
      result = described_class.unreachable_status_for_service(orch_svc)
      expect(result).to include(:service => "orchestrator", :state => :unreachable, :alerts => [])
    end
  end

  context "try_get_status" do
    timeout = 5
    it "calls the status API as expected" do
      conn_double = double("connection")
      response_double = double("I'm a response")
      expect(Puppet::Network::HttpPool).to receive(:http_instance).and_return(conn_double)
      expect(conn_double).to receive(:get).with("#{status_url}/v1/services?timeout=#{timeout}").and_return(response_double)
      allow(response_double).to receive(:body).and_return(status_response)
      # is expected_service_configs a hash of hashes or list of hashes?
      described_class.try_get_status(status_url, [orch_svc], timeout)
    end

    it "returns an array of status hashes"

    it "marks unreachable services as unreachable"
  end

  context "errors:" do
    context "when non-JSON response is returned" do
      it "logs an error"
      it "marks service as unreachable"
    end
  end

  context "services_for" do
    context "if a given service on a given host from the list of services" do
      context "is there" do
        it "return it"
      end

      context "is not there" do
        it "???"
      end
    end
  end

  context "node_role" do
    context "if the node is present in services.conf" do
      let(:nodes_config) { [{:certname => 'test_node', :role => 'great_server'}]}
      it "returns the role" do
        expect(described_class.node_role(nodes_config, 'test_node')).to eq('great_server')
      end
    end

    context "if the node is not present in services.conf" do
      let(:nodes_config) { [] }
      it "returns nil" do
        expect(described_class.node_role(nodes_config, 'test_node')).to be_nil
      end
    end
  end

  context "load_services_config" do
    it "loads the file and symbolizes the keys"

    context "with path override" do
      it "uses the given path"
    end
  end

  context "validate_and_select_services_from_config" do
    context "expected behavior:" do
      it "returns a ??? of services for the given certname by filtering the config data"
    end
    context "errors:" do
      context "required services not in file" do
        it "raises an error"
        # current error handling relies on wrong assumptions about what services.conf will always look like or what is being done. Fix.
      end
    end
  end

  context "ensure_all_services_running" do
    context "given a list of service statuses" do
      context "if all are running" do
        it "returns true"
      end
      context "if any are not running" do
        it "raises error with message explaining what is not running"
      end
    end
  end

  ### Status formatting

  context "service_has_changing_state" do
  end
  context "service_has_error_state" do
  end
  context "service_has_important_alert" do
  end
  context "format_summary_message" do
  end
  context "format_status" do
  end
end
