require 'spec_helper'

describe 'puppet_enterprise::master::code_manager' do
  let(:facts) do
    {
      'clientcert'        => 'awesomecert',
    }
  end
  let(:params)  { {} }
  let(:config_dir) { "/etc/puppetlabs/puppetserver" }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetserver': }
      package { 'pe-puppetserver': }
    PRE_COND
  end

  context 'no control repo configuration provided' do
    it { should contain_pe_hocon_setting('code-manager.sources').with_ensure('absent') }
  end

  context 'with a proxy' do
    let(:proxy) { "http://user:password@proxy.example.com:3128" }
    before(:each) { params['proxy'] = proxy }

    it { should contain_pe_hocon_setting('code-manager.proxy').with_ensure('present') }
    it { should contain_pe_hocon_setting('code-manager.proxy').with_value(proxy) }
  end

  context 'without forge_settings' do
    it { should contain_pe_hocon_setting('code-manager.forge').with_ensure('absent') }
    it { should contain_pe_hocon_setting('code-manager.forge-settings').with_ensure('absent') }
  end

  context 'with forge_settings' do
    before(:each) { params['forge_settings'] = {'proxy' => 'https://forge.proxy'} }
    it { should contain_pe_hocon_setting('code-manager.forge').with_ensure('present') }
    it { should contain_pe_hocon_setting('code-manager.forge').with_value({'proxy' => 'https://forge.proxy'}) }
    it { should contain_pe_hocon_setting('code-manager.forge-settings').with_ensure('absent') }
  end

  context 'timeouts' do
    context 'defaults' do
      it { should contain_pe_hocon_setting('code-manager.timeouts.fetch').with_value(30) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.deploy').with_value(600) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.hook').with_value(30) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.shutdown').with_value(610) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.wait').with_value(700) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.sync').with_value(60) }
    end

    context 'overrides' do
      before(:each) {
        params['timeouts_fetch'] = 5
        params['timeouts_deploy'] = 5
        params['timeouts_hook'] = 5
        params['timeouts_shutdown'] = 5
        params['timeouts_wait'] = 5
        params['timeouts_sync'] = 5
      }
      it { should contain_pe_hocon_setting('code-manager.timeouts.fetch').with_value(5) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.deploy').with_value(5) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.hook').with_value(5) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.shutdown').with_value(5) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.wait').with_value(5) }
      it { should contain_pe_hocon_setting('code-manager.timeouts.sync').with_value(5) }
    end
  end

  context 'with a remote repo information provided' do
    before :each do
      params['remote'] = 'git@github.com:puppetlabs/puppetlabs-modules'
    end

    it do
      sources = { 'puppet' => { 'remote' => 'git@github.com:puppetlabs/puppetlabs-modules'}}
      should contain_pe_hocon_setting('code-manager.sources').with_value(sources)
    end
  end

  context 'with sources provided' do
    before :each do
      params['sources'] = { 'puppet' => {} }
    end

    it do
      should contain_pe_hocon_setting('code-manager.sources').with_value(params['sources'])
    end
  end

  context 'without invalid-branches' do
    it { should contain_pe_hocon_setting('code-manager.invalid-branches').with_ensure('absent') }
  end

  context 'with invalid-branches provided' do
    before :each do
      params['invalid_branches'] = 'correct'
    end

    it do
      should contain_pe_hocon_setting('code-manager.invalid-branches').with_ensure('present')
      should contain_pe_hocon_setting('code-manager.invalid-branches').with_value('correct')
    end
  end

  context 'with file_sync_auto_commit' do
    it { should contain_pe_hocon_setting('code-manager.environmentdir').with_value('/etc/puppetlabs/code-staging/environments') }
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('code-manager-v1') }
  end

  context 'Without a private_key' do
    it do
      should contain_pe_hocon_setting('code-manager.git').with_ensure('absent')
    end

    context 'With a git_settings_hash' do
      before(:each) { params['git_settings'] = {"proxy" => "https://example.com"} }

      it do
        should contain_pe_hocon_setting('code-manager.git').with_value(params['git_settings'])
      end
    end
  end

  context 'with a private key provided' do
    before(:each) { params['private_key'] = '/etc/puppetlabs/code-manager/id_rsa' }

    it do
      should contain_pe_hocon_setting('code-manager.private-key').with_ensure('absent')
      should contain_pe_hocon_setting('code-manager.git').with_value({"private-key" => params['private_key']})
    end

    context "with private_key and git_settings" do
      before(:each) { params['git_settings'] = {"proxy" => "https://example.com"} }

      it do
        expect {should compile}.to raise_error(/Both private_key and git/)
      end
    end
  end

  context 'webserver' do
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("code-manager")
      .with('container'          => 'puppetserver',
            'certname'           => 'awesomecert',
            'ssl_listen_address' => '0.0.0.0',
            'ssl_listen_port'    => 8170,
            'localcacert'        => '/etc/puppetlabs/puppet/ssl/certs/ca.pem',
            'hostcrl'            => '/etc/puppetlabs/puppet/ssl/crl.pem',
            'access_log_config'  => "#{config_dir}/code-manager-request-logging.xml",
            'ssl_protocols'      => ['TLSv1.2']) }
  end

  context 'purge whitelists' do
    context 'the default whitelist' do
      it { should contain_pe_hocon_setting('code-manager.purge-whitelist').
                    with_value(['.resource_types']) }
    end

    context 'can be specified via hiera' do
      before(:each) { params['purge_whitelist'] = ['vendor', '.resource_types'] }

      it { should contain_pe_hocon_setting('code-manager.purge-whitelist').
                    with_value(['vendor', '.resource_types']) }
    end
  end
end
