require 'spec_helper'

describe 'puppet_enterprise::master::file_sync_disabled' do
  let(:config_dir) { "/etc/puppetlabs/puppetserver" }
  let(:params) do
    {
      'code_id_command'      => '/usr/bin/justin-is-amazing.sh',
      'code_content_command' => '/usr/bin/justin-loves-this-idea.sh',
    }
  end

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetserver': }
      package { 'pe-puppetserver': }
    PRE_COND
  end

  context "remove the file-sync.conf file" do
    it { should contain_file("#{config_dir}/conf.d/file-sync.conf").with_ensure('absent')}
  end

  context "manages the versioned-code.conf file and settings" do
    it { should contain_file("#{config_dir}/conf.d/versioned-code.conf").with_ensure('present')}
    it { should contain_pe_hocon_setting('versioned-code.code-id-command').with_value(params['code_id_command'])}
    it { should contain_pe_hocon_setting('versioned-code.code-content-command').with_value(params['code_content_command'])}
  end

  context "explodes with only one set" do
    let(:params) do
      { 'code_id_command' => '/usr/bin/justin-is-amazing.sh' }
    end
    it { should_not contain_file("#{config_dir}/conf.d/versioned-code.conf"), "The catalogue DOES contain the file. HOW CAN THIS BE?"}
    it { should_not contain_pe_hocon_setting('versioned-code.code-id-command'), "The catalogue DOES contain the setting. HOW CAN THIS BE?"}
    it { should_not contain_pe_hocon_setting('versioned-code.code-content-command'), "The catalogue DOES contain the setting. HOW CAN THIS BE?"}
    it { should contain_notify('missing-code-command-configuration') }
  end

  context "does nothing if neither is set" do
    let(:params) { {} }
    it { should contain_file("#{config_dir}/conf.d/versioned-code.conf").with_ensure('absent'), "The catalogue DOES contain the file. HOW CAN THIS BE?"}
    it { should_not contain_pe_hocon_setting('versioned-code.code-id-command'), "The catalogue DOES contain the setting. HOW CAN THIS BE?"}
    it { should_not contain_pe_hocon_setting('versioned-code.code-content-command'), "The catalogue DOES contain the setting. HOW CAN THIS BE?"}
    it { should_not contain_notify('missing-code-command-configuration') }
  end

  context "add the versioned code service to the bootstrap.cfg" do
    # Make sure we aren't adding the disabled service.
    it { should_not contain_pe_concat__fragment('puppetserver file-sync-versioned-code-disabled-service') }

    it { should contain_pe_concat__fragment('puppetserver versioned-code-service') }
  end

  context "remove the authorization rules" do
    it { should contain_pe_puppet_authorization__rule('puppetlabs file sync api')
      .with_path('/etc/puppetlabs/puppetserver/conf.d/auth.conf')
      .with_notify('Service[pe-puppetserver]') }
    it { should contain_pe_puppet_authorization__rule('puppetlabs file sync repo')
      .with_path('/etc/puppetlabs/puppetserver/conf.d/auth.conf')
      .with_notify('Service[pe-puppetserver]') }
  end
end
