require 'spec_helper'

describe 'puppet_enterprise::master::puppetserver' do
  let(:facts) { {} }
  let(:params)   do
    {
      'certname'                                  => 'master.rspec',
      'metrics_server_id'                         => 'localhost',
      'metrics_jmx_enabled'                       => true,
      'metrics_graphite_enabled'                  => false,
      'metrics_puppetserver_metrics_allowed'      => ["foo", "bar"],
      'profiler_enabled'                          => true,
      'puppetserver_jruby_puppet_master_code_dir' => '/etc/puppetlabs/code'
    }
  end

  let(:config_dir)  { "/etc/puppetlabs/puppetserver" }
  let(:authconf) { "#{config_dir}/conf.d/auth.conf" }

  context 'puppetserver status logging with default params' do
    it { should contain_pe_hocon_setting('status.debug-logging').with_ensure('absent') }
  end

  context 'puppetserver status logging enabled with default interval' do
    before :each do
      params['status_logging_enabled'] = true
    end
    it { should contain_pe_hocon_setting('status.debug-logging.interval-minutes').with_ensure('present') }
    it { should contain_pe_hocon_setting('status.debug-logging.interval-minutes').with_value(30) }
  end

  context 'puppetserver status logging with custom logging interval' do
    before :each do
      params['status_logging_enabled'] = true
      params['status_logging_interval_minutes'] = 314
    end
    it { should contain_pe_hocon_setting('status.debug-logging.interval-minutes').with_ensure('present') }
    it { should contain_pe_hocon_setting('status.debug-logging.interval-minutes').with_value(314) }
  end

  context 'puppet-admin certs' do
    before(:each) do
      facts['clientcert'] = 'awesomecert'
    end

    context 'deprecated list removed' do
      it { should contain_pe_hocon_setting('puppet-admin').with_ensure('absent') }
    end

    context 'default list' do
      it { should contain_pe_puppet_authorization__rule('puppetlabs environment cache')
        .with_match_request_path('/puppet-admin-api/v1/environment-cache')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow(['awesomecert'])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
      it { should contain_pe_puppet_authorization__rule('puppetlabs jruby pool')
        .with_match_request_path('/puppet-admin-api/v1/jruby-pool')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow(['awesomecert'])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
     it { should contain_pe_hocon_setting('jruby-puppet.compile-mode').with(
         'value' => 'jit',
         'path' => '/etc/puppetlabs/puppetserver/conf.d/pe-puppet-server.conf',
         'setting' => 'jruby-puppet.compile-mode',
         'ensure' => 'present',
     )}
    end

    context 'using custom certs' do
      before :each do
        params['puppet_admin_certs'] = ['foo', 'bar']
      end

      it { should contain_pe_puppet_authorization__rule('puppetlabs environment cache')
        .with_match_request_path('/puppet-admin-api/v1/environment-cache')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow(['awesomecert', 'foo', 'bar'])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
      it { should contain_pe_puppet_authorization__rule('puppetlabs jruby pool')
        .with_match_request_path('/puppet-admin-api/v1/jruby-pool')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow(['awesomecert', 'foo', 'bar'])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
    end

    context 'using custom base_puppet_admin_certs' do
      before :each do
        params['puppet_admin_certs'] = []
        params['base_puppet_admin_certs'] = []
      end

      it { should contain_pe_puppet_authorization__rule('puppetlabs environment cache')
        .with_match_request_path('/puppet-admin-api/v1/environment-cache')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow([])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
      it { should contain_pe_puppet_authorization__rule('puppetlabs jruby pool')
        .with_match_request_path('/puppet-admin-api/v1/jruby-pool')
        .with_match_request_type('path')
        .with_match_request_method('delete')
        .with_allow([])
        .with_sort_order(500)
        .with_path(authconf)
        .with_notify('Service[pe-puppetserver]') }
    end
  end

  context 'custom codedir' do
    before :each do
        params['puppetserver_jruby_puppet_master_code_dir'] = '/tmp/code'
    end
    it { should contain_pe_hocon_setting('jruby-puppet.master-code-dir').with_value('/tmp/code') }
  end

  context "managing files" do
    it { should contain_pe_concat("#{config_dir}/bootstrap.cfg").with_mode('0640') }
    it { should contain_file("#{config_dir}/conf.d/webserver.conf").with_mode('0640') }
    it { should contain_pe_hocon_setting('webserver.puppet-server.static-content').with_ensure('absent') }
    it { should contain_pe_hocon_setting('jruby-puppet.borrow-timeout').with_ensure('absent') }
    it { should contain_pe_hocon_setting('jruby-puppet.max-active-instances').with_ensure('absent') }
    it { should contain_pe_hocon_setting('jruby-puppet.max-requests-per-instance').with_ensure('present') }
    it { should contain_pe_hocon_setting('jruby-puppet.max-requests-per-instance').with_value(100000) }
    it { should contain_pe_hocon_setting('jruby-puppet.ruby-load-path').with_value(['/opt/puppetlabs/puppet/lib/ruby/vendor_ruby', '/opt/puppetlabs/puppet/cache/lib']) }
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_ensure('absent') }
    it { should contain_pe_hocon_setting('os-settings.remove').with_ensure('absent') }

    it { should contain_pe_hocon_setting('http-client.cipher-suites').with_ensure('absent') }
    it { should contain_pe_hocon_setting('http-client.idle-timeout-milliseconds').with_ensure('absent') }
    it { should contain_pe_hocon_setting('http-client.connect-timeout-milliseconds').with_ensure('absent') }

    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("puppet-server")
      .with('access_log_config' => '/etc/puppetlabs/puppetserver/request-logging.xml')
    }

    it { should contain_pe_hocon_setting('web-router-service/master-service').with_value('/puppet') }
    it do
      should contain_pe_hocon_setting('web-router-service/master-service')
        .with_setting('web-router-service."puppetlabs.services.master.master-service/master-service"')
    end
    it { should_not contain_pe_hocon_setting('web-router-service/pe-legacy-routes-service').with_value('') }
    it do
      should contain_pe_hocon_setting('web-router-service/legacy-routes-service')
                 .with_setting('web-router-service."puppetlabs.services.legacy-routes.legacy-routes-service/legacy-routes-service"')
    end
    it do
      should contain_pe_hocon_setting('web-router-service/legacy-routes-service')
        .with_setting('web-router-service."puppetlabs.services.legacy-routes.legacy-routes-service/legacy-routes-service"')
    end
    it { should contain_pe_hocon_setting('web-router-service/certificate-authority-service').with_value('/puppet-ca') }
    it do
      should contain_pe_hocon_setting('web-router-service/certificate-authority-service')
        .with_setting('web-router-service."puppetlabs.services.ca.certificate-authority-service/certificate-authority-service"')
    end
    it { should contain_pe_hocon_setting('web-router-service/reverse-proxy-ca-service').with_value('') }
    it do
      should contain_pe_hocon_setting('web-router-service/reverse-proxy-ca-service')
        .with_setting('web-router-service."puppetlabs.enterprise.services.reverse-proxy.reverse-proxy-ca-service/reverse-proxy-ca-service"')
    end
    it { should contain_pe_hocon_setting('web-router-service/puppet-admin-service').with_value('/puppet-admin-api') }
    it do
      should contain_pe_hocon_setting('web-router-service/puppet-admin-service')
        .with_setting('web-router-service."puppetlabs.services.puppet-admin.puppet-admin-service/puppet-admin-service"')
    end
    it { should contain_pe_hocon_setting('web-router-service/status-service').with_value('/status') }
    it do
      should contain_pe_hocon_setting('web-router-service/status-service')
             .with_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-service/status-service"')
    end
    it { should contain_pe_hocon_setting('web-router-service/master-service')
             .with_setting('web-router-service."puppetlabs.services.master.master-service/master-service"')}

    it { should contain_file("#{config_dir}/conf.d/pe-puppet-server.conf").with_mode('0640') }
    it { should contain_file("#{config_dir}/conf.d/metrics.conf").with_mode('0640') }
    it { should_not contain_file("#{config_dir}/conf.d/webserver.conf").with_content(%r[max-threads:\s*\d*]) }

    it { should contain_pe_hocon_setting('metrics.registries.puppetserver.reporters.graphite.enabled').with_value(false) }
    it { should contain_pe_hocon_setting('metrics.registries.puppetserver.reporters.jmx.enabled').with_value(true) }
    it { should contain_pe_hocon_setting('metrics.registries.default.reporters.jmx.enabled').with_value(true) }
    it { should contain_pe_hocon_setting('metrics.registries.puppetserver.metrics-allowed').with_value(["foo", "bar"]) }

    it { should contain_pe_hocon_setting('metrics.reporters.graphite.enabled').with_ensure('absent') }
    it { should contain_pe_hocon_setting('metrics.reporters.jmx').with_ensure('absent') }
  end

  context "managing the metrics-webservice" do
    context "with metrics_webservice_enabled set to false" do
      before :each do
        params['metrics_webservice_enabled'] = false
      end
      it { should contain_pe_hocon_setting('metrics-service/metrics-webservice').with_ensure('absent') }
      it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('metrics-webservice').with_ensure('absent') }
    end

    context "with metrics_webservice_enabled set to true" do
      before :each do
        params['metrics_webservice_enabled'] = true
      end
      it { should contain_pe_hocon_setting('metrics-service/metrics-webservice').with_ensure('present') }
      it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('metrics-webservice')
        .with_namespace('puppetlabs.trapperkeeper.services.metrics.metrics-service') }
    end
  end

  context "managing webserver.puppet-server" do
    before :each do
      params['puppetserver_webserver_ssl_host'] = 'test.host.com'
      params['puppetserver_webserver_ssl_port'] = '1234'
    end

    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("puppet-server")
      .with('container'          => 'puppetserver',
            'ssl_listen_address' => 'test.host.com',
            'ssl_listen_port'    => 1234,
            'default_server'     => true,
            )
    }
  end

  context "managing service defaults" do
    it { should contain_puppet_enterprise__trapperkeeper__init_defaults('puppetserver')
      .with({ 'user'                 => 'pe-puppet',
              'group'                => 'pe-puppet',
              'service_stop_retries' => 60,
              'start_timeout'        => 300,
              'jruby_jar'            => '/opt/puppetlabs/server/apps/puppetserver/jruby-9k.jar', })
    }

    context "with overrides" do
      before(:each) do
        params['service_stop_retries'] = 12345
        params['start_timeout'] = 67890
        params['jruby_jar'] = 'foo'
      end

      it { should contain_puppet_enterprise__trapperkeeper__init_defaults('puppetserver')
        .with({ 'service_stop_retries' => 12345,
                'start_timeout'        => 67890,
                'jruby_jar'            => 'foo', })
      }
    end
  end

  context "managing jetty cipher-suites" do
    before :each do
      params['cipher_suites'] = [
        'TLS_RSA_WITH_AES_256_CBC_SHA256',
	      'TLS_RSA_WITH_AES_256_CBC_SHA',
        'TLS_RSA_WITH_AES_128_CBC_SHA256',
        'TLS_RSA_WITH_AES_128_CBC_SHA'
      ]
    end
    it { should contain_pe_hocon_setting('http-client.cipher-suites').with_value(['TLS_RSA_WITH_AES_256_CBC_SHA256', 'TLS_RSA_WITH_AES_256_CBC_SHA', 'TLS_RSA_WITH_AES_128_CBC_SHA256', 'TLS_RSA_WITH_AES_128_CBC_SHA']) }
  end

  context "managing jetty ssl-protocols" do
    before :each do
      params['ssl_protocols'] = [
        'TLSv1',
        'TLSv1.1',
        'TLSv1.2'
      ]
    end
    it { should contain_pe_hocon_setting('http-client.ssl-protocols').with_value(['TLSv1', 'TLSv1.1', 'TLSv1.2']) }
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("puppet-server")
      .with('ssl_protocols' => ['TLSv1', 'TLSv1.1', 'TLSv1.2'])
    }
  end

  context "managing jetty idle-timeout-milliseconds" do
    before :each do
      params['idle_timeout_milliseconds'] = 12000000
    end
    it { should contain_pe_hocon_setting('http-client.idle-timeout-milliseconds').with_value('12000000') }
  end

  context "managing jetty max-requests-per-instance" do
    before :each do
      params['jruby_max_requests_per_instance'] = 42
    end
    it { should contain_pe_hocon_setting('jruby-puppet.max-requests-per-instance').with_value('42') }
  end

  context "managing jetty connect-timeout-milliseconds" do
    before :each do
      params['connect_timeout_milliseconds'] = 1200000
    end
    it { should contain_pe_hocon_setting('http-client.connect-timeout-milliseconds').with_value('1200000') }
  end

  context "managing jetty max-threads" do
    before :each do
      params['tk_jetty_max_threads'] = 10
    end

    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("puppet-server")
      .with('tk_jetty_max_threads' => 10)
    }
  end

  context "managing jruby borrow_timeout" do
    before :each do
      params['jruby_borrow_timeout'] = 1000
    end

    it { should contain_pe_hocon_setting('jruby-puppet.borrow-timeout').with_value('1000') }
  end

  context "managing the number of jruby instances" do
    before :each do
      params['jruby_max_active_instances'] = 5
    end

    it { should contain_pe_hocon_setting('jruby-puppet.max-active-instances').with_value('5') }
  end

  context "managing jruby environment class cache enabled (not specified) and code manager auto configure (true)" do
    before :each do
      params['code_manager_auto_configure'] = true
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(true) }
  end

  context "managing jruby environment class cache enabled (not specified) and code manager auto configure (false)" do
    before :each do
      params['code_manager_auto_configure'] = false
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_ensure('absent') }
  end

  context "managing jruby environment class cache enabled (true) and code manager auto configure (not specified)" do
    before :each do
      params['jruby_environment_class_cache_enabled'] = true
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(true) }
  end

  context "managing jruby environment class cache enabled (true) and code manager auto configure (true)" do
    before :each do
      params['code_manager_auto_configure'] = true
      params['jruby_environment_class_cache_enabled'] = true
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(true) }
  end

  context "managing jruby environment class cache enabled (true) and code manager auto configure (false)" do
    before :each do
      params['code_manager_auto_configure'] = false
      params['jruby_environment_class_cache_enabled'] = true
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(true) }
  end

  context "managing jruby environment class cache enabled (false) and code manager auto configure not specified" do
    before :each do
      params['jruby_environment_class_cache_enabled'] = false
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(false) }
  end

  context "managing jruby environment class cache enabled (false) and code manager auto configure (true)" do
    before :each do
      params['code_manager_auto_configure'] = true
      params['jruby_environment_class_cache_enabled'] = false
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(false) }
  end

  context "managing jruby environment class cache enabled (false) and code manager auto configure (false)" do
    before :each do
      params['code_manager_auto_configure'] = false
      params['jruby_environment_class_cache_enabled'] = false
    end
    it { should contain_pe_hocon_setting('jruby-puppet.environment-class-cache-enabled').with_value(false) }
  end

  context "disable use_legacy_auth_conf" do
    it { should contain_pe_hocon_setting('jruby-puppet.use-legacy-auth-conf').with_value(false)}
  end

  context "managing the pre-commit-hook-commands" do
    context "it has a sane default" do
      it { should contain_pe_hocon_setting('pe-puppetserver.pre-commit-hook-commands').with_value(['/opt/puppetlabs/server/bin/generate-puppet-types.rb']) }
    end

    context "it can be overridden" do
      before :each do
        params['pre_commit_hook_commands'] = ['/bin/touch']
      end

      it { should contain_pe_hocon_setting('pe-puppetserver.pre-commit-hook-commands').with_value(['/bin/touch']) }
    end
  end

  context "serving static files" do
    before :each do
      params['static_files'] = {
        "/baz-files" => "/foo/bar/baz",
        "/others" => "/foo/bar/others"
      }
    end
    it { should contain_pe_hocon_setting('webserver.puppet-server.static-content').with_value(%r[.*resource.*/foo/bar/baz.*path.*/baz-files.*follow-links.*true])}
    it { should contain_pe_hocon_setting('webserver.puppet-server.static-content').with_value(%r[.*resource.*/foo/bar/others.*path.*/others.*follow-links.*true])}
  end

  context "managing services" do
    it { should contain_service('pe-puppetserver').with_ensure('running') }
  end

  context "configuring console services consumers" do
    it { should contain_pe_hocon_setting('rbac-consumer.api-url').with_value('https://console.rspec:4433/rbac-api') }
    it { should contain_pe_hocon_setting('activity-consumer.api-url').with_value('https://console.rspec:4433/activity-api') }
  end

  context "configuring bootstrap.cfg" do
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('puppetserver-analytics-service')
                  .with_service('puppetserver-analytics-service')
                  .with_namespace('puppetlabs.enterprise.services.analytics.analytics-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('legacy-routes-service').with_namespace('puppetlabs.services.legacy-routes.legacy-routes-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('pe-legacy-routes-service')
                    .with_ensure('absent')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('pe-metrics-puppet-profiler-service')
                    .with_namespace('puppetlabs.enterprise.services.puppet-profiler.pe-puppet-profiler-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('puppet-profiler-service')
                    .with_namespace('puppetlabs.services.puppet-profiler.puppet-profiler-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('pe-master-service')
                    .with_namespace('puppetlabs.enterprise.services.master.pe-master-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('master-service')
                    .with_namespace('puppetlabs.services.master.master-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('pe-jruby-metrics-service')
                    .with_namespace('puppetlabs.enterprise.services.jruby.pe-jruby-metrics-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('jruby-metrics-service')
                    .with_namespace('puppetlabs.services.jruby.jruby-metrics-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('pe-legacy-routes-service').with_namespace('puppetlabs.enterprise.services.legacy-routes.pe-legacy-routes-service')}
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('filesystem-watch-service').with_namespace('puppetlabs.trapperkeeper.services.watcher.filesystem-watch-service') }
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('remote-rbac-consumer-service')
                    .with_namespace('puppetlabs.rbac-client.services.rbac')}
  end

  context "when managing default java_args" do
    context "1 GB of RAM => 512 MB heap" do
      before(:each) { facts['memorysize'] = '1073741824' }
      it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetserver').with(
        :java_args => { 'Xmx' => '512m',
                        'Xms' => '512m',
                        "Djava.io.tmpdir="=>"/opt/puppetlabs/server/apps/puppetserver/tmp" },
        )
      }
    end

    context "2 GB of RAM => 1 GB heap" do
      before(:each) { facts['memorysize'] = '2147483648' }
      it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetserver').with(
        :java_args => { 'Xmx' => '1024m',
                        'Xms' => '1024m',
                        "Djava.io.tmpdir="=>"/opt/puppetlabs/server/apps/puppetserver/tmp" },
        )
      }
    end

    context "4 GB of RAM => 2 GB heap" do
      before(:each) { facts['memorysize'] = '4294967296' }
      it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetserver').with(
        :java_args => { 'Xmx' => '2048m',
                        'Xms' => '2048m',
                        "Djava.io.tmpdir="=>"/opt/puppetlabs/server/apps/puppetserver/tmp",
                        'XX:ReservedCodeCacheSize=' => '512m',},
        )
      }
    end
  end

  context "with custom java_args" do
    before(:each) { params['java_args'] = { 'Xms' => '128m', 'Xmx' => '333m' }
                    params['enable_gc_logging'] = true }
    it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetserver').with(
      :java_args => { 'Xmx' => '333m',
                      'Xms' => '128m',
                      "Djava.io.tmpdir="=>"/opt/puppetlabs/server/apps/puppetserver/tmp" },
      :enable_gc_logging => true,
      )
    }
  end

  context "passing custom djava_io_tmpdir" do
    before(:each) {
      params['java_args'] = { 'Xms' => '128m', 'Xmx' => '333m' }
      params['djava_io_tmpdir'] = '/var/my_tmp'
    }
    it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetserver').with(
      :java_args => { 'Xmx' => '333m',
                      'Xms' => '128m',
                      "Djava.io.tmpdir="=>"/var/my_tmp" },
      :enable_gc_logging => true,
      )
    }
  end

  context "managing the global config file" do
    let(:node) { 'master.rspec'}

    it { should contain_puppet_enterprise__trapperkeeper__global_settings('puppetserver')
      .with({ 'service_user' => 'pe-puppet',
              'service_group' => 'pe-puppet',
              'hostname'    => 'master.rspec',
              'ssl_cert'    => "/etc/puppetlabs/puppet/ssl/certs/master.rspec.pem",
              'ssl_key'     => "/etc/puppetlabs/puppet/ssl/private_keys/master.rspec.pem",
              'localcacert' => "/etc/puppetlabs/puppet/ssl/certs/ca.pem",
            })
        }
  end

  context "setting max-queued-requests and max-retry-delay" do
    before(:each) {
      params['jruby_puppet_max_queued_requests'] = 100
      params['jruby_puppet_max_retry_delay'] = 600
    }

    it { should contain_pe_hocon_setting('jruby-puppet.max-queued-requests').with(
      'value' => 100,
      'path' => '/etc/puppetlabs/puppetserver/conf.d/pe-puppet-server.conf',
      'setting' => 'jruby-puppet.max-queued-requests',
    )}

    it { should contain_pe_hocon_setting('jruby-puppet.max-retry-delay').with(
        'value' => 600,
        'path' => '/etc/puppetlabs/puppetserver/conf.d/pe-puppet-server.conf',
        'setting' => 'jruby-puppet.max-retry-delay',
    )}
  end

  context "turn off jit compiler" do
    before(:each) {
      params['jruby_compile_mode'] = 'off'
    }

    it { should contain_pe_hocon_setting('jruby-puppet.compile-mode').with(
      'value' => 'off',
      'path' => '/etc/puppetlabs/puppetserver/conf.d/pe-puppet-server.conf',
      'setting' => 'jruby-puppet.compile-mode',
      'ensure' => 'present',
    )}
  end
end
