require 'spec_helper'

describe 'puppet_enterprise::master' do

  let(:params) do
    {
      'certname' => 'master.rspec',
      'metrics_server_id' => 'localhost',
      'metrics_jmx_enabled' => true,
      'metrics_graphite_enabled' => false,
      'metrics_puppetserver_metrics_allowed' => [],
      'profiler_enabled' => true,
      'puppetserver_jruby_puppet_master_code_dir' => '/etc/puppetlabs/code'
    }
  end

  context "managing files" do
    it { should contain_file("/var/log/puppetlabs/puppet").with_mode('0640') }
    it { should contain_file("/etc/puppetlabs/puppet/fileserver.conf")
      .with({ 'owner'   => 'root',
              'group'   => 'root',
              'mode'    => '0644' })
    }

    it { should contain_pe_ini_setting("puppetserver puppetconf certname").with_value('master.rspec') }
    it { should contain_pe_ini_setting("puppetserver puppetconf always_cache_features").with_ensure('absent') }
    it { should contain_pe_ini_setting("puppetserver puppetconf always_retry_plugins")
      .with_ensure('present')
      .with_value(false) }
    it { should contain_pe_ini_setting("puppetserver puppetconf user").with_value('pe-puppet') }
    it { should contain_pe_ini_setting("puppetserver puppetconf group").with_value('pe-puppet') }
    it { should contain_pe_ini_setting('puppetserver static catalogs').with_ensure('absent') }
    it { should_not contain_pe_ini_setting("parser").with_value('current') }
    it { should contain_class('puppet_enterprise::master::puppetserver') }
    it { should satisfy_all_relationships }

    context "managing parser - invalid parser" do
      before(:each) { params['parser'] = 'past' }
      it { expect { should satisfy_all_relationships }.to raise_error(Puppet::Error) }
    end

    context "managing symlinks" do
        before(:each) { params['manage_symlinks'] = true }
        it { should contain_file('/usr/local/bin/pe-man').with_tag('pe-master-symlinks') }
        it { should contain_file('/usr/local/bin/r10k').with_tag('pe-master-symlinks') }
        it { should contain_file('/usr/local/bin/eyaml').with_tag('pe-master-symlinks') }
    end
  end

  context "with static_catalogs" do
    before(:each) { params['static_catalogs'] = true }
    it { should contain_pe_ini_setting('puppetserver static catalogs').with_ensure('present').with_value(true) }
    it { should satisfy_all_relationships }
  end

  context "with static_catalog false" do
    before(:each) { params['static_catalogs'] = false }
    it { should contain_pe_ini_setting('puppetserver static catalogs').with_ensure('present').with_value(false) }
    it { should satisfy_all_relationships }
  end

  context "managing puppetsever" do
    it { should contain_package('pe-puppetserver') }
    it { should contain_package('pe-java') }
    it { should contain_class('puppet_enterprise::master::puppetserver') }
    it { should satisfy_all_relationships }
  end

  context "managing pe_build file" do
    # bust cache
    let(:node) { "pe_build.rspec" }
    before :each do
      Puppet::Parser::Functions.newfunction(:pe_build_version, :type => :rvalue) do |args|
        '1.2.3'
      end
    end

    it do
      should contain_file('/opt/puppetlabs/server/pe_build')
        .with_content('1.2.3')
    end
  end

  context 'when managing max_requests_per_instance' do
    before(:each) {
      params['jruby_max_requests_per_instance'] = 11111
    }
    it { should contain_class('puppet_enterprise::master::puppetserver').with(
      :jruby_max_requests_per_instance => 11111
    )}
  end

  context 'passing through code_manager_auto_configure to the pe::master::puppetserver class' do
    context 'when code_manager_auto_configure false' do
      before(:each) { params['code_manager_auto_configure'] = false }
      it { should contain_class('puppet_enterprise::master::puppetserver').with(:code_manager_auto_configure => false) }
      it { should contain_pe_ini_setting("puppetconf environment_timeout setting").with_value( '0' ) }
    end
    context 'when code_manager_auto_configure true' do
      before(:each) { params['code_manager_auto_configure'] = true }
      it { should contain_class('puppet_enterprise::master::puppetserver').with(:code_manager_auto_configure => true) }
      it { should contain_pe_ini_setting("puppetconf environment_timeout setting").with_value( 'unlimited' ) }
    end
  end

  context 'when configuring a specific environment_timeout' do
    before(:each) { params['environment_timeout'] = '3m' }
    it { should contain_pe_ini_setting("puppetconf environment_timeout setting").with_value( '3m' ) }
  end

  context 'enable i18n by default' do
    it { should contain_pe_ini_setting("puppetconf disable_i18n setting").with_value( false ) }
  end

  context 'when configuring a specific disable_i18n' do
    before(:each) { params['disable_i18n'] = true }
    it { should contain_pe_ini_setting("puppetconf disable_i18n setting").with_value( true ) }
  end

  context 'when managing pe-backup-tools' do
    it { should contain_package("pe-backup-tools") }
  end
end
