require 'spec_helper'

describe 'puppet_enterprise::profile::agent' do
  let(:facts) do
    {
      'kernel'                    => 'Linux',
      'platform_symlink_writable' => true,
    }
  end
  let(:params) do
    {
      'manage_symlinks'   => true,
      'pcp_broker_host'   => 'pcp_broker.rspec',
      'pcp_broker_port'   => 4245,
    }
  end

  context "when parameter disables management of symlinks" do
    before :each do
      params['manage_symlinks'] = false
    end

    it { should_not contain_file('/usr/local/bin/facter') }
    it { should_not contain_file('/usr/local/bin/puppet') }
    it { should_not contain_file('/usr/local/bin/hiera') }
    it { should satisfy_all_relationships }
  end

  context "when pxp is disabled" do
    before :each do
      params['pxp_enabled'] = false
      facts['puppetversion'] = '4.3.0'
    end

    context "with the identity fact in Facter 3.2" do
      before :each do
        facts['identity'] = {
          'uid' => 0,
          'user' => 'root',
          'gid' => 0,
          'group' => 'root',
          'privileged' => true,
        }
      end

      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_broker.rspec:4245/pcp/'])
                   .with_enabled(false) }
      it { should satisfy_all_relationships }
    end

    context "with the identity fact prior to Facter 3.2" do
      before :each do
        facts['identity'] = {
          'uid' => 0,
          'user' => 'root',
          'gid' => 0,
          'group' => 'root',
        }
      end

      it { should_not contain_class('puppet_enterprise::pxp_agent') }
    end

    context "without the identity fact" do
      it { should_not contain_class('puppet_enterprise::pxp_agent') }
    end
  end

  context "when non boolean is passed for parameter manage_symlinks" do
    before :each do
      params['manage_symlinks'] = 'IamAString'
    end

    it { should_not compile }
  end

  context "when platform symlink fact reports system path is not writable" do
    before :each do
      facts['platform_symlink_writable'] = false
    end

    it { should_not contain_file('/usr/local/bin/facter') }
    it { should_not contain_file('/usr/local/bin/puppet') }
    it { should_not contain_file('/usr/local/bin/hiera') }
    it { should satisfy_all_relationships }
  end

  context "when using inherited parameter defaults" do
    before :each do
      params.delete('manage_symlinks')
      facts['puppetversion'] = '4.3.0'
    end

    context "with the identity fact in Facter 3.2" do
      before :each do
        facts['identity'] = {
          'uid' => 0,
          'user' => 'root',
          'gid' => 0,
          'group' => 'root',
          'privileged' => true,
        }
      end

      it { should contain_file('/usr/local/bin/facter').with_tag('pe-agent-symlinks') }
      it { should contain_file('/usr/local/bin/puppet').with_tag('pe-agent-symlinks') }
      it { should contain_file('/usr/local/bin/hiera').with_tag('pe-agent-symlinks') }
      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_broker.rspec:4245/pcp/'])
                   .with_master_uris(['https://master.rspec:8140'])
                   .with_pcp_version(nil)
                   .with_enabled(true) }
      it { should satisfy_all_relationships }
      it { should_not contain_file('/usr/local/bin/pe-man').with_tag('pe-master-symlinks') }
      it { should_not contain_file('/usr/local/bin/r10k').with_tag('pe-master-symlinks') }
      it { should_not contain_file('/usr/local/bin/eyaml').with_tag('pe-master-symlinks') }
    end

    context "with the identity fact prior to Facter 3.2" do
      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_broker.rspec:4245/pcp/'])
                   .with_master_uris(['https://master.rspec:8140'])
                   .with_pcp_version(nil)
                   .with_enabled(true) }
    end

    context "without the identity fact" do
      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_broker.rspec:4245/pcp/'])
                   .with_master_uris(['https://master.rspec:8140'])
                   .with_pcp_version(nil)
                   .with_enabled(true) }
    end
  end

  context "when using inherited parameter defaults with puppet version 4.9 or later" do
    before :each do
      facts['puppetversion'] = '4.9.0'
    end

    it { should contain_class('puppet_enterprise::pxp_agent')
                 .with_broker_ws_uri(['wss://pcp_broker.rspec:4245/pcp2/'])
                 .with_pcp_version('2')}
  end

  context "when using inherited parameter defaults with puppet version below 4.3" do
    before :each do
      params.delete('manage_symlinks')
      facts['puppetversion'] = '4.2.0'
    end

    it { should_not contain_class('puppet_enterprise::pxp_agent') }
  end

  context "when providing a list of pcp broker" do
    before :each do
      params['pcp_broker_list'] = ['pcp_test1.com:1234','pcp_test2.com:1234']
    end

    it { should contain_class('puppet_enterprise::pxp_agent')
                 .with_broker_ws_uri(['wss://pcp_test1.com:1234/pcp/',
                                      'wss://pcp_test2.com:1234/pcp/'])}

    context "With a new puppet-agent version" do
      before :each do
        facts['puppetversion'] = '4.9.0'
      end

      it { should contain_class('puppet_enterprise::pxp_agent')
                 .with_broker_ws_uri(['wss://pcp_test1.com:1234/pcp2/',
                                      'wss://pcp_test2.com:1234/pcp2/'])}
    end
  end

  context "when providing a list of pcp broker uris" do
    before :each do
      params['pcp_broker_ws_uris'] = ['wss://pcp_test1.com:1234/pcp/','wss://pcp_test2.com:1234/pcp']
    end

    it { should contain_class('puppet_enterprise::pxp_agent')
                 .with_broker_ws_uri(['wss://pcp_test1.com:1234/pcp/',
                                      'wss://pcp_test2.com:1234/pcp'])}

    context "ignores PCP v2" do
      before :each do
        facts['puppetversion'] = '4.9.0'
      end

      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_test1.com:1234/pcp/',
                                        'wss://pcp_test2.com:1234/pcp'])
                   .with_pcp_version(nil)}
    end

    context "When pcp_broker_list is also set" do
      before :each do
        params['pcp_broker_list'] = ['pcp_test1.com:5678','pcp_test2.com:5678']
      end

      it { should contain_class('puppet_enterprise::pxp_agent')
                   .with_broker_ws_uri(['wss://pcp_test1.com:5678/pcp/',
                                        'wss://pcp_test2.com:5678/pcp/'])
                   .with_pcp_version(nil)}
    end
  end

  context "when providing a list of server uris" do
    before :each do
      params['server_list'] = ['master_test1.com','master_test2.com:8140']
    end

    it { should contain_class('puppet_enterprise::pxp_agent')
                 .with_master_uris(['master_test1.com',
                                    'master_test2.com:8140']) }
  end

  context "when providing a list of master uris" do
    before :each do
      params['master_uris'] = ['master_test1.com:1234','master_test2.com:8140']
    end

    it { should contain_class('puppet_enterprise::pxp_agent')
                    .with_master_uris(['master_test1.com:1234',
                                       'master_test2.com:8140']) }
  end

  context "when managing puppet.conf on Linux" do
    before :each do
      facts['puppetversion'] = '4.6.0'
      params['manage_puppet_conf'] = true
      params['server_list'] = ['one', 'two', 'three']
      facts['os'] = {'family' => 'RedHat'}
    end

    it { should contain_pe_ini_setting('agent conf file server_list')
                 .with_path('/etc/puppetlabs/puppet/puppet.conf')
                 .with_setting('server_list')
                 .with_value('one,two,three') }
  end

  context "when managing puppet.conf on Windows" do
    before :each do
      facts['puppetversion'] = '4.6.0'
      params['manage_puppet_conf'] = true
      params['server_list'] = ['one', 'two', 'three']
      facts['os'] = {'family' => 'windows'}
      facts['common_appdata'] = 'C:/ProgramData'
    end

    it { should contain_pe_ini_setting('agent conf file server_list')
                 .with_path('C:/ProgramData/PuppetLabs/puppet/etc/puppet.conf')
                 .with_setting('server_list')
                 .with_value('one,two,three') }
  end

  context "when managing puppet.conf on an older puppet version" do
    before :each do
      facts['puppetversion'] = '3.8.0'
      params['manage_puppet_conf'] = true
      params['server_list'] = ['one', 'two', 'three']
    end

    it { should_not contain_pe_ini_setting('agent conf file server_list') }
  end

  context "when managing package inventory" do
    before(:each) do
      facts['identity'] = {
        'privileged' => true
      }
    end

    it "should create the package_inventory_enabled file when the parameter is true" do
      params['package_inventory_enabled'] = true
      is_expected.to contain_file("/opt/puppetlabs/puppet/cache/state/package_inventory_enabled")
        .with_ensure("present")
    end

    it "should not create the package_inventory_enabled file when the parameter is false" do
      params['package_inventory_enabled'] = false
      is_expected.to contain_file("/opt/puppetlabs/puppet/cache/state/package_inventory_enabled")
        .with_ensure("absent")
    end

    it "should not create the package_inventory_enabled file by default" do
      is_expected.to contain_file("/opt/puppetlabs/puppet/cache/state/package_inventory_enabled")
        .with_ensure("absent")
    end

    it "should create the correct package_inventory_enabled file on windows" do
      params['package_inventory_enabled'] = true
      facts['os'] = { 'family' => 'windows' }
      facts['common_appdata'] = '/ProgramData'
      is_expected.to compile
      is_expected.to contain_file("/ProgramData/PuppetLabs/puppet/cache/state/package_inventory_enabled")
        .with_ensure("present")
    end

    it "does not include package_inventory_enabled file resource for a non-root user" do
      facts['identity'] = {
        'privileged' => false
      }
      is_expected.to_not contain_file("/opt/puppetlabs/puppet/cache/state/package_inventory_enabled")
    end

    it "does not include package_inventory_enabled file resource for agents <= 2016.2.1 (1.5.3)" do
      facts['identity'].delete('privileged')
      is_expected.to_not contain_file("/opt/puppetlabs/puppet/cache/state/package_inventory_enabled")
    end
  end

  context "when using linux install the uninstall script" do
    before :each do
      mock_puppet_file('/opt/puppetlabs/bin/puppet-enterprise-uninstaller', 'uninstall')
    end
    it { should contain_file('/opt/puppetlabs/bin/puppet-enterprise-uninstaller')
      .with_mode('0544') }
    it { should satisfy_all_relationships }
  end

  context "when using linux dont install the uninstall script if it's empty" do
    before :each do
      facts['prevent_rspec_caching'] = 'agent-with-no-uninstaller-script-on-master'
    end
    it { should_not contain_file('/opt/puppetlabs/bin/puppet-enterprise-uninstaller') }
    it { should satisfy_all_relationships }
  end

  context "when using windows do not install the uninstall script" do
    before :each do
      facts['kernel'] = 'windows'
    end

    it { should_not contain_file('/opt/puppetlabs/bin/puppet-enterprise-uninstaller') }
    it { should satisfy_all_relationships }
  end

  context "when using MacOS do not install the uninstall script" do
    before :each do
      facts['kernel'] = 'Darwin'
    end

    it { should_not contain_file('/opt/puppetlabs/bin/puppet-enterprise-uninstaller') }
    it { should satisfy_all_relationships }
  end

  it { should satisfy_all_relationships }
end
