require 'spec_helper'

describe 'puppet_enterprise::profile::console::console_services_config' do
  let(:params) do
    {
      'certname' => 'console_services.rspec',
      'classifier_url_prefix' => '/classifier-api'
    }
  end
  let(:cert_dir) { '/opt/puppetlabs/server/data/console-services/certs' }
  let(:hiera_config) { nil }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-console-services': }
    PRE_COND
  end

  context "managing the global config file" do
    it { should contain_puppet_enterprise__trapperkeeper__global_settings('console-services')
      .with({ 'login_path'       => '/auth/login',
              'version_path'     => '/opt/puppetlabs/server/pe_version',
            })
        }
  end

  context "when configuring jetty SSL" do
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("console")
      .with('container'          => 'console-services',
            'ssl_listen_address' => '0.0.0.0',
            'ssl_listen_port'    => 4431,
            'default_server'     => true,
            'ssl_protocols'      => ['TLSv1.2'],
            'client_auth'        => 'none',
            'localcacert'        => '/etc/puppetlabs/puppet/ssl/certs/ca.pem',
            'tk_jetty_request_header_max_size' => 65536,
            'ssl_key'            => "#{cert_dir}/console_services.rspec.private_key.pem",
            'ssl_cert'           => "#{cert_dir}/console_services.rspec.cert.pem",
            'access_log_config'  => '/etc/puppetlabs/console-services/request-logging.xml',
            )
    }
    it { should contain_pe_hocon_setting('console-services.webserver.console.host').with_value('127.0.0.1') }
    it { should contain_pe_hocon_setting('console-services.webserver.console.port').with_value('4430') }

    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("api")
      .with('container'          => 'console-services',
            'ssl_listen_address' => '0.0.0.0',
            'ssl_listen_port'    => 4433,
            'ssl_protocols'      => ['TLSv1.2'],
            'localcacert'        => '/etc/puppetlabs/puppet/ssl/certs/ca.pem',
            'ssl_key'            => "#{cert_dir}/console_services.rspec.private_key.pem",
            'ssl_cert'           => "#{cert_dir}/console_services.rspec.cert.pem",
            'access_log_config'  => '/etc/puppetlabs/console-services/request-logging-api.xml',
            )
    }
    it { should contain_pe_hocon_setting('console-services.webserver.api.host').with_value('127.0.0.1') }
    it { should contain_pe_hocon_setting('console-services.webserver.api.port').with_value('4432') }
    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.activity.services/activity-service"').with_value(
        { "route" => '/activity-api',
          "server" => 'api',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.activity.services/activity-service"'
      ).with_setting('web-router-service."puppetlabs.activity.services/activity-service"')
    end

    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.rbac.services.http.api/rbac-http-api-service"').with_value(
        { "route" => '/rbac-api',
          "server" => 'api',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.rbac.services.http.api/rbac-http-api-service"'
      ).with_setting('web-router-service."puppetlabs.rbac.services.http.api/rbac-http-api-service"')
    end
    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.pe-console-ui.service/pe-console-ui-service".pe-console-app').with_value(
        { "route" => '/',
          "server" => 'console',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.pe-console-ui.service/pe-console-ui-service".pe-console-app'
      ).with_setting('web-router-service."puppetlabs.pe-console-ui.service/pe-console-ui-service".pe-console-app')
    end

    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.pe-console-auth-ui.service/pe-console-auth-ui-service".authn-app').with_value(
        { "route" => '/auth',
          "server" => 'console',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.pe-console-auth-ui.service/pe-console-auth-ui-service".authn-app'
      ).with_setting('web-router-service."puppetlabs.pe-console-auth-ui.service/pe-console-auth-ui-service".authn-app')
    end

    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.classifier.main/classifier-service"').with_value(
        { "route" => '/classifier-api',
          "server" => 'api',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.classifier.main/classifier-service"'
      ).with_setting('web-router-service."puppetlabs.classifier.main/classifier-service"')
    end

    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-service/status-service"').with_value(
        { "route" => '/status',
          "server" => 'api',}
      )
    end
    it do
      should contain_pe_hocon_setting(
        'web-router-service."puppetlabs.trapperkeeper.services.status.status-service/status-service"'
      ).with_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-service/status-service"')
    end

    it do
      should contain_pe_hocon_setting('web-router-service."puppetlabs.trapperkeeper.services.metrics.metrics-service/metrics-webservice"').with_value(
        { "route" => '/metrics',
          "server" => 'api',}
      )
    end

    context "when status proxy is enabled" do
      before(:each) do
        params['status_proxy_enabled'] = true
        params['status_proxy_port'] = 5000
      end

      it do
        should contain_pe_hocon_setting(
          'web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"'
        ).with_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"')
      end

      it do
        should contain_pe_hocon_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"').with_value(
          { "route" => '/status',
            "server" => 'status-proxy',}
        )
      end
    end

    context "when status proxy is disabled" do
      before(:each) do
        params['status_proxy_enabled'] = false
        params['status_proxy_port'] = 5000
      end

      it do
        should contain_pe_hocon_setting(
          'web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"'
        ).with_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"').with_ensure(:absent)
      end

      it do
        should contain_pe_hocon_setting('web-router-service."puppetlabs.trapperkeeper.services.status.status-proxy-service/status-proxy-service"').with_value(
          { "route" => '/status',
            "server" => 'status-proxy',}
        ).with_ensure(:absent)
      end
    end

    context "removal of deprecated web-router-service" do
      it do
        should contain_pe_hocon_setting('web-router-service.remove-rbac-ui-service')
          .with_ensure('absent')
          .with_setting('web-router-service."puppetlabs.rbac-ui.service/rbac-ui-service"')
      end

      it do
        should contain_pe_hocon_setting('web-router-service.remove-helpers-service')
          .with_ensure('absent')
          .with_setting('web-router-service."puppetlabs.proxy.services.proxy/helpers-service"')
      end

      it do
        should contain_pe_hocon_setting('web-router-service.remove-classifier-ui-service')
          .with_ensure('absent')
          .with_setting('web-router-service."puppetlabs.classifier-ui.service/classifier-ui-service"')
      end
    end
  end

  context "when specifying jetty max-threads" do
    before :each do
      params['tk_jetty_max_threads_api'] = 12
      params['tk_jetty_max_threads_console'] = 34
    end
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("console")
      .with('tk_jetty_max_threads' => 34)
    }
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("api")
      .with('tk_jetty_max_threads' => 12)
    }
  end

  context "when specifying jetty request-header-max-size" do
    before :each do
      params['tk_jetty_request_header_max_size'] = 42
    end
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("console")
      .with('tk_jetty_request_header_max_size' => 42)
    }
  end

  context "when specifying an RBAC URL prefix" do
    before :each do
      params['rbac_url_prefix'] = '/prefix'
    end
    it { should contain_pe_hocon_setting('web-router-service."puppetlabs.rbac.services.http.api/rbac-http-api-service"').with_value(
      { "route" => '/prefix',
        "server" => 'api',})}
  end
end
