require 'spec_helper'

describe 'puppet_enterprise::profile::console::proxy' do
  let(:params) do
    {
      'certname'                           => 'console_services.rspec',
      'trapperkeeper_proxy_listen_address' => '0.0.0.0',
      'trapperkeeper_proxy_listen_port'    => 1234,
      'ssl_listen_port'                    => 6789,
    }
  end
  let(:config_dir) { '/etc/puppetlabs/nginx/conf.d' }
  let(:proxy_conf_file) { "#{config_dir}/proxy.conf" }
  let(:cert_dir) { '/opt/puppetlabs/server/data/console-services/certs' }
  let(:console_host) { 'console.rspec' }
  let(:localcacert) { '/etc/puppetlabs/puppet/ssl/certs/ca.pem' }

  let(:default_ssl_cert) { "#{cert_dir}/console_services.rspec.cert.pem" }
  let(:default_ssl_priv_key) { "#{cert_dir}/console_services.rspec.private_key.pem" }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
        console_host                 => 'console.rspec',
        puppetdb_host                => 'puppetdb.rspec',
        puppetdb_port                => 8081,
      }
      class {'puppet_enterprise::console_services':
        master_host           => 'master.rspec',
        client_certname       => 'console_services.rspec',
        puppetdb_host         => ['puppetdb.rspec'],
        puppetdb_port         => [8081],
      }
      include puppet_enterprise::profile::console::certs
      file { '/etc/puppetlabs/console-services/conf.d/webserver.conf': }
    PRE_COND
  end

  def should_have_directive(name, value)
    should contain_pe_nginx__directive(name).with_value(value)
  end

  context "using this class" do
    it { should contain_class('puppet_enterprise::profile::console::proxy::nginx_conf') }
  end

  %w[ssl_verify_client ssl_prefer_server_ciphers].each do |param|
    context "invalid parameter #{param}" do
      before :each do
        params[param] = 'this should tots fail'
      end

      it { should compile.and_raise_error(/does not match/) }
    end
  end

  context "when the parameters are valid" do
    it { should_have_directive('server_name', 'console.rspec') }
    it { should_have_directive('listen', '6789 ssl') }
    it { should_have_directive('ssl_certificate', default_ssl_cert) }
    it { should_have_directive('ssl_certificate_key', default_ssl_priv_key) }
    it { should_have_directive('proxy_pass', 'http://0.0.0.0:1234') }
    it { should_have_directive('proxy_redirect', 'http://0.0.0.0:1234 /') }
    it { should_have_directive('proxy_set_header x-forwarded-for', 'X-Forwarded-For $proxy_add_x_forwarded_for') }
    it { should_have_directive('proxy_read_timeout', '120')}
    it { should contain_file(proxy_conf_file).with(:ensure => "file", :owner => "root", :group => "root", :mode => "0644") }
  end

  context "disables pe-nginx service on a replica" do
    before :each do
      params['replication_mode'] = 'replica'
    end

    it { should contain_service('pe-nginx').with_ensure('stopped') }
    it { should contain_service('pe-nginx').with('enable' => false) }
  end

  %w[source none].each do |mode|
    context "enables pe-nginx service when replication mode is '${mode}'" do
      before :each do
        params['replication_mode'] = mode
      end

      it { should contain_service('pe-nginx').with_ensure('running') }
      it { should contain_service('pe-nginx').with('enable' => true) }
    end
  end

  context "when managing server_name" do
    before :each do
      params['server_name'] = 'my_custom_dns_name'
    end
      it { should_have_directive('server_name', 'my_custom_dns_name') }
  end

  context "managing ssl_protocols" do
    # This case is required for backward-compatibility
    it "accepts a space-separated string" do
      params['ssl_protocols'] = 'TLSv1.1 TLSv1.2'
      should contain_pe_nginx__directive('ssl_protocols').with_value('TLSv1.1 TLSv1.2')
    end

    it "accepts an array" do
      params['ssl_protocols'] = ['TLSv1.1', 'TLSv1.2']
      should contain_pe_nginx__directive('ssl_protocols').with_value('TLSv1.1 TLSv1.2')
    end
  end

  context "custom proxy read timeout" do
    context "value is set to integer" do
      before :each do
        params['proxy_read_timeout'] = 99
      end

      it { should_have_directive('proxy_read_timeout', '99')}
    end

    context "value is set to integer string" do
      before :each do
        params['proxy_read_timeout'] = '99'
      end

      it { should_have_directive('proxy_read_timeout', '99')}
    end

    context "value is set to non-integer" do
      before :each do
        params['proxy_read_timeout'] = 'infinite'
      end

      it { should compile.and_raise_error(/not an integer/) }
    end

    context "when no ssl_listen_address is given" do
      before :each do
        params['ssl_listen_address'] = '1.2.3.4'
      end

      it { should_have_directive('listen', '1.2.3.4:6789 ssl') }
      it { should_have_directive('proxy_pass', 'http://0.0.0.0:1234') }
      it { should_have_directive('proxy_redirect', 'http://0.0.0.0:1234 /') }
      it { should contain_file(proxy_conf_file).with(:ensure => "file", :owner => "root", :group => "root", :mode => "0644") }
    end

    context "when 0.0.0.0 ssl_listen_address is given" do
      before :each do
        params['ssl_listen_address'] = '0.0.0.0'
      end

      it { should_have_directive('listen', '6789 ssl') }
      it { should_have_directive('proxy_pass', 'http://0.0.0.0:1234') }
      it { should_have_directive('proxy_redirect', 'http://0.0.0.0:1234 /') }
      it { should contain_file(proxy_conf_file).with(:ensure => "file", :owner => "root", :group => "root", :mode => "0644") }
    end
  end

  context "custom ssl configurations" do
    context "when browser_ssl_cert and browser_ssl_private_key are set" do
      before :each do
        params['browser_ssl_cert'] = '/some/path/to/a/browser.cert.pem'
        params['browser_ssl_private_key'] = '/some/path/to/a/browser.private_key.pem'
      end

      it { should_have_directive('server_name', 'console.rspec') }
      it { should_have_directive('listen', '6789 ssl') }
      it { should_have_directive('ssl_certificate', '/some/path/to/a/browser.cert.pem') }
      it { should_have_directive('ssl_certificate_key', '/some/path/to/a/browser.private_key.pem') }
      it { should_have_directive('proxy_pass', 'http://0.0.0.0:1234') }
      it { should_have_directive('proxy_redirect', 'http://0.0.0.0:1234 /') }
      it { should contain_file(proxy_conf_file).with(:ensure => "file", :owner => "root", :group => "root", :mode => "0644") }
    end

    context "when only browser_ssl_cert is set" do
      before :each do
        params['browser_ssl_cert'] = '/some/path/to/a/browser.cert.pem'
      end

      it { should compile.and_raise_error(/browser_ssl_private_key must also be set/) }
    end
    context "when only browser_ssl_private_key is set" do
      before :each do
        params['browser_ssl_private_key'] = '/some/path/to/a/browser.private_key.pem'
      end

      it { should compile.and_raise_error(/browser_ssl_cert must also be set/) }
    end
  end
end
