require 'spec_helper'
require 'digest/sha1'


describe 'puppet_enterprise::profile::database' do
  require 'puppet/resource/catalog'
  require 'puppet/indirector/memory'
  class ::Puppet::Resource::Catalog::StoreConfigsTesting < Puppet::Indirector::Memory; end

  RSpec.shared_context('pe-conf') do
    let(:tempdir) { Dir.mktmpdir('hiera') }
    let(:hiera_config) { "#{tempdir}/hiera.yaml" }

    around(:each) do |example|
      hiera_yaml = <<-HIERA
version: 5
defaults:  # Used for any hierarchy level that omits these keys.
  datadir: .
  data_hash: hocon_data  # Use the built-in hocon backend.
hierarchy:
  - name: "Puppet Enterprise configuration"
    path: "pe.conf"
      HIERA
      File.write("#{tempdir}/hiera.yaml", hiera_yaml)
      File.write("#{tempdir}/pe.conf", pe_conf)

      begin
        example.run
      ensure
        FileUtils.rm_rf(tempdir, :secure => true)
      end
    end
  end

  let(:facts) do
    {
      'memorysize'        => '2.00 GB',
      'memory'            => { 'system' => { 'total_bytes' => 2147483648 } },
      'processors'        => { 'count' => 4 },
    }
  end
  let(:params) do
    {
      :certname => certname,
    }
  end
  let(:certname) { 'puppetdb.local.domain' }
  let(:certname_sha) { Digest::SHA1.hexdigest(certname) }
  let(:pglogical_node_name) { sprintf('n%.14s', certname_sha) }
  let(:pglogical_subscription_name) { sprintf('s%.14s', certname_sha) }
  let(:sharedir) { '/etc/puppetlabs/puppet/ssl' }
  let(:pgsqldir) { '/opt/puppetlabs/server/data/postgresql/9.6/data' }

  context "validating parameters" do
    context "listen_addresses" do
      before(:each) do
        params.merge!({ 'database_listen_addresses' => 'localhost' })
      end

      it { should contain_class('pe_postgresql::server').with_listen_addresses('localhost') }
    end
  end

  context "when the parameters are valid" do
    it do
      should contain_class('pe_postgresql::server')
        .with(:listen_addresses => '*')
        .with(:package_ensure  => 'latest')
    end
    it { should contain_class('pe_postgresql::server::contrib').with('package_ensure' => 'latest') }
    it do
      should contain_class('pe_postgresql::client')
        .with('package_ensure' => 'latest')
        .with('package_name'   => 'pe-postgresql96')
        .with('bindir'         => '/opt/puppetlabs/server/bin')
    end
    it do
      should contain_package('pe-puppet-enterprise-release')
        .with_before([
          'Class[Pe_postgresql::Client]',
          'Class[Pe_postgresql::Server]',
          'Class[Pe_postgresql::Server::Contrib]',
          'Package[pe-postgresql-common]',
        ])
    end
    it do
      should contain_package('pe-postgresql96-pgrepack')
        .with_require([
          'Package[postgresql-server]',
        ])
    end
    it do
      should contain_package('pe-postgresql96-pglogical')
        .with_require([
          'Package[postgresql-server]',
        ])
    end
    it do
      is_expected.to contain_package('pe-postgresql-common')
        .with_before([
          'Class[Pe_postgresql::Client]',
          'Class[Pe_postgresql::Server]',
          'Class[Pe_postgresql::Server::Contrib]',
        ])
    end

    it { should contain_service('postgresqld').with('name'                    => 'pe-postgresql', 'ensure' => 'running', 'enable' => true) }
    it { should contain_file('/opt/puppetlabs/server/data/postgresql/9.6').with(:ensure => 'directory') }

    context "when managing the databases" do
      before(:each) do
        params.merge!({ 'classifier_database_password' => 'blah',
                                     'rbac_database_password' => 'blah',
                                     'activity_database_password' => 'blah',
                                     'orchestrator_database_password' => 'blah',
                                     'inventory_database_password' => 'blah', })
      end

      it { should contain_pe_postgresql__server__tablespace('pe-puppetdb') }
      it { should contain_pe_postgresql__server__role('pe-puppetdb').with_superuser(false) }
      it { should contain_pe_postgresql__server__db('pe-puppetdb').with_user('pe-puppetdb') }
      it { should contain_puppet_enterprise__pg__extension('pe-puppetdb/pg_trgm').with_database('pe-puppetdb').with_extension('pg_trgm') }
      it { should contain_puppet_enterprise__pg__extension('pe-puppetdb/pgcrypto').with_database('pe-puppetdb').with_extension('pgcrypto') }

      it { should contain_pe_postgresql__server__tablespace('pe-classifier') }
      it { should contain_pe_postgresql__server__role('pe-classifier').with_superuser(true) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-classifier-read').with_write_access(false) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-classifier-write').with_write_access(true) }
      it { should contain_pe_postgresql__server__db('pe-classifier').with_user('pe-classifier') }

      it { should contain_pe_postgresql__server__tablespace('pe-rbac') }
      it { should contain_pe_postgresql__server__role('pe-rbac').with_superuser(true) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-rbac-read').with_write_access(false) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-rbac-write').with_write_access(true) }
      it { should contain_pe_postgresql__server__db('pe-rbac').with_user('pe-rbac') }
      it { should contain_puppet_enterprise__pg__extension('pe-rbac/citext').with_database('pe-rbac').with_extension('citext') }
      it { should contain_puppet_enterprise__pg__extension('pe-rbac/pgcrypto').with_database('pe-rbac').with_extension('pgcrypto') }

      it { should contain_pe_postgresql__server__tablespace('pe-activity') }
      it { should contain_pe_postgresql__server__role('pe-activity').with_superuser(true) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-activity-read').with_write_access(false) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-activity-write').with_write_access(true) }
      it { should contain_pe_postgresql__server__db('pe-activity').with_user('pe-activity') }

      it { should contain_pe_postgresql__server__tablespace('pe-orchestrator') }
      it { should contain_pe_postgresql__server__role('pe-orchestrator').with_superuser(true) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-orchestrator-read').with_write_access(false) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-orchestrator-write').with_write_access(true) }
      it { should contain_pe_postgresql__server__db('pe-orchestrator').with_user('pe-orchestrator') }

      it { should contain_pe_postgresql__server__tablespace('pe-inventory') }
      it { should contain_pe_postgresql__server__role('pe-inventory').with_superuser(true) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-inventory-read').with_write_access(false) }
      it { should contain_puppet_enterprise__pg__ordinary_user('pe-inventory-write').with_write_access(true) }
      it { should contain_pe_postgresql__server__db('pe-inventory').with_user('pe-inventory') }

      it { should satisfy_all_relationships }
    end

    context "when setting database_ssl to false" do
      let(:pre_condition) {
      <<-PRE_COND
        class {'puppet_enterprise':
          puppet_master_host => 'master.rspec',
          database_ssl       => false,
        }
      PRE_COND
      }

      it { should_not compile.and_raise_error('You may not set $puppet_enterprise::database_ssl to false when PE manages PostgreSQL.')  }
    end

    context "when setting database names equal to each other" do
      before(:each) do
        params.merge!({ 'puppetdb_database_name' => 'blah',
                                     'classifier_database_name' => 'blah',
                                     'rbac_database_name' => 'blah',
                                     'activity_database_name' => 'blah',
                                     'orchestrator_database_name' => 'blah',
                                     'inventory_database_name' => 'blah', })
      end

      it { should_not compile.and_raise_error('Database names must be unique')  }
    end

    context "when hacking around RedHat-related bugs in the puppetlabs-postgresql module" do
      before :each do
        facts['os'] = {
          'family'  => 'RedHat',
          'name'    => 'RedHat',
          'release' => {
            'major' => '6'
          },
        }
      end

      it { should contain_file('/etc/sysconfig/pgsql').with(:ensure => 'directory') }
      it { should contain_file('/etc/sysconfig/pe-pgsql').with(:ensure => 'directory') }
      it { should contain_file('/etc/sysconfig/pe-pgsql/pe-postgresql').with(:ensure => 'link', :target => '/etc/sysconfig/pgsql/postgresql') }
    end

    context "managing certs" do
      it { should contain_file("#{pgsqldir}/certs").with_mode('0600') }
      it { should contain_file("#{pgsqldir}/certs/_local.cert.pem").with(:source => "#{sharedir}/certs/#{certname}.pem", :mode => '0400') }
      it { should contain_file("#{pgsqldir}/certs/_local.private_key.pem").with(:source => "#{sharedir}/private_keys/#{certname}.pem", :mode => '0400') }
    end

    context "when configuring postgresql.conf" do
      it { should contain_pe_postgresql__server__config_entry('ssl').with(:value => 'on') }
      it { should contain_pe_postgresql__server__config_entry('ssl_cert_file')
                   .with(:value => "/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.cert.pem") }
      it { should contain_pe_postgresql__server__config_entry('ssl_key_file')
                   .with(:value => "/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.private_key.pem") }
      it { should contain_pe_postgresql__server__config_entry('ssl_ciphers') }
      it { should contain_pe_postgresql__server__config_entry('effective_cache_size').with(:value => '921MB') }
      it { should contain_pe_postgresql__server__config_entry('shared_buffers').with(:value => '384MB') }
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '256MB') }
      it { should contain_pe_postgresql__server__config_entry('wal_buffers').with(:value => '8MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '4MB') }
      it { should contain_pe_postgresql__server__config_entry('max_wal_size').with(:value => '6144MB') }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_segments').with(:ensure => :absent) }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_completion_target').with(:value => '0.90') }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_timeout').with(:value => '10min') }
      it { should contain_pe_postgresql__server__config_entry('log_min_duration_statement').with(:value => '5000') }
      it { should contain_pe_postgresql__server__config_entry('max_connections').with(:value => '400') }
      it { should contain_pe_postgresql__server__config_entry('log_line_prefix').with(:value => '%m [db:%d,sess:%c,pid:%p,vtid:%v,tid:%x] ') }
      it { should contain_pe_postgresql__server__config_entry('ssl_ca_file').with(:value => "/etc/puppetlabs/puppet/ssl/certs/ca.pem") }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_vacuum_scale_factor').with(:value => '0.08') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_analyze_scale_factor').with(:value => '0.04') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '3') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '85MB') }
      it { should contain_pe_postgresql__server__config_entry('log_checkpoints').with(:value => 'on') }
      it { should contain_pe_postgresql__server__config_entry('log_connections').with(:value => 'on') }
      it { should contain_pe_postgresql__server__config_entry('log_disconnections').with(:value => 'on') }
      it { should contain_pe_postgresql__server__config_entry('log_lock_waits').with(:value => 'on') }
      it { should contain_pe_postgresql__server__config_entry('log_temp_files').with(:value => '4096') }
      it { should contain_pe_postgresql__server__config_entry('log_autovacuum_min_duration').with(:value => '0') }
      it { should contain_pe_postgresql__server__config_entry('wal_level').with(:value => 'logical') }
      it { should contain_pe_postgresql__server__config_entry('shared_preload_libraries').with(:value => 'pglogical') }
    end

    context "when configuring postgresql.conf with custom parameters" do
      before :each do
        params['shared_preload_libraries'] = ['pglogical', 'auto_explain']
        params['wal_level'] = 'hot_standby'
      end
      it { should contain_pe_postgresql__server__config_entry('wal_level').with(:value => 'hot_standby') }
      it { should contain_pe_postgresql__server__config_entry('shared_preload_libraries').with(:value => 'pglogical,auto_explain') }
    end

    context "when configuring postgresql.conf with auto_explain settings" do
      before :each do
        params['auto_explain_settings'] = { 'auto_explain.log_min_duration' => '10s',
                                            'auto_explain.log_verbose'      => 'true' }
      end
      it { should contain_pe_postgresql__server__config_entry('auto_explain.log_min_duration').with(:value => '10s') }
      it { should contain_pe_postgresql__server__config_entry('auto_explain.log_verbose').with(:value => 'true') }
      it { should contain_pe_postgresql__server__config_entry('shared_preload_libraries').with(:value => 'pglogical,auto_explain') }
    end

    context "when configuring postgresql.conf with 8GB of Memory and 8 CPUs" do
      before :each do
        facts['memory'] = { 'system' => { 'total_bytes' => 8589934592 } }
        facts['memorysize'] = '8 GiB'
        facts['processors'] = { 'count' => 8 }
      end
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '1024MB') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '3') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '341MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '4MB') }

    end

    context "when configuring postgresql.conf with 16GB of Memory and 16 CPUs" do
      before :each do
        facts['memory'] = { 'system' => { 'total_bytes' => 17179869184 } }
        facts['memorysize'] = '16 GiB'
        facts['processors'] = { 'count' => 16 }
      end
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '2048MB') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '5') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '409MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '4MB') }
    end

    context "when configuring postgresql.conf with 32GB of Memory and 32 CPUs" do
      before :each do
        facts['memory'] = { 'system' => { 'total_bytes' => 34359738368 } }
        facts['memorysize'] = '32 GiB'
        facts['processors'] = { 'count' => 32 }
      end
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '4096MB') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '8') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '512MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '4MB') }
    end

    context "when configuring postgresql.conf with 64GB of Memory and 32 CPUs" do
      before :each do
        facts['memory'] = { 'system' => { 'total_bytes' => 68719476736 } }
        facts['memorysize'] = '64 GiB'
        facts['processors'] = { 'count' => 32 }
      end
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '8192MB') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '8') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '1024MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '8MB') }
      it { should contain_pe_postgresql__server__config_entry('log_temp_files').with(:value => '8192') }
    end

    context "when configuring postgresql.conf with 128GB of Memory and 32 CPUs" do
      before :each do
        facts['memory'] = { 'system' => { 'total_bytes' => 137438953472 } }
        facts['memorysize'] = '128 GiB'
        facts['processors'] = { 'count' => 32 }
      end
      it { should contain_pe_postgresql__server__config_entry('maintenance_work_mem').with(:value => '16384MB') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_max_workers').with(:value => '8') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_work_mem').with(:value => '2048MB') }
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '16MB') }
      it { should contain_pe_postgresql__server__config_entry('log_temp_files').with(:value => '16384') }
    end

    context "when configuring postgresql.conf with 256GB of Memory" do
      before :each do
        facts['memorysize'] = '256 GiB'
      end
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '16MB') }
    end

    context "when configuring postgresql.conf with a custom work_mem, log_temp_files should match" do
      before :each do
        params['work_mem'] = '34MB'
      end
      it { should contain_pe_postgresql__server__config_entry('work_mem').with(:value => '34MB') }
      it { should contain_pe_postgresql__server__config_entry('log_temp_files').with(:value => '34816') }
    end

    context "when configuring postgresql.conf with custom settings" do
      before :each do
        params.merge!( { 'max_connections'                  => 543,
                         'autovacuum_vacuum_scale_factor'   => 0.15,
                         'autovacuum_analyze_scale_factor'  => 0.07,
                         'log_checkpoints'                  => 'off',
                         'log_connections'                  => 'off',
                         'log_disconnections'               => 'off',
                         'log_lock_waits'                   => 'off',
                         'log_temp_files'                   => 7,
                         'log_autovacuum_min_duration'      => 7,
                         'checkpoint_segments'              => 192,
                         'checkpoint_completion_target'     => 0.6,
                         'checkpoint_timeout'               => '21min' } )
      end
      it { should contain_pe_postgresql__server__config_entry('max_connections').with(:value => '543') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_vacuum_scale_factor').with(:value => '0.15') }
      it { should contain_pe_postgresql__server__config_entry('autovacuum_analyze_scale_factor').with(:value => '0.07') }
      it { should contain_pe_postgresql__server__config_entry('log_checkpoints').with(:value => 'off') }
      it { should contain_pe_postgresql__server__config_entry('log_connections').with(:value => 'off') }
      it { should contain_pe_postgresql__server__config_entry('log_disconnections').with(:value => 'off') }
      it { should contain_pe_postgresql__server__config_entry('log_lock_waits').with(:value => 'off') }
      it { should contain_pe_postgresql__server__config_entry('log_temp_files').with(:value => '7') }
      it { should contain_pe_postgresql__server__config_entry('log_autovacuum_min_duration').with(:value => '7') }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_segments').with(:ensure => :absent) }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_completion_target').with(:value => '0.60') }
      it { should contain_pe_postgresql__server__config_entry('checkpoint_timeout').with(:value => '21min') }
    end

    context "when configuring postgresql.conf with custom max_connections setting below 200" do
      before :each do
        params['max_connections'] = 1
      end
      it { should contain_pe_postgresql__server__config_entry('max_connections').with(:value => '200') }
    end

    context "when configuring postgresql.conf with low memory" do
      before :each do
        facts['memorysize'] = '512MB'
      end

      it { should contain_pe_postgresql__server__config_entry('effective_cache_size').with(:value => '128MB') }
      it { should contain_pe_postgresql__server__config_entry('shared_buffers').with(:value => '32MB') }
    end

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-puppetdb as pe-puppetdb")
                .with(:user => 'pe-puppetdb',
                      :database => 'pe-puppetdb',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-activity as pe-activity")
                .with(:user => 'pe-activity',
                      :database => 'pe-activity',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-classifier as pe-classifier")
                .with(:user => 'pe-classifier',
                      :database => 'pe-classifier',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-rbac as pe-rbac")
                .with(:user => 'pe-rbac',
                      :database => 'pe-rbac',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-orchestrator as pe-orchestrator")
                .with(:user => 'pe-orchestrator',
                      :database => 'pe-orchestrator',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it {should contain_puppet_enterprise__pg__cert_whitelist_entry("Allow puppetdb.local.domain to connect to pe-inventory as pe-inventory")
                .with(:user => 'pe-inventory',
                      :database => 'pe-inventory',
                      :allowed_client_certname => 'puppetdb.local.domain')}

    it { should contain_pe_postgresql__server__role('pe-activity-read') }
    it { should contain_puppet_enterprise__pg__default_read_grant('pe-activity grant read perms on new objects from pe-activity to pe-activity-read')
                 .with(:database => 'pe-activity',
                       :schema => 'public',
                       :table_creator => 'pe-activity',
                       :table_reader => 'pe-activity-read') }

    it { should contain_puppet_enterprise__pg__default_read_grant('pe-activity grant read perms on new objects from pe-ha-replication to pe-activity-read')
                 .with(:database => 'pe-activity',
                       :schema => 'public',
                       :table_creator => 'pe-ha-replication',
                       :table_reader => 'pe-activity-read') }

    context "when replication_mode is source" do
      before :each do
        params[:replication_mode] = 'source'
        params[:replica_hostnames] = ['replica.rspec']
      end

      it {
        ['activity', 'classifier', 'rbac', 'orchestrator', 'inventory'].each do |app|
          should contain_puppet_enterprise__app_database(app).with(:replication_mode => 'source')
        end
      }

      it { should contain_puppet_enterprise__pg__extension('pe-activity/pglogical')
                   .with(:extension => 'pglogical') }

      it { should contain_puppet_enterprise__pg__pglogical__node("pe-activity/#{pglogical_node_name}")
                   .with(:database => 'pe-activity',
                         :node_name => pglogical_node_name,
                         :host => 'puppetdb.local.domain',
                         :user => 'pe-ha-replication',
                         :ssl_cert_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.cert.pem',
                         :ssl_key_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.private_key.pem',
                         :ssl_ca_file => '/etc/puppetlabs/puppet/ssl/certs/ca.pem')}

      it {should contain_puppet_enterprise__pg__pglogical__subscription("pe-activity/#{pglogical_subscription_name}")
                  .with(:ensure => 'absent',
                        :database => 'pe-activity',
                        :subscription_name => pglogical_subscription_name,
                        :keepalives_idle => 30,
                        :keepalives_count => 2) }

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow replication connections by pe-ha-replication from replica.rspec')
                   .with(:user => 'pe-ha-replication',
                         :database => 'replication',
                         :allowed_client_certname => 'replica.rspec')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow puppetdb.local.domain to connect to pe-activity as pe-activity')
                   .with(:user => 'pe-activity',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'puppetdb.local.domain')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow puppetdb.local.domain to connect to pe-activity as pe-ha-replication')
                   .with(:user => 'pe-ha-replication',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'puppetdb.local.domain')}
    end

    context "when replication_mode is replica" do
      let(:certname) { 'replica.rspec' }

      before :each do
        params[:replication_mode] = 'replica'
        params[:replication_source_hostname] = 'master.rspec'
      end

      it {
        ['activity', 'classifier', 'rbac', 'orchestrator', 'inventory'].each do |app|
          should contain_puppet_enterprise__app_database(app).with(:replication_mode => 'replica')
        end
      }

      it { should contain_puppet_enterprise__pg__extension('pe-activity/pglogical')
                   .with(:extension => 'pglogical') }

      it { should contain_puppet_enterprise__pg__pglogical__node("pe-activity/#{pglogical_node_name}")
                   .with(:database => 'pe-activity',
                         :node_name => pglogical_node_name,
                         :host => 'replica.rspec',
                         :user => 'pe-ha-replication',
                         :ssl_cert_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.cert.pem',
                         :ssl_key_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.private_key.pem',
                         :ssl_ca_file =>  '/etc/puppetlabs/puppet/ssl/certs/ca.pem')}

      it { should contain_puppet_enterprise__pg__pglogical__subscription("pe-activity/#{pglogical_subscription_name}")
                   .with(:ensure => 'present',
                         :database => 'pe-activity',
                         :subscription_name => pglogical_subscription_name,
                         :host => 'master.rspec',
                         :user => 'pe-ha-replication',
                         :ssl_cert_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.cert.pem',
                         :ssl_key_file => '/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.private_key.pem',
                         :ssl_ca_file => '/etc/puppetlabs/puppet/ssl/certs/ca.pem')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow replica.rspec to connect to pe-activity as pe-activity')
                   .with(:user => 'pe-activity',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'replica.rspec')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow replica.rspec to connect to pe-activity as pe-activity-read')
                   .with(:user => 'pe-activity-read',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'replica.rspec')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow replica.rspec to connect to pe-activity as pe-ha-replication')
                   .with(:user => 'pe-ha-replication',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'replica.rspec')}

      it { should contain_puppet_enterprise__pg__cert_whitelist_entry('Allow master.rspec to connect to pe-activity as pe-ha-replication')
                   .with(:user => 'pe-ha-replication',
                         :database => 'pe-activity',
                         :allowed_client_certname => 'master.rspec')}

      context 'and using experimental two-way replication' do
        include_context('pe-conf')

        let(:pe_conf) do
          <<-PE_CONF
            "puppet_enterprise::ha_two_way_replication": true
          PE_CONF
        end

        it do
          is_expected.to contain_puppet_enterprise__pg__cert_whitelist_entry('Allow replication connections by pe-ha-replication from master.rspec')
        end
      end
    end

    context "when configuring pg_hba.conf" do
      it { should contain_pe_postgresql__server__pg_hba_rule('local access as pe-postgres user').with(
        :type        => 'local',
        :database    => 'all',
        :user        => 'pe-postgres',
        :auth_method => 'peer',
      )}
      it { should_not contain_pe_postgresql__server__pg_hba_rule('allow access to all ipv6') }
      it { should_not contain_pe_postgresql__server__pg_hba_rule('allow access to all users') }
    end

    context "with cert-based auth disabled" do
      let(:pre_condition) {
      <<-PRE_COND
        class {'puppet_enterprise':
          puppet_master_host => 'master.rspec',
          database_cert_auth => false,
        }
      PRE_COND
      }
      before(:each) do
        params.merge!({ 'puppetdb_database_password' => 'blah' })
      end

      it { should contain_puppet_enterprise__app_database('puppetdb').with_database_password('blah') }

      context "configuring pg_hba.conf" do
        it { should contain_pe_postgresql__server__pg_hba_rule('local access as pe-postgres user').with(
          :type        => 'local',
          :database    => 'all',
          :user        => 'pe-postgres',
          :auth_method => 'peer',
        )}
        it { should contain_pe_postgresql__server__pg_hba_rule('allow access to all ipv6').with(
          :type        => 'hostssl',
          :database    => 'all',
          :user        => 'all',
          :auth_method => 'md5',
        )}
        it { should contain_pe_postgresql__server__pg_hba_rule('allow access to all users').with(
          :type        => 'hostssl',
          :database    => 'all',
          :user        => 'all',
          :auth_method => 'md5',
        )}
      end
    end

    context "with cert-based auth and password auth enabled" do
      let(:pre_condition) {
      <<-PRE_COND
        class {'puppet_enterprise':
          puppet_master_host => 'master.rspec',
          database_cert_auth => true,
        }
      PRE_COND
      }
      before(:each) do
        params.merge!({ 'puppetdb_database_password' => 'blah' })
      end
      it { should_not compile.and_raise_error('$puppetdb_database_password and $puppet_enterprise::database_cert_auth are both set. You may not use both certificate and password authentication')  }
    end

    context "with cert-based auth and password auth both disabled" do
      let(:pre_condition) {
      <<-PRE_COND
        class {'puppet_enterprise':
          puppet_master_host => 'master.rspec',
          database_cert_auth => false,
        }
      PRE_COND
      }
      it { should_not compile.and_raise_error('You must chose a PuppetDB database authetication method. Please set $puppet_enterprise::database_cert_auth or $puppetdb_database_password')  }
    end

    context "when adding additional pg_hba_rules" do
      before :each do
        params['additional_pg_hba_rules'] = { 'test_rule_1' => { 'type' => 'host',
                                                                 'database' => 'all',
                                                                 'user'     => 'all',
                                                                 'auth_method' => 'md5',
                                                                 'address'  => '0.0.0.0/24',
                                                                 'order'    => '50' },
                                              'test_rule_2' => { 'type' => 'host',
                                                                 'database' => 'postgres',
                                                                 'user'     => 'my_user',
                                                                 'address'  => '127.0.0.1',
                                                                 'auth_method' => 'ident',
                                                                 'order'    => '51' },
                                             }
      end
        it { should contain_pe_postgresql__server__pg_hba_rule('test_rule_1').with(
          :type        => 'host',
          :database    => 'all',
          :user        => 'all',
          :auth_method => 'md5',
          :order       => '50'
        )}
        it { should contain_pe_postgresql__server__pg_hba_rule('test_rule_2').with(
          :type        => 'host',
          :database    => 'postgres',
          :user        => 'my_user',
          :auth_method => 'ident',
          :order       => '51'
        )}
    end

    context "when configuring individaul databases" do
      it { should contain_puppet_enterprise__app_database('puppetdb').with(
        :extensions => ['pg_repack','pg_trgm','pgcrypto'],
        ) }
      it { should contain_puppet_enterprise__app_database('classifier').with(
        :extensions => ['pg_repack'],
        ) }
      it { should contain_puppet_enterprise__app_database('rbac').with(
        :extensions => ['pg_repack','citext','pgcrypto'],
        ) }
      it { should contain_puppet_enterprise__app_database('activity').with(
        :extensions => ['pg_repack'],
        ) }
      it { should contain_puppet_enterprise__app_database('orchestrator').with(
        :extensions => ['pg_repack'],
        ) }
      it { should contain_puppet_enterprise__app_database('inventory').with(
        :extensions => ['pg_repack','pgcrypto'],
        ) }
    end

    context "when configuring max_wal_size" do

      it do
        should contain_pe_postgresql__server__config_entry('checkpoint_segments')
          .with_ensure('absent')
      end

      context "no max_wal_size" do
        context "and no checkpoint_segments" do
          it do
            should contain_pe_postgresql__server__config_entry('max_wal_size')
             .with_value('6144MB')
          end
        end

        context "and checkpoint_segments greater than 6144MB" do
          before(:each) do
            params.merge!({ 'checkpoint_segments' => '192' })
          end

          it do
            should contain_pe_postgresql__server__config_entry('max_wal_size')
             .with_value('9216MB')
          end
        end

        context "and checkpoint_segments less than 1000MB" do
          before(:each) do
            params.merge!({ 'checkpoint_segments' => '16' })
          end

          it do
            should contain_pe_postgresql__server__config_entry('max_wal_size')
             .with_value('6144MB')
          end
        end
      end

      context "with max_wal_size" do
        context "and no checkpoint_segments"
        context "and checkpoint_segments greater than max_wal_size"
        context "and checkpoint_segments less than max_wal_size"
      end
    end

    context "when dynamically adding puppetdb_hosts to pg_hba.conf" do
      let(:node) { 'database.storeconfigstrue.1.test' }
      before :each do
        Puppet.settings[:storeconfigs] = true
        Puppet.settings[:storeconfigs_backend] = 'store_configs_testing'

        Puppet::Parser::Functions.newfunction(:puppetdb_query,
                                              :type => :rvalue,
                                              :arity => 1) do |args|
          [{'certname' => 'dbquery_puppetdb_1'},
           {'certname' => 'dbquery_puppetdb_2'}].shuffle
        end
      end
      it { should contain_puppet_enterprise__app_database('puppetdb').with(
        :client_certnames => ['puppetdb.local.domain', 'dbquery_puppetdb_1',
                              'dbquery_puppetdb_2', 'puppetdb.rspec'],
        ) }
    end

    context "when setting puppetdb_hosts parameter puppetdb_hosts are not gathered from puppetdb_query" do
      before :each do
        params['puppetdb_hosts'] = ['my_test_host']
      end
      it { should contain_puppet_enterprise__app_database('puppetdb').with(
        :client_certnames => ['puppetdb.local.domain', 'my_test_host'],
        ) }
    end

    context 'multi version packaging' do

      RSpec.shared_examples('common package') do
        it do
          is_expected.to contain_package('pe-puppet-enterprise-release')
            .with_before([
              'Class[Pe_postgresql::Client]',
              'Class[Pe_postgresql::Server]',
              'Class[Pe_postgresql::Server::Contrib]',
              'Package[pe-postgresql-common]'
            ])
        end

        it do
          is_expected.to contain_package('pe-postgresql-common')
            .with_before([
              "Class[Pe_postgresql::Client]",
              "Class[Pe_postgresql::Server]",
              "Class[Pe_postgresql::Server::Contrib]",
            ])
        end

        it do
          is_expected.to contain_pe_postgresql__server__database('pe-postgres')
        end
      end

      context 'installing pe-postgresql11' do
        include_context('pe-conf')

        let(:pe_conf) do
          <<-PE_CONF
            "puppet_enterprise::postgres_version_override": "11"
          PE_CONF
        end

        include_examples('common package')

        it do
          is_expected.to contain_package('postgresql-client')
            .with_name('pe-postgresql11')
        end

        it do
          is_expected.to contain_package('postgresql-server')
            .with_name('pe-postgresql11-server')
        end

        it do
          is_expected.to contain_package('postgresql-contrib')
            .with_name('pe-postgresql11-contrib')
        end

        it do
          is_expected.to contain_package('pe-postgresql11-pgrepack')
        end

        it do
          is_expected.to_not contain_package('pe-postgresql11-pglogical')
        end

        it do
          is_expected.to_not contain_pe_postgresql__server__config_entry('shared_preload_libraries')
            .with_value('pglogical')
        end
      end
    end
  end
end
