require 'spec_helper'

describe 'puppet_enterprise::profile::master' do
  before :each do
    Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
      '1.3.3'
    end
  end

  let(:facts) do
    {
      'fqdn'            => 'master.rspec',
      'clientcert'      => 'awesomecert',
      'aio_agent_build' => '1.3.3',
    }
  end
  let(:params) do
    {
      'console_host'                 => 'console.rspec',
      'certname'                     => 'master.rspec',
      'console_client_certname'      => 'console.rspec',
      'console_server_certname'      => 'blah.rspec',
      'classifier_client_certname'   => 'classifier.rspec',
      'orchestrator_client_certname' => 'orchestrator.rspec',
      'puppetdb_host'                => ['puppetdb.rspec'],
      'ca_host'                      => 'ca.rspec',
      'metrics_graphite_host'                    => 'graphite',
      'metrics_graphite_port'                    => 2003,
      'metrics_graphite_update_interval_seconds' => 30,
    }
  end

  context "managing files" do
    it { should contain_pe_hocon_setting('metrics.reporters.graphite.host').with_value('graphite') }
    it { should contain_pe_hocon_setting('metrics.reporters.graphite.port').with_value(2003) }
    it { should contain_pe_hocon_setting('metrics.reporters.graphite.update-interval-seconds').with_value(30) }
  end

  context "managing crl" do
    around(:each) do |example|
      begin
        cacrl = Tempfile.new('cacrl')
        File.write(cacrl, "blah blah blah I'm a CRL blah blah blah")
        Puppet.settings[:cacrl] = cacrl.path
        hostcrl = Tempfile.new('hostcrl')
        File.open(hostcrl, 'w') { |f| f.write "I'm another CRL that doesn't blah so much" }
        Puppet.settings[:hostcrl] = hostcrl.path
        example.run
      ensure
        cacrl.unlink
        hostcrl.unlink
      end
    end

    context 'puppet agent -t' do
      before :each do
        facts['servername'] = 'master.rspec'
      end

      it { should contain_file("/etc/puppetlabs/puppet/ssl/crl.pem").with_content("blah blah blah I'm a CRL blah blah blah") }
    end

    context 'puppet apply' do
      it { should_not contain_file("/etc/puppetlabs/puppet/ssl/crl.pem").with_content("blah blah blah I'm a CRL blah blah blah") }
    end

    context 'without crl files' do
      before :each do
        facts['servername'] = 'master_no_crl.rspec'
        Puppet.settings[:cacrl] = 'nonexistant'
        Puppet.settings[:hostcrl] = 'nonexistant'
      end

      it { should_not contain_file("/etc/puppetlabs/puppet/ssl/crl.pem") }
    end
  end

  context "with classifier" do
    before :each do
      params['classifier_host'] = 'classifier.rspec'
    end
    it { should contain_class('puppet_enterprise::profile::master::classifier') }
    it { should satisfy_all_relationships }
  end

  context "without classifier" do
    before :each do
      params['classifier_host'] = ''
    end
    it { should_not contain_class('puppet_enterprise::profile::master::classifier') }
  end

  context 'managing auth.conf' do
    it { should contain_class('puppet_enterprise::profile::master::auth_conf')
                 .with_console_client_certname('console.rspec')
                 .with_classifier_client_certname('classifier.rspec')
                 .with_orchestrator_client_certname('orchestrator.rspec')
                 .with_require('Package[pe-puppetserver]')
                 .with_notify('Service[pe-puppetserver]')}
  end

  context "with puppetdb" do
    it { should contain_class('puppet_enterprise::profile::master::puppetdb').with(
                  'puppetdb_host' => ['puppetdb.rspec'],
      ) }
  end

  context "managing puppet.conf" do
    it { should contain_pe_ini_setting('module_groups').with_value('base+pe_only') }
  end

  context "managing master" do
    it { should contain_class('puppet_enterprise::master').with(:certname => params['certname']) }
  end

  context "when using default certname" do
    before :each do
      params.delete('certname')
    end
    it { should contain_class('puppet_enterprise::master').with(:certname => facts['clientcert']) }
  end

  context "determining version match" do
    before(:each) do
      # The hostname of the compiling server
      facts['servername'] = 'master.rspec'
      facts['fqdn'] = 'master.rspec'
    end

    context "on the master of masters (puppet apply)" do
      context "when installing" do
        before :each do
          facts.delete('pe_server_version')
          facts.delete('pe_version')
          # These server variables (which we are pretending are facts) are not
          # present when applying
          facts.delete('servername')
          facts.delete('aio_agent_build')

          Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
            nil
          end
        end

        it "does not raise an error regardless of match so as not to interfere with initial master of master's install/upgrade" do
          should contain_class('puppet_enterprise::profile::master')
        end
      end

      context "when upgrading (puppet apply)" do
        context "from 3.8.x" do
          before :each do
            facts.delete('pe_server_version')
            facts.delete('aio_agent_build')
            facts['pe_version'] = '3.8.2'
          end

          it "does not raise an error regardless of match so as not to interfere with initial master of master's install/upgrade" do
            should contain_class('puppet_enterprise::profile::master')
          end
        end

        context "from 2015.2.x" do
          before :each do
            facts['pe_server_version'] = '2015.2.0'
            facts['aio_agent_build'] = '1.2.2'
            facts.delete('pe_version')
            Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
              '1.2.2'
            end
          end

          it "does not raise an error regardless of match so as not to interfere with initial master of master's install/upgrade" do
            should contain_class('puppet_enterprise::profile::master')
          end
        end
      end

      context "during an agent run (steady state)" do
        before :each do
          facts['pe_server_version'] = '2015.2.1'
          facts['aio_agent_build'] = '1.3.3'
          facts.delete('pe_version')
          Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
            '1.3.3'
          end
        end

        it "compiles" do
          should contain_class('puppet_enterprise::profile::master')
        end
      end

    end

    context "for a compile master" do
      before :each do
        facts['servername'] = 'master.rspec'
        facts['fqdn'] = 'compile-master.rspec'
        Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
          '1.3.3'
        end
      end

      context "when installing (after frictionless agent install)" do
        before :each do
          facts['pe_server_version'] = '2015.2.1'
          facts['aio_agent_build'] = '1.3.3'
          facts.delete('pe_version')
        end

        it "the catalog compiles" do
          should contain_class('puppet_enterprise::profile::master')
        end
      end

      context "when upgrading" do
        context "from 3.8.x" do
          before :each do
            facts.delete('pe_server_version')
            facts.delete('aio_agent_build')
            facts['pe_version'] = '3.8.2'
          end

          it "raises a version match error despite aio_agent_build not being present" do
            expect { catalogue }.to raise_error(Puppet::Error, /Please ensure that the PE versions are consistent across all Puppet masters/)
          end
        end

        context "from 2015.2.x" do
          before :each do
            facts.delete('pe_version')
          end

          context "and master versions are inconsistent" do
            before :each do
              facts['pe_server_version'] = '2015.2.0'
              facts['aio_agent_build'] = '1.2.2'
            end

            it "fails to compile" do
              expect { catalogue }.to raise_error(Puppet::Error, /Please ensure that the PE versions are consistent across all Puppet masters/)
            end
          end

          context "when compiling and compile master versions match" do
            before :each do
              facts['pe_server_version'] = '2015.2.1'
              facts['aio_agent_build'] = '1.3.3'
            end

            it "the catalog compiles" do
              should contain_class('puppet_enterprise::profile::master')
            end
          end
        end
      end

    end

    it { should compile }
  end

  context "providing packages mount" do
    it do
      should contain_augeas('fileserver.conf pe_packages').with_changes([
        "set /files/etc/puppetlabs/puppet/fileserver.conf/pe_packages/path /opt/puppetlabs/server/data/packages/public",
        "set /files/etc/puppetlabs/puppet/fileserver.conf/pe_packages/allow *",
      ])
    end
  end

  context "managing symlinks" do
    before(:each) do
      facts['platform_symlink_writable'] = true
      params['manage_symlinks'] = true
    end

    it { should contain_file('/usr/local/bin/r10k') }
  end

  context "not managing symlinks" do
    before(:each) do
      facts['platform_symlink_writable'] = true
      params['manage_symlinks'] = false
    end

    it { should_not contain_file('/usr/local/bin/r10k') }
  end

  context "not managing symlinks because fact reports target is not writable" do
    before(:each) do
      facts['platform_symlink_writable'] = false
      params['manage_symlinks'] = true
    end

    it { should_not contain_file('/usr/local/bin/r10k') }
  end

  context "managing pe_r10k with remote" do
    before(:each) { params['r10k_remote'] = 'git@gitserver.rspec' }
    it { should contain_package('pe-r10k') }
    it { should contain_file('r10k.yaml').with_content(/.*git@gitserver\.rspec.*/) }
    it { should satisfy_all_relationships }
  end

  context "managing pe_r10k with remote and private key" do
    # The mode for the r10k_private_key should match the mode for the
    # /etc/puppetlabs/puppetserver/ssh directory
    let(:r10k_key_mode) { '0600' }
    before :each do
      params['r10k_remote']      = 'git@gitserver.rspec'
      params['r10k_private_key'] = '/etc/puppetlabs/puppetserver/ssh/key.rspec'
    end
    it { should contain_package('pe-r10k') }
    it { should contain_file('r10k.yaml').with_content(%r[.*/etc/puppetlabs/puppetserver/ssh/key\.rspec.*]) }
    it do
      should contain_puppet_enterprise__set_owner_group_permissions('/etc/puppetlabs/puppetserver/ssh')
        .with_file_mode(r10k_key_mode)
    end
    it do
      should contain_puppet_enterprise__set_owner_group_permissions('/etc/puppetlabs/puppetserver/ssh/key.rspec')
        .with_file_mode(r10k_key_mode)
    end
    it { should satisfy_all_relationships }
  end

  context "managing pe_r10k with proxy server" do
    proxy = 'https://user:p455w0rd@proxy.example.com'
    before :each do
      params['r10k_remote']      = 'git@gitserver.rspec'
      params['r10k_private_key'] = '/path/to/key.rspec'
      params['r10k_proxy'] = proxy
    end
    it { should contain_file('r10k.yaml').with_content(/^proxy:\s+#{proxy}/) }
    it { should satisfy_all_relationships }
  end

  context "managing pe_r10k with postrun command" do
    before :each do
      params['r10k_remote']      = 'git@gitserver.rspec'
      params['r10k_postrun']     = ['curl', 'www.puppet.com']
    end
    it { should contain_file('r10k.yaml').with_content(/\["curl", "www\.puppet\.com"\]/) }
    it { should satisfy_all_relationships }
  end

  context "managing pe_r10k without postrun command" do
    before :each do
      params['r10k_remote']      = 'git@gitserver.rspec'
    end
    it { should_not contain_file('r10k.yaml').with_content(/postrun/) }
    it { should satisfy_all_relationships }
  end

  context 'not managing r10k explicitly using parameters' do
    it { should contain_class('pe_r10k::package') }
    it { should_not contain_file('r10k.yaml') }
  end

  context 'when running on MoM' do
    before(:each) { facts['servername'] = 'master.rspec' }

    context 'when autoconfiguring code_manager' do
      before(:each) {
        params['code_manager_auto_configure'] = true
        params['r10k_git_provider'] = 'shellgit'
      }
      it { should contain_class('puppet_enterprise::master::code_manager') }
      it { should contain_file('r10k.yaml').with_content(/write_lock/) }
      it { should contain_pe_hocon_setting('code-manager.git-provider').with_value('shellgit') }
    end

    context 'when not autoconfiguring code_manager' do
      it { should_not contain_class('puppet_enterprise::master::code_manager') }
      it { should_not contain_class('puppet_enterprise::master::file_sync') }

      context 'when still managing r10k config' do
        before(:each) { params['r10k_remote'] = 'git@gitserver.rspec' }
        it { should contain_file('r10k.yaml').without_content(/write_lock/) }
      end
    end

    context 'when file sync is enabled' do
      before(:each) { params['code_manager_auto_configure'] = true }
      it { should contain_class('puppet_enterprise::master::file_sync').with_storage_service_disabled(false)}
      it { should_not contain_class('puppet_enterprise::master::code_manager').with_file_sync_autocommit(true) }

      context "when there are replicas" do
        before :each do
          params['provisioned_replicas'] = ['replica.dev']
        end

        it { should contain_pe_puppet_authorization__rule('puppetlabs file sync ca-data repo')
                     .with_match_request_path('/file-sync-git/ca-data')
                     .with_allow(['replica.dev'])}
      end
    end

    it "doesn't have pcp-broker" do
      should_not contain_class('puppet_enterprise::master::pcp_broker')
    end

    it { should contain_class('puppet_enterprise::trapperkeeper::analytics') }
  end

  context 'when running on a compile master' do
    before(:each) { facts['servername'] = 'compile.rspec' }

    context 'when autoconfiguring code_manager' do
      before(:each) { params['code_manager_auto_configure'] = true }
      it { should_not contain_class('puppet_enterprise::master::code_manager') }
    end
    context 'when file sync is enabled' do
      before(:each) { params['file_sync_enabled'] = true}
      it { should contain_class('puppet_enterprise::master::file_sync').with_storage_service_disabled(true)}
    end
    context 'when file sync is disabled' do
      before(:each) { params['file_sync_enabled'] = false}
      it { should contain_class('puppet_enterprise::master::file_sync_disabled')}
    end

    context 'managing pcp-broker' do
      it { should contain_class('puppet_enterprise::master::pcp_broker')
                   .with_orchestrator_hosts(['master.rspec']) }

      context "when there are replicas" do
        before :each do
          params['ha_enabled_replicas'] = ['replica.dev']
        end

        it { should contain_class('puppet_enterprise::master::pcp_broker')
                     .with_orchestrator_hosts(['master.rspec', 'replica.dev']) }
      end
    end

    it { should_not contain_class('puppet_enterprise::trapperkeeper::analytics')}
  end

  context 'when running on a primary master' do
    before(:each) {
      params['code_manager_auto_configure'] = true
      params['replication_mode'] = 'source'
      facts['certname'] = 'master.rspec'
      facts['servername'] = 'master.rspec'
    }
    it { should contain_class('puppet_enterprise::master::code_manager') }
  end

  context 'when running on a replica' do
    before(:each) {
      params['code_manager_auto_configure'] = true
      params['replication_mode'] = 'replica'
      facts['certname'] = 'replica.rspec'
    }

    context 'when a replica has failed over' do
      before(:each) {
        facts['servername'] = 'replica.rspec'
      }
      it { should_not contain_class('puppet_enterprise::master::code_manager') }
      it { should_not contain_class('puppet_enterprise::trapperkeeper::analytics') }
    end

    context 'when a replica has not failed over' do
      before(:each) {
        facts['servername'] = 'master.rspec'
      }
      it { should_not contain_class('puppet_enterprise::master::code_manager') }
      it { should_not contain_class('puppet_enterprise::trapperkeeper::analytics') }
    end
  end

  context 'passing through code_manager_auto_configure to the pe::master class' do
    context 'when code_manager_auto_configure false' do
      before(:each) { params['code_manager_auto_configure'] = false }
      it { should contain_class('puppet_enterprise::master').with(:code_manager_auto_configure => false) }
    end
    context 'when code_manager_auto_configure true' do
      before(:each) { params['code_manager_auto_configure'] = true }
      it { should contain_class('puppet_enterprise::master').with(:code_manager_auto_configure => true) }
    end
  end

  context "default environmentpath setting is removed from puppet.conf" do
    it { should contain_pe_ini_setting('environmentpath_setting').with_ensure('absent') }
  end

  context "custom environmentpath setting in puppet.conf" do
    before(:each) { params['environmentpath'] = '/tmp' }
    it { should contain_pe_ini_setting('environmentpath_setting').with_value('/tmp') }
  end

  context "default codedir setting is removed from puppet.conf" do
    it { should contain_pe_ini_setting('codedir_setting').with_ensure('absent') }
  end

  context "custom codedir setting in puppet.conf" do
    before(:each) { params['codedir'] = '/tmp/code' }
    it { should contain_pe_ini_setting('codedir_setting').with_value('/tmp/code') }
  end

  it { should satisfy_all_relationships }

  context "when not enabling app_management" do
    before(:each) {params['app_management'] = 'false' }
    it { should contain_pe_ini_setting("puppetserver puppetconf app_management").with_ensure('absent') }
  end

  context "default for app_management" do
    it { should contain_pe_ini_setting("puppetserver puppetconf app_management").with_ensure('absent') }
  end

  context "removes ankeny app_management config" do
    it { should contain_pe_ini_setting("deprecated puppetserver puppetconf app_management").with_ensure('absent') }
  end

  context "acting as a controller node" do
    it { should contain_class("puppet_enterprise::profile::controller") }
  end

  context "with check_for_updates unspecified" do
    it { should contain_file('/etc/puppetlabs/puppetserver/opt-out').with_ensure('absent').with_require('Package[pe-puppetserver]') }
    it { should satisfy_all_relationships }
  end

  context "with check_for_updates opt out disabled" do
    before :each do
      params['check_for_updates'] = false
    end

    it { should contain_file('/etc/puppetlabs/puppetserver/opt-out').with_ensure('file').with_require('Package[pe-puppetserver]') }
    it { should satisfy_all_relationships }
  end

  context "with multiple puppetdbs" do
    before :each do
      params['puppetdb_host'] = ['pdb1.rspec', 'pdb2.rspec']
      params['puppetdb_port'] = [8081,8081]
    end

    it { should contain_pe_hocon_setting('http-client.connect-timeout-milliseconds')
                 .with_value(5000) }
  end

  context "with multiple classifiers" do
    before :each do
      params['classifier_host'] = ['nc1.rspec', 'nc2.rspec']
      params['classifier_port'] = [4433,4433]
    end

    it { should contain_pe_hocon_setting('http-client.connect-timeout-milliseconds')
                 .with_value(5000) }
  end
end
