require 'spec_helper'

describe 'puppet_enterprise::profile::orchestrator' do
  require 'puppet/resource/catalog'
  require 'puppet/indirector/memory'
  class ::Puppet::Resource::Catalog::StoreConfigsTesting < Puppet::Indirector::Memory; end

  before(:each) do
    Puppet::Parser::Functions.newfunction(:puppetdb_query, :type => :rvalue, :arity => 1) do |args|
      [{'certname' => 'compile.rspec'}]
    end
  end

  let(:facts) do
    {}
  end

  let(:params) do
    {
      'certname' => 'orchestrator.rspec',
      'database_host' => 'db.rspec',
      'database_port' => 5432,
      'master_host' => 'master.rspec',
      'puppetdb_host' => ['puppetdb.rspec'],
      'classifier_host' => 'classifier.rspec',
      'rbac_host' => 'rbac.rspec',
      'activity_host' => 'activity.rspec',
    }
  end
  let(:service) { 'Service[pe-orchestration-services]' }

  context "when managing default java_args" do
    it { should contain_puppet_enterprise__trapperkeeper__java_args('orchestration-services').with(
      :java_args => { 'Xmx' => '704m',
                      'Xms' => '704m', }
      )
    }
  end

  context "with custom java_args" do
    before(:each) { params['java_args'] = { 'Xms' => '128m', 'Xmx' => '333m' }
                    params['enable_gc_logging'] = true }
    it { should contain_puppet_enterprise__trapperkeeper__java_args('orchestration-services').with(
      :java_args => { 'Xmx' => '333m',
                      'Xms' => '128m', },
      :enable_gc_logging => true,
      )
    }
  end

  context "with valid parameters" do
    context "configuring the webserver" do
      it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("pcp-broker")
        .with('container'          => 'orchestration-services',
              'ssl_listen_address' => '0.0.0.0',
              'ssl_listen_port'    => 8142,
              'ssl_cert'           => "/etc/puppetlabs/orchestration-services/ssl/orchestrator.rspec.cert.pem",
              'ssl_key'            => "/etc/puppetlabs/orchestration-services/ssl/orchestrator.rspec.private_key.pem",
              )
      }
      it { should contain_pe_hocon_setting("orchestration-services.web-router-service.broker-service").with(
        :setting => 'web-router-service."puppetlabs.pcp.broker.service/broker-service"',
        :value => {
          'v1' => {
            'route' => '/pcp',
            'server' => 'pcp-broker'
          },
          'v2' => {
            'route' => '/pcp2',
            'server' => 'pcp-broker'
          },
          'metrics' => {
            'route' => '/',
            'server' => 'pcp-broker'
          }
        },
        :notify => service)}
      it { should contain_pe_hocon_setting("orchestration-services.web-router-service.metrics-webservice").with( value: { 'route' => '/metrics', 'server' => 'orchestrator' })}
    end

    context "managing SSL protocols" do
      before :each do
        params['ssl_protocols'] = ['TLSv1.1', 'TLSv1.2']
      end
      it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("pcp-broker")
        .with('ssl_protocols' => ['TLSv1.1', 'TLSv1.2'])
      }
      it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("orchestrator")
        .with('ssl_protocols' => ['TLSv1.1', 'TLSv1.2'])
      }
    end

    it { should contain_puppet_enterprise__certs('orchestration-services').with(:certname => 'master.rspec',
                                                                                :container => 'orchestration-services') }

    context "configuring bootstrap.cfg" do
      it { should contain_pe_concat__fragment('orchestration-services scheduler-service') }
      it { should contain_pe_concat__fragment('orchestration-services status-service') }
      it { should contain_pe_concat__fragment('orchestration-services metrics-service') }
      it { should contain_pe_concat__fragment('orchestration-services metrics-webservice') }
      it { should contain_pe_concat__fragment('orchestration-services webrouting-service') }
    end
    it { should contain_file("/etc/puppetlabs/orchestration-services/conf.d/pcp-broker.conf").with(:owner => 'pe-orchestration-services',
                                                                                                   :group => 'pe-orchestration-services',
                                                                                                   :mode  => '0640') }

    context "configures analytics client" do
      it { should contain_puppet_enterprise__trapperkeeper__analytics_client_settings('orchestration-services')
        .with({ 'container' => 'orchestration-services',
                'owner'     => 'pe-orchestration-services',
              })
      }
    end

    context "managing the global config file" do
      it { should contain_puppet_enterprise__trapperkeeper__global_settings('orchestration-services')
        .with({ 'certname' => 'master.rspec',
                'ssl_cert' => "/etc/puppetlabs/orchestration-services/ssl/master.rspec.cert.pem",
                'ssl_key'  => "/etc/puppetlabs/orchestration-services/ssl/master.rspec.private_key.pem",
                'localcacert' => "/etc/puppetlabs/puppet/ssl/certs/ca.pem",
              })
      }
    end

    context "managing pe-bolt-server" do
      it { should contain_class('puppet_enterprise::profile::bolt_server') }
    end

    context "managing pe-plan-executor" do
      it { should contain_class('puppet_enterprise::profile::plan_executor') }
    end

    context "managing orchestrator config" do
      it { should contain_service('pe-orchestration-services').with(:ensure => 'running', :enable => true) }

      it { should contain_puppet_enterprise__trapperkeeper__orchestrator('orchestration-services')
                   .with(:database_host => 'db.rspec',
                         :database_port => 5432,
                         :database_user => 'pe-orchestrator-write',
                         :classifier_url => 'https://classifier.rspec:4433/classifier-api',
                         :rbac_url => 'https://rbac.rspec:4433/rbac-api',
                         :activity_url => 'https://activity.rspec:4433/activity-api',
                         :notify => service) }
      it { should contain_puppet_enterprise__trapperkeeper__pcp_broker('orchestration-services').with(:accept_consumers => '2',
                                                                                                      :delivery_consumers => '2',
                                                                                                      :controller_uris => ['wss://orchestrator.rspec:8143/server'],
                                                                                                      :notify => service) }

      it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("orchestrator")
        .with('container'          => 'orchestration-services',
              'ssl_listen_address' => '0.0.0.0',
              'ssl_listen_port'    => 8143,
              'ssl_cert'           => "/etc/puppetlabs/orchestration-services/ssl/orchestrator.rspec.cert.pem",
              'ssl_key'            => "/etc/puppetlabs/orchestration-services/ssl/orchestrator.rspec.private_key.pem",
              'default_server'     => true,
              'access_log_config'  => '/etc/puppetlabs/orchestration-services/request-logging.xml',
              )
      }
      it { should contain_pe_hocon_setting("orchestration-services.web-router-service.orchestrator-service")
        .with_value('route' => '/orchestrator/v1', 'server' => 'orchestrator').with_notify(service) }
      it { should contain_pe_hocon_setting("orchestration-services.web-router-service.orchestrator-dispatch-service")
        .with_value('route' => '/server', 'server' => 'orchestrator').with_notify(service) }
      it { should contain_pe_hocon_setting("orchestration-services.web-router-service.status-service")
        .with_value('route' => '/status', 'server' => 'orchestrator').with_notify(service) }

      context "enabling app-management" do
        before(:each) { params['use_application_services'] = true }
        it { should contain_pe_hocon_setting("orchestration-services.orchestrator.app-management").with(:setting => 'orchestrator.app-management',
                                                                                                        :ensure => 'present',
                                                                                                        :value => true) }
      end
    end

    context "when stopping orchestration-services" do
      before(:each) { params['run_service'] = false }
      it { should contain_service('pe-orchestration-services').with(:ensure => 'stopped', :enable => false) }
    end

    context "with storeconfigs enabled" do
      let(:node) { 'orchestrator.storeconfigstrue.1.test' }

      before :each do
        Puppet.settings[:storeconfigs] = true
        Puppet.settings[:storeconfigs_backend] = 'store_configs_testing'

        # We have to change some parameter to cause a recompile with the storeconfigs setting...
        params['pcp_timeout'] = 10
      end

      it { should contain_pe_puppet_authorization__rule('dispatch: allow pcp-brokers').with_allow(['master.rspec', 'compile.rspec']) }
    end
  end
end
