require 'spec_helper'

describe 'puppet_enterprise::profile::plan_executor' do
  let(:pre_condition) do
    <<-PRE
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-orchestration-services': }
    PRE
  end
  let(:orchestration_service) { 'Service[pe-orchestration-services]' }
  let(:ruby_service_name) { 'pe-plan-executor' }

  context 'when using the ruby variant' do
    let(:params) do
      {
        'variant'  => 'ruby',
        'certname' => 'plan.rspec',
        'settings' => { 'workers' => 1,
                        'modulepath' => '/path/to/modules',
                        'orchestrator-url' => 'https://abcde.com:123' }
      }
    end

    it { should contain_puppet_enterprise__orchestrator__ruby_service('plan-executor')
                  .with(:certname => params['certname'],
                        :container => 'plan-executor',
                        :ssl_listen_port => '62659',
                        :settings => params['settings']) }

    it { should contain_pe_hocon_setting('orchestrator.plan-executor-enable')
                  .with_value('external')
                  .that_notifies(orchestration_service) }


    context 'under sysvinit' do
      let(:facts) do
        {
          'os' => {
            'family'  => 'RedHat',
            'release' => {
              'major' => '6'
            }
          }
        }
      end

      it { should contain_file("/etc/rc.d/init.d/#{ruby_service_name}") }
      it { should contain_file("/etc/sysconfig/#{ruby_service_name}").with(replace: false) }
    end

    context 'under systemd' do
      context 'on a debian-based distribution' do
        let(:facts) do
          {
            'os' => {
              'family'  => 'Debian'
            }
          }
        end

        it { should contain_file("/lib/systemd/system/#{ruby_service_name}.service") }
      end

      context 'on a redhat-based distribution' do
        let(:facts) do
          {
            'os' => {
              'family'  => 'RedHat',
              'release' => {
                'major' => '7'
              }
            }
          }
        end

        it { should contain_file("/usr/lib/systemd/system/#{ruby_service_name}.service") }
      end
    end
  end

  context 'when using the jruby variant' do
    let(:params) do
      {
        'variant'                         => 'jruby',
        'modulepath'                      => '/path/to/modules',
        'jruby_compile_mode'              => 'compile-mode',
        'jruby_max_requests_per_instance' => 42,
      }
    end

    it { should_not contain_puppet_enterprise__orchestrator__ruby_service('plan-executor') }

    {
      'orchestrator.plan-executor-enable'            => 'jruby',
      'orchestrator.modulepath'                      => ['/path/to/modules'],
      'orchestrator.url'                             => 'https://master.rspec:8143/orchestrator/v1',
      'orchestrator.jruby.compile-mode'              => 'compile-mode',
      'orchestrator.jruby.gem-home'                  => '/opt/puppetlabs/server/apps/bolt-server/lib/ruby',
      'orchestrator.jruby.gem-path'                  => '/opt/puppetlabs/server/apps/bolt-server/lib/ruby:/opt/puppetlabs/puppet/lib/ruby/vendor_gems:/opt/puppetlabs/puppet/lib/ruby/gems/2.5.0',
      'orchestrator.jruby.max-requests-per-instance' => 42,
      'orchestrator.jruby.ruby-load-path'            => ['/opt/puppetlabs/server/apps/bolt-server/lib/ruby', '/opt/puppetlabs/puppet/lib/ruby/vendor_ruby']
    }.each do |name, value|
      it { should contain_pe_hocon_setting(name).with_value(value).that_notifies(orchestration_service) }
    end
  end

  context 'when using neither variant' do
    let(:params) { {} }

    it { should_not contain_puppet_enterprise__orchestrator__ruby_service('plan-executor') }

    it { should contain_pe_hocon_setting('orchestrator.plan-executor-enable')
                  .with(ensure: 'absent')
                  .that_notifies(orchestration_service) }

    it { should contain_service(ruby_service_name)
                  .with(ensure: 'stopped', enable: false)
                  .that_notifies(orchestration_service) }
  end
end
