require 'spec_helper'

describe 'puppet_enterprise::puppetdb' do
  let(:params) do
    {
      'certname'            => 'puppetdb.local.domain',
      'database_host'       => 'database.local.domain',
      'cert_whitelist_path' => '/etc/puppetlabs/puppetdb/certificate-whitelist',
      'database_port'       => 5432,
      'database_user'       => 'pe-puppetdb',
      'database_properties' => '',
      'command_processing_threads' => 4,
      'concurrent_writes' => 4,
    }
  end
  let(:sharedir) { '/opt/puppet/share/puppet-dashboard' }
  let(:ssldir) { '/etc/puppetlabs/puppet/ssl' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
        database_host                => 'database.local.domain',
      }
      file { '/etc/puppetlabs/puppetdb/certificate-whitelist': }
    PRE_COND
  end

  context "when managing packages" do
    it { should contain_package('pe-java') }
    it { should contain_package('pe-puppetdb') }
  end

  context "when managing default java_args" do
    it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetdb').with(
      :java_args => { 'Xmx' => '256m',
                      'Xms' => '256m', }
      )
    }
  end

  context "with custom java_args" do
    before(:each) { params['java_args'] = { 'Xms' => '128m', 'Xmx' => '333m' }
                    params['enable_gc_logging'] = true }
    it { should contain_puppet_enterprise__trapperkeeper__java_args('puppetdb').with(
      :java_args => { 'Xmx' => '333m',
                      'Xms' => '128m', },
      :enable_gc_logging => true,
      )
    }
  end

  context "managing service defaults" do
    it { should contain_puppet_enterprise__trapperkeeper__init_defaults('puppetdb')
      .with({ 'service_stop_retries' => 60,
              'start_timeout'        => 14400, })
    }

    context "with overrides" do
      before(:each) do
        params['service_stop_retries'] = 12345
        params['start_timeout'] = 67890
      end

      it { should contain_puppet_enterprise__trapperkeeper__init_defaults('puppetdb')
        .with({ 'service_stop_retries' => 12345,
                'start_timeout'        => 67890, })
      }
    end
  end

  it { should contain_class('puppet_enterprise::puppetdb::database_ini')
       .with_node_purge_ttl(nil) }
  it { should contain_class('puppet_enterprise::puppetdb::jetty_ini') }
  it { should contain_class('puppet_enterprise::puppetdb::rbac_consumer_conf') }
  it { should contain_class('puppet_enterprise::puppetdb::service') }
  it { should contain_file('/var/log/puppetlabs/puppetdb').with(
    'ensure' => 'directory',
    'owner'  => 'pe-puppetdb',
    'group'  => 'pe-puppetdb',
    'mode'   => '0640'
  )}
  it { should contain_file('/var/log/puppetlabs/puppetdb/puppetdb.log').with(
    'ensure'  => 'present',
    'owner'   => 'pe-puppetdb',
    'group'   => 'pe-puppetdb',
    'mode'    => '0640',
  )}

  context "when managing node_purge_ttl" do
    before(:each) do
      params['node_purge_ttl'] = '22d'
    end
    it { should contain_class('puppet_enterprise::puppetdb::database_ini')
         .with_node_purge_ttl('22d') }
  end

  context "when not managing sync.ini" do
    it { should_not contain_class('puppet_enterprise::puppetdb::sync_ini') }
  end

  context "when managing sync.ini" do
    before(:each) { params['sync_peers'] = [] }
    it { should contain_class('puppet_enterprise::puppetdb::sync_ini') }
  end

  describe 'when using default values' do
    it { should contain_pe_ini_setting('[read-database]-maximum-pool-size').
         with(
          'ensure'  => 'present',
          'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
          'section' => 'read-database',
          'setting' => 'maximum-pool-size',
          'value'   => 25
    )}
    it { should contain_pe_ini_setting('[read-database]-puppetdb_psdatabase_username').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
           'section' => 'read-database',
           'setting' => 'username',
           'value'   => 'pe-puppetdb'
    )}
    it { should_not contain_pe_ini_setting('[read-database]-puppetdb_psdatabase_password') }
    it { should contain_pe_ini_setting('[read-database]-puppetdb_subname').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
           'section' => 'read-database',
           'setting' => 'subname',
           'value'   => '//database.local.domain:5432/pe-puppetdb'
    )}
  end
  describe 'when setting read database connections' do
    before(:each) do
      params['read_maximum_pool_size'] = 41
    end
    it { should contain_pe_ini_setting('[read-database]-maximum-pool-size').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
           'section' => 'read-database',
           'setting' => 'maximum-pool-size',
           'value'   => 41
    )}
  end
  describe 'when using a box with 32 cores' do
    before(:each) do
      params['command_processing_threads'] = 16
    end
    it { should contain_pe_ini_setting('[database]-maximum-pool-size').
         with(
          'ensure'  => 'present',
          'path'    => '/etc/puppetlabs/puppetdb/conf.d/database.ini',
          'section' => 'database',
          'setting' => 'maximum-pool-size',
          'value'   => 32
    )}
  end
  describe 'when using a box with 32 cores and overwriting maximum-pool-size' do
    before(:each) do
      params['command_processing_threads'] = 16
      params['write_maximum_pool_size']    = 18
    end
    it { should contain_pe_ini_setting('[database]-maximum-pool-size').
         with(
          'ensure'  => 'present',
          'path'    => '/etc/puppetlabs/puppetdb/conf.d/database.ini',
          'section' => 'database',
          'setting' => 'maximum-pool-size',
          'value'   => 18
    )}
  end
  describe 'when setting read and write databases seperately' do
    before(:each) do
      params['database_host']            = 'write-database.local.domain'
      params['read_database_host']       = 'read-database.local.domain'
      params['database_name']            = 'write-pdb-database'
      params['read_database_name']       = 'read-pdb-database'
      params['database_port']            = 1234
      params['read_database_port']       = 5678
      params['database_user']            = 'write-pe-puppetdb'
      params['read_database_user']       = 'read-pe-puppetdb'
      params['database_properties']      = ''
      params['read_database_properties'] = ''
    end

    it { should contain_pe_ini_setting('[read-database]-puppetdb_psdatabase_username').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
           'section' => 'read-database',
           'setting' => 'username',
           'value'   => 'read-pe-puppetdb'
    )}
    it { should contain_pe_ini_setting('[database]-puppetdb_psdatabase_username').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/database.ini',
           'section' => 'database',
           'setting' => 'username',
           'value'   => 'write-pe-puppetdb'
    )}
    it { should_not contain_pe_ini_setting('[read-database]-puppetdb_psdatabase_password') }
    it { should_not contain_pe_ini_setting('[database]-puppetdb_psdatabase_password') }
    it { should contain_pe_ini_setting('[read-database]-puppetdb_subname').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/read_database.ini',
           'section' => 'read-database',
           'setting' => 'subname',
           'value'   => '//read-database.local.domain:5678/read-pdb-database'
    )}
    it { should contain_pe_ini_setting('[database]-puppetdb_subname').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/database.ini',
           'section' => 'database',
           'setting' => 'subname',
           'value'   => '//write-database.local.domain:1234/write-pdb-database'
    )}
  end
end
