require 'spec_helper'

describe 'puppet_enterprise::pxp_agent' do

  let(:agent_conf) { '/etc/puppetlabs/pxp-agent/pxp-agent.conf' }

  def should_have_setting(path, setting, value)
    settings = JSON.load(catalogue.resource('file', path)[:content])
    expect(settings[setting]).to eq(value)
  end

  def should_not_have_setting(path, setting)
    settings = JSON.load(catalogue.resource('file', path)[:content])
    expect(settings[setting]).to be_nil
  end

  let(:facts) do
    {
      'clientcert'        => 'awesomecert',
    }
  end

  let(:params) do
    {
      'broker_ws_uri' => 'wss://pxp-broker.example.com:8142/pcp/',
    }
  end

  context "on a windows machine" do
    before :each do
      facts['os'] = { 'family' => 'windows' }
      facts['common_appdata'] = '/ProgramData'
    end

    it "the catalog compiles" do
      is_expected.to contain_file('/ProgramData/PuppetLabs/pxp-agent/etc/pxp-agent.conf')
    end
  end

  context "on a RedHat machine" do
    it "the catalog compiles" do
      is_expected.to compile
    end

    it { should contain_file(agent_conf).with_notify('Service[pxp-agent]') }
  end

  context "pxp-agent.conf with PCP v1" do
    before :each do
      facts['puppetversion'] = '4.6.0'
    end

    it { should contain_file(agent_conf) }
    it { should_have_setting(agent_conf, 'broker-ws-uris', ['wss://pxp-broker.example.com:8142/pcp/']) }
    it { should_have_setting(agent_conf, 'ssl-key', '/etc/puppetlabs/puppet/ssl/private_keys/awesomecert.pem') }
    it { should_have_setting(agent_conf, 'ssl-cert', '/etc/puppetlabs/puppet/ssl/certs/awesomecert.pem') }
    it { should_have_setting(agent_conf, 'loglevel', 'info') }
    it { should_not_have_setting(agent_conf, 'master-uris') }
    it { should_not_have_setting(agent_conf, 'ping-interval') }
    it { should_not_have_setting(agent_conf, 'spool-dir-purge-ttl') }
    it { should_not_have_setting(agent_conf, 'task-cache-dir-purge-ttl') }
  end

  context "pxp-agent.conf on older agents" do
    before :each do
      facts['puppetversion'] = '4.3.0'
    end

    it { should contain_file(agent_conf) }
    it { should_have_setting(agent_conf, 'broker-ws-uri', 'wss://pxp-broker.example.com:8142/pcp/') }
    it { should_have_setting(agent_conf, 'ssl-key', '/etc/puppetlabs/puppet/ssl/private_keys/awesomecert.pem') }
    it { should_have_setting(agent_conf, 'ssl-cert', '/etc/puppetlabs/puppet/ssl/certs/awesomecert.pem') }
    it { should_have_setting(agent_conf, 'loglevel', 'info') }
    it { should_not_have_setting(agent_conf, 'master-uris') }
    it { should_not_have_setting(agent_conf, 'ping-interval') }
    it { should_not_have_setting(agent_conf, 'spool-dir-purge-ttl') }
    it { should_not_have_setting(agent_conf, 'task-cache-dir-purge-ttl') }
  end

  context "pxp-agent.conf with pxp-agent 1.4" do
    before :each do
      facts['puppetversion'] = '4.9.0'
    end

    it { should_not_have_setting(agent_conf, 'ping-interval') }
  end

  context "pxp-agent.conf with ping-interval overwritten" do
    before :each do
      facts['puppetversion'] = '4.9.0'
      params['ping_interval'] = 120
    end

    it { should_have_setting(agent_conf, 'ping-interval', 120) }
  end

  context "pxp-agent.conf with loglevel overwritten" do
    before :each do
      params['pxp_loglevel'] = 'debug'
    end

    it { should_have_setting(agent_conf, 'loglevel', 'debug') }
  end

  context "pxp-agent.conf with logfile specified" do
    before :each do
      params['pxp_logfile'] = '/pxp.log'
    end

    it { should_have_setting(agent_conf, 'logfile', '/pxp.log') }
  end

  context "pxp-agent.conf with master-uris specified" do
    before :each do
      params['master_uris'] = ['https://foo:1', 'https://foo:2']
    end

    context 'old agents' do
      before :each do
        facts['aio_agent_version'] = '1.10.1'
      end

      it { should_not_have_setting(agent_conf, 'master-uris') }
    end

    context "newer agents" do
      before :each do
        facts['aio_agent_version'] = '5.2.0'
      end

      it { should_have_setting(agent_conf, 'master-uris', ['https://foo:1', 'https://foo:2']) }
    end
  end

  context "pxp-agent.conf with spool-dir-purge-ttl specified" do
    before :each do
      params['spool_dir_purge_ttl'] = '1h'
    end

    context 'old agents' do
      before :each do
        facts['puppetversion'] = '4.3.0'
      end

      it { should_not_have_setting(agent_conf, 'spool-dir-purge-ttl') }
    end

    context 'newer agents' do
      before :each do
        facts['puppetversion'] = '4.4.0'
      end

      it { should_have_setting(agent_conf, 'spool-dir-purge-ttl', '1h') }
    end
  end

  context "pxp-agent.conf with task-cache-dir-purge-ttl specified" do
    before :each do
      params['task_cache_dir_purge_ttl'] = '1h'
    end

    context 'old agents' do
      before :each do
        facts['aio_agent_version'] = '5.3.3'
      end

      it { should_not_have_setting(agent_conf, 'task-cache-dir-purge-ttl') }
    end

    context 'newer agents' do
      before :each do
        facts['aio_agent_version'] = '5.4.0'
      end

      it { should_have_setting(agent_conf, 'task-cache-dir-purge-ttl', '1h') }
    end
  end

  context "pxp-agent.conf with task_download_timeout and task_download_connect_timeout" do
    before :each do
      params['task_download_timeout'] = 6
      params['task_download_connect_timeout'] = 7
    end

    context 'old agents' do
      before :each do
        facts['aio_agent_version'] = '5.5.0'
      end

      it { should_not_have_setting(agent_conf, 'task-download-timeout') }
      it { should_not_have_setting(agent_conf, 'task-cache-download_connect-timeout') }
    end

    context 'newer agents' do
      before :each do
        facts['aio_agent_version'] = '5.5.2'
      end

      it { should_have_setting(agent_conf, 'task-download-timeout', 6) }
      it { should_have_setting(agent_conf, 'task-download-connect-timeout', 7) }
    end
  end

  context "pxp-agent.conf with master_proxy and broker_proxy" do
    before :each do
      params['master_proxy'] = 'master_proxy.com:8132'
      params['broker_proxy'] = 'broker_proxy.com:8132'
    end

    context 'old agents' do
      before :each do
        facts['aio_agent_version'] = '5.5.0'
      end

      it { should_not_have_setting(agent_conf, 'master-proxy') }
      it { should_not_have_setting(agent_conf, 'broker-ws-proxy') }
    end

    context 'newer agents' do
      before :each do
        facts['aio_agent_version'] = '5.5.4'
      end

      it { should_have_setting(agent_conf, 'master-proxy', 'master_proxy.com:8132') }
      it { should_have_setting(agent_conf, 'broker-ws-proxy', 'broker_proxy.com:8132') }
    end
  end

  context "pxp-agent.conf with extra_options specified" do
    before :each do
      params['extra_options'] = {
        'spool-dir' => '/opt/puppet/custom/spool'
      }
    end

    it { should_have_setting(agent_conf, 'spool-dir', '/opt/puppet/custom/spool') }
  end

  context "pxp-agent.conf with extra_options that match existing options" do
    before :each do
      # task-download-timeout is only valid on newer versions
      facts['aio_agent_version'] = '5.5.3'

      params['extra_options'] = {
        'task-download-timeout' => 20
      }

      params['task_download_timeout'] = 10
    end

    # Explicit parameters should win
    it { should_have_setting(agent_conf, 'task-download-timeout', 10) }
  end

  context "when enabled" do
    it { should contain_class('puppet_enterprise::pxp_agent::service').with_enabled(true) }
  end

  context "when disabled" do
    before :each do
      params['enabled'] = false
    end

    it { should contain_class('puppet_enterprise::pxp_agent::service').with_enabled(false) }
  end

  context "when PCP version is specified" do
    before :each do
      params['pcp_version'] = '2'
    end

    it { should_have_setting(agent_conf, 'pcp-version', '2') }
  end

  it { should satisfy_all_relationships }
end
