require 'spec_helper'

describe 'puppet_enterprise::repo::config' do
  def platform_tag_for(os, facts)
    case os
    when /redhat/i
      then "el-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    when /ubuntu/i
      then "ubuntu-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    when /sles/i
      then "sles-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    else
      raise("Don't know how to create a platform_tag fact for os '#{os}'")
    end
  end

  # PE-25851: This is to test that if a customer is managing /etc/yum.repos.d/ in their
  # site.pp (or the sles equivalent), then we don't get dependency cycles. Note that this fix
  # and spec test change only work for EL and SLES, not Ubuntu, as Ubuntu requires some files
  # to be present. The puppet_enterprise::repo class included here is simply to simulate when
  # this file resource is present in site.pp by creating a seaparate stage. It would not need to 
  # be present in a real site.pp.
  let(:pre_condition) do
    <<-PRE_COND
    class {'puppet_enterprise':
      puppet_master_host           => 'master.rspec',
    }
    file { '/etc/yum.repos.d/':
      ensure => 'directory',
      owner => 'root',
      group => 'root',
      mode => '0755',
    }
    file {'/etc/zypp/repos.d':
      ensure => 'directory',
      owner => 'root',
      group => 'root',
      mode => '0755',
    }
    file { '/etc/apt/apt.conf.d/' :
      ensure => 'directory',
      owner => 'root',
      group => 'root',
      mode => '0755',
    }
    file { '/etc/apt/sources.list.d' :
      ensure => 'directory',
      owner => 'root',
      group => 'root',
      mode => '0755',
    }
    class { 'puppet_enterprise::packages': }
    PRE_COND
  end

  RSpec.shared_examples 'repo' do |os|
    if os =~ /^redhat/
      it { should contain_file('/etc/yum.repos.d/pe_repo.repo').with_ensure('absent') }
      it { should contain_file('/etc/yum.repos.d/pc_repo.repo').with_ensure('absent') }
      it { should contain_yumrepo(repo_name)
        .with_ensure('present')
        .with_baseurl(repo_source)
        .with_enabled(true)
        .with_gpgcheck('1')
      }
      it{ is_expected.to compile }
    elsif os =~ /^ubuntu/
      it { should contain_file('/etc/apt/apt.conf.d/90pe_repo').with_ensure('absent') }
      it { should contain_file('/etc/apt/apt.conf.d/90pc_repo').with_ensure('absent') }
      it { should contain_file("/etc/apt/sources.list.d/#{repo_name}.list")
        .with_ensure('file')
        .with_owner('0')
        .with_group('0')
        .with_content("deb #{repo_source} ./")
        .with_notify('Exec[pe_apt_update]')
      }
      it{ is_expected.to compile }
    elsif os=~ /^sles/
      it { should contain_file('/etc/zypp/repos.d/pe_repo.repo').with_ensure('absent') }
      it { should contain_file('/etc/zypp/repos.d/pc_repo.repo').with_ensure('absent') }
      it do
        repo_settings.each do |setting, value|
          should contain_pe_ini_setting("zypper #{repo_name} #{setting}")
            .with_path("/etc/zypp/repos.d/#{repo_name}.repo",)
            .with_section(repo_name)
            .with_setting(setting)
            .with_value(value)
        end
      end
      it { should contain_pe_ini_setting("zypper #{repo_name} baseurl")
        .with_path("/etc/zypp/repos.d/#{repo_name}.repo")
        .with_section(repo_name)
        .with_setting('baseurl')
        .with_value("#{repo_source}?ssl_verify=no")
        .with_notify('Exec[pe_zyp_update]')
      }
      it { should contain_exec("pe_zyp_update")
        .with_command("/usr/bin/zypper ref #{repo_name}")
        .with_logoutput('on_failure')
        .with_refreshonly(true)
      }
      it{ is_expected.to compile }
    end
  end

  on_supported_os.each do |os, base_facts|
    let(:pe_server_version) { '2016.5.0' }
    let(:repo_name) { 'puppet_enterprise' }
    let(:repo_settings) do
      {
      'name'        => repo_name,
      'enabled'     => '1',
      'autorefresh' => '0',
      'type'        => 'rpm-md',
      }
    end
    context "on #{os}" do
      let(:facts) do
        base_facts.merge({
          :pe_server_version => pe_server_version,
          :pe_build          => pe_server_version,
          :platform_tag      => platform_tag,
          :aio_agent_version => '5.5.6',
        })
      end
      let(:platform_tag) { platform_tag_for(os, base_facts) }
      let(:aio_agent_version) { '5.5.6' }

      context 'when managing repo config on an infrastructure node' do
        let(:node) { 'foo.rspec' }
        let(:repo_source) do
          "https://master.rspec:8140/packages/#{pe_server_version}/puppet_enterprise"
        end
        if os =~ /^ubuntu/
           it { should contain_file("/etc/apt/apt.conf.d/90#{repo_name}")
            .with_ensure('file')
            .with_owner('0')
            .with_group('0')
            .with_content("Acquire::https::master.rspec::Verify-Peer false;\nAcquire::http::Proxy::master.rspec DIRECT;")
            .with_notify('Exec[pe_apt_update]')
          }
        end
        include_examples 'repo', os
      end

      context 'when managing repo config on a primary node' do
        let(:node) { 'master.rspec' }
        let(:repo_source) do
          "file:///opt/puppetlabs/server/data/packages/public/#{pe_server_version}/puppet_enterprise"
        end
        if os =~ /^ubuntu/
          it { should contain_file(''"/etc/apt/apt.conf.d/90#{repo_name}").with_ensure('absent') }
        end
        include_examples 'repo', os
      end
    end
  end
end
