require 'spec_helper'
require 'digest'

describe 'puppet_enterprise::app_database', :type => :define do
  let(:title) { 'testing' }
  let(:primary) { 'master.rspec' }
  let(:pg_ident_conf_path) { '/opt/puppetlabs/server/data/postgresql/version/data/pg_ident.conf' }
  let(:base_params) do
    {
      :certname            => primary,
      :database_name       => 'test',
      :write_user          => 'writer',
      :tablespace_name     => 'test',
      :pg_ident_conf_path  => pg_ident_conf_path,
      :client_certnames    => [primary],
      :tablespace_location => '/opt/puppetlabs/server/data/postgresql/test',
      :ip_mask_allow_all_users_ssl   => '0.0.0.0/0',
      :ipv6_mask_allow_all_users_ssl => '::/0',
    }
  end
  let(:params) do
    base_params
  end
  let(:common_precondition) do
    <<-EOS
      # These are the puppet_enterprise::profile::database fragments
      # necessary to be able to generate an app_database.pp defined type.
      class { 'puppet_enterprise':
        puppet_master_host => '#{primary}',
      }
      pe_concat { '#{pg_ident_conf_path}':
      }
      class { 'pe_postgresql::globals':
        pg_hba_conf_defaults => false,
      }
      include pe_postgresql::server
    EOS
  end

  let(:pre_condition) do
    common_precondition
  end

  it { is_expected.to compile.with_all_deps }

  RSpec.shared_examples('cert auth') do |cert,database,user,expect_role = true|
    context "for #{cert} to #{database} as #{user}" do
      let(:hba_title) { "Allow certificate mapped connections to #{database} as #{user}" }
      let(:ident_title) { "Allow #{cert} to connect to #{database} as #{user}" }
      let(:map) { "#{database}-#{user}-map" }

      if expect_role
        it do
          is_expected.to contain_pe_postgresql__server__role(user)
            .with_password_hash(false)
            .with_superuser(user == 'super')
        end
      end

      it do
        is_expected.to contain_pe_postgresql__server__pg_hba_rule("#{hba_title} (ipv4)")
          .with_auth_option("map=#{map} clientcert=1")
          .with_address('0.0.0.0/0')
      end

      it do
        is_expected.to contain_pe_postgresql__server__pg_hba_rule("#{hba_title} (ipv6)")
          .with_auth_option("map=#{map} clientcert=1")
          .with_address('::/0')
      end

      it do
        is_expected.to contain_puppet_enterprise__pg__ident_entry(ident_title)
          .with_database(database)
          .with_user(user)
          .with_client_certname(cert)
          .with_ident_map_key(map)
      end
    end
  end

  RSpec.shared_examples('common') do |database, dbowner|
    it do
      is_expected.to contain_pe_postgresql__server__tablespace(database)
        .with_location("/opt/puppetlabs/server/data/postgresql/#{database}")
    end

    it do
      is_expected.to contain_pe_postgresql__server__db(database)
        .with_user(dbowner)
        .with_password(nil)
        .with_tablespace(database)
    end

    it { is_expected.to contain_puppet_enterprise__psql("#{database} revoke public table create perms") }
    it { is_expected.to contain_puppet_enterprise__psql("grant table create to #{dbowner} on #{database}") }
  end

  RSpec.shared_examples('super user cert auth') do |options|
    let(:database) { params[:database_name] }
    let(:super_user) { options[:super] }
    let(:read_user) { options[:read] }
    let(:write_user) { options[:write] }
    let(:replication_user) { options[:replication_user] || 'none' }
    let(:replicas) { options[:replicas] || [] }
    let(:client_certnames) { options[:client_certnames] }

    let(:ordinary_users) { [read_user, write_user].compact }
    let(:roles) { [read_user, write_user, super_user].compact }
    let(:all_users) { [super_user, read_user, write_user, options[:replication_user]].compact }

    it do
      is_expected.to contain_puppet_enterprise__pg__enforce_ownership("#{database} objects should be owned by #{super_user}")
        .with_database(database)
        .with_schema('public')
        .with_owner(super_user)
    end
    it { is_expected.to have_puppet_enterprise__pg__enforce_ownership_resource_count(1) }

    it do
      is_expected.to contain_puppet_enterprise__pg__ordinary_user(read_user)
        .with_user_name(read_user)
        .with_database(database)
        .with_database_password('')
        .with_write_access(false)
        .with_db_owner(super_user)
        .with_replication_user(replication_user)
    end

    it do
      is_expected.to contain_puppet_enterprise__pg__ordinary_user('writer')
        .with_user_name('writer')
        .with_database('test')
        .with_database_password('')
        .with_write_access(true)
        .with_db_owner('super')
        .with_replication_user(replication_user)
    end

    it { is_expected.to have_puppet_enterprise__pg__ordinary_user_resource_count(ordinary_users.size) }
    it { is_expected.to have_pe_postgresql__server__role_resource_count(roles.size) }

    context 'the certificate authentication' do
      options[:client_certnames].each do |cert|
        options.values_at(:read, :write, :super).each do |u|
          include_examples('cert auth', cert, 'test', u)
        end

        if options.include?(:replication_user)
          include_examples('cert auth', cert, 'test', options[:replication_user], false)
        end
      end

      it do
        is_expected.to have_pe_postgresql__server__pg_hba_rule_resource_count(all_users.size * 2)
      end

      (options[:replicas] || []).each do |r|
        it do
          is_expected.to contain_puppet_enterprise__pg__ident_entry("Allow #{r} to connect to #{database} as #{replication_user}")
            .with_database(database)
            .with_user(replication_user)
            .with_client_certname(r)
            .with_ident_map_key("#{database}-#{replication_user}-map")
        end
      end

      it do
        client_count = client_certnames.size
        other_replicas = replicas - client_certnames
        # each client can connect with any of the users
        client_entries = client_count * all_users.size
        # each replica can connect with replication user
        is_expected.to have_puppet_enterprise__pg__ident_entry_resource_count(client_entries + other_replicas.size)
      end
    end
  end

  RSpec.shared_examples('no pglogical resources') do
    it { is_expected.to have_puppet_enterprise__pg__extension_resource_count(0) }
    it { is_expected.to have_puppet_enterprise__pg__pglogical__node_resource_count(0) }
    it { is_expected.to have_puppet_enterprise__pg__pglogical__replication_set_resource_count(0) }
    it { is_expected.to have_puppet_enterprise__pg__pglogical__subscription_resource_count(0) }
  end

  context 'monolithic primary' do

    context 'without super_user' do
      include_examples('common', 'test', 'writer')

      include_examples('no pglogical resources')

      it { is_expected.to have_puppet_enterprise__pg__enforce_ownership_resource_count(0) }
      it { is_expected.to have_puppet_enterprise__pg__ordinary_user_resource_count(0) }
      it { is_expected.to have_pe_postgresql__server__role_resource_count(1) }

      context 'the certificate authentication' do
        include_examples('cert auth', 'master.rspec', 'test', 'writer')

        it { is_expected.to have_pe_postgresql__server__pg_hba_rule_resource_count(2) }
        it { is_expected.to have_puppet_enterprise__pg__ident_entry_resource_count(1) }
      end
    end

    context 'with super_user' do
      let(:params) do
        base_params.merge(
          :super_user => 'super',
          :read_user  => 'reader'
        )
      end

      include_examples('common', 'test', 'super')

      include_examples('no pglogical resources')

      include_examples('super user cert auth', {
        :client_certnames => ['master.rspec'],
        :read     => 'reader',
        :write    => 'writer',
        :super    => 'super',
      })
    end
  end

  context 'ha' do
    RSpec.shared_examples('ha extensions') do
      it { is_expected.not_to contain_puppet_enterprise__pg__extension('test/pglogical_origin') }
      it { is_expected.to contain_puppet_enterprise__pg__extension('test/pglogical') }
      it { is_expected.to have_puppet_enterprise__pg__extension_resource_count(1) }
    end

    RSpec.shared_examples('ha pglogical node') do |database,cert,user|
      it do
        is_expected.to contain_puppet_enterprise__pg__pglogical__node("#{database}/#{node_name}")
          .with_node_name(node_name)
          .with_host(cert)
          .with_database(database)
          .with_user(user)
      end
      it { is_expected.to have_puppet_enterprise__pg__pglogical__node_resource_count(1) }
    end

    def sha_it(cert)
      sprintf("%.14s", Digest::SHA1.hexdigest(cert))
    end

    RSpec.shared_examples('ha pglogical source') do |database,cert,user|
      include_examples('ha pglogical node', database, cert, user)

      it do
        is_expected.to contain_puppet_enterprise__pg__pglogical__replication_set("#{database}/default")
          .with_ensure('populated')
          .with_database(database)
          .with_replication_set_name('default')
          .with_schema('public')
      end
      it { is_expected.to have_puppet_enterprise__pg__pglogical__replication_set_resource_count(1) }
    end

    RSpec.shared_examples('ha pglogical replica') do |database,cert,user|
      include_examples('ha pglogical node', database, cert, user)

      it do
        is_expected.to contain_puppet_enterprise__pg__pglogical__subscription("#{database}/#{subscription_name}")
          .with_ensure('present')
          .with_host('master.rspec')
          .with_database(database)
          .with_user(user)
          .with_synchronize_structure(true)
      end
      it { is_expected.to have_puppet_enterprise__pg__pglogical__subscription_resource_count(1) }

      it do
        is_expected.to contain_puppet_enterprise__psql("pglogical_subscription #{database}/#{subscription_name} create-sql")
          .with_command(/synchronize_structure := true/)
      end
    end

    let(:primary) { 'master.rspec' }
    let(:replicas) { ['replica1.rspec', 'replica2.rspec'] }
    let(:ha_params) do
      base_params.merge(
        :super_user                  => 'super',
        :read_user                   => 'reader',
        :replication_user            => 'replicator',
        :replication_source_hostname => primary,
        :replica_hostnames           => replicas
      )
    end
    let(:hashed) { sha_it(params[:certname]) }
    let(:node_name) { "n#{hashed}" }
    let(:subscription_name) { "s#{hashed}" }

    let(:ha_precondition) do
      <<-EOS
        pe_postgresql::server::database { 'postgres': }
      EOS
    end
    let(:pre_condition) do
      common_precondition + ha_precondition
    end

    context 'source' do
      let(:params) do
        ha_params.merge(
          :replication_mode => 'source',
        )
      end

      RSpec.shared_examples('source replication') do
        include_examples('common', 'test', 'super')

        include_examples('ha extensions')

        include_examples('ha pglogical source', 'test', 'master.rspec', 'replicator')

        include_examples('super user cert auth', {
          :client_certnames => ['master.rspec'],
          :read     => 'reader',
          :write    => 'writer',
          :super    => 'super',
          :replication_user => 'replicator',
          :replicas => ['replica1.rspec', 'replica2.rspec'],
        })
      end

      include_examples('source replication')

      it 'ensures subscription absent' do
        is_expected.to contain_puppet_enterprise__pg__pglogical__subscription("#{database}/#{subscription_name}")
          .with_ensure('absent')
      end
      it { is_expected.to have_puppet_enterprise__pg__pglogical__subscription_resource_count(1) }

      context 'with two-way replication' do
        let(:params) do
          ha_params.merge(
            :replication_mode => 'source',
            :two_way_replication => true,
          )
        end

        include_examples('source replication')

        it 'does not subscribe before replicas are available' do
          is_expected.to have_puppet_enterprise__pg__pglogical__subscription_resource_count(0)
        end

        context 'once replicas are up' do
          let(:params) do
            ha_params.merge(
              :replication_mode => 'source',
              :two_way_replication => true,
              :enabled_replicas => replicas,
            )
          end

          it 'has subscriptions to each replica' do
            replicas.each do |replica|
              replica_node_name = "n#{sha_it(replica)}"
              is_expected.to contain_puppet_enterprise__pg__pglogical__subscription("test/#{subscription_name}_#{replica_node_name}")
                .with_ensure('present')
                .with_host(replica)
                .with_database('test')
                .with_user('replicator')
                .with_synchronize_structure(false)
            end
          end
          it { is_expected.to have_puppet_enterprise__pg__pglogical__subscription_resource_count(2) }

          it 'does not synchronize_structure for its subscriptions' do
            replicas.each do |replica|
              replica_node_name = "n#{sha_it(replica)}"
              is_expected.to contain_puppet_enterprise__psql("pglogical_subscription test/#{subscription_name}_#{replica_node_name} create-sql")
                .with_command(/synchronize_structure := false/)
            end
          end
        end
      end
    end

    context 'replica' do
      let(:params) do
        ha_params.merge(
          :certname         => 'replica1.rspec',
          :replication_mode => 'replica',
          :client_certnames => [primary, 'replica1.rspec'],
        )
      end

      RSpec.shared_examples('replica replication') do
        include_examples('common', 'test', 'super')

        include_examples('ha extensions')

        include_examples('ha pglogical replica', 'test', 'replica1.rspec', 'replicator')

        include_examples('super user cert auth', {
          :client_certnames  => ['master.rspec', 'replica1.rspec'],
          :read     => 'reader',
          :write    => 'writer',
          :super    => 'super',
          :replication_user => 'replicator',
          :replicas => ['replica1.rspec', 'replica2.rspec'],
        })
      end

      include_examples('replica replication')

      it do
        is_expected.to contain_puppet_enterprise__pg__pglogical__replication_set("#{database}/default")
          .with_ensure('empty')
      end
      it { is_expected.to have_puppet_enterprise__pg__pglogical__replication_set_resource_count(1) }

      context 'with two-way-replication' do
        let(:params) do
          ha_params.merge(
            :certname         => 'replica1.rspec',
            :replication_mode => 'replica',
            :client_certnames => [primary, 'replica1.rspec'],
            :two_way_replication => true,
          )
        end

        include_examples('replica replication')

        it do
          is_expected.to contain_puppet_enterprise__pg__pglogical__replication_set("#{database}/default")
            .with_ensure('populated')
            .with_database(database)
            .with_replication_set_name('default')
            .with_schema('public')
        end
        it { is_expected.to have_puppet_enterprise__pg__pglogical__replication_set_resource_count(1) }
      end
    end
  end
end
