require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::classifier' do
  let(:params) do
    {
      'database_host'           => 'db.rspec',
      'database_port'           => 54321,
      'database_user'           => 'custom_user',
      'database_name'           => 'pe-classifier',
      'database_password'       => 'secretly',
      'database_migration_user' => 'migration_user',
      'client_certname'         => 'console.rspec',
      'synchronization_period'  => 180,
      'prune_days_threshold'    => 14,
    }
  end
  let(:title)  { 'console-services' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-classifier': }
      service { 'pe-console-services': }
      package { 'pe-console-services': }
    PRE_COND
  end

  context "when the parameters are valid" do
    it do
      should contain_file("/etc/puppetlabs/#{title}/conf.d/classifier.conf").with(
        :owner => "pe-#{title}",
        :group => "pe-#{title}",
        :mode => "0640"
      )
    end

    it { should contain_puppet_enterprise__trapperkeeper__database_settings('classifier').with(
      'container'           => 'console-services',
      'database_host'       => 'db.rspec',
      'database_name'       => 'pe-classifier',
      'database_port'       => '54321',
      'database_properties' => '',
      'database_user'       => 'custom_user',
      'database_password'   => 'secretly',
      'migration_user'      => 'migration_user',
      'group'               => 'pe-console-services',
      'user'                => 'pe-console-services',
    ) }


    it { should contain_pe_hocon_setting("#{title}.classifier.database.subname").with_value('//db.rspec:54321/pe-classifier') }
    it { should contain_pe_hocon_setting("#{title}.classifier.database.user").with_value('custom_user') }
    it { should contain_pe_hocon_setting("#{title}.classifier.database.password").with_value('secretly') }
    it { should contain_pe_hocon_setting("#{title}.classifier.database.migration-user").with_value('migration_user') }
    it { should contain_pe_hocon_setting("#{title}.classifier.database.migration-password").with_value('secretly') }
    it { should contain_pe_hocon_setting("#{title}.classifier.puppet-master").with_ensure('present') }
    it { should contain_pe_hocon_setting("#{title}.classifier.ssl-key").with_value("/opt/puppetlabs/server/data/#{title}/certs/console.rspec.private_key.pem") }
    it { should contain_pe_hocon_setting("#{title}.classifier.ssl-cert").with_value("/opt/puppetlabs/server/data/#{title}/certs/console.rspec.cert.pem") }
    it { should contain_pe_hocon_setting("#{title}.classifier.ssl-ca-cert").with_value('/etc/puppetlabs/puppet/ssl/certs/ca.pem') }
    it { should contain_pe_hocon_setting("#{title}.classifier.synchronization-period").with_value('180') }
    it { should contain_pe_hocon_setting("#{title}.classifier.prune-days-threshold").with_value('14') }
    it { should contain_pe_hocon_setting("#{title}.classifier.node-check-in-storage").with_value(false) }
    it { should contain_pe_hocon_setting("#{title}.classifier.allow-config-data").with_value(false) }
    it { should contain_pe_concat__fragment("#{title} classifier-service") }
    it { should contain_pe_concat__fragment("#{title} activity-reporting-service") }
    it { should contain_pe_concat__fragment("#{title} jetty9-service") }
  end

  context "with custom synchronization period" do
    before(:each) do
      params.merge!({ 'synchronization_period' => 300 })
    end
    it { should contain_pe_hocon_setting("#{title}.classifier.synchronization-period").with_value('300') }
  end

  context "when enabling node-check-in-storage" do
    before(:each) do
      params.merge!({ 'node_check_in_storage' => true })
    end
    it { should contain_pe_hocon_setting("#{title}.classifier.node-check-in-storage").with_value(true) }
  end

  context "managing allow_config_data" do
    before :each do
      params['allow_config_data'] = true
    end
    it { should contain_pe_hocon_setting("#{title}.classifier.allow-config-data").with_value(true) }
  end

  context "with custom master host" do
    before(:each) do
      params.merge!({ 'master_host' => 'custom.puppet.master' })
    end
    it { should contain_pe_hocon_setting("#{title}.classifier.puppet-master").with_value('https://custom.puppet.master:8140') }
  end

end
