require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::console_services' do
  require 'puppet/resource/catalog'
  require 'puppet/indirector/memory'
  class ::Puppet::Resource::Catalog::StoreConfigsTesting < Puppet::Indirector::Memory; end

  before :all do
  end

  let(:params) do
    {
      'proxy_idle_timeout'       => 60,
      'puppetdb_host'            => ['puppetdb.rspec'],
      'master_host'              => 'master.rspec',
      'puppetdb_port'            => [54321],
      'classifier_host'          => 'classifier.rspec',
      'classifier_port'          => '4242',
      'classifier_url_prefix'    => '/classifier-api',
      'rbac_host'                => 'rbac.rspec',
      'rbac_port'                => '4244',
      'rbac_url_prefix'          => '/rbac-api',
      'session_timeout'          => 30,
      'session_maximum_lifetime' => '6y',
      'activity_host'            => 'activity.rspec',
      'activity_port'            => '4243',
      'activity_url_prefix'      => '/activity-api',
      'client_certname'          => 'classifier_ui.rspec',
      'status_proxy_enabled'     => false,
      'orchestrator_host'        => 'orchestrator.rspec',
      'orchestrator_port'        => '4246',
      'orchestrator_url_prefix'  => '/orchestrator',
      'service_alert_timeout'    => 5000,
      'agent_installer_host'     => 'master.rspec',
      'agent_installer_port'     => 8140,
    }
  end
  let(:title) { 'console-services' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-console-services': }
      package { 'pe-console-services': }
    PRE_COND
  end

  context "when the parameters are valid" do
    it { should contain_file("/etc/puppetlabs/console-services/conf.d/console.conf").with(:owner => "pe-console-services",
                                                                                             :group => "pe-console-services",
                                                                                             :mode => "0640") }
    it { should contain_pe_hocon_setting("#{title}.console.assets-dir").with_value('dist') }
    it { should contain_pe_hocon_setting("#{title}.console.rbac-server").with_value('http://rbac.rspec:4244/rbac-api') }
    it { should contain_pe_hocon_setting("#{title}.console.orchestrator-server").with_value('https://orchestrator.rspec:4246/orchestrator') }
    it { should contain_pe_hocon_setting("#{title}.console.classifier-server").with_value('http://classifier.rspec:4242/classifier-api') }
    it { should contain_pe_hocon_setting("#{title}.console.activity-server").with_value('http://activity.rspec:4243/activity-api') }
    it { should contain_pe_hocon_setting("#{title}.console.puppetdb-server").with_value('https://puppetdb.rspec:54321') }
    it { should contain_pe_hocon_setting("#{title}.console.certs.ssl-key").with_value('/opt/puppetlabs/server/data/console-services/certs/classifier_ui.rspec.private_key.pem') }
    it { should contain_pe_hocon_setting("#{title}.console.certs.ssl-cert").with_value('/opt/puppetlabs/server/data/console-services/certs/classifier_ui.rspec.cert.pem') }
    it { should contain_pe_hocon_setting("#{title}.console.certs.ssl-ca-cert").with_value('/etc/puppetlabs/puppet/ssl/certs/ca.pem') }
    it { should contain_pe_hocon_setting("#{title}.console.proxy-idle-timeout").with_value('60') }
    it { should contain_pe_hocon_setting("#{title}.console.pcp-request-timeout").with_value(5) }
    it { should contain_pe_hocon_setting("#{title}.console.display-local-time").with_value('false') }
    it { should contain_pe_hocon_setting("#{title}.console.no-longer-reporting-cutoff").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.console.agent-installer-url").with_ensure('present').with_value('https://master.rspec:8140') }
    it { should contain_pe_concat__fragment('console-services pe-console-ui-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services pe-console-auth-ui-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services jetty9-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-authn-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-authn-middleware').with_ensure('absent') }
    it { should contain_pe_concat__fragment('console-services rbac-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-consumer-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-status-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-storage-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-authz-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services rbac-authn-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services audit-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services webrouting-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services status-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services metrics-service').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services metrics-webservice').with_ensure('present') }
    it { should contain_pe_concat__fragment('console-services scheduler-service') }
    it { should_not contain_pe_concat__fragment('console-services status-proxy-service') }
    context "when status proxy is enabled" do
      before(:each) do
        params['status_proxy_enabled'] = true
      end
      it { should contain_pe_concat__fragment('console-services status-proxy-service') }
    end

    context "service alert" do
      context "with puppetdb_query stubbed" do
        let(:node) { 'puppetmaster.storeconfigstrue.1.test' }
        before :each do
          Puppet::Parser::Functions.newfunction(:puppetdb_query,
                                                :type => :rvalue,
                                                :arity => 1) do |args|
            case args[0]
            when ['from', 'resources',
                  ['extract', ['certname', 'parameters'],
                   ['and',
                    ['=', ['node','active'], true],
                    ['=', 'type', 'Class'],
                    ['=', 'title', 'Puppet_enterprise::Profile::Console']]]]
              [{'certname' => 'source.host', 'parameters' => {'replication_mode' => 'source'}},
               {'certname' => 'replica.host', 'parameters' => {'replication_mode' => 'replica'}},
               {'certname' => 'compile.host', 'parameters' => {'replication_mode' => 'none'}}]
            when ['from', 'resources',
                  ['extract', ['certname', 'title', 'parameters'],
                   ['and', ['=', ['node', 'active'], true],
                    ['=', 'type', 'Class'],
                    ['or', ['=', 'title', 'Puppet_enterprise::Master::Puppetserver'],
                     ['=', 'title', 'Puppet_enterprise::Master::Code_manager'],
                     ['=', 'title', 'Puppet_enterprise::Profile::Puppetdb'],
                     ['=', 'title', 'Puppet_enterprise::Profile::Orchestrator'],
                     ['=', 'title', 'Puppet_enterprise::Profile::Console']]]]]
              [{'certname' => 'source.host',
                'title' => 'Puppet_enterprise::Master::Puppetserver',
                'parameters' => {'puppetserver_webserver_ssl_port' => '8140'}},
               {'certname' => 'replica.host',
                'title' => 'Puppet_enterprise::Master::Puppetserver',
                'parameters' => {'puppetserver_webserver_ssl_port' => '8141'}},
               {'certname' => 'compile.host',
                'title' => 'Puppet_enterprise::Master::Puppetserver',
                'parameters' => {'puppetserver_webserver_ssl_port' => '8142'}},
               {'certname' => 'replica.host',
                'title' => 'Puppet_enterprise::Profile::Console',
                'parameters' => {'console_services_api_ssl_listen_port' => '8989'}},
               {'certname' => 'source.host',
                'title' => 'Puppet_enterprise::Master::Code_manager',
                'parameters' => {'puppet_master_port' => '10000'}},
               {'certname' => 'other.host',
                'title' => 'Puppet_enterprise::Master::Code_manager',
                'parameters' => {'puppet_master_port' => '10001'}},
               {'certname' => 'puppetdb1.host',
                'title' => 'Puppet_enterprise::Profile::Puppetdb',
                'parameters' => {'ssl_listen_port' => '8888'}},
               {'certname' => 'puppetdb2.host',
                'title' => 'Puppet_enterprise::Profile::Puppetdb',
                'parameters' => {'ssl_listen_port' => '4444'}}]
            else
              raise Puppet::Error,
                    "Unexpected query: #{args[0]}"
            end
          end
          Puppet.settings[:storeconfigs] = true
          Puppet.settings[:storeconfigs_backend] = 'store_configs_testing'
        end

        it { should contain_pe_hocon_setting('console-services.console.service-alert')
        .with_setting('console.service-alert')
        .with_type('array')
        .with_value([{'type'=>'activity',
                      'url'=>'https://replica.host:8989',
                      'replication_mode'=>'replica'},
                     {'type'=>'classifier',
                      'url'=>'https://replica.host:8989',
                      'replication_mode'=>'replica'},
                     {'type' => 'code-manager',
                      'url' => 'https://other.host:10001',
                      'replication_mode' => 'none'},
                     {'type' => 'code-manager',
                      'url' => 'https://source.host:10000',
                      'replication_mode' => 'source'},
                     {'type' => 'master',
                      'url' => 'https://compile.host:8142',
                      'replication_mode' => 'none'},
                     {'type' => 'master',
                      'url' => 'https://replica.host:8141',
                      'replication_mode' => 'replica'},
                     {'type' => 'master',
                      'url' => 'https://source.host:8140',
                      'replication_mode' => 'source'},
                     {'type'=>'puppetdb',
                      'url'=>'https://puppetdb1.host:8888',
                      'replication_mode'=>'none'},
                     {'type'=>'puppetdb',
                      'url'=>'https://puppetdb2.host:4444',
                      'replication_mode'=>'none'},
                     {'type'=>'rbac',
                      'url'=>'https://replica.host:8989',
                      'replication_mode'=>'replica'}
                     ])
        }
      end

      it { should contain_pe_hocon_setting("#{title}.console.service-alert-timeout").with_value(5000) }
    end
  end

  context "when the node is a replica" do
    before(:each) do
      params['replication_mode'] = 'replica'
    end

    it { should contain_pe_concat__fragment('console-services pe-console-ui-service').with_ensure('absent') }
    it { should contain_pe_concat__fragment('console-services pe-console-auth-ui-service').with_ensure('absent') }
  end

  context "when params are undef" do
    before(:each) do
      params['proxy_idle_timeout'] = ''
      params.delete('session_timeout')
      params['session_maximum_lifetime'] = ''
      params.delete('max_connections_per_route')
      params.delete('max_connections_total')
    end

    it { should contain_pe_hocon_setting("#{title}.console.proxy-idle-timeout").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.console.session-timeout").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.console.session-maximum-lifetime").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.console.max-connections-per-route").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.console.max-connections-total").with_ensure('absent') }
  end

  context "using a malformed session maximum lifetime" do
    ["avocado", {:baz => :qux}, 42.0, "10k"].each do |lifetime|
      before(:each) do
        params['session_maximum_lifetime'] = lifetime
      end
      it { should compile.and_raise_error(/session_maximum_lifetime must either be an integer or a string of digits optionally followed by "s", "m", "h", "d", or "y"./) }
    end
  end

  context "when PuppetDB HA is configured" do
    before(:each) do
      params['puppetdb_host'] = ['puppetdb.rspec', 'replica.vm']
      params['puppetdb_port'] = [54321, 8081]
    end

    # We only grab the first PuppetDB from the list of PuppetDB's for the
    # Console when HA is configured
    it { should contain_pe_hocon_setting("#{title}.console.puppetdb-server").with_value('https://puppetdb.rspec:54321') }
  end

  context "when a custom cache ttl is in use" do
    before(:each) do
      params['query_cache_ttl'] = 1337
    end

    it { should contain_pe_hocon_setting("#{title}.console.query-cache-ttl").with_value(1337) }
  end

  context "when max-connections-per-route is set" do
    before(:each) do
      params['max_connections_per_route'] = 7
    end

    it { should contain_pe_hocon_setting("#{title}.console.max-connections-per-route").with_value(7) }
  end

  context "when max-connections-total is set" do
    before(:each) do
      params['max_connections_total'] = 9
    end

    it { should contain_pe_hocon_setting("#{title}.console.max-connections-total").with_value(9) }
  end

  context "when configuring no_longer_reporting_cutoff" do
    before(:each) do
      params['no_longer_reporting_cutoff'] = 1111
    end
    it { should contain_pe_hocon_setting("#{title}.console.no-longer-reporting-cutoff").with_ensure('present').with_value(1111) }
  end

  context "when configuring agent_installer_host and port" do
    before(:each) do
      params['agent_installer_host'] = 'foo'
      params['agent_installer_port'] = 8141
    end
    it { should contain_pe_hocon_setting("#{title}.console.agent-installer-url").with_ensure('present').with_value('https://foo:8141') }
  end
end
