require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::orchestrator' do
  let(:params) do
    {
      'database_host' => 'db.rspec',
      'master_url' => 'https://master.rspec:8140',
      'puppetdb_url' => 'https://puppetdb.rspec:8081',
      'classifier_url' => 'https://classifier.rspec:4433/classifier-api',
      'console_services_url' => 'https://console.rspec:4433',
      'rbac_url' => 'https://rbac.rspec:4433/rbac-api',
      'activity_url' => 'https://activity.rspec:4433/activity-api',
      'pcp_broker_url' => 'wss://pcp.rspec:8142/pcp',
      'console_url' => 'https://console.rspec',
      'inventory_service_url' => 'https://inventory-service.rspec:8143/inventory',
      'allow_unauthenticated_status' => true,
      'bolt_server_max_connections' => 100,
      'ace_server_max_connections' => 10,
      'plan_executor_workers'       => 1,
    }
  end
  let(:title) { 'orchestration-services' }
  let(:config_dir) { "/etc/puppetlabs/#{title}" }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-orchestration-services': }
      package { 'pe-orchestration-services': }
    PRE_COND
  end

  it { should contain_file("#{config_dir}/conf.d/orchestrator.conf").with(:owner => 'pe-orchestration-services',
                                                                       :group => 'pe-orchestration-services',
                                                                       :mode  => '0640') }

  context "configuring the SSL client" do
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ssl-cert").with_value("#{config_dir}/ssl/master.rspec.cert.pem") }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ssl-key").with_value("#{config_dir}/ssl/master.rspec.private_key.pem") }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ssl-ca-cert").with_value("/etc/puppetlabs/puppet/ssl/certs/ca.pem") }
  end

  context "managing integration with other services" do
    it { should contain_pe_hocon_setting("#{title}.orchestrator.master-url").with_value('https://master.rspec:8140') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.puppetdb-url").with_value('https://puppetdb.rspec:8081') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.classifier-service").with_value('https://classifier.rspec:4433/classifier-api') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.console-services-url").with_value('https://console.rspec:4433') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.bolt-server-url").with_value('https://master.rspec:62658') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.bolt-server-conn-timeout").with_value(10000) }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.bolt-server-max-connections").with_value(100) }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ace-server-url").with_value('https://master.rspec:44633') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ace-server-conn-timeout").with_value(10000) }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.ace-server-max-connections").with_value(10) }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.plan-executor-url").with_value('https://master.rspec:62659') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.plan-executor-conn-timeout").with_value(10000) }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.plan-executor-workers").with_value(1) }
    it { should contain_pe_hocon_setting("#{title}.rbac-consumer.api-url").with_value('https://rbac.rspec:4433/rbac-api') }
    it { should contain_pe_hocon_setting("#{title}.activity-consumer.api-url").with_value('https://activity.rspec:4433/activity-api') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.pcp-broker-url").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.console-url").with_value('https://console.rspec') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.inventory-service-url").with_value('https://inventory-service.rspec:8143/inventory') }
  end

  context "configuring database settings" do
    it { should contain_pe_hocon_setting("#{title}.orchestrator.database.subname").with_value('//db.rspec:5432/pe-orchestrator') }
    it { should contain_pe_hocon_setting("#{title}.orchestrator.database.user").with_value('pe-orchestrator-write') }
    it { should_not contain_pe_hocon_setting("#{title}.orchestrator.database.password") }
  end

  context "configuring bootstrap.cfg" do
    it { should contain_pe_concat__fragment('orchestration-services orchestrator-service') }
    it { should contain_pe_concat__fragment('orchestration-services orchestrator-dispatch-service') }
    it { should contain_pe_concat__fragment('orchestration-services jetty9-service') }
    it { should contain_pe_concat__fragment('orchestration-services remote-rbac-consumer-service') }
    it { should contain_pe_concat__fragment('orchestration-services remote-activity-reporter') }
    it { should contain_pe_concat__fragment('orchestration-services jruby-pool-manager-service') }
  end

  context "configuring authorization" do
    it { should contain_pe_puppet_authorization__rule('dispatch: allow pcp-brokers')
      .with_match_request_path('/server')
      .with_match_request_type('path')
      .with_allow(['master.rspec']) }

    context 'when authentication is required for the status api' do
      before :each do
        params['allow_unauthenticated_status'] = false
      end

      it { should contain_pe_puppet_authorization__rule('orchestrator simple status endpoint')
                .with_allow_unauthenticated(true) }

      it { should contain_pe_puppet_authorization__rule('orchestrator status service')
                .with_allow_unauthenticated(false)
                .with_allow('*') }
    end
  end

  context "configuring other settings" do
    context "with pcp-timeout set" do
      before :each do
        params['pcp_timeout'] = 10
      end
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.pcp-timeout").with(:ensure => 'present', :value => 10) }
    end

    context "without pcp-timeout set" do
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.pcp-timeout").with(:ensure => 'absent') }
    end

    context "using a global concurrency compile setting" do
      before :each do
        params['global_concurrent_compiles'] = 8
      end

      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.global-concurrent-compiles").with(:ensure => 'present', :value => 8) }
    end

    context "without global-concurrent-compiles set" do
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.global-concurrent-compiles").with(:ensure => 'absent') }
    end

    context "using a task concurrency setting" do
      before :each do
        params['task_concurrency'] = 1024
      end

      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.task-concurrency").with(:ensure => 'present', :value => 1024) }
    end

    context "without task-concurrency set" do
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.task-concurrency").with(:ensure => 'absent') }
    end

    context "with job-prune-threshold specified" do
      before :each do
        params['job_prune_threshold'] = 21
      end

      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.job-prune-days-threshold").with(:ensure => 'present', :value => 21) }
    end

    context "without job-prune-threshold set" do
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.job-prune-days-threshold").with(:ensure => 'absent') }
    end

    context "with app-management specified" do
      before :each do
        params['app_management'] = true
      end

      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.app-management").with(:ensure => 'present', :value => true) }
    end

    context "without app-management set" do
      it { should contain_pe_hocon_setting("orchestration-services.orchestrator.app-management").with(:ensure => 'absent') }
    end
  end
end
