require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::rbac' do
  let(:params) do
    {
      'certname'                => 'blah.foo.bar',
      'database_host'           => 'db.rspec',
      'database_port'           => 54321,
      'database_password'       => 'secretly',
      'database_name'           => 'pe-rbac',
      'database_migration_user' => 'migration_user',
    }
  end
  let(:customization_params) do
    {
      'password_reset_expiration'    => 24,
      'token_auth_lifetime'          => "666m",
      'token_maximum_lifetime'       => '9y',
      'failed_attempts_lockout'      => 3,
      'ds_trust_chain'               => "/foo/bar",
      'account_expiry_check_minutes' => 20,
      'account_expiry_days'          => 60,
    }
  end
  let(:title)          { 'console-services' }
  let(:config_file)    { '/etc/puppetlabs/console-services/conf.d/rbac.conf'}

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-console-services': }
      package { 'pe-console-services': }
    PRE_COND
  end

  context "when the parameters are valid" do
    it { should contain_pe_hocon_setting("#{title}.rbac.certificate-whitelist").with_value("/etc/puppetlabs/#{title}/rbac-certificate-whitelist") }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-private-key").with_value("/opt/puppetlabs/server/data/#{title}/certs/blah.foo.bar.private_key.pem") }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-public-key").with_value("/opt/puppetlabs/server/data/#{title}/certs/blah.foo.bar.cert.pem") }
    it { should contain_pe_hocon_setting("#{title}.rbac.database.subname").with_value('//db.rspec:54321/pe-rbac') }
    it { should contain_pe_hocon_setting("#{title}.rbac.database.maximum-pool-size").with_value(10) }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-maximum-lifetime").with_value("10y") }
    it { should contain_pe_hocon_setting("#{title}.rbac.database.connection-timeout").with_value(60000) }
    it { should contain_pe_hocon_setting("#{title}.rbac.database.connection-check-timeout").with_value(5000) }

    it { should contain_puppet_enterprise__trapperkeeper__database_settings('rbac').with(
      'container'           => 'console-services',
      'database_host'       => 'db.rspec',
      'database_name'       => 'pe-rbac',
      'database_password'   => 'secretly',
      'database_port'       => '54321',
      'database_properties' => '',
      'database_user'       => 'pe-rbac-write',
      'migration_user'      => 'migration_user',
      'migration_password'  => 'secretly',
      'group'               => 'pe-console-services',
      'user'                => 'pe-console-services',
    ) }

    it do
      should contain_file(config_file).with(
        :owner => "pe-console-services",
        :group => "pe-console-services",
        :mode => "0640"
      )
    end
    it { should contain_pe_hocon_setting("#{title}.rbac.database.migration-user").with_value('migration_user') }
    it { should contain_pe_hocon_setting("#{title}.rbac.database.migration-password").with_value('secretly') }
    it { should contain_pe_hocon_setting("#{title}.rbac.password-reset-expiration").with_ensure('absent') }
    # this setting has been migrated to console.conf
    it { should contain_pe_hocon_setting("#{title}.rbac.session-timeout").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-auth-lifetime").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.failed-attempts-lockout").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.ds-trust-chain").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.account-expiry-check-minutes").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.account-expiry-days").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.ssl-key").with_value("/opt/puppetlabs/server/data/#{title}/certs/blah.foo.bar.private_key.pem") }
    it { should contain_pe_hocon_setting("#{title}.rbac.ssl-cert").with_value("/opt/puppetlabs/server/data/#{title}/certs/blah.foo.bar.cert.pem") }
    it { should contain_pe_hocon_setting("#{title}.rbac.ssl-ca-cert").with_value('/etc/puppetlabs/puppet/ssl/certs/ca.pem') }
    it { should contain_pe_concat__fragment("#{title} rbac-service") }
    it { should contain_pe_concat__fragment("#{title} rbac-storage-service") }
    it { should contain_pe_concat__fragment("#{title} rbac-http-api-service") }
    it { should contain_pe_concat__fragment("#{title} activity-reporting-service") }
    it { should contain_pe_concat__fragment("#{title} jetty9-service") }
  end

  context "using a malformed lifetime" do
    ["bananas", {:foo => :bar}, 1.0, "10k"].each do |lifetime|
      before(:each) do
        params.merge!({'token_auth_lifetime' => lifetime})
      end
      it { should compile.and_raise_error(/token_auth_lifetime must either be an integer/) }
    end
  end

  context "using customization parameters" do
    before(:each) do
      params.merge!(customization_params)
    end
    it { should contain_pe_hocon_setting("#{title}.rbac.password-reset-expiration").with_value('24') }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-maximum-lifetime").with_value("9y") }
    # this setting has been migrated to console.conf
    it { should contain_pe_hocon_setting("#{title}.rbac.session-timeout").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.rbac.token-auth-lifetime").with_value('666m') }
    it { should contain_pe_hocon_setting("#{title}.rbac.failed-attempts-lockout").with_value('3') }
    it { should contain_pe_hocon_setting("#{title}.rbac.ds-trust-chain").with_value('/foo/bar') }
    it { should contain_pe_hocon_setting("#{title}.rbac.account-expiry-check-minutes").with_value('20') }
    it { should contain_pe_hocon_setting("#{title}.rbac.account-expiry-days").with_value('60') }
  end
end
