require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::webserver_settings' do
  let(:params) do
    {
      'container'          => 'puppetserver',
      'ssl_listen_address' => '0.0.0.0',
      'ssl_listen_port'    => 8142,
    }
  end
  let(:node) { 'compile.rspec' }
  let(:title) { 'pcp-broker' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetserver': }
    PRE_COND
  end

  context "configuring the webserver" do
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-host')
      .with_setting('webserver.pcp-broker.ssl-host')
      .with_value('0.0.0.0') }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-port')
      .with_setting('webserver.pcp-broker.ssl-port')
      .with_value(8142) }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.client-auth')
      .with_setting('webserver.pcp-broker.client-auth')
      .with_value('want') }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-ca-cert')
      .with_setting('webserver.pcp-broker.ssl-ca-cert')
      .with_value('/etc/puppetlabs/puppet/ssl/certs/ca.pem') }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-cert')
      .with_setting('webserver.pcp-broker.ssl-cert')
      .with_value("/etc/puppetlabs/puppet/ssl/certs/#{node}.pem") }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-key')
      .with_setting('webserver.pcp-broker.ssl-key')
      .with_value("/etc/puppetlabs/puppet/ssl/private_keys/#{node}.pem") }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-crl-path')
      .with_setting('webserver.pcp-broker.ssl-crl-path')
      .with_value("/etc/puppetlabs/puppet/ssl/crl.pem") }
  end

  context "configuring the webserver host and port" do
    before :each do
        params['ssl_listen_address'] = '127.0.0.1'
        params['ssl_listen_port']  = 3453
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-host')
      .with_setting('webserver.pcp-broker.ssl-host')
      .with_value('127.0.0.1') }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-port')
      .with_setting('webserver.pcp-broker.ssl-port')
      .with_value(3453) }
  end

  context "configuring the webserver cert and key defaults" do
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-cert')
      .with_setting('webserver.pcp-broker.ssl-cert')
      .with_value("/etc/puppetlabs/puppet/ssl/certs/#{node}.pem") }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-key')
      .with_setting('webserver.pcp-broker.ssl-key')
      .with_value("/etc/puppetlabs/puppet/ssl/private_keys/#{node}.pem") }
  end

  context "configuring the webserver cert and key non-defaults" do
    before :each do
        params['ssl_cert'] = '/tmp/ssl/cert.pem'
        params['ssl_key']  = '/tmp/ssl/private_keys/cert.pem'
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-cert')
      .with_setting('webserver.pcp-broker.ssl-cert')
      .with_value("/tmp/ssl/cert.pem") }
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-key')
      .with_setting('webserver.pcp-broker.ssl-key')
      .with_value("/tmp/ssl/private_keys/cert.pem") }
  end

  context "configuring the webserver with access-log-config" do
    before :each do
        params['access_log_config'] = '/tmp/log'
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.access-log-config')
      .with_ensure('present')
      .with_setting('webserver.pcp-broker.access-log-config')
      .with_value('/tmp/log')
    }
  end

  context "configuring the webserver without access-log-config" do
    before :each do
        params['access_log_config'] = :undef
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.access-log-config')
      .with_setting('webserver.pcp-broker.access-log-config')
      .with_ensure('absent')
    }
  end

  context "configuring the webserver with jetty-max-threads" do
    before :each do
        params['tk_jetty_max_threads'] = 123
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.max-threads')
      .with_setting('webserver.pcp-broker.max-threads')
      .with_ensure('present')
      .with_value(123)
    }
  end

  context "configuring the webserver without jetty-max-threads" do
    before :each do
        params['tk_jetty_max_threads'] = :undef
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.max-threads')
      .with_setting('webserver.pcp-broker.max-threads')
      .with_ensure('absent')
    }
  end

  context "configuring the webserver with jetty-max-request-header-size" do
    before :each do
        params['tk_jetty_request_header_max_size'] = 12345
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.request-header-max-size')
      .with_setting('webserver.pcp-broker.request-header-max-size')
      .with_ensure('present')
      .with_value(12345)
    }
  end

  context "configuring the webserver without jetty-max-request-header-size" do
    before :each do
        params['tk_jetty_request_header_max_size'] = :undef
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.request-header-max-size')
      .with_setting('webserver.pcp-broker.request-header-max-size')
      .with_ensure('absent')
    }
  end

  context "configuring the webserver with default-server" do
    before :each do
        params['default_server'] = true
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.default-server')
      .with_setting('webserver.pcp-broker.default-server')
      .with_ensure('present')
      .with_value(true)
    }
  end

  context "configuring the webserver without default-server" do
    before :each do
        params['default_server'] = :undef
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.default-server')
      .with_setting('webserver.pcp-broker.default-server')
      .with_ensure('absent')
    }
  end

  context "configuring the webserver with ssl_protocols" do
    before :each do
        params['ssl_protocols'] = ['TLSv1.3']
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-protocols')
      .with_setting('webserver.pcp-broker.ssl-protocols')
      .with_ensure('present')
      .with_type('array')
      .with_value(['TLSv1.3'])
    }
  end

  context "configuring the webserver without ssl_protocols" do
    before :each do
        params['ssl_protocols'] = []
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.ssl-protocols')
      .with_setting('webserver.pcp-broker.ssl-protocols')
      .with_ensure('absent')
    }
  end

  context "configuring the webserver with client_auth = want" do
    before :each do
        params['client_auth'] = 'want'
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.client-auth')
      .with_setting('webserver.pcp-broker.client-auth')
      .with_ensure('present')
      .with_value('want')
    }
  end

  context "configuring the webserver with client_auth = none" do
    before :each do
        params['client_auth'] = 'none'
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.client-auth')
      .with_setting('webserver.pcp-broker.client-auth')
      .with_ensure('present')
      .with_value('none')
    }
  end

  context "configuring ssl cipher suites" do
    before :each do
        params['ssl_cipher_suites'] = ['ECDHE-ECDSA-AES256-GCM-SHA384',
                                       'ECDHE-RSA-AES256-GCM-SHA384',
                                       'ECDHE-ECDSA-CHACHA20-POLY1305']
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.cipher-suites')
      .with_setting('webserver.pcp-broker.cipher-suites')
      .with_ensure('present')
      .with_value(['ECDHE-ECDSA-AES256-GCM-SHA384',
                   'ECDHE-RSA-AES256-GCM-SHA384',
                   'ECDHE-ECDSA-CHACHA20-POLY1305'])
    }
  end

  context "configuring ssl cipher suites" do
    before :each do
        params['ssl_cipher_suites'] = []
    end
    it { should contain_pe_hocon_setting('puppetserver.webserver.pcp-broker.cipher-suites')
      .with_setting('webserver.pcp-broker.cipher-suites')
      .with_ensure('absent')
    }
  end
end
