#! /usr/bin/env ruby -S rspec
require 'spec_helper'

describe "pe_compile_master" do
  it "should exist" do
    expect(Puppet::Parser::Functions.function("pe_compile_master")).to eq("function_pe_compile_master")
  end

  it "should raise a ParseError if there is more than one argument" do
    expect(subject).to run.with_params('1', '2').and_raise_error(Puppet::ParseError)
  end

  it "should raise a ParseError if the argument is not a valid accepted argument" do
    expect(subject).to run.with_params('1').and_raise_error(Puppet::ParseError)
  end

  context "with servername/fqdn mocked" do
    let(:fqdn) { nil }
    let(:servername) { nil }

    before(:each) do
      expect(scope).to receive(:function_pe_servername).with([]).and_return(servername).at_least(:once)
      expect(scope).to receive(:lookupvar).with('fqdn').and_return(fqdn).at_least(:once)
    end

    context "servername unset, fqdn set" do
      let(:fqdn) { 'foobar' }
      let(:servername) { nil }

      it "should return true" do
        expect(subject).to run.with_params().and_return(false)
      end
    end

    context "servername is the same as fqdn" do
      let(:fqdn) { 'foobar' }
      let(:servername) { 'foobar' }

      it "should return false" do
        expect(subject).to run.with_params().and_return(false)
      end
    end

    context "servername is different from fqdn" do
      let(:fqdn) { 'foobar' }
      let(:servername) { 'bazbaz' }

      it "should return false" do
        expect(subject).to run.with_params().and_return(true)
      end
    end

    context "where servername and fqdn do not match (possibly a compile master) and replication_mode is set to" do
      let(:fqdn) { 'foobar' }
      let(:servername) { 'bazbaz' }

      context "'source' or 'replica'" do
        it "should return false" do
          ["source", "replica"].each do |mode|
            expect(subject).to run.with_params(mode).and_return(false)
          end
        end
      end

      context "'none' or nil" do
        it "should return true" do
          ["none", nil].each do |mode|
            expect(subject).to run.with_params(mode).and_return(true)
          end
        end
      end
    end

    context "where servername and fqdn do match (not a compile master) and replication_mode is set to" do
      let(:fqdn) { 'foobar' }
      let(:servername) { 'foobar' }

      context "any of the four values" do
        it "should return false" do
          ["source", "replica", "none", nil].each do |mode|
            expect(subject).to run.with_params(mode).and_return(false)
          end
        end
      end
    end
  end

  context "servername is set, fqdn unset" do
    let(:fqdn) { nil }

    it "should raise a ParseError" do
      expect(scope).to receive(:lookupvar).with('fqdn').and_return(fqdn)
      expect(subject).to run.with_params().and_raise_error(Puppet::ParseError)
    end
  end
end
