#!/usr/bin/env rspec

require 'spec_helper'
require 'facter/inventory'

describe 'inventory facts' do
  let(:gem_instance) { double('yum_instance') }
  let(:yum_instance) { double('yum_instance') }
  let(:gem_provider) { double('gem_provider') }
  let(:yum_provider) { double('yum_provider') }

  before :each do
    allow(gem_instance).to receive(:properties).and_return({:name => 'apache',
                                                            :ensure => '1.1.1',
                                                            :provider => :gem})
    allow(yum_instance).to receive(:properties).and_return({:name => 'ruby',
                                                            :ensure => ['1.8.7', '1.9.3'],
                                                            :provider => :yum})
    allow(gem_provider).to receive(:instances).and_return([gem_instance])
    allow(yum_provider).to receive(:instances).and_return([yum_instance])

    package_type = double('package_type')
    allow(package_type).to receive(:providers_by_source).and_return([gem_provider, yum_provider])

    allow(Puppet::Type).to receive(:type).and_return(package_type)
  end

  context 'package collection' do
    it 'should contain all the packages' do
      expected = [['apache', '1.1.1', 'gem'],
                  ['ruby', '1.8.7', 'yum'],
                  ['ruby', '1.9.3', 'yum']]
      expect(Packages.new.gather_inventory).to eq(expected)
    end

    it 'should contain all the packages from working providers if one provider fails' do
      expected = [['ruby', '1.8.7', 'yum'],
                  ['ruby', '1.9.3', 'yum']]

      allow(gem_provider).to receive(:instances).and_raise("explosions")
      expect(Packages.new.gather_inventory).to eq(expected)
    end

    it 'should scrub any non-UTF8 content produced by providers' do
      allow(yum_instance).to receive(:properties).and_return({:name => "m\xE4ni interesting furry animals",
                                                              :ensure => ["1.0.0\xAE"],
                                                              :provider => :yum})

      expect(Packages.new.gather_inventory.to_json).to be_valid_encoding
    end

    it 'should remove any null bytes produced by providers' do
      allow(yum_instance).to receive(:properties).and_return({:name => "majestik m\0\0se",
                                                              :ensure => ["1.\0.\0"],
                                                              :provider => :yum})

      output = Packages.new.gather_inventory.to_json

      expect(output).not_to include("\0")
      expect(output).not_to include("\\u0000")
    end
  end

  context 'the inventory and metadata facts' do
    before :each do
      Facter.flush
    end

    context 'when setting the inventory fact' do
      it 'will not populate when disabled' do
        allow_any_instance_of(Packages).to receive(:enabled?).and_return(false)
        expect(Facter.value('_puppet_inventory_1')).to be nil
      end

      it 'wil be populated when enabled' do
        allow_any_instance_of(Packages).to receive(:enabled?).and_return(true)
        expect(Facter.value('_puppet_inventory_1')).not_to be nil
      end
    end

    context 'when setting the metadata fact' do
      it 'sets last_collection_time to 0 if inventory collection is disabled' do
        allow_any_instance_of(Packages).to receive(:enabled?).and_return(false)
        expect(Facter.value('puppet_inventory_metadata')['packages']['last_collection_time']).to eq("0.0s")
      end

      it 'sets enabled to true if enabled' do
        allow_any_instance_of(Packages).to receive(:enabled?).and_return(true)
        expect(Facter.value('puppet_inventory_metadata')['packages']['collection_enabled']).to be true
      end

      it 'sets enabled to false when disabled' do
        allow_any_instance_of(Packages).to receive(:enabled?).and_return(false)
        expect(Facter.value('puppet_inventory_metadata')['packages']['collection_enabled']).to be false
      end
    end
  end
end
