require 'spec_helper'
require_relative  '../../mono-repo'

RSpec::Matchers.define(:execute_with) do |args|
  match do |klass|
    @stdout_tmp = StringIO.new

    begin
      @runner = klass.invoke(args.split, @stdout_tmp)
    rescue ParseOptions::HelpExit
      @result = true # would exit with a 0
    end

    begin
      @result = @runner.execute if @runner
    rescue RunShell::ActionFailed => e
      @failure_exception = e
    end

    @output_matched = (@output_expressions || []).all? do |oe|
      @stdout_tmp.string.match(oe)
    end

    @commands_matched = (@command_expressions || []).all? do |ce|
      TestExecutor.commands.find { |c| c.match(ce) }
    end

    @expected_result = true if !instance_variable_defined?(:@expected_result)
    @result_matched = @result == @expected_result

    @result_matched && @output_matched && @commands_matched
  end

  chain(:and_return) do |expected_result|
    @expected_result = expected_result
  end

  chain(:and_output) do |output_expressions|
    @output_expressions = [output_expressions].flatten
  end

  chain(:and_generate_commands) do |command_expressions|
    @command_expressions = [command_expressions].flatten
  end

  chain(:and_fail_with) do |failure_expression|
    @expected_result = nil
    @failure_expected = true
    @output_expressions ||= []
    @output_expressions << failure_expression
    @output_expressions << /Action failed!/
  end

  failure_message do |actual|
    message = ["expected that #{@runner} would:"]
    message << " * return '#{@expected_result}' but it returned '#{@result}'" unless @result_matched
    if !@output_matched
      message << " * have found these expressions:"
      message << @output_expressions.pretty_inspect
      message << "   in this output:"
      message << @stdout_tmp.string
    end
    if !@commands_matched
      message << " * have found these commands:"
      message << @command_expressions.pretty_inspect
      message << "   in this command list:"
      message << TestExecutor.commands.pretty_inspect
    end
    message.join("\n")
  end
end

describe 'mono-repo.rb' do

  class TestExecutor

    DEFAULT_REMOTE_TAGS = <<-EOS
c50b452697b9061ac74d4ec04eea27151a1e6e12        refs/tags/2018.1.1
f65900bcd24a1d5ddcdcaa22e2a6eb644d56fc27        refs/tags/2018.1.1^{}
d85b9332f2bee1be2b2e676d8489e5f5ccbc0fd6        refs/tags/2018.1.1-rc0
390b8b7284d810c85ccda982069d91050fcfa990        refs/tags/2018.1.1-rc0^{}
8f60bea5806c2580bedbdab1b84b4af9d76fe791        refs/tags/2018.1.3
de2da6734a91465d12f868e90d5866369cefce17        refs/tags/2018.1.3^{}
    EOS

    DEFAULT_BRANCHES = <<-EOS
* abranch
    EOS

    DEFAULT_MODULE_TAGS = <<-EOS
module/puppet_enterprise/2018.1.1
module/puppet_enterprise/2018.1.2
module/puppet_enterprise/2018.1.3
    EOS

    def self.commands
      @commands ||= []
    end

    def self.responses
      @responses ||= {
        /git ls-remote -t/ => [DEFAULT_REMOTE_TAGS, Status.new],
        /git tag -l (?!'module)/ => ['atag', Status.new],
        %r{git tag -l 'module/.*} => [DEFAULT_MODULE_TAGS, Status.new],
        /git branch --list/ => [DEFAULT_BRANCHES, Status.new],
      }
    end

    def self.find_response(command)
      response = TestExecutor.responses[command]
      if response.nil?
        tests = TestExecutor.responses.keys.select { |k| k.kind_of?(Regexp) }
        matched_command = tests.find do |test|
          test.match(command)
        end
        response = TestExecutor.responses[matched_command]
      end
      response ||=  ["", Status.new]
      return *response
    end

    def self.clear
      @commands = []
    end

    def self.add_response(command, response)
      _response = response.kind_of?(Array) ?
        [response[0], Status.new(response[1])] :
        [response, Status.new]
      responses[command] = _response
    end

    class Status
      attr_accessor :success

      def initialize(success = true)
        self.success = success
      end

      def success?
        success == true || success == 0
      end
    end

    def exec(command)
      TestExecutor.commands << command
      response = TestExecutor.find_response(command)
      return *response
    end
  end

  before(:each) do
    allow(RunShell).to receive(:executor).and_return(TestExecutor.new)
  end

  after(:each) do
    TestExecutor.clear
  end

  it 'returns help if no arguments given' do
    expect do
      expect(Dispatcher.new([]).run).to eq(false)
    end.to output(/Usage:.*mono-repo.rb/m).to_stdout
  end

  it 'returns help if an invalid option is given' do
    expect do
      expect(Dispatcher.new(['generate', '--foo']).run).to eq(1)
    end.to output(/Usage.*mono-repo.rb.*!! invalid option: --foo/m).to_stdout
  end

  it 'returns help for the filter-module-branch command' do
    expect { Dispatcher.new(['filter-module-branch']).run }.to(
      output(/Usage:.*mono-repo.rb.*filter-module-branch -m/m).to_stdout.and(
        raise_error(MonoRepoSubCommand::MonoRepoException,/No module name given/)
      )
    )
  end

  it 'executes the generate command' do
    expect do
      expect(Dispatcher.new(['generate']).run).to eq(true)
    end.to output(/Generate the kearney monorepo branch/m).to_stdout
  end

  it 'returns help for the push command' do
    expect { Dispatcher.new(['push']).run }.to(
      output(/Usage:.*mono-repo.rb.*push \(--tags/m).to_stdout.and(
        raise_error(MonoRepoSubCommand::MonoRepoException,/No targets to push/)
      )
    )
  end

  it 'executes the create-remotes command' do
    expect do
      expect(Dispatcher.new(['create-remotes']).run).to eq(true)
    end.to output(/Create and fetch the module remotes/m).to_stdout
  end

  describe 'FilterModuleBranch' do
    let(:subject) { FilterModuleBranch }

    it 'parses options' do
      fmb = FilterModuleBranch.invoke(['-m', 'foo', '-b', 'bar'])
      expect(fmb.module_name).to eq('foo')
      expect(fmb.branch_name).to eq('bar')
    end

    it 'filters a branch' do
      is_expected.to execute_with('-m foo -b bar')
        .and_output(/Create the filtered branch: bar-foo/)
        .and_generate_commands(/git filter-branch.*bar-foo/m)
    end
  end

  describe 'MonoRepoBranch' do
    let(:subject) { MonoRepoBranch }

    it 'parses options' do
      mrb = MonoRepoBranch.invoke([])
      expect(mrb.destructive?).to eq(false)
      expect(mrb.skip_filter_branches?).to eq(false)
    end

    it 'generates the latest branch' do
      is_expected.to execute_with('')
        .and_output([
          /Generate the kearney monorepo branch/,
          /Recreating current module branches/,
        ])
        .and_generate_commands([
          /git filter-branch.*Original Commit/m,
          /git merge .* kearney-puppet_enterprise/,
          /git merge .* kearney-pe_install/,
        ])
    end

    it 'raises an error if --branch is not a downstream branch target' do
      testio = StringIO.new
      expect { subject.invoke('--branch foo'.split, testio).execute }.to(
        raise_error(MonoRepoSubCommand::MonoRepoException, /The downstream branch to build must be one of these targets/)
      )
    end

    it 'halts if generating a downstream --branch when source branch does not exist' do
      TestExecutor.add_response('git branch --list kearney', '')
      is_expected.to execute_with('--branch johnson')
        .and_fail_with(/The kearney source monorepo branch does not exist/)
    end

    it 'generates an older downstream --branch' do
      TestExecutor.add_response('git branch --list kearney', 'kearney')
      is_expected.to execute_with('--branch johnson')
        .and_output([
          /Generate the johnson monorepo branch from references in kearney/,
        ])
        .and_generate_commands([
          /git rev-parse/,
          /git log .* --grep/,
          /git merge/,
        ])
    end

    it 'does not recreate branch without --destructive'
  end

  describe 'MonoRepoPush' do
    let(:subject) { MonoRepoPush }
    let(:module_branches) do
      [
        '2015.2.x-pe_concat',
        '2015.3.x-pe_staging',
        '2016.2.x-pe_hocon',
        '2016.2.x-pe_puppet_authorization',
        '2016.4.x-pe_java_ks',
        '2016.5.x-pe_accounts',
        '2016.5.x-pe_inifile',
        '2016.5.x-pe_postgresql',
        'glisan-pe_nginx',
        'kearney-pe_infrastructure',
        'kearney-pe_install',
        'kearney-pe_manager',
        'kearney-pe_repo',
        'kearney-puppet_enterprise',
      ]
    end

    it 'parses options' do
      fmb = MonoRepoPush.invoke(['--tags', '--remote', 'foo'])
      expect(fmb.tags?).to eq(true)
      expect(fmb.remote).to eq('foo')
    end

    it 'pushes tags' do
      is_expected.to execute_with('--tags --remote foo')
        .and_output(/Push the tags to the remote foo/)
        .and_generate_commands(%r{git push foo module/puppet_enterprise/2018\.1\.1 module/puppet_enterprise/2018\.1\.2})
    end

    it 'pushes module branches' do
      is_expected.to execute_with('--module-branches=bar --remote foo')
        .and_output(/Push the bar module branches to the remote foo/)
        .and_generate_commands(%r{git push foo #{module_branches.join(' ')}})
    end

    it 'pushes all'

    it 'purges tags' do
      is_expected.to execute_with('--tags --remote foo --purge --force')
        .and_output(/Purge the tags from the remote foo/)
        .and_generate_commands(%r{git push foo :module/puppet_enterprise/2018\.1\.1 :module/puppet_enterprise/2018\.1\.2 :module/puppet_enterprise/2018\.1\.3})
    end

    it 'purges module branches' do
      purged_module_branches = module_branches.map { |b| ":#{b}" }
      is_expected.to execute_with('--module-branches=bar --remote foo --purge --force')
        .and_output(/Purge the bar module branches from the remote foo/)
        .and_generate_commands(%r{git push foo #{purged_module_branches.join(' ')}})
    end

    it 'purges all'

    it 'demands confirmation if purging without --force' do
      expect { is_expected.to execute_with('--module-branches=bar --remote foo --purge') }.to(
        raise_error(MonoRepoSubCommand::MonoRepoException, /Request to purge not validated/)
      )
    end
  end

  describe 'CreateRemotes' do
    let(:subject) { CreateRemotes }

    it 'responds to --help' do
      is_expected.to execute_with('--help')
        .and_output(/create-remotes:.*Create remote references/m)
    end

    it 'generates remote refs for all the modules' do
      TestExecutor.add_response('git remote show puppet_enterprise', [%q{fatal: 'puppet_enterprise' does not appear to be a git repository}, 128])
      TestExecutor.add_response('git remote show pe_install', [%q{fatal: 'puppet_enterprise' does not appear to be a git repository}, 128])

      is_expected.to execute_with('')
        .and_output(/Create and fetch the module remotes/)
        .and_generate_commands([
          /git remote show puppet_enterprise/,
          /git remote add puppet_enterprise/,
          /git fetch puppet_enterprise/,
          /git remote show pe_install/,
          /git remote add pe_install/,
          /git fetch pe_install/,
        ])
    end
  end
end
