#! /usr/bin/env ruby

require 'optparse'
require 'pp'
require 'open3'

module ParseOptions

  class HelpExit < RuntimeError; end

  # Placeholder for specific help text for the subcommand
  def command_banner
    ''
  end

  # Placeholder for specific option parsers for the subcommand
  def set_command_options(parser); end

  def option_parser
    parser = OptionParser.new

    common_banner = <<-EOS
Usage:
  mono-repo.rb <generate|filter-module-branch|push|create-remotes> [options]

  To get help for a specific subcommand run the subcommand with the --help flag.

    EOS

    parser.banner = common_banner + command_banner

    set_command_options(parser)

    parser.on('--debug', 'Add full command output') do
      self.options[:debug] = true
    end

    parser.on('-h', '--help', 'This message.') do
      io.puts parser
      raise ParseOptions::HelpExit
    end

    return parser
  end

  # Main routine for parsing commandline options
  def parse_options(args = {})
    self.option_parser.parse!(args)
  rescue OptionParser::InvalidOption => e
    io.puts option_parser
    io.puts "\n!! #{e.message}\n"
    raise(e)
  end
end

# Subclassed by each subcommand for option parsing.
class BaseOptions
  include ParseOptions

  attr_accessor :options, :commands, :io

  def initialize(args, io = $stdout)
    self.io = io
    self.options = {}
    self.commands = parse_options(args)
  end
end

# This class is initialized and called by the script; it dispatches
# to subcommands based on the argument left over after options are parsed.
class Dispatcher

  include ParseOptions

  attr_accessor :args, :options, :io

  def initialize(args)
    self.args = args.dup
    self.options = {}
    self.io = $stdout
  end

  def run
    action = self.args.first
    klass = case action
    when 'generate'
      MonoRepoBranch
    when 'filter-module-branch'
      FilterModuleBranch
    when 'push'
      MonoRepoPush
    when 'create-remotes'
      CreateRemotes
    end

    code = false
    if klass
      begin
        subcommand = klass.invoke(args)
        code = subcommand.execute
      rescue RunShell::ActionFailed,OptionParser::InvalidOption
        code = 1
      rescue ParseOptions::HelpExit
        code = 0
      end
    else
      io.puts option_parser
    end
    return code
  end
end

# Shared code for executing commands through the shell
module RunShell

  class ActionFailed < StandardError; end

  # Whether or not the command was invoked with --debug
  def debugging?
    @debug
  end

  # Whether or not something has set @muzzle to true to temporarily block output
  def muzzled?
    @muzzle
  end

  def colorize(color_code, string)
    "\e[#{color_code}m#{string}\e[0m"
  end

  def red(string)
    colorize(31, string)
  end

  def green(string)
    colorize(32, string)
  end

  def cyan(string)
    colorize(36, string)
  end

  def grey(string)
    colorize(37, string)
  end

  def out(string, options = {})
    bump_level = options[:bump_level] || 0
    io.puts indent(string, bump_level)
    return true
  end

  def indent(string, bump_level = 0)
    indentation = "  " * (@level + bump_level)
    string.split("\n").map { |s| indentation + s }.join("\n")
  end

  def action(message, options = {})
    @original_muzzle = @muzzle
    @muzzle = options[:muzzle]
    @level ||= 0
    out(green("* #{message} "))
    out(grey('(output suppressed...run with --debug for all output)'), :bump_level => 1) if muzzled? && !debugging?
    @level += 1
    successful = yield
    if !successful
      out(red("Action failed!"))
      raise(ActionFailed, "Failed on step: #{message}")
    end
    return successful
  ensure
    @muzzle = @original_muzzle
    @level -= 1
  end

  class Executor
    def exec(command)
      Open3.capture2e(command)
    end
  end

  def self.executor
    Executor.new
  end

  def run(command, options = {})
    capture = options[:capture]
    test = options[:test]

    stdout_and_err, status = RunShell.executor.exec(command)

    failed = !status.success?
    out(cyan(command)) if debugging? || !muzzled?
    if debugging? || (!muzzled? && failed && !test)
      output = status.success? ? stdout_and_err : red(stdout_and_err)
      out(output, :bump_level => 1)
      out(grey("status: #{status}"), :bump_level => 2 )
    end
    return capture ? stdout_and_err : status.success?
  end

  # Execute a command and return the captured stdout and err streams (in one String).
  def capture(command)
    run(command, :capture => true)
  end

  # Execute a command and return true for a successful status or false if it fails.
  # Failure is not printed to stdout unless debugging.
  def test(command)
    run(command, :test => true)
  end
end

# Shell actions shared by multiple subcommands
module CommonActions
  def branch_exists?(name)
    branch_output = capture("git branch --list #{name}")
    # The checked out branch is prefixed with '* '
    branch_output.gsub(/^\* /, '').strip == name
  end

  def delete_branch(name, ok_to_destroy = false)
    action("Delete the #{name} branch, if it exists") do
      case
      when branch_exists?(name)
        if ok_to_destroy
          # You can't delete a branch if you happen to have it checked out,
          # so checkout master first...which assumes we aren't generating a
          # 'master' branch
          run("git checkout master") &&
            run("git branch -D #{name}")
        else
          out(red("The #{name} branch already exists!"))
          out("If you want it deleted and recreated from scratch please re-run this script with the --destructive flag")
          false
        end
      else
        out(grey('no branch to delete'), :bump_level => 1)
      end
    end
  end

  def create_branch(name, start, options = {})
    tracking = options[:tracking]
    action("Create #{name} #{tracking ? 'tracking' : 'pointing to'} #{start}, unless it exists") do
      case
      when !branch_exists?(name)
        tracking_arg = tracking ? '--track ' : ''
        run("git branch #{tracking_arg}#{name} #{start}")
      else
        out(grey('branch already exists'), :bump_level => 1)
      end
    end
  end

  def checkout(name)
    action("Checkout branch #{name}") do
      run("git checkout #{name}")
    end
  end

  def create_module_remote(module_name)
    action("Create the #{module_name} remote") do
      if test("git remote show #{module_name}")
        out(grey('remote already exists'), :bump_level => 1)
      else
        run("git remote add #{module_name} git@github.com:puppetlabs/puppetlabs-#{module_name}")
      end
    end
  end

  def fetch(module_name)
    action ("Fetch the #{module_name} remote") do
      run("git fetch #{module_name}")
    end
  end
end

# Abstract class for common functionality shared across the various
# mono-repo subcommands.
class MonoRepoSubCommand
  include RunShell
  include CommonActions

  class MonoRepoException < StandardError; end

  attr_accessor :options, :io

  def self.invoke(args, stdout_io = $stdout)
    parser = self::Options.new(args, stdout_io)
    self.new(parser.options.merge(:stdout_io => stdout_io))
  end

  def initialize(options)
    self.options = options
    self.io = options[:stdout_io] || $stdout
    @debug = options[:debug]
    @level = options[:level] || 0
  end

  def usage
    io.puts self.class::Options.new([], io).option_parser
  end

  def failure(message)
    usage
    io.puts(red("\n#{message}\n"))
    raise(MonoRepoException, message)
  end

  def execute
    fail('Reimplement in a concrete subcommand class.')
  end
end

# This class can be used to filter one remote PE module repo
# branch into a local branch isolated within a subdirectory.
#
# So if called for the puppetlabs-puppet_enterprise modules kearney branch, it
# will create puppet_enterprise remote in the current module and then take
# puppet_enterprise/kearney and produce a local branch called
# kearney-puppet_enterprise with the module now namespaced under the
# puppet_enterprise/ subdirectory.  All of the tags relevant to the branch will
# be regenerated under modules/puppet_enterprise/<tag> as well.
#
# Because tags are shared, tag handling is tricky.
#
# The main modules, puppet_enterprise, pe_install, pe_manager, etc., are likely
# to share key tags, such as 2018.1.4, 2019.0.0 and so forth. There can only be one
# tag in a repository, and since we're adding remotes for multiple unrelated
# repositories, there are conflicts for the tags.
#
# Whatever module remote is added first (say puppetlabs-puppet_enterprise) will
# fetch its tags. A later addition of puppetlabs-pe_install will only fetch
# tags that are not already present.
#
# This is why the delete_tags step is used to clear out overlapping tags before
# fetching a module for filter branching, and to clear them out afterwards.
class FilterModuleBranch < MonoRepoSubCommand

  class Options < BaseOptions
    def command_banner
      <<-EOS
      filter-module-branch -m <MODULE> -b <BRANCH>:

        Generate a local <branch_name>-<module_name> branch that recreates a
        remote <repo_name>/<branch_name> within a subdirectory namespaced as
        <prefix>/<module_name>

      EOS
    end

    def set_command_options(parser)
      parser.on('-m', '--module_name=MODULE', 'The module to run through filter-module-branch.') do |a|
        self.options[:module_name] = a
      end

      parser.on('-b', '--branch_name=BRANCH', 'The branch of the module to run through filter-module-branch.') do |a|
        self.options[:branch_name] = a
      end
    end
  end

  attr_accessor :module_name, :branch_name

  def initialize(options)
    super(options)
    self.module_name = options[:module_name]
    failure('No module name given') if module_name.nil?
    self.branch_name = options[:branch_name]
    failure('No branch given') if branch_name.nil?
  end

  def filtered_branch_name
    "#{branch_name}-#{module_name}"
  end

  # Takes the output of git ls-remote -t <module_name> and returns
  # just the tag names. So this text output:
  #
  #   293c3e78381a5b17ae912abb60572dc24be71128        refs/tags/2018.1.0
  #   fbba12fdc0cc49838208de65a38cb7619c905b1b        refs/tags/2018.1.0^{}
  #   96ae9e7bf5bcb507dc79463a13bce365d3d5f82b        refs/tags/2018.1.0-rc0
  #   de4a842d63b9c86154a96c6bad5859e3f42c00cf        refs/tags/2018.1.0-rc0^{}
  #   ecbe29713d7aef9082530e4fd8639003c9619f00        refs/tags/2018.1.0-rc1
  #   64f2a3f849205a4884bf2bb30971bfcf8a0c430d        refs/tags/2018.1.0-rc1^{}
  #
  # gets returned as an array of strings:
  #
  #   [
  #     "2018.1.0",
  #     "2018.1.0-rc0",
  #     "2018.1.0-rc1",
  #   ]
  def list_remote_tags
    output = nil

    action("Get a list of all tags in the #{module_name} remote") do
      output = capture("git ls-remote -t #{module_name}")
    end

    tags = output.split("\n")
    tags.map! do |t|
      ref = t.split.last
      ref.split('/').last
    end
    tags.reject! { |t| t =~ /\^{}$/ }
  end

  def delete_tags(tags)
    action("Delete the tags", :muzzle => true) do
      successful = tags.map do |tag|
        capture("git tag -l #{tag}").strip == tag ?
          run("git tag -d #{tag}") :
          true
      end
      successful.all? { |t| t == true }
    end
  end

  # Use git filter-branch to move to subdir and regenerate tags under
  # module/#{module_name}/*
  def filter_branch
    subdirectory_prefix = "modules/#{module_name}"
    action("Filter the #{filtered_branch_name} to shift it into #{subdirectory_prefix}") do
      # Keep a reference to the original git commit sha
      env_filter = "ORIGINAL_COMMIT_SHA=$GIT_COMMIT"
      # Reset each commit tree to be under our subdirectory namespace
      index_filter = "rm $GIT_INDEX_FILE && git read-tree --prefix #{subdirectory_prefix} $GIT_COMMIT"
      # Reset each tag to be prefixed as module/#{module_name}/
      tag_filter = %Q{sed -e "s!^!module/#{module_name}/!"}
      # Append the original commit SHA to each new commit
      msg_filter = %Q{cat && echo "\n\nOriginal Commit: puppetlabs/puppetlabs-#{module_name}@$ORIGINAL_COMMIT_SHA"}
      run(%Q{git filter-branch --force --env-filter '#{env_filter}' --index-filter '#{index_filter}' --tag-name-filter '#{tag_filter}' --msg-filter '#{msg_filter}' #{filtered_branch_name}})
    end
  end

  def execute
    action("Create the filtered branch: #{filtered_branch_name}") do
      create_module_remote(module_name)
      remote_tags = list_remote_tags
      delete_tags(remote_tags)
      fetch(module_name)
      delete_branch(filtered_branch_name, true)
      create_branch(filtered_branch_name, "#{module_name}/#{branch_name}", :tracking => true)
      checkout(filtered_branch_name)
      filter_branch
      delete_tags(remote_tags)
    end
  end
end

# Metadata mapping repo/branches to a target monorepo branch.
# This could be moved over to a config file.
module BranchTargets
  COMMON = {
    'pe_accounts' => '2016.5.x',
    'pe_concat' => '2015.2.x',
    'pe_hocon' => '2018.1.x',
    'pe_inifile' => '2016.5.x',
    'pe_java_ks' => '2016.4.x',
    'pe_nginx' => 'glisan',
    'pe_postgresql' => '2016.5.x',
    'pe_puppet_authorization' => '2016.2.x',
    'pe_staging' => '2015.3.x',
  }.freeze

  def self.generate_branch_targets(for_branch)
    {
      'puppet_enterprise' => for_branch,
      'pe_install' => for_branch,
      'pe_manager' => for_branch,
      'pe_infrastructure' => for_branch,
      'pe_repo' => for_branch,
    }.merge(COMMON).freeze
  end

  KEARNEY = generate_branch_targets('kearney')
  JOHNSON = generate_branch_targets('johnson')
  IRVING = generate_branch_targets('irving')

  TARGET = KEARNEY
  TARGET_BRANCH_NAME = 'kearney'

  DOWNSTREAM_BRANCHES = [
    'johnson',
    'irving',
  ]

  def self.downstream_branches
    DOWNSTREAM_BRANCHES.join(', ')
  end

  # Output a branch map for help text.
  def self.help_map(target)
    branches = const_get(target.upcase)
    branches.map { |k,v| "          #{k} : #{v}" }.join("\n")
  end

  def self.get_branch_targets(for_branch)
    const_get(for_branch.upcase)
  end
end

# Subcommand for generating or regenerating the monorepo branch from a set
# of module branches.
class MonoRepoBranch < MonoRepoSubCommand

  class Options < BaseOptions
    def command_banner
      source_branch = BranchTargets::TARGET_BRANCH_NAME
      downstream_branches = BranchTargets::downstream_branches
      first_downstream_branch = BranchTargets::DOWNSTREAM_BRANCHES.first

      <<-EOS
      generate:

        Build the source monorepo branch by running filter-module-branch on all
        of these module and branch references:

#{BranchTargets.help_map(source_branch)}

        and then merging the resultant re-written <branch_name>-<module_name> branches
        into the #{source_branch} branch.

        Alternatively, if the latest source branch has already been built (currently
        #{source_branch}), then you can build one of it's older, downstream
        branches (#{downstream_branches}) from references in #{source_branch}
        that match up with the HEADS of the downstream module branches.

        (see --branch)

        For example, to create the #{first_downstream_branch} branch,
        for each reference in:

#{BranchTargets.help_map(first_downstream_branch)}

        Every filtered <branch_name>-<module-name> which alredy exists from building
        #{source_branch} will be merged in as is. All other module targets
        such as puppet_enterprise will merge in the #{source_branch} commit
        that matches the current puppet_enterprise/#{first_downstream_branch} (based
        on the Original Commit tracker in the rewritten branches...)

        This #{first_downstream_branch} monorepo branch should then have all the
        re-written #{first_downstream_branch} module commits and should merge up
        cleanly.

      EOS
    end

    def set_command_options(parser)
      parser.on("--branch=BRANCH", "Generate a downstream monorepo BRANCH for one of the available targets [#{BranchTargets.downstream_branches}].") do |b|
        self.options[:downstream_branch] = b
      end

      parser.on('--destructive', 'Destroy and recreate the final branch') do
        self.options[:destructive] = true
      end

      parser.on('--skip-filter-branches', 'When generating/regenerating the monorepo branch, skip creating each module filter branch (assumes these have already been generated).') do
        self.options[:skip_filter_branches] = true
      end
    end
  end

  include BranchTargets

  attr_accessor :downstream_branch

  def initialize(options)
    super(options)
    self.downstream_branch = options[:downstream_branch]
    downstream_branches = BranchTargets::DOWNSTREAM_BRANCHES
    failure("The downstream branch to build must be one of these targets: #{downstream_branches}") if !self.downstream_branch.nil? && !downstream_branches.include?(self.downstream_branch)
  end

  def generate_module_branch(module_name, branch)
    mono = FilterModuleBranch.new(
      options.merge(
        :module_name => module_name,
        :branch_name => branch,
        :level => 2
      )
    )
    mono.execute
  end

  def destructive?
    !!options[:destructive]
  end

  def skip_filter_branches?
    !!options[:skip_filter_branches]
  end

  def source_branch
    TARGET_BRANCH_NAME
  end

  def generating_source_branch?
    downstream_branch.nil?
  end

  def generating_downstream_branch?
    !generating_source_branch?
  end

  def recreate_module_branches
    action("Recreating current module branches") do
      KEARNEY.each do |m,b|
        generate_module_branch(m, b)
      end
    end
  end

  def merge_filtered_module_branches(branch_targets)
    branch_targets.map do |m,b|
      filtered_module_branch = "#{b}-#{m}"
      run("git merge --no-ff --log #{filtered_module_branch} --allow-unrelated-histories")
    end
  end

  def merge_from_source_monorepo_branch(branch_targets)
    branch_targets.map do |m,b|
      module_branch = "#{m}/#{b}"
      module_branch_head = capture("git rev-parse #{module_branch}").strip
      original_commit_message = "Original Commit: puppetlabs/puppetlabs-#{m}@#{module_branch_head}"
      filtered_ref = capture("git log --format=format:%H --grep '#{original_commit_message}' kearney").strip
      run("git merge --no-ff --log #{filtered_ref} --allow-unrelated-histories")
    end
  end

  def merge_all_refs_into_monorepo_branch(target_branch)
    branch_targets = BranchTargets.get_branch_targets(target_branch)
    action("Merge all module branch refs into the monorepo branch") do
      # Ensure we have the correct target branch checked out first
      run("git checkout #{target_branch}")
      successful = generating_source_branch? ?
        merge_filtered_module_branches(branch_targets) :
        merge_from_source_monorepo_branch(branch_targets)
      successful.all? { |s| s == true }
    end
  end

  def verify_branch_exists?(branch_name)
    action("Verify that the #{branch_name} source monorepo branch exists") do
      if branch_exists?(branch_name)
        true
      else
        out(red("The #{branch_name} source monorepo branch does not exist, so we cannot create a downstream branch from it."))
        false
      end
    end
  end

  def execute
    description = destructive? ? 'Recreate' : 'Generate'
    target_branch = generating_source_branch? ? source_branch : downstream_branch
    object = generating_source_branch? ?
      "module targets" :
      "references in #{source_branch}"

    action("#{description} the #{target_branch} monorepo branch from #{object}") do
      verify_branch_exists?(source_branch) if generating_downstream_branch?
      delete_branch(target_branch, destructive?)
      recreate_module_branches if generating_source_branch? && !skip_filter_branches?
      create_branch(target_branch, 'master')
      checkout(target_branch)
      merge_all_refs_into_monorepo_branch(target_branch)
    end
  end
end

# Subcommand for pushing the artifacts related to a branch: tags, sub module
# branches.
#
# Also provides a mechanism for purging all of these artifacts from a remote
# if you are recreating a updated version of the mono repo master branch.
#
# Useful for testing, purging and regenerating monorepo builds, since any
# given build of a monorepo branch falls out of date over time until we cutover
# to using it instead of the individual module branches.
class MonoRepoPush < MonoRepoSubCommand

  class Options < BaseOptions
    def command_banner
      <<-EOS
      push (--tags | --module-branches=<BRANCH> | --all=<BRANCH>) --remote=<REMOTE> [--purge]:

        Git push either module tags or module branches or both up to a given remote.
        Alternatively can be used to purge the same from a given remote.

      EOS
    end

    def set_command_options(parser)
      parser.on('--tags', 'Push all tags in the "module/*" namespace up to --remote') do
        self.options[:tags] = true
      end

      parser.on('--module-branches=BRANCH', 'Push all of the re-written <BRANCH>-<*module*> sub branches up to the --remote. (Since these are present in the monorepo branch, this is only useful if you want them for reference.)') do |b|
        self.options[:branch_name] = b
      end

      parser.on('--all=BRANCH', 'Push all module tags and module-branches matching <BRANCH> to the given --remote') do |b|
        self.options[:tags] = true
        self.options[:branch_name] = b
      end

      parser.on('--remote=REMOTE', 'The git remote specification to push to.') do |r|
        self.options[:remote] = r
      end

      parser.on('--purge', 'Remove the selected artifacts from the chosen --remote.') do
        self.options[:purge] = true
      end

      parser.on('--force', 'Force actions without confirmation (used to force a purge).') do
        self.options[:force] = true
      end
    end
  end

  include BranchTargets

  attr_accessor :branch_name, :remote

  def initialize(options)
    super(options)
    self.branch_name = options[:branch_name]
    failure('No targets to push') if !tags? && branch_name.nil?
    self.remote = options[:remote]
    failure('No remote given') if remote.nil?
  end

  def tags?
    options[:tags]
  end

  def branches?
    !options[:branch_name].nil?
  end

  def purge?
    options[:purge]
  end

  def force?
    options[:force]
  end

  def valid?
    valid = !purge? || (purge? && force?)
    !!valid
  end

  def get_local_module_tags
    output = nil
    action('Capture a list of all local tags in the module/* namespace') do
      output = capture(%q{git tag -l 'module/*'})
    end
    output.split("\n")
  end

  def get_local_module_branches
    KEARNEY.map do |mod,branch|
      "#{branch}-#{mod}"
    end.sort
  end

  # Prepend ':' to each reference, which will cause git push to
  # delete that reference on the remote.
  def rename_remote_refs_to_delete(refs)
    refs.map { |r| ":#{r}" }
  end

  def _push(type, refs, purge = false)
    verb = purge ? "Purge" : "Push"
    action("#{verb} the #{type}") do
      final_refs = purge ? rename_remote_refs_to_delete(refs) : refs
      run("git push #{remote} #{final_refs.join(' ')}")
    end
  end

  def execute
    artifacts = []
    artifacts << 'tags' if tags?
    artifacts << "#{branch_name} module branches" if branches?
    artifacts = artifacts.join(' and ')

    verb = purge? ? "Purge" : "Push"
    preposition = purge? ? "from" : "to"

    failure('Request to purge not validated') if !valid?

    action("#{verb} the #{artifacts} #{preposition} the remote #{remote}") do
      success = true
      success &&= _push('tags', get_local_module_tags, purge?) if tags?
      success &&= _push('branches', get_local_module_branches, purge?) if branches?
      success
    end
  end
end

# This is a simple helper to automate generating remote repository references
# for the original PE module repositories.
class CreateRemotes < MonoRepoSubCommand

  class Options < BaseOptions
    def command_banner
      modules = BranchTargets::KEARNEY.keys
      <<-EOS
      create-remotes:

        Create remote references for each of the original PE modules and fetches.
        If the module remote already exists, only a fetch is done.

        #{modules}

      EOS
    end
  end

  include BranchTargets

  def modules
    BranchTargets::KEARNEY.keys
  end

  def execute
    action('Create and fetch the module remotes') do
      successful = modules.map do |m|
        create_module_remote(m)
        fetch(m)
      end
      successful.all? { |s| s == true }
    end
  end
end

# Execute if the script is called on the command line.
if $0 == __FILE__
  monorepo = Dispatcher.new(ARGV)
  result = monorepo.run
  exit(result)
end
